/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "SrvParms.h"

#include "mapstuff.h"
#include "verinfo.h"
#include "worldmain.h"
#include <vector>

#include "wolfpack.h"
#include "network.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "SrvParms.cpp"

////////////////////
// name:	cKeyWord, cSection etc.
// purpose:	handle the key=value stuff in server.scp
// history:	by Duke, Oct 2003
// remarks:	I'm still not content with the way the default values are handled, so do not *deploy* these classes !
//
enum eKeyWordTypes
{
	KWT_dec_uint = 1,	// decimal stored as an unsigned int
	KWT_dec_int,		// decimal stored as an int
	KWT_flo_flo,		// float
	KWT_str_str,		// a string ;)
	KWT_cha_char,		// a single character
	KWT_dec_char,		// a number stored in a char
	KWT_str_sp1			// special handler 1: for defaultfood
};

class cKeyWord
{
	char _key[50];
	eKeyWordTypes _type;
	void *_data;
public:
	cKeyWord(char* key, eKeyWordTypes type, unsigned int* data, const unsigned int def);
	cKeyWord(char* key, eKeyWordTypes type, int* data, const int def);
	cKeyWord(char* key, eKeyWordTypes type, float*data, const double def);
	cKeyWord(char* key, eKeyWordTypes type, char*data, const char* def);
	cKeyWord(char* key, eKeyWordTypes type, unsigned char* data, const unsigned char def);
	void Save(FILE* fp);
	bool Load(char* key, char* parm);
};

cKeyWord::cKeyWord(char* key, eKeyWordTypes type, unsigned int* data, const unsigned int def)
{
	strcpy(_key,key);
	_type=type;
	_data=data;
	switch(_type)
	{
	case KWT_dec_uint: *((unsigned int*)_data) = def; break;
	default:	LogErrorVar("invalid type of keyword %i",_type);
	}
}

cKeyWord::cKeyWord(char* key, eKeyWordTypes type, int* data, const int def)
{
	strcpy(_key,key);
	_type=type;
	_data=data;
	switch(_type)
	{
	case KWT_dec_int: *((int*)_data) = (int)def; break;
	default:	LogErrorVar("invalid type of keyword %i",_type);
	}
}

// float constants can't be converted to void*s, so we need another constructor
// update: forget it, results in 'ambigous call'
cKeyWord::cKeyWord(char* key, eKeyWordTypes type, float*data, const double def)
{
	strcpy(_key,key);
	_type=type;
	_data=data;
	switch(_type)
	{
	case KWT_flo_flo: *((float*)_data) = def; break;
	default:	LogErrorVar("invalid type of keyword %i",_type);
	}
}

cKeyWord::cKeyWord(char* key, eKeyWordTypes type, char* data, const char* def)
{
	strcpy(_key,key);
	_type=type;
	_data=data;
	switch(_type)
	{
	case KWT_str_str: strcpy((char*)_data, def); break;
	default:	LogErrorVar("invalid type of keyword %i",_type);
	}
}

cKeyWord::cKeyWord(char* key, eKeyWordTypes type, unsigned char* data, const unsigned char def)
{
	strcpy(_key,key);
	_type=type;
	_data=data;
	switch(_type)
	{
	case KWT_cha_char:
	case KWT_dec_char:
	case KWT_str_sp1: *data = def; break;
	default:	LogErrorVar("invalid type of keyword %i",_type);
	}
}

bool cKeyWord::Load(char* key, char* parm)
{
	if (strcmp(_key,key)) return false;	// not what we're looking for

	char* p = NULL;

	switch(_type)
	{
	case KWT_dec_uint:
	case KWT_dec_int: *((int*)_data) = str2num(parm);	break;
	case KWT_flo_flo: *((float*)_data) = atof(parm);	break;
	case KWT_cha_char:*((char*)_data) = *parm;			break;
	case KWT_dec_char:*((char*)_data) = str2num(parm);	break;
	case KWT_str_str:
		strcpy((char*)_data, parm);
		p = (char*)_data + strlen(parm) - 1;
		while (p >= _data)
		{
			if (*p >=33 && *p <=126)	// valid char
				break;
			*p = 0;
			p--;  
		}
		break;
	case KWT_str_sp1:
		*((unsigned char*)_data) = 0;
		if (strstr(parm,"BAKED"))		SrvParms->defaultfood |= FOOD_BAKED;
		if (strstr(parm,"CHEESE"))		SrvParms->defaultfood |= FOOD_CHEESE;
		if (strstr(parm,"FISH"))		SrvParms->defaultfood |= FOOD_FISH;
		if (strstr(parm,"FRUITS"))		SrvParms->defaultfood |= FOOD_FRUIT;
		if (strstr(parm,"MAGIC"))		SrvParms->defaultfood |= FOOD_MAGIC;
		if (strstr(parm,"MEAT"))		SrvParms->defaultfood |= FOOD_MEAT;
		if (strstr(parm,"RAW"))			SrvParms->defaultfood |= FOOD_RAW;
		if (strstr(parm,"VEGETABLES"))	SrvParms->defaultfood |= FOOD_VEGETABLE;
		break;
	default:	LogErrorVar("invalid type of keyword %i",_type);
	}
	return true;
}

void cKeyWord::Save(FILE* fp)
{
	switch(_type)
	{
	case KWT_dec_uint:
	case KWT_dec_int:	fprintf(fp,"%s %i\n", _key, *((int*)_data));	break;
	case KWT_str_str:	fprintf(fp,"%s %s\n", _key, (char*)_data);		break;
	case KWT_flo_flo:	fprintf(fp,"%s %f\n", _key, *((float*)_data));	break;
	case KWT_cha_char:	fprintf(fp,"%s %c\n", _key, *((char*)_data));	break;
	case KWT_dec_char:	fprintf(fp,"%s %i\n", _key, *((char*)_data));	break;
	case KWT_str_sp1:
		{
		char tempfood[255] = "DEFAULTFOOD ";
		if ((SrvParms->defaultfood&FOOD_MEAT) == FOOD_MEAT) strcat(tempfood,"MEAT+");
		if ((SrvParms->defaultfood&FOOD_FISH) == FOOD_FISH) strcat(tempfood,"FISH+");
		if ((SrvParms->defaultfood&FOOD_CHEESE) == FOOD_CHEESE) strcat(tempfood,"CHEESE+");
		if ((SrvParms->defaultfood&FOOD_FRUIT) == FOOD_FRUIT) strcat(tempfood,"FRUITS+");
		if ((SrvParms->defaultfood&FOOD_VEGETABLE) == FOOD_VEGETABLE) strcat(tempfood,"VEGETABLES+");
		if ((SrvParms->defaultfood&FOOD_BAKED) == FOOD_BAKED) strcat(tempfood,"BAKED+");
		if ((SrvParms->defaultfood&FOOD_RAW) == FOOD_RAW) strcat(tempfood,"RAW+");
		if ((SrvParms->defaultfood&FOOD_MAGIC) == FOOD_MAGIC) strcat(tempfood,"MAGIC+");

		int len = strlen(tempfood);
		if (len>12) tempfood[len-1] = 0; //remove the last "+"
		else strcpy(tempfood,"DEFAULTFOOD MEAT+FISH+CHEESE+FRUITS+VEGETABLES+BAKED+RAW+MAGIC"); //eat everything by default
		fprintf(fp,"%s\n", tempfood);
		}
		break;
	default:	LogErrorVar("invalid type of keyword %i",_type);
	}
}


class cSection
{
protected:
	char _secname[50];
	vector<cKeyWord*> _keys;
public:
	cSection(char* sname);
	void Add(cKeyWord* kw);
	bool Load(char* key);
	bool LoadKeywords(char* key, char* parm);
	void SaveKeywords(FILE* fp);
	void Save(FILE* fp);
};

cSection::cSection(char* sname)
{
	strcpy(_secname,sname);
}

void cSection::Add(cKeyWord* kw)
{
	if (!kw) return;
	_keys.push_back(kw);
}

bool cSection::LoadKeywords(char* key, char* parm)
{
	unsigned int i;
	for (i=0;i < _keys.size();i++)
	{
		if (_keys[i]->Load(key,parm))
			return true;
	}
	return false;
}

bool cSection::Load(char* key)
{
	if (strcmp(_secname,key)) return false;	// not what we're looking for

	int loopexit=0;
	do
	{
		readwsc2();
		if (!LoadKeywords(script1,script2) && !(script1[0]=='}'))
			LogWarningVar("Invalid keyword '%s' in server script",script1);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
	return true;
}

void cSection::SaveKeywords(FILE* fp)
{
	unsigned int i;
	for (i=0;i < _keys.size();i++)
	{
		_keys[i]->Save(fp);
	}
}

void cSection::Save(FILE* fp)
{
	fprintf(fp, "SECTION %s\n",_secname);
	fprintf(fp, "{\n");
	SaveKeywords(fp);
	fprintf(fp, "}\n\n");
}


class cParmScript
{
protected:
	vector<cSection*> _sects;
public:
	cParmScript();
	void Add(cSection* sect);
	void Load(FILE* fp);
	bool LoadSections(char* key);
	void SaveSections(FILE* fp);
	void Save();
};

cParmScript::cParmScript()
{
}

void cParmScript::Add(cSection* kw)
{
	if (!kw) return;
	_sects.push_back(kw);
}

bool cParmScript::LoadSections(char* key)
{
	unsigned int i;
	for (i=0;i < _sects.size();i++)
	{
		if (_sects[i]->Load(key))
			return true;
	}
	return false;
}

void cParmScript::Load(FILE* fp)
{
	int loopexit=0;
	do
	{
		readwsc2();
		if (!strcmp(script1,"SECTION"))
			LoadSections(script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}

void cParmScript::SaveSections(FILE* fp)
{
	unsigned int i;
	for (i=0;i < _sects.size();i++)
	{
		_sects[i]->Save(fp);
	}
}

void cParmScript::Save()
{
	FILE *file;
	file = fopen("server.scp", "w");
	
	fprintf(file, "// LONEWOLF Server Script\n");
	fprintf(file, "// Generated by %s Version %s\n\n",  wp_version.betareleasestring.c_str() , wp_version.verstring.c_str() );

	SaveSections(file);

	fprintf(file, "EOF\n\n");
	fclose(file);
}

// important: loaddefault has to be called before. 
// currently it is. makes no sense to change that too.

void loadclientsallowed(void)
{
	int num_c=0, s=0, loopexit=0;
	vector<string> cls;    

	do
	{
		readwscFullLine();
		
		if (!strcmp(script1,"SERVER_DEFAULT")) { s=1; break; }
		else if (!strcmp(script1,"ALL")) { s=2; break; }
		else if (strlen(script1)>3) 
		{ 		
			num_c++;
			cls.push_back(script1);
		}
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );  

	if (num_c==0 && s==0) s=1; // no entry == pick server_default

	switch(s)
	{
	case 0: 
		clientsAllowed.clear();  // delete the entry from loaddefaults
		copy(cls.begin(), cls.end(), back_inserter(clientsAllowed) ); // copy temp vector to perm vector
		break;

	case 1: 
		break; // already in vector from loaddefaults

	case 2: 
		clientsAllowed.clear();    // delete server_defualt entry from loaddefaults
		clientsAllowed.push_back("ALL"); // now add the correct one
		break;
	}
}


void cSrvParms::Init()
{
	ParmScript = new cParmScript();

    clientsAllowed.push_back("SERVER_DEFAULT");

	SrvParms->antifastwalk=0;
	SrvParms->antiinjection=0;

	SrvParms->CryptKey1=0;
	SrvParms->CryptKey2=0;

	serv.sServer =  "A LoneWolf shard" ;
	serv.sIP="127.0.0.1";
	serv.uiFull=0;
	serv.uiIndex=0;
	serv.uiPort=2593;
	serv.uiTime=0;
	serv.AliasServer="NONE";
	serv.AliasIP="192.168.0.1";

	// Should we check character age for delete?	

	SrvParms->packetsendstyle=PSS_SMARTBUFFER;

	SrvParms->antimacro_exceptions=0;
	SrvParms->antimacro_inputtype=0;
	SrvParms->antimacro_queryfrequency=20; // default = 5*20=100minutes between queries
	SrvParms->antimacro_on=0;
	SrvParms->antimacro_querytype=0;
	
	SrvParms->snoopdelay=7;

	SrvParms->ra_port = 2594;
	SrvParms->EnableRA = false;

	cSection* sp = new cSection("MULS");
	sp->Add(new cKeyWord("MULPATH",				KWT_str_str,	&SrvParms->mulpath[0],		""));
	sp->Add(new cKeyWord("AUTODETECT",			KWT_dec_char,	&SrvParms->mul_autodetect,	1));
	ParmScript->Add(sp);

	sp = new cSection("BACKUP");
	sp->Add(new cKeyWord("ARCHIVEPATH",			KWT_str_str,	&SrvParms->archivepath[0],	""));
	sp->Add(new cKeyWord("BACKUP_SAVE_RATIO",	KWT_dec_int,	&SrvParms->backup_save_ratio,1));
	ParmScript->Add(sp);

	sp = new cSection("LOGGING");
	sp->Add(new cKeyWord("LOGGINGPATH",			KWT_str_str,	&SrvParms->logging_path[0],	""));
	sp->Add(new cKeyWord("SERVER_LOG",			KWT_dec_char,	&SrvParms->server_log,		0));
	sp->Add(new cKeyWord("SPEECH_LOG",			KWT_dec_char,	&SrvParms->speech_log,		0));
	sp->Add(new cKeyWord("PVP_LOG",				KWT_dec_char,	&SrvParms->pvp_log,			0));
	sp->Add(new cKeyWord("GM_LOG",				KWT_dec_char,	&SrvParms->gm_log,			0));
	sp->Add(new cKeyWord("CHAR_LOG",			KWT_dec_int,	&SrvParms->char_log,		0));
	ParmScript->Add(sp);

	sp = new cSection("WORLDSAVE");
	sp->Add(new cKeyWord("SAVEINTERVAL",		KWT_dec_uint,	&SrvParms->saveinterval,300));
	ParmScript->Add(sp);

	sp = new cSection("HTML");
	sp->Add(new cKeyWord("HTMLSAVEINTERVAL",	KWT_dec_int,	&SrvParms->html,		-1));
	ParmScript->Add(sp);

	sp = new cSection("MSGBOARDS");
	sp->Add(new cKeyWord("MSGBOARDPATH",		KWT_str_str,	&msgboardpath[0],		""));
	sp->Add(new cKeyWord("MSGPOSTACCESS",		KWT_dec_int,	&msgpostaccess,			0));
	sp->Add(new cKeyWord("MSGPOSTREMOVE",		KWT_dec_int,	&msgpostremove,			0));
	sp->Add(new cKeyWord("MSGRETENTION",		KWT_dec_int,	&msgretention,			30));
	sp->Add(new cKeyWord("ESCORTACTIVE",		KWT_dec_int,	&escortactive,			1));
	sp->Add(new cKeyWord("ESCORTINITEXPIRE",	KWT_dec_int,	&escortinitexpire,		86400));//(24 hours)
	sp->Add(new cKeyWord("ESCORTACTIVEEXPIRE",	KWT_dec_int,	&escortactiveexpire,	1800));	//(1/2 Hour)
	sp->Add(new cKeyWord("ESCORTDONEEXPIRE",	KWT_dec_int,	&escortdoneexpire,		1800));
	sp->Add(new cKeyWord("BOUNTYSACTIVE",		KWT_dec_int,	&bountysactive,			1));
	sp->Add(new cKeyWord("BOUNTYSEXPIRE",		KWT_dec_int,	&bountysexpire,			0));	//(0-Never - in days)
	ParmScript->Add(sp);

	sp = new cSection("SHOWINFOS");
	sp->Add(new cKeyWord("ANNOUNCE_WORLDSAVES",	KWT_dec_uint,	&announceworldsaves,	1));	// 0=no
	sp->Add(new cKeyWord("JOINMSG",				KWT_dec_uint,	&joinmsg,				1));	// 0=no
	sp->Add(new cKeyWord("PARTMSG",				KWT_dec_uint,	&partmsg,				1));	// 0=no
	sp->Add(new cKeyWord("SHOWDEATHANIM",		KWT_dec_uint,	&showdeathanim,			1));	// 0=no
	sp->Add(new cKeyWord("SHOW_CVCS_INFO_AT_LOGIN",KWT_dec_uint,&showCVCS,				1));	// 0=no // blackwind
	sp->Add(new cKeyWord("ERRORS_TO_CONSOLE",	KWT_dec_uint,	&errors_to_console,		0));	// 0=no
	sp->Add(new cKeyWord("LOGMODE",				KWT_dec_uint,	&logmode,				0));	// 0=nothing, bit x01 = network, x02 = mapstuff
	sp->Add(new cKeyWord("PACKETSTATS",			KWT_dec_uint,	&packetstats,			0));	// 0=no
	ParmScript->Add(sp);

	sp = new cSection("ACCOUNTS");
	sp->Add(new cKeyWord("AUTO_CREATE_ACCTS",	KWT_dec_uint,	&auto_a_create,			0));	// 0=no
	sp->Add(new cKeyWord("AUTO_RELOAD_ACCTS",	KWT_dec_uint,	&auto_a_reload,			0));	// 0=no
	sp->Add(new cKeyWord("CHECKCHAR_AGE",		KWT_dec_uint,	&checkcharage,			0));	// 0=no
	ParmScript->Add(sp);

	sp = new cSection("IT");
	sp->Add(new cKeyWord("COMMANDPREFIX",		KWT_cha_char,	&commandPrefix,			'\''));
	sp->Add(new cKeyWord("CHATPREFIX",			KWT_cha_char,	&chatPrefix,			'%'));
	sp->Add(new cKeyWord("SKILLCAP",			KWT_dec_uint,	&skillcap,				999999));	// default=no cap
	sp->Add(new cKeyWord("STATCAP",				KWT_dec_int,	&statcap,				999));		// default=no cap
	sp->Add(new cKeyWord("SKILLADVANCEMODIFIER",KWT_dec_uint,	&skilladvancemodifier,	1000));		// Default Modulo val for skill advance
	sp->Add(new cKeyWord("STATSADVANCEMODIFIER",KWT_dec_uint,	&statsadvancemodifier,	500));		// Default Modulo val for stats advance
	sp->Add(new cKeyWord("DECAYTIMER",			KWT_dec_uint,	&decaytimer,			300));		// 5 minute decay rate
	sp->Add(new cKeyWord("PLAYERCORPSEDECAYMULTIPLIER",KWT_dec_uint,&playercorpsedecaymultiplier,3));// Player's corpse decays 3x the normal decay rate
	sp->Add(new cKeyWord("LOOTDECAYSWITHCORPSE",KWT_dec_uint,	&lootdecayswithcorpse,	1));		// JM - 0 Loot disappears with corpse, 1 loot gets left when corpse decays
	sp->Add(new cKeyWord("INVISTIMER",			KWT_dec_uint,	&invisibiliytimer,		60));		// invisibility spell lasts 1 minute
	sp->Add(new cKeyWord("SKILLDELAY",			KWT_dec_uint,	&skilldelay,			5));		// Skill usage delay (5 seconds)
	sp->Add(new cKeyWord("SKILLLEVEL",			KWT_dec_uint,	&skilllevel,			5));		// up to how many times the minskill they can learn
	sp->Add(new cKeyWord("OBJECTDELAY",			KWT_dec_uint,	&objectdelay,			1));		// in seconds
	sp->Add(new cKeyWord("BANDAGEDELAY",		KWT_dec_uint,	&bandagedelay,			6));		// in seconds
	sp->Add(new cKeyWord("BANDAGEINCOMBAT",		KWT_dec_uint,	&bandageincombat,		1));		// 0=no, 1=yes; can use bandages while healer and/or patient are in combat ?
	sp->Add(new cKeyWord("BANDAGECANRES",		KWT_dec_uint,	&bandagecanres,			1));		// 0=no, 1=yes; can resurrect with bandages
	sp->Add(new cKeyWord("GATETIMER",			KWT_dec_uint,	&gatetimer,				30));
	sp->Add(new cKeyWord("GUARDSACTIVE",		KWT_dec_char,	&guardsactive,			1));
	sp->Add(new cKeyWord("BG_SOUNDS",			KWT_dec_char,	&bg_sounds,				2));
	sp->Add(new cKeyWord("POISONTIMER",			KWT_dec_uint,	&poisontimer,			180));		// lb
	sp->Add(new cKeyWord("SAVESPAWNREGIONS",	KWT_dec_uint,	&savespawns,			1));
	sp->Add(new cKeyWord("DEFAULTPRIVONE",		KWT_dec_uint,	&defaultpriv1,			0));
	sp->Add(new cKeyWord("DEFAULTPRIVTWO",		KWT_dec_uint,	&defaultpriv2,			0));
	sp->Add(new cKeyWord("ROGUE",				KWT_dec_uint,	&rogue,					1));
	sp->Add(new cKeyWord("INACTIVITYTIMEOUT",	KWT_dec_uint,	&inactivitytimeout,		300));		// seconds of inactivity until player will be disconnected
	sp->Add(new cKeyWord("CHAR_TIME_OUT",		KWT_dec_uint,	&quittime,				300));		// Instalog
	sp->Add(new cKeyWord("ALWAYSRELOGIN",		KWT_dec_char,	&alwaysrelogin,			0));
	sp->Add(new cKeyWord("MAXSTEALTHSTEPS",		KWT_dec_int,	&maxstealthsteps,		10));		// AntiChrist - stealth=100.0 --> 10 steps allowed
	sp->Add(new cKeyWord("RUNNINGSTAMINASTEPS",	KWT_dec_uint,	&runningstaminasteps,	15));		// AntiChrist - how often ( in steps ) stamina decreases while running
	sp->Add(new cKeyWord("PERSECUTION",			KWT_dec_int,	&persecute,				1));		// AntiChrist
	sp->Add(new cKeyWord("SPIRITSPEAKTIMER",	KWT_dec_uint,	&spiritspeaktimer,		30));		// spiritspeak lasts at least 30 seconds, additional time is calculated by Skill & INT;
	sp->Add(new cKeyWord("TAMED_DISAPPEAR",		KWT_dec_uint,	&tamed_disappear,		1));		// Ripper
	sp->Add(new cKeyWord("HOUSEINTOWN",			KWT_dec_uint,	&houseintown,			0));		// Ripper
	sp->Add(new cKeyWord("SHOPRESTOCK",			KWT_dec_uint,	&shoprestock,			1));		// Ripper
	sp->Add(new cKeyWord("HOUSEDECAY_SECS",		KWT_dec_uint,	&housedecay_secs,		604800));	// LB, one week
	sp->Add(new cKeyWord("DEFAULT_JAIL_TIME",	KWT_dec_uint,	&default_jail_time,		86400));	// blackwind
	sp->Add(new cKeyWord("NEWMAGICTEST",		KWT_dec_char,	&newmagictest,			0));
	sp->Add(new cKeyWord("ENGRAVING",			KWT_dec_int,	&engraving,				1));		// Values= 0(Disabled) or 1(Enabled) - allow engraving of weapons, etc.. when enabled
	sp->Add(new cKeyWord("SMELTABLEITEMS",		KWT_dec_int,	&smeltableitems,		1));		// Values= 0(Disabled) or 1(Enabled) - allow smelting of weapons, etc.. when enabled
	sp->Add(new cKeyWord("DEFAULTGMVIEW",		KWT_dec_int,	&defaultgmview,			2));
	sp->Add(new cKeyWord("IDLESERVERCHATTER",	KWT_dec_char,	&IDLESERVERCHATTER,		0));
	sp->Add(new cKeyWord("THEHARDWAY",			KWT_dec_int,	&thehardway,			0));
	sp->Add(new cKeyWord("FISHTOBP",			KWT_dec_int,	&fishtobp,				0));
	sp->Add(new cKeyWord("SEXTANTDISPLAY",		KWT_dec_uint,	&sextantDisplay,		1));
	sp->Add(new cKeyWord("SHOESWEAROUT",		KWT_dec_int,	&shoeswearout,			-1));
	ParmScript->Add(sp);

	sp = new cSection("CLIENT_MODES");
	sp->Add(new cKeyWord("NPCPOPUPMENUES",		KWT_dec_char,	&npcpopmenues,			0));
	sp->Add(new cKeyWord("AOSMODE",				KWT_dec_char,	&AOS_Mode,				0));
	sp->Add(new cKeyWord("SEMODE",			    KWT_dec_char,	&SE_Mode,				0));
	//sp->Add(new cKeyWord("ONE_PPA",		    KWT_dec_char,	&siege_Mode,			0));
	ParmScript->Add(sp);


	sp = new cSection("BOOKS");
	sp->Add(new cKeyWord("PATH",				KWT_str_str,	&bookpath[0],	""));
	ParmScript->Add(sp);

	sp = new cSection("CONTAINERS");
	sp->Add(new cKeyWord("BANKBOXLIMIT",		KWT_dec_uint,	&BankboxLimit,			125));
	sp->Add(new cKeyWord("STRONGBOXLIMIT",		KWT_dec_uint,	&StrongboxLimit,		25));
	sp->Add(new cKeyWord("SECURECONTLIMIT",		KWT_dec_uint,	&SecurecontLimit,		125));
	sp->Add(new cKeyWord("CONTAINERLIMIT",		KWT_dec_uint,	&ContainerLimit,		125));
	sp->Add(new cKeyWord("CONTAINERWEIGHT",		KWT_dec_uint,	&ContainerWeight,		400));
	sp->Add(new cKeyWord("BOATHOLDWEIGHT",		KWT_dec_uint,	&BoatHoldWeight,		1200));
	ParmScript->Add(sp);

	sp = new cSection("MONEY");
	sp->Add(new cKeyWord("STARTGOLD",			KWT_dec_uint,	&startgold,			100));
	sp->Add(new cKeyWord("GOLDWEIGHT",			KWT_flo_flo,	&goldweight,		0.005f));//what a goldpiece weighs this is in hundreths of a stone! AntiChrist
	sp->Add(new cKeyWord("STABLING_FEE",		KWT_flo_flo,	&stablingfee,		0.25));
	sp->Add(new cKeyWord("USESPECIALBANK",		KWT_dec_int,	&usespecialbank,	0));	//AntiChrist - specialbank
	sp->Add(new cKeyWord("SPECIALBANKTRIGGER",	KWT_str_str,	&specialbanktrigger[0],"WARE"));
	ParmScript->Add(sp);

	sp = new cSection("PLAYERPOSITIONFILE");
	sp->Add(new cKeyWord("FILENAME",			KWT_str_str,	&playerpositionsfile[0],	""));
	sp->Add(new cKeyWord("REFRESHTIME",			KWT_dec_int,	&playerpositionsrefresh,	0));
	sp->Add(new cKeyWord("BACKUPRATIO",			KWT_dec_int,	&playerposfilebackupratio,	0));
	ParmScript->Add(sp);

	sp = new cSection("SPEED");
	sp->Add(new cKeyWord("ANTINPCAGGREGATIONS",	KWT_dec_int,	&speed.antinpcaggregation,	0));
	sp->Add(new cKeyWord("SLEEP_IDLE",			KWT_dec_int,	&speed.idlesleep,			1000));	// 1 sec
    sp->Add(new cKeyWord("SLEEP_PERLOOP",		KWT_dec_int,	&speed.loopsleep,			10));
	sp->Add(new cKeyWord("MAXSOCKETWAIT",		KWT_dec_int,	&speed.maxsocketwait,		10));
	sp->Add(new cKeyWord("CHECK_ITEMS",			KWT_flo_flo,	&speed.itemtime,			1.5));
	sp->Add(new cKeyWord("CHECK_NPCS",			KWT_flo_flo,	&speed.npctime,				1.0));
	sp->Add(new cKeyWord("CHECK_NPCAI",			KWT_flo_flo,	&speed.npcaitime,			1.2));
	sp->Add(new cKeyWord("CHECK_SPAWNREGIONS",	KWT_dec_uint,	&speed.srtime,				30));	
	sp->Add(new cKeyWord("BOAT_SPEED",			KWT_flo_flo,	&boatspeed,					0.75));	
	ParmScript->Add(sp);
	
	sp = new cSection("COMBAT");
	sp->Add(new cKeyWord("COMBAT_HIT_MESSAGE",	KWT_dec_uint,	&combathitmessage,		1));
	sp->Add(new cKeyWord("MAX_ABSORBTION",		KWT_dec_uint,	&maxabsorbtion,			20));	// 20 Arm (single armour piece) -- Magius(CHE)
	sp->Add(new cKeyWord("MAX_NON_HUMAN_ABSORBTION",KWT_dec_uint,&maxnohabsorbtion,		100));	// 100 Arm (total armour) -- Magius(CHE)
	sp->Add(new cKeyWord("MONSTERS_VS_ANIMALS",	KWT_dec_uint,	&monsters_vs_animals,	0));	// By default monsters won't attack animals
	sp->Add(new cKeyWord("ANIMALS_ATTACK_CHANCE",KWT_dec_uint,	&animals_attack_chance,	15));	// a 15% chance
	sp->Add(new cKeyWord("ANIMALS_GUARDED",		KWT_dec_uint,	&animals_guarded,		0)); 	// By default players can attack animals without summoning guards
	sp->Add(new cKeyWord("NPC_DAMAGE_RATE",		KWT_dec_uint,	&npcdamage,				2));
	sp->Add(new cKeyWord("NPC_BASE_FLEEAT",		KWT_dec_uint,	&npc_base_fleeat,		20));	// 20% of hp
	sp->Add(new cKeyWord("NPC_BASE_REATTACKAT",	KWT_dec_uint,	&npc_base_reattackat,	40));
	sp->Add(new cKeyWord("ATTACKSTAMINA",		KWT_dec_int,	&attackstamina,			-2));	// AntiChrist - attacker looses stamina when hits
	sp->Add(new cKeyWord("ATTACK_DISTANCE",		KWT_dec_uint,	&attack_distance,		13));	// blackwind 
	ParmScript->Add(sp);

	sp = new cSection("VENDOR");
	sp->Add(new cKeyWord("SELLBYNAME",			KWT_dec_uint,	&sellbyname,		1));
	sp->Add(new cKeyWord("SELLFROMSUBPACKS",	KWT_dec_int,	&sellfromsubpacks,	0));
	sp->Add(new cKeyWord("SELLMAXITEM",			KWT_dec_uint,	&sellmaxitem,		5));
	sp->Add(new cKeyWord("TRADESYSTEM",			KWT_dec_uint,	&trade_system,		1));
	sp->Add(new cKeyWord("RANKSYSTEM",			KWT_dec_uint,	&rank_system,		1));
	sp->Add(new cKeyWord("CHECKBANK",			KWT_dec_int,	&CheckBank,			2000)); // Ripper..2000 default
	sp->Add(new cKeyWord("VENDORGREET",			KWT_dec_int,	&VendorGreet,		1));	// Ripper..do greet by default.
	ParmScript->Add(sp);

	sp = new cSection("REGENERATE");
	sp->Add(new cKeyWord("HITPOINTS_REGENRATE",	KWT_dec_uint,	&hitpointrate,		8));	// Seconds to heal ONE hp
	sp->Add(new cKeyWord("STAMINA_REGENRATE",	KWT_dec_uint,	&staminarate,		3));	// Seconds to heal ONE stm
	sp->Add(new cKeyWord("MANA_REGENRATE",		KWT_dec_uint,	&manarate,			5));	// Seconds to heal ONE mn
	sp->Add(new cKeyWord("ARMOR_AFFECT_MANA_REGEN",KWT_dec_int,	&armoraffectmana,	1));	// meditation allowed
	ParmScript->Add(sp);

	sp = new cSection("HUNGER");
	sp->Add(new cKeyWord("DEFAULTFOOD",			KWT_str_sp1,	&defaultfood,	0xFF));	// all chars can eat every food by default (thrawn)
	sp->Add(new cKeyWord("HUNGERRATE",			KWT_dec_uint,	&hungerrate,	6000));	// seconds
	sp->Add(new cKeyWord("HUNGER_DAMAGE",		KWT_dec_uint,	&hungerdamage,	2));	// 2 Hitpoints
	sp->Add(new cKeyWord("HUNGER_DAMAGE_RATE",	KWT_dec_uint,	&hungerdamagerate,	10));	// seconds
	ParmScript->Add(sp);

	sp = new cSection("RESOURCE");
	sp->Add(new cKeyWord("MINECHECK",			KWT_dec_uint,	&minecheck,				1));
	sp->Add(new cKeyWord("ORE_PER_AREA",		KWT_dec_uint,	&resource.ore,			10));
	sp->Add(new cKeyWord("ORE_RESPAWN_TIME",	KWT_dec_uint,	&resource.oretime,		600));
	sp->Add(new cKeyWord("ORE_RESPAWN_RATE",	KWT_dec_uint,	&resource.orerate,		3));
	sp->Add(new cKeyWord("ORE_RESPAWN_AREA",	KWT_dec_uint,	&resource.orearea,		10));
	sp->Add(new cKeyWord("ORE_TRIGGER",			KWT_dec_int,	&resource.miningtrigger,0));
	sp->Add(new cKeyWord("ORE_STAMINA",			KWT_dec_int,	&resource.miningstamina,-5));
	sp->Add(new cKeyWord("LOGS_PER_AREA",		KWT_dec_uint,	&resource.logs,			3));
	sp->Add(new cKeyWord("LOG_RESPAWN_TIME",	KWT_dec_uint,	&resource.logtime,		600));
	sp->Add(new cKeyWord("LOG_RESPAWN_RATE",	KWT_dec_uint,	&resource.lograte,		3));
	sp->Add(new cKeyWord("LOG_RESPAWN_AREA",	KWT_dec_uint,	&resource.logarea,		10));
	sp->Add(new cKeyWord("LOG_TRIGGER",			KWT_dec_int,	&resource.logtrigger,	0));
	sp->Add(new cKeyWord("LOG_STAMINA",			KWT_dec_int,	&resource.logstamina,	-5));
	sp->Add(new cKeyWord("FISHES_PER_AREA",		KWT_dec_uint,	&resource.fishes,		5));
	sp->Add(new cKeyWord("FISH_RESPAWN_TIME",	KWT_dec_uint,	&resource.fishtime,		600));
	sp->Add(new cKeyWord("FISH_RESPAWN_RATE",	KWT_dec_uint,	&resource.fishrate,		3));
	sp->Add(new cKeyWord("FISH_RESPAWN_AREA",	KWT_dec_uint,	&resource.fisharea,		5));
	sp->Add(new cKeyWord("FISH_TRIGGER",		KWT_dec_int,	&resource.fishtrigger,	0));
	sp->Add(new cKeyWord("FISH_STAMINA",		KWT_dec_int,	&resource.fishstamina,	-2));
	ParmScript->Add(sp);
	
	sp = new cSection("REPSYS");
	sp->Add(new cKeyWord("MURDER_DECAY",		KWT_dec_int,	&repsys.murderdecay,	420));
	sp->Add(new cKeyWord("MAXKILLS",			KWT_dec_int,	&repsys.maxkills,		4));
	sp->Add(new cKeyWord("CRIMINAL_TIME",		KWT_dec_int,	&repsys.crimtime,		120));
	ParmScript->Add(sp);

	sp = new cSection("TRACKING");
	sp->Add(new cKeyWord("BASE_TRACKING_RANGE",	KWT_dec_uint,	&tracking_data.baserange,	10));	// tracking range is at least TRACKINGRANGE, additional distance is calculated by Skill,INT
	sp->Add(new cKeyWord("MAX_TRACKING_TARGETS",KWT_dec_uint,	&tracking_data.maxtargets,	MAXTRACKINGTARGETS));
	sp->Add(new cKeyWord("BASE_TRACKING_TIME",	KWT_dec_uint,	&tracking_data.basetimer,	30));	// tracking last at least TRACKINGTIMER seconds, additional time is calculated by Skill,INT, & DEX
	sp->Add(new cKeyWord("TRACKING_MESSAGE_REDISPLAY_TIME",	KWT_dec_uint,	&tracking_data.redisplaytime,	30));	// tracking will display above the character every TRACKINGDISPLAYTIME seconds
	ParmScript->Add(sp);

	sp = new cSection("BEGGING");
	sp->Add(new cKeyWord("BEGGING_RANGE",		KWT_dec_uint,	&begging_data.range,	3));
	sp->Add(new cKeyWord("BEGGING_TIME",		KWT_dec_int,	&begging_data.timer,	300));
	sp->Add(new cKeyWord("BEGGING_TEXT0",		KWT_str_str,	begging_data.text[0],	"Could thou spare a few coins?"));
	sp->Add(new cKeyWord("BEGGING_TEXT1",		KWT_str_str,	begging_data.text[1],	"Hey buddy can you spare some gold?"));
	sp->Add(new cKeyWord("BEGGING_TEXT2",		KWT_str_str,	begging_data.text[2],	"I have a family to feed, think of the children."));
	ParmScript->Add(sp);

	sp = new cSection("FISHING");
	sp->Add(new cKeyWord("BASE_FISHING_TIME",	KWT_dec_uint,	&fishing_data.basetime,			10));	// it takes at least FISHINGTIMEBASE sec to fish;
	sp->Add(new cKeyWord("RANDOM_FISHING_TIME",	KWT_dec_uint,	&fishing_data.randomtime,		5));
	sp->Add(new cKeyWord("FISHING_LOOT_PROBABILITY",KWT_dec_uint,&fishing_data.loot_probability,	12));
	ParmScript->Add(sp);

	sp = new cSection("HARDCODED_ITEMS");
	sp->Add(new cKeyWord("SLOTMACHINE1",		KWT_dec_int,	&hci.Slotmachine1,	3185));
	sp->Add(new cKeyWord("SLOTMACHINE2",		KWT_dec_int,	&hci.Slotmachine2,	3186));
	sp->Add(new cKeyWord("SLOTMACHINE3",		KWT_dec_int,	&hci.Slotmachine3,	3187));
	sp->Add(new cKeyWord("SLOTMACHINE4",		KWT_dec_int,	&hci.Slotmachine4,	2003));
	sp->Add(new cKeyWord("SLOTMACHINE5",		KWT_dec_int,	&hci.Slotmachine5,	2004));
	sp->Add(new cKeyWord("SLOTMACHINE6",		KWT_dec_int,	&hci.Slotmachine6,	2005));
	sp->Add(new cKeyWord("SLOTMACHINE7",		KWT_dec_int,	&hci.Slotmachine7,	2006));
	sp->Add(new cKeyWord("SLOTMACHINE8",		KWT_dec_int,	&hci.Slotmachine8,	2007));
	sp->Add(new cKeyWord("ORDERSHIELD",			KWT_dec_int,	&hci.OrderShield,	29));
	sp->Add(new cKeyWord("CHAOSSHIELD",			KWT_dec_int,	&hci.ChaosShield,	28));
	sp->Add(new cKeyWord("DECIPHEREDMAP",		KWT_dec_int,	&hci.DecipheredMap,	70025));
	sp->Add(new cKeyWord("TREASURECHESTBASE",	KWT_dec_int,	&hci.TreasureChestBase,	20000));
	sp->Add(new cKeyWord("BALLOFJUSTICE",		KWT_dec_int,	&hci.BallOfJustice,	50040));
	sp->Add(new cKeyWord("KEY",					KWT_dec_int,	&hci.Key,			339));
	sp->Add(new cKeyWord("MAGICSCROLLBASE",		KWT_dec_int,	&hci.MagicScrollBase,800));
	ParmScript->Add(sp);

	sp = new cSection("HARDCODED_NPCS");
	sp->Add(new cKeyWord("ENERGYVORTEX",		KWT_dec_int,	&hcn.EnergyVortex,	295	));
	sp->Add(new cKeyWord("AIRELEMENTAL",		KWT_dec_int,	&hcn.AirElemental,	291	));
	sp->Add(new cKeyWord("DEAMON",				KWT_dec_int,	&hcn.Deamon,		290	));
	sp->Add(new cKeyWord("EARTHELEMENTAL",		KWT_dec_int,	&hcn.EarthElemental,292	));
	sp->Add(new cKeyWord("FIREELEMENTAL",		KWT_dec_int,	&hcn.FireElemental,	293	));
	sp->Add(new cKeyWord("WATERELEMENTAL",		KWT_dec_int,	&hcn.WaterElemental,294	));
	sp->Add(new cKeyWord("BLADESPIRITS",		KWT_dec_int,	&hcn.BladeSpirits,	296	));
	sp->Add(new cKeyWord("MERCENARY",			KWT_dec_int,	&hcn.Mercenary,		227	));
	sp->Add(new cKeyWord("PLAYERVENDOR",		KWT_dec_int,	&hcn.PlayerVendor,	2117));
	sp->Add(new cKeyWord("MOUNTA0",				KWT_dec_int,	&hcn.MountA0,		8	));
	sp->Add(new cKeyWord("MOUNTA1",				KWT_dec_int,	&hcn.MountA1,		9	));
	sp->Add(new cKeyWord("MOUNTA2",				KWT_dec_int,	&hcn.MountA2,		10	));
	sp->Add(new cKeyWord("MOUNTA3",				KWT_dec_int,	&hcn.MountA3,		429	));
	sp->Add(new cKeyWord("MOUNTA4",				KWT_dec_int,	&hcn.MountA4,		427	));
	sp->Add(new cKeyWord("MOUNTA5",				KWT_dec_int,	&hcn.MountA5,		428	));
	sp->Add(new cKeyWord("MOUNTA6",				KWT_dec_int,	&hcn.MountA6,		54	));
	sp->Add(new cKeyWord("MOUNT9FA",			KWT_dec_int,	&hcn.Mount9Fa,		430	));
	sp->Add(new cKeyWord("MOUNT9FB",			KWT_dec_int,	&hcn.Mount9Fb,		7	));
	sp->Add(new cKeyWord("SUMMONLIST",			KWT_dec_int,	&hcn.SummonList,	10000));
	ParmScript->Add(sp);

	sp = new cSection("OFFICIALLY_NOT_THERE");
	sp->Add(new cKeyWord("800*600_LOCK",		KWT_dec_char,   &_800_600_lock, 1));
	sp->Add(new cKeyWord("HARDCODED_SUPPORTED_CLIENTS_LIST",KWT_dec_char,   &HardcodedClientsList, 1));
	sp->Add(new cKeyWord("RUSSIAN_HACK", 		KWT_dec_char,   &unicodehack, 0));
	sp->Add(new cKeyWord("RISKYTRIGGERS", 		KWT_dec_char,   &riskyTriggers, 0));
	ParmScript->Add(sp);
	
	sp = new cSection("TIME_LIGHT");
	sp->Add(new cKeyWord("DAY",					KWT_dec_int,	&timeLight.day,		0));
	sp->Add(new cKeyWord("HOUR",				KWT_dec_int,	&timeLight.hour,	12));
	sp->Add(new cKeyWord("MINUTE",				KWT_dec_int,	&timeLight.minute,	1));
	sp->Add(new cKeyWord("AMPM",				KWT_dec_int,	&timeLight.ampm,	0));	//Initial time is noon.
	sp->Add(new cKeyWord("MOON1UPDATE",			KWT_dec_int,	&timeLight.moon1update,	1));
	sp->Add(new cKeyWord("MOON2UPDATE",			KWT_dec_int,	&timeLight.moon2update,	1));
	sp->Add(new cKeyWord("MOON1",				KWT_dec_char,	&timeLight.moon1,	0));
	sp->Add(new cKeyWord("MOON2",				KWT_dec_char,	&timeLight.moon2,	0));
	sp->Add(new cKeyWord("DUNGEONLIGHTLEVEL",	KWT_dec_char,	&timeLight.dungeonlightlevel,	21));
	sp->Add(new cKeyWord("WORLDFIXEDLEVEL",		KWT_dec_char,	&timeLight.worldfixedlevel,	255));
	sp->Add(new cKeyWord("WORLDCURLEVEL",		KWT_dec_char,	&timeLight.worldcurlevel,	0));
	sp->Add(new cKeyWord("WORLDBRIGHTLEVEL",	KWT_dec_char,	&timeLight.worldbrightlevel,	1));
	sp->Add(new cKeyWord("WORLDDARKLEVEL",		KWT_dec_char,	&timeLight.worlddarklevel,	16)); //used to be 0x1a - too dark
	sp->Add(new cKeyWord("SEASON",				KWT_dec_char,	&timeLight.season,	0));
	sp->Add(new cKeyWord("SECONDSPERUOMINUTE",	KWT_dec_int,	&timeLight.secondsperuominute,	5)); //Number of realtime seconds for a UO minute.
	sp->Add(new cKeyWord("INLORPOTIONFIXEDTIME",KWT_dec_int,	&timeLight.fixedInlorPotionTime,1));
	sp->Add(new cKeyWord("INLORPOTIONVARTIME",	KWT_dec_int,	&timeLight.varInlorPotionTime,	5));
	sp->Add(new cKeyWord("INLORSPELLFIXEDTIME",	KWT_dec_int,	&timeLight.fixedInlorSpellTime,	1));
	sp->Add(new cKeyWord("INLORSPELLVARTIME",	KWT_dec_int,	&timeLight.varInlorSpellTime,	5));
	ParmScript->Add(sp);
}

void loadanticheat()
{
	int loopexit=0;
	do
	{
	  readwsc2();		

	  if (!strcmp(script1,"ANTIFASTWALKHACKTOOLS")) SrvParms->antifastwalk =  (char) str2num(script2);
	  else if (!strcmp(script1,"ANTIINJECTION"))    SrvParms->antiinjection = (char) str2num(script2);
	  else if (!strcmp(script1,"CRYPTKEY1"))        SrvParms->CryptKey1 =     str2num(script2);
	  else if (!strcmp(script1,"CRYPTKEY2"))        SrvParms->CryptKey2 =     str2num(script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}


void loadnetworking()
{
	int loopexit=0;
	do
	{
		readwsc2();	
		if (!strcmp(script1,"GAMESERVERPORT"))  serv.uiPort = (short) str2num(script2);     
		else if (!strcmp(script1,"PACKETSENDSTYLE:")) 
		{	   
			if ( (!strcmp ( script2, "UNBUFFERED") ) )  SrvParms->packetsendstyle  = PSS_UNBUFFERED;
			else if ( (!strcmp ( script2, "SMARTBUFFER") ) ) SrvParms->packetsendstyle  = PSS_SMARTBUFFER;		 
			else SrvParms->packetsendstyle = PSS_SMARTBUFFER;
			
		}
	}
	while (  (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}


void loadminiloginserver()
{
	int loopexit=0;
	do
	{
		readwsc2();
		if (!strcmp(script1,"SHARDNAME"))               serv.sServer = script2;
		else if (!strcmp(script1,"CLIENTRELAYADDRESS")) serv.sIP = script2;
		else if (!strcmp(script1,"ALIASADDRESS"))       serv.AliasIP = script2;
		else if (!strcmp(script1,"ALIASNAME"))          serv.AliasServer = script2;
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}


void loadantimacro()
{
	int loopexit=0;
	do
	{
		readwsc2();
		if (!strcmp(script1,     "ENABLED"))              SrvParms->antimacro_on=str2num(script2);	 
		else if (!strcmp(script1,"QUERYCHECKINTERVALL"))  SrvParms->antimacro_queryfrequency=str2num(script2);
		else if (!strcmp(script1,"QUERYTYPE"))            SrvParms->antimacro_querytype=str2num(script2);	 
		else if (!strcmp(script1,"INPUTTYPE"))            SrvParms->antimacro_inputtype=str2num(script2);
		else if (!strcmp(script1,"EXCEPTIONSTYPE"))       SrvParms->antimacro_exceptions=str2num(script2);
	}
	while (  (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}

void loadremote_admin()
{
	int loopexit=0;
	do
	{
		readwsc2();
		if (!strcmp(script1,"ENABLE")) SrvParms->EnableRA = true;
		else if (!strcmp(script1,"DISABLE")) SrvParms->EnableRA = false;
		else if (!strcmp(script1,"PORT")) SrvParms->ra_port = str2num(script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS));
}

void cSrvParms::LoadScript(char *fn) // Load a server script
{
	int loopexit=0;
	wscfile = fopen(fn, "r");
	if (wscfile==NULL)
	{
		LogMessageVar("file '%s' not found...defaults are used",fn);
		return;
	}
	do
	{
		readwsc2();
		if (!strcmp(script1, "SECTION"))
		{
	        if (!strcmp(script2, "GAMESERVERNETWORKING")) loadnetworking();
			else if (!strcmp(script2, "ANTICHEAT")) loadanticheat();
			else if (!strcmp(script2, "MINILOGINSERVER")) loadminiloginserver();
			else if (!strcmp(script2, "ANTIMACRO")) loadantimacro();
			else if (!strcmp(script2, "CLIENTS_ALLOWED")) loadclientsallowed();
			else if (ParmScript->LoadSections(script2)) continue;
			else if (!strcmp(script2, "REMOTE_ADMIN")) loadremote_admin();
		}
	}
	while ( (strcmp(script1, "EOF")) && (++loopexit < MAXLOOPS) );
	fclose(wscfile);
	wscfile = NULL;

	debug_network=SrvParms->logmode&1;
	if (!repsys.murderdecay) repsys.murderdecay=420;
	if (!repsys.maxkills) repsys.maxkills=4;
	if (!repsys.crimtime) repsys.crimtime=120;
	if (skilladvancemodifier <= 0) skilladvancemodifier=1;	// must be nonzero && positive
	if (statsadvancemodifier <= 0) statsadvancemodifier=1;	// must be nonzero && positive
	if (skilllevel < 2 || skilllevel > 10) skilllevel=5;
	for(unsigned int i = 0; i < strlen(specialbanktrigger); i++)
		specialbanktrigger[i] = toupper(specialbanktrigger[i]);
	if (tracking_data.maxtargets>MAXTRACKINGTARGETS)
		tracking_data.maxtargets=MAXTRACKINGTARGETS;
	timeLight.uotickcount=1;
	if (hci.MagicScrollBase != 800 && hci.MagicScrollBase != 4205)
	{
		hci.MagicScrollBase = 800;
		LogWarning("Invalid value for MagicScrollBase found. Must be 800 or 4205 !");
	}
}

void cSrvParms::Load() // Load server script
{
	LoadScript("server.scp");
	LoadScript("shards_server.scp");
}

void cSrvParms::Save(void)
{
	FILE *file;
	file = fopen("server.scp", "w");
    vector<string>::const_iterator vis; 
	
	fprintf(file, "// LONEWOLF Server Script\n");
	fprintf(file, "// Generated by %s Version %s\n\n",  wp_version.betareleasestring.c_str() , wp_version.verstring.c_str() );

	fprintf(file, "SECTION CLIENTS_ALLOWED\n");
	fprintf(file, "{\n");	
   
    for (vis=clientsAllowed.begin(); vis != clientsAllowed.end();  ++vis) 
    {
	  fprintf(file, "%s\n", vis->c_str()); 
    }  

	fprintf(file, "}\n\n");

    fprintf(file, "SECTION GAMESERVERNETWORKING\n");		
	fprintf(file, "{\n");
	fprintf(file, "GAMESERVERPORT %i\n", serv.uiPort);	// First entry, make it more visible.

	switch(SrvParms->packetsendstyle)
	{
	case PSS_UNBUFFERED:	fprintf(file, "PACKETSENDSTYLE: UNBUFFERED\n");		break;
	case PSS_SMARTBUFFER:	fprintf(file, "PACKETSENDSTYLE: SMARTBUFFER\n");	break;		
	default:				fprintf(file, "PACKETSENDSTYLE: SMARTBUFFER\n");	break;
	}
	
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION REMOTE_ADMIN\n");
	fprintf(file, "{\n");
	if (SrvParms->EnableRA)
		fprintf(file, "ENABLE\n");
	else
		fprintf(file, "DISABLE\n");
	
	fprintf(file, "PORT %i\n", SrvParms->ra_port);
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION ANTICHEAT\n");
	fprintf(file, "{\n");
	fprintf(file, "ANTIFASTWALKHACKTOOLS %i\n",SrvParms->antifastwalk);
	fprintf(file, "ANTIINJECTION %i\n",SrvParms->antiinjection);
	fprintf(file, "CRYPTKEY1 %lu\n",SrvParms->CryptKey1);
	fprintf(file, "CRYPTKEY2 %lu\n",SrvParms->CryptKey2);
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION ANTIMACRO\n");
	fprintf(file, "{\n");
	fprintf(file, "ENABLED %i\n", SrvParms->antimacro_on);
	fprintf(file, "QUERYCHECKINTERVALL %i\n", SrvParms->antimacro_queryfrequency); 
	fprintf(file, "QUERYTYPE %i\n", SrvParms->antimacro_querytype); 
	fprintf(file, "INPUTTYPE %i\n", SrvParms->antimacro_inputtype); 
	fprintf(file, "EXCEPTIONSTYPE %i\n", SrvParms->antimacro_exceptions); 
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION MINILOGINSERVER\n");
	fprintf(file, "{\n");
	fprintf(file, "SHARDNAME %s\n",serv.sServer.c_str() );
	fprintf(file, "CLIENTRELAYADDRESS %s\n",serv.sIP.c_str() );		 
	fprintf(file, "ALIASNAME %s\n",serv.AliasServer.c_str() );		 
	fprintf(file, "ALIASADDRESS %s\n",serv.AliasIP.c_str() );		 	
	fprintf(file, "}\n\n");
	
	ParmScript->SaveSections(file);

	fprintf(file, "EOF\n\n");
	fclose(file);
	file = NULL;
}

