/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//
#include "commands.h"

#include "Client.h"
#include "cmdtable.h"
#include "gumps.h"
#include "mapstuff.h"
#include "network.h"
#include "SndPkg.h"
#include "sregions.h"
#include "globals.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "commands.cpp"

// inline command to do targeting - coulda made this a
// macro but this is pretty much as fast...
static inline void _do_target(P_PLAYER Player, TARGET_S *ts)
{
	target(Player, ts->a1, ts->txt);
}

#define CMD_EXEC	void (*) (int)
#define CMD_EXECP	void (*) (P_PLAYER)


void InitPages() // Initilizes the gmpages[] and counspages[] arrays
{
	int i;
	for(i=1;i<MAXPAGES;i++)
	{
		gmpages[i].name.erase();
		gmpages[i].reason.erase();
		gmpages[i].serial = 0;
		gmpages[i].timeofcall[0]=0;
		gmpages[i].handled=1;
	}
	for(i=1;i<MAXPAGES;i++)
	{
		counspages[i].name.erase();
		counspages[i].reason.erase();
		counspages[i].serial = 0;
		counspages[i].timeofcall[0]=0;
		counspages[i].handled=1;
	}
}

void donewithcall(P_PLAYER Player, int type)
{
	int cn = Player->callnum;
	if (cn > 0) //Player is on a call
	{
		if (type==1) //Player is a GM
		{
			gmpages[cn].handled=1;
			gmpages[cn].name.erase();
			gmpages[cn].reason.erase();
			gmpages[cn].serial = 0;
			Player->Msg("Call removed from the GM queue.");
		}
		else //Player is a counselor
		{
			counspages[cn].handled=1;
			counspages[cn].name.erase();
			counspages[cn].reason.erase();
			counspages[cn].serial = 0;
			Player->Msg("Call removed from the Counselor queue.");
		}
		Player->callnum=0;
	}
	else
	{
		Player->Msg("You are currently not on a call");
	}
}


std::vector<std::string> cCommands::split(std::string sep,std::string value)
{
	std::vector<std::string> rvalue ;
	std::string::size_type index,index2 ;

	index = value.find(sep);
	if (index == std::string::npos)
	{
		// there is no sperator in this at all, return the string
		rvalue.push_back(value);
	}
	else
	{
		while (index != std::string::npos)
		{
			index2 = value.find(sep,index+sep.size());
			if (index2 == std::string::npos)
			{
				// There is no other seperator
				rvalue.push_back(value.substr(0,index));
				rvalue.push_back(value.substr(index + sep.size()));
				index = std::string::npos;


			}
			else
			{

				rvalue.push_back(value.substr(0,index));
				rvalue.push_back(value.substr(index + sep.size(),index2-index-sep.size()));
				value = value.substr(index2+sep.size());
				index = value.find(sep);
			}
		}
	}
	return rvalue ;
}



/* extensively modified 8/2/99 crackerjack@crackerjack.net -
 * see cmdtable.cpp for more details */
void cCommands::Command(P_PLAYER Player, string speech) // Client entred a '/' command like /ADD
{
	unsigned char *comm;
	unsigned char nonuni[512];

	if (!Player) return;

	if (Player->unicode)
		cCommands::cmd_offset = 1;
	else
		cCommands::cmd_offset = 1;

	cCommands::command_line = speech;
	//cCommands::params = cCommands::command_line.split(" ");
	cCommands::params = split(" ",cCommands::command_line);
	strcpy((char*)nonuni, speech.c_str());
	strcpy(tbuffer, (char*)nonuni);

	cline = (char*) &nonuni[0];
	splitline();
	if (tnum<1)
		return;
	// Let's ignore the command prefix;
	comm = &nonuni[1];
	strupr((char*)comm); // only strupr the "comamnd word" :) (Thrawn)

	bool found = false;
	vector<struct cmdtable_s>::iterator it;
	for (it=command_table.begin(); it!=command_table.end(); ++it)
	{
		if (!strcmp((*it).cmd_name, (char*)comm))
		{
			found = true;
			break;
		}
	}

	if (!found)
	{
		Player->Msg("Unrecognized command.");
		return;
	}
	else
	{
		struct cmdtable_s& Cmd = *it;
		if ((Player->isTrueGM() && !Player->isGM()) ||		// a restricted GM outside his region(s)
			!Player->isAdmin() &&(Cmd.cmd_priv_m!=255)&&
			(!(Player->cmds.get(Cmd.cmd_priv_m)&
			(0-0xFFFFFFFF<<Cmd.cmd_priv_b))))
		{
			Player->Msg("Access denied.");
			return;
		}

		UOXSOCKET s = Player->cs;

		switch(Cmd.cmd_type)
		{
		case CMD_FUNCP:
			(*((CMD_EXECP)Cmd.cmd_extra)) (Player);
			break;
		case CMD_TARGET:
			_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			break;
		case CMD_TARGETX:
			if (tnum==2)
			{
				addx[s]=makenumber(1);
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes one number as an argument.");
			break;
		case CMD_TARGETXY:
			if (tnum==3)
			{
				addx[s]=makenumber(1);
				addy[s]=makenumber(2);
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes two numbers as arguments.");
			break;
		case CMD_TARGETXYZ:
			if (tnum==4)
			{
				addx[s]=makenumber(1);
				addy[s]=makenumber(2);
				addz[s]=makenumber(3);
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes three numbers as arguments.");
			break;
		case CMD_TARGETPOS:
			if (tnum>=4 && tnum<=5)
			{
				cCoord cpos(makenumber(1),makenumber(2),makenumber(3),makenumber(4));
				Player->TD._Target1Pos = cpos;
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes three or four numbers as arguments.");
			break;
		case CMD_TARGETHX:
			if (tnum==2)
			{
				addx[s]=hexnumber(1);
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes one hex number as an argument.");
			break;
		case CMD_TARGETHXY:
			if (tnum==3)
			{
				addx[s]=hexnumber(1);
				addy[s]=hexnumber(2);
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes two hex numbers as arguments.");
			break;
		case CMD_TARGETHXYZ:
			if (tnum==4)
			{
				addx[s]=hexnumber(1);
				addy[s]=hexnumber(2);
				addz[s]=hexnumber(3);
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes three hex numbers as arguments.");
			break;
		case CMD_TARGETID1:
			if (tnum==2)
			{
				addid1[s]=makenumber(1);
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes one number as an argument.");
			break;
		case CMD_TARGETID2:
			if (tnum==3)
			{
				addid1[s] = static_cast<unsigned char>(makenumber(1));
				addid2[s] = static_cast<unsigned char>(makenumber(2));
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes two numbers as arguments.");
			break;
		case CMD_TARGETID3:
			if (tnum==4)
			{
				addid1[s] = static_cast<unsigned char>(makenumber(1));
				addid2[s] = static_cast<unsigned char>(makenumber(2));
				addid3[s] = static_cast<unsigned char>(makenumber(3));
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes three numbers as arguments.");
			break;
		case CMD_TARGETID4:
			if (tnum==5)
			{
				addid1[s] = static_cast<unsigned char>(makenumber(1));
				addid2[s] = static_cast<unsigned char>(makenumber(2));
				addid3[s] = static_cast<unsigned char>(makenumber(3));
				addid4[s] = static_cast<unsigned char>(makenumber(4));
				//Console.send("1: %i 2: %i 3: %i 4: %i\n",addid1[s],addid2[s],addid2[s],addid3[s],addid4[s]);
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes four numbers as arguments.");
			break;
		case CMD_TARGETHID1:
			if (tnum==2)
			{
				addid1[s] = static_cast<unsigned char>(hexnumber(1));
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes one hex number as an argument.");
			break;
		case CMD_TARGETHID2:
			if (tnum==3)
			{
				addid1[s] = static_cast<unsigned char>(hexnumber(1));
				addid2[s] = static_cast<unsigned char>(hexnumber(2));
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes two hex numbers as arguments.");
			break;
		case CMD_TARGETHID3:
			if (tnum==4)
			{
				addid1[s] = static_cast<unsigned char>(hexnumber(1));
				addid2[s] = static_cast<unsigned char>(hexnumber(2));
				addid3[s] = static_cast<unsigned char>(hexnumber(3));
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes three hex numbers as arguments.");
			break;
		case CMD_TARGETHID4:
			if (tnum==5)
			{
				addid1[s] = static_cast<unsigned char>(hexnumber(1));
				addid2[s] = static_cast<unsigned char>(hexnumber(2));
				addid3[s] = static_cast<unsigned char>(hexnumber(3));
				addid4[s] = static_cast<unsigned char>(hexnumber(4));
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes four hex numbers as arguments.");
			break;
		case CMD_TARGETTMP:
			if (tnum==2)
			{
				tempint[s]=makenumber(1);
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes a number as an argument.");
			break;
		case CMD_TARGETHTMP:
			if (tnum==2)
			{
				tempint[s]=hexnumber(1);
				_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			}
			else
				Player->Msg("This command takes a hex number as an argument.");
			break;

		case CMD_TARGET4OPT:
			addx[s]		= hexnumber(1);
			addy[s]		= hexnumber(2);
			addz[s]		= hexnumber(3);
			tempint[s]	= hexnumber(4);
			addid1[s]	= static_cast<unsigned char>(tnum);
			_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			break;

		case CMD_TARGETSTR:
			if (strlen(Commands->GetAllParams().c_str()) <= 120)
				strcpy(xtext[s], Commands->GetAllParams().c_str());
			_do_target(Player, (TARGET_S *)Cmd.cmd_extra);
			break;

		default:
			Player->Msg("BUG: Command has a bad command type set!");
			break;
		}
		return;
	}

	Player->Msg("BUG: Should never reach end of command() function!");
}

//mstring cCommands::GetAllParams(void)
#define COMMANDWHITESPACE " \t\v\f\0 "
std::string cCommands::GetAllParams(void)
{
	unsigned int pos = 0;
	//mstring dummy;
	std::string dummy ;
	std::string whitespace(COMMANDWHITESPACE);
	pos = command_line.find_first_of(" ");
	if (pos != std::string::npos)
	{
		dummy = command_line.substr(pos);
		std::string::size_type index = dummy.find_first_not_of(whitespace) ;
		std::string::size_type index2 = dummy.find_last_not_of(whitespace);
		if (index != std::string::npos && index2 != std::string::npos)
		{
			dummy = dummy.substr(index,index2-index+1);
		}
		else if (index2==std::string::npos)
		{
			dummy="";
		}


		//dummy.trim();
	}
	return dummy;
}


void cCommands::MakeShop(P_CHAR pc, int shoptype)
{
	if (!pc) return;

	pc->shop=shoptype;
	if (pc->GetItemOnLayer(0x1A) == NULL)
	{
		P_ITEM p1A=Items->SpawnItem(ITY_Container,1,"#",0x2AF8,0);
		if (p1A)
		{
			p1A->newbieOn();
			pc->Wear(p1A,0x1A);
		}
	}
	
	if (pc->GetItemOnLayer(0x1B) == NULL)
	{
		P_ITEM p1B=Items->SpawnItem(ITY_Container,1,"#",0x2AF8,0);
		if (p1B)
		{
			p1B->newbieOn();
			pc->Wear(p1B,0x1B);
		}
	}
	
	if (pc->GetItemOnLayer(0x1C) == NULL)
	{
		P_ITEM p1C=Items->SpawnItem(ITY_Container,1,"#",0x2AF8,0);
		if (p1C)
		{
			p1C->newbieOn();
			pc->Wear(p1C,0x1C);
		}
	}
}

void cCommands::NextCall(P_PLAYER Player, int type)
{
	// Type is the same as it is in showgmqueue()
	int i;
	int x = 0;
	
	if (Player->callnum != 0)
	{
		donewithcall(Player, type);
	}
	if (type == 1) // Player is a GM
	{
		for (i = 1; i < MAXPAGES; i++)
		{
			if (gmpages[i].handled == 0)
			{
				P_CHAR pc = FindCharBySerial(gmpages[i].serial);
				if (pc)
				{
					Player->Msg("");
					sprintf(temp, "Transporting to next call: %s", gmpages[i].name.c_str());
					Player->Msg(temp);
					sprintf(temp, "Problem: %s.", gmpages[i].reason.c_str());
					Player->Msg(temp);
					sprintf(temp, "Serial number %x", gmpages[i].serial);
					Player->Msg(temp);
					sprintf(temp, "Paged at %s.", gmpages[i].timeofcall);
					Player->Msg(temp);
					gmpages[i].handled = 1;
					Player->travelTo(pc->pos);
					Player->callnum = i;
					x++;
				}// if
				if (x > 0)
					break;
			}// if
		}// for
		if (x == 0)
			Player->Msg("The GM queue is currently empty");
	} // end first IF
	else // Player is only a counselor
	{
		x = 0;
		for (i = 1; i < MAXPAGES; i++)
		{
			if (counspages[i].handled == 0)
			{
				P_CHAR pc = FindCharBySerial(counspages[i].serial);
				if (pc)
				{
					Player->Msg("");
					sprintf(temp, "Transporting to next call: %s", counspages[i].name.c_str());
					Player->Msg(temp);
					sprintf(temp, "Problem: %s.", counspages[i].reason.c_str());
					Player->Msg(temp);
					sprintf(temp, "Serial number %x", counspages[i].serial);
					Player->Msg(temp);
					sprintf(temp, "Paged at %s.", counspages[i].timeofcall);
					Player->Msg(temp);
					gmpages[i].handled = 1;
					Player->travelTo(pc->pos);
					Player->callnum = i;
					x++;
					break;
				}// if
			}// else
			if (x > 0)
				break;
		}// for
		if (x == 0)
			Player->Msg("The Counselor queue is currently empty");
	}// if
}

void cCommands::KillSpawn(P_PLAYER Player, int r)  //courtesy of Revana
{
	int killed=0;

	if (r<=0 || r>=255) return;

	Player->Msg("Killing spawn, this may cause lag...");

	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++)
	{
		if (pc->spawnregion==r)
		{
			bolteffect(pc, true);
			soundeffect2(pc, 0x0029);
			Npcs->DeleteChar(pc);
			killed++;
		}
	}

	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
	{
		if (pi->spawnregion==r)
		{			
			Items->DeleItem(pi->unconst());
			killed++;
		}
	}

	gcollect();
	Player->Msg("Done.");
	char tstr[512];
	sprintf(tstr, "%i of spawnregion %i have been killed.",killed,r);
	Player->Msg(tstr);
}

void cCommands::RegSpawnMax(P_PLAYER Player, int r) // rewrite LB
{
	char temps[100];

	spawnregion_st *SR = GetSpawnRegion(r);
	if (!SR) return;

	int spawn = GetSpawnDelta(SR);

	sprintf(temps, "Region %d is Spawning %d items/NPCs, this might cause some lag.", r, spawn);
	sysbroadcast(temps);

	SpawnSomeMore(SR, spawn);

	sprintf(temps, "Done. %d total NPCs/items spawned in Spawnregion %d.",spawn,r);
	Player->Msg(temps);
}

void cCommands::RegSpawnNum(P_PLAYER Player, int r, int n) // rewrite by LB
{
	char temps[100];
	
	spawnregion_st *SR = GetSpawnRegion(r);
	if (!SR) return;
	
	int spawn = GetSpawnDelta(SR);
	if (n > spawn) 
	{		
		sprintf(temps, "%d too many for region %d, spawning %d to reach MAX instead.",n, r, spawn);
		Player->Msg(temps);
		n=spawn;
	}		
	sprintf(temps, "Region %d is Spawning: %d NPCs/items, this will cause some lag.", r, spawn);
	sysbroadcast(temps);
	
	SpawnSomeMore(SR, spawn);
	
	sprintf(temps, "Done. %d total NPCs/items spawned in Spawnregion %d.",spawn,r);
	Player->Msg(temps);		
}

void cCommands::KillAll(P_PLAYER Player, int percent, const char* sysmsg)
{
	Player->Msg("Killing all characters, this may cause some lag...");
	sysbroadcast(sysmsg);

	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++)
	{
		if (!pc->isGM())
		{
			if (rand()%100+1<=percent)
			{
				bolteffect(pc, true);
				soundeffect2(pc, 0x0029);
				deathstuff(pc);
			}
		}
	}
	Player->Msg("Done.");
}

//////////////
// Name:	CPage
// History:	UnKnown
// Purpose:	handle help button (Calls Counselor Call Menus up)
//
void cCommands::CPage(P_PLAYER Player, char *reason)
{
	int i, x=0;
	int x2=0;
	
	for(i=1;i<MAXPAGES;i++)
	{
		if (counspages[i].handled==1)
		{
			counspages[i].handled=0;
			counspages[i].name = Player->name;
			counspages[i].reason = reason;
			counspages[i].serial = Player->serial;
			time_t current_time = time(0);
			struct tm *local = localtime(&current_time);
			sprintf(counspages[i].timeofcall, "%02d:%02d:%02d", local->tm_hour, local->tm_min, local->tm_sec);
		
			Player->playercallnum=i;
			Player->pagegm=2;
			x2++;
			break;
		}
	}
	if (x2==0)
	{
		Player->Msg("The Counselor Queue is currently full. Contact the shard operator");
		Player->Msg("and ask them to increase the size of the queue.");
	}
	else
	{
		if (strcmp(reason,"OTHER"))
		{
			Player->pagegm=0;
			char serstring[33];
			LongToHexChunks(Player->serial,serstring);
			sprintf(temp, "Counselor Page from %s %s: %s", Player->name, serstring, reason);

			P_CHAR pc;
			AllPlayersIterator api;
			for (pc = api.First(); pc != NULL; pc = api.Next())
			{
				if (pc->isCounselor())
				{
					x=1;
					pc->Msg(temp);
				}				
				
				if (x==1)
				{
					Player->Msg("Available Counselors have been notified of your request.");
				}
				else Player->Msg("There was no Counselor available to take your call.");
			}
		}
		else Player->Msg("Please enter the reason for your Counselor request");
	}
}

////////////
// Name:	GMPage
// History:	by unknown
// Purpose:	Help button (Calls GM Call Menus up)
//
void cCommands::GMPage(P_PLAYER Player, char *reason)
{
	int i, x=0;
	int x2=0;
	
	for(i=1;i<MAXPAGES;i++)
	{
		if (gmpages[i].handled==1)
		{
			gmpages[i].handled=0;
			gmpages[i].name = Player->name;
			gmpages[i].reason = reason;
			gmpages[i].serial = Player->serial;
			time_t current_time = time(0);
			struct tm *local = localtime(&current_time);
			sprintf(gmpages[i].timeofcall, "%02d:%02d:%02d", local->tm_hour, local->tm_min, local->tm_sec);
		
			Player->playercallnum=i;
			Player->pagegm=1;
			x2++;
			break;
		}
	}
	if (x2==0)
	{
		Player->Msg("The GM Queue is currently full. Contact the shard operator");
		Player->Msg("and ask them to increase the size of the queue.");
	}
	else
	{
		if (strcmp(reason,"OTHER"))
		{
			Player->pagegm=0;
			char serstring[33];
			LongToHexChunks(Player->serial,serstring);
			sprintf(temp, "Page from %s %s: %s", Player->name, serstring, reason);
			
			P_CHAR pc;
			AllPlayersIterator api;
			for (pc = api.First(); pc != NULL; pc = api.Next())
			{
				if (pc->isGM())				
				{
					x=1;
					pc->Msg(temp);
				}
				if (x==1)
				{
					Player->Msg("Available Game Masters have been notified of your request.");
				}
				else Player->Msg("There was no Game Master available to take your call.");
			}
		}
		else Player->Msg("Please enter the reason for your GM request");
	}
}

void cCommands::DyeItem(P_PLAYER Player, int serial, unsigned short color) // Rehue an item
{
	if (!Player) return;

	P_ITEM pi = FindItemBySerial(serial);
	if (pi)
	{
		if (!dyeall[Player->cs])
		{
			if (color<0x0002 || color>0x03E9)
			{
				color=0x03E9;
			}
		}
		
		if (!(color&0xC000))
		{
			pi->setColor(color);
		}
		
		if (color==17969)
		{
			pi->setColor(color);
		}
		pi->refresh();//AntiChrist
		
		soundeffect(Player,0x023e); // plays the dye sound, LB
		return;
	}
	
	P_CHAR pc = FindCharBySerial(serial);
	if (pc)
	{
		if (!Player->isGM()) return; // Only gms dye characters
		int k=color;
		
		int body=pc->id();
		int b=k&0x4000; 
		
		if ( ( ( k>>8 ) < 0x80 ) && body >= 0x0190 && body <= 0x0193 ) k+= 0x8000;
		
		if (b==16384 && (body >=0x0190 && body<=0x03e1)) k=0xf000; // but assigning the only "transparent" value that works, namly semi-trasnparency.
		
		if (k!=0x8000) 
		{	
			pc->skin = pc->xskin = k;
			teleport(pc,pc->pos);
		}
	}
	soundeffect(Player, 0x023E); // plays the dye sound, LB
}


void cCommands::SetItemTrigger(P_PLAYER Player, P_ITEM pi)
{
	if (!pi) return;

	Player->Msg("Item triggered");
	pi->trigger=addx[Player->cs];
}

void cCommands::SetTriggerType(P_PLAYER Player, P_ITEM pi)
{
	if (!pi) return;

	Player->Msg("Trigger type set");
	pi->trigtype=addx[Player->cs];
}

void cCommands::SetTriggerWord(P_PLAYER Player, P_CHAR pc)
{
	if (pc)
	{
		Player->Msg("Trigger word set");
		strcpy(pc->trigword,xtext[Player->cs]);
	}
}

void cCommands::SetNPCTrigger(P_PLAYER Player, P_CHAR pc)
{
	if (pc)
	{
		//   if (pc->npc)
		//   {
		Player->Msg("NPC triggered");
		pc->trigger=addx[Player->cs];
		//   }else{
		//    Player->Msg("You can not trigger Player Characters");
		//   }
	}
}

void cCommands::AddHere(P_PLAYER Player, char z)
{
	int pileable=0;
	UOXSOCKET s = Player->cs;
	short id = (addid1[s]<<8)+addid2[s];
	
	if (isTruelyPileable(id)) pileable=1;
	
	cCoord tpos = Player->pos;
	tpos.z = z;
	P_ITEM pi=Items->SpawnItem(ITY_None, 1, "#", id, 0, tpos);
	if (pi)
	{
		pi->doordir=0;
		pi->resetPriv();
	}
	addid1[s]=0;
	addid2[s]=0;
}

void cCommands::WhoCommand(P_PLAYER Player, int type,int buttonnum)
{
	char sect[512];
	int length, length2, textlines;
	unsigned int line, i;
	char menuarray[7*(MAXCLIENT)+50][50];  /** lord binary **/
	char menuarray1[7*(MAXCLIENT)+50][50]; /** the ( IS important !!! **/
	unsigned int linecount=0;
	unsigned int linecount1=0;

	if (buttonnum < 0)
		return;


	int serial=whomenudata[buttonnum];
	P_CHAR pc_c = FindCharBySerial(serial); // find selected char ...
	if (!pc_c) 
	{
		Player->Msg("selected character not found");
		return;
	}
	
	//--static pages
	strcpy(menuarray[linecount++], "nomove");
	strcpy(menuarray[linecount++], "noclose");
	strcpy(menuarray[linecount++], "page 0");
	strcpy(menuarray[linecount++], "resizepic 0 0 5120 260 280");    //The background
	strcpy(menuarray[linecount++], "button 20 240 2130 2129 1 0 1"); //OKAY
	strcpy(menuarray[linecount++], "text 20 10 300 0");           //text <Spaces from Left> <Space from top> <Length, Color?> <# in order>
	strcpy(menuarray[linecount++], "text 20 30 300 1");
	
	//--Command Button Page 
	strcpy(menuarray[linecount++], "page 1");
	strcpy(menuarray[linecount++], "text 20 60 300 2");	//goto text
	strcpy(menuarray[linecount++], "button 150 60 1209 1210 1 0 200"); //goto button
	strcpy(menuarray[linecount++], "text 20 80 300 3");	//gettext
	strcpy(menuarray[linecount++], "button 150 80 1209 1210 1 0 201"); //get button
	strcpy(menuarray[linecount++], "text 20 100 300 4");	//Jail text
	strcpy(menuarray[linecount++], "button 150 100 1209 1210 1 0 202"); //Jail button
	strcpy(menuarray[linecount++], "text 20 120 300 5");	//Release text
	strcpy(menuarray[linecount++], "button 150 120 1209 1210 1 0 203"); //Release button
	strcpy(menuarray[linecount++], "text 20 140 300 6");	//Kick user text
	strcpy(menuarray[linecount++], "button 150 140 1209 1210 1 0 204"); //kick button
	strcpy(menuarray[linecount++], "text 20 180 300 7");

	length=21;
	length2=1;
	
	for(line=0;line<linecount;line++)
	{
		if (strlen(menuarray[line])==0)
			break;
		{
			length+=strlen(menuarray[line])+4;
			length2+=strlen(menuarray[line])+4;
		}
	}
	
	length+=3;
	textlines=0;
	line=0;
	
	sprintf(menuarray1[linecount1++], "User %i selected (account %i)",buttonnum,pc_c->account);
	sprintf(menuarray1[linecount1++], "Name: %s",pc_c->name);   
	sprintf(menuarray1[linecount1++], "Goto Character:");
	sprintf(menuarray1[linecount1++], "Get Character:");
	sprintf(menuarray1[linecount1++], "Jail Character:");
	sprintf(menuarray1[linecount1++], "Release Character:");
	sprintf(menuarray1[linecount1++], "Kick Character:");
	sprintf(menuarray1[linecount1++], "Serial# [%i]",pc_c->serial);   
	
	
	for(line=0;line<linecount1;line++)
	{
		if (strlen(menuarray1[line])==0)
			break;
		length+=strlen(menuarray1[line])*2 +2;
		textlines++;
	}

	unsigned char gump1[21]={0,};
	unsigned char gump2[3]={0,};
	unsigned char gump3[2]={0,};

	LongToCharPtr(1, &gump1[3]); // can be anything except 0		
	gump1[10]=type; // Gump Number
	gump1[19]=length2>>8;
	gump1[20]=length2%256;

	if (length>=32000)
	{
		Player->Msg("Whomenu gump data too huge, plz try wholist filtering");
		return;
	}

	XStartPkt(0xb0, length);
	XAddPkt(&gump1[0+3], 21-3);
	
	for(line=0;line<linecount;line++)
	{
		sprintf(sect, "{ %s }", menuarray[line]);
		XAddPkt((unsigned char*)&sect[0], strlen(sect));	
	}
	
	gump2[1]=textlines>>8;
	gump2[2]=textlines%256;
	
	XAddPkt(&gump2[0], 3);
	
	for(line=0;line<linecount1;line++)
	{
		if (strlen(menuarray1[line])==0)
			break;
		{
			gump3[0]=strlen(menuarray1[line])>>8;
			gump3[1]=strlen(menuarray1[line])%256;
			XAddPkt(&gump3[0], 2);
			gump3[0]=0;
			for (i=0;i<strlen(menuarray1[line]);i++)
			{
				gump3[1]=menuarray1[line][i];
				XAddPkt(&gump3[0], 2);
			}
		}
	}

	XFinishPkt(Player->cs);
}

bool cCommands::MakePlace(P_PLAYER Player, int i, cCoord& pos) // Decode a teleport location number into X/Y/Z
{
	if (!findLocation(i,pos))
	{
		Player->Msg("invalid place");
		return false;
	}
	return true;
}


void cCommands::DupeItem(P_PLAYER Player, P_ITEM pi, int amount)
{
	if (!pi) return;
	if (pi->corpse) return;
	if (!Player) return;

	P_ITEM Pack = Player->getBackpack();
	if (!Pack) return;

    P_ITEM Ic = Items->CloneItem(pi);
	if (!Ic) return;
	
	Ic->SetContSerial(Pack->serial);
	Ic->SetOwnSerial(pi->ownserial);
	Ic->SetSpawnSerial(pi->spawnserial);
	Ic->layer=0;	// it's created in a backpack
	Ic->amount=amount;
	
	Ic->refresh();//AntiChrist
}

void cCommands::ShowGMQue(P_PLAYER Player, int type) // Shows next unhandled call in the GM queue
{
	
	// Type is 0 if it is a Counselor doing the command (or a GM doing /cq) and 1 if it is a GM
	
	int i;
	int x=0;
	
	if (type==1) //Player is a GM
	{
		for(i=1;i<MAXPAGES;i++)
		{
			if (gmpages[i].handled==0)
			{
				if (x==0)
				{
					Player->Msg("");
					sprintf(temp,"Next unhandled page from %s", gmpages[i].name.c_str());
					Player->Msg(temp);
					sprintf(temp,"Problem: %s.", gmpages[i].reason.c_str());
					Player->Msg(temp);
					sprintf(temp,"Serial number %x", gmpages[i].serial);
					Player->Msg(temp);
					sprintf(temp,"Paged at %s.", gmpages[i].timeofcall);
					Player->Msg(temp);
				}
				x++;
			}
		}
		if (x>0)
		{
			sprintf(temp,"Total pages in queue: %i",x);
			Player->Msg("");
			Player->Msg(temp);
		}
		else Player->Msg("The GM queue is currently empty");
	} //end of first if
	else //Player is a counselor so show counselor queue
	{
		for(i=1;i<MAXPAGES;i++)
		{
			if (counspages[i].handled==0)
			{
				if (x==0)
				{
					Player->Msg("");
					sprintf(temp,"Next unhandled page from %s", counspages[i].name.c_str());
					Player->Msg(temp);
					sprintf(temp,"Problem: %s.", counspages[i].reason.c_str());
					Player->Msg(temp);
					sprintf(temp,"Serial number %x", counspages[i].serial);
					Player->Msg(temp);
					sprintf(temp,"Paged at %s.", counspages[i].timeofcall);
					Player->Msg(temp);
				}
				x++;
			}
		}
		if (x>0)
		{
			sprintf(temp,"Total pages in queue: %i",x);
			Player->Msg("");
			Player->Msg(temp);
		}
		else Player->Msg("The Counselor queue is currently empty");
	}
}
// new wipe function, basically it prints output on the console when someone wipes so that
// if a malicious GM wipes the world you know who to blame

void cCommands::Wipe(P_PLAYER Player)
{
	LogMessageVar("LoneWolf: %s has initiated an item wipe",Player->name);
	
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi=aii.GetData())!=aii.End(); aii++)
	{
		if (pi->isInWorld() && pi->wipe==0)
		{
			Items->DeleItem(pi->unconst());
		}
	}
	sysbroadcast("All items have been wiped."); 
}

void cCommands::PartialWipe(P_PLAYER Player, const cMapRectangle& rect, bool inverse)
{
	LogMessageVar("LoneWolf: %s has initiated a partial item wipe",Player->name);
	
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi=aii.GetData())!=aii.End(); aii++)
	{
		if (!pi->isInWorld() || pi->wipe)
			continue;
		if (pi->pos.IsIn(rect)) 
		{
			if (!inverse)
				Items->DeleItem(pi->unconst());
		}
		else
		{
			if (inverse && pi->pos.map == rect.mapNo)
				Items->DeleItem(pi->unconst());
		}
	}
}

void cCommands::Possess(P_PLAYER Player) 
{
	Player->Msg("wiping wordfile, hold on...");
	Player->Msg("just kidding");
	Player->Msg("command currently disabled");
}

void cCommands::GotoCur(P_PLAYER Player)	// Goes to the current call in the GM/Counsellor Queue
{
	if (Player->callnum<=0)
	{
		Player->Msg("You are not currently on a call.");
	}
	else
	{  
		P_CHAR pc = FindCharBySerial( gmpages[Player->callnum].serial );
		if (pc)
		{
			Player->travelTo(pc->pos);
			Player->Msg("Transporting to your current call.");
		}
	}
}

void cCommands::GMtransfer(P_PLAYER Player)
// Escilate a Counsellor Page into the GM Queue
{
	int i;
	int x2=0;
	
	if (Player->callnum>=0)
	{
		if (!Player->isGM()) //Char is a counselor
		{
			for(i=1;i<MAXPAGES;i++)
			{
				if (gmpages[i].handled==1)
				{
					gmpages[i].handled=0;
					gmpages[i].name = counspages[Player->callnum].name;
					gmpages[i].reason = counspages[Player->callnum].reason;
					gmpages[i].serial = counspages[Player->callnum].serial;
					time_t current_time = time(0);
					struct tm *local = localtime(&current_time);
					sprintf(gmpages[i].timeofcall, "%02d:%02d:%02d", local->tm_hour, local->tm_min, local->tm_sec);
					x2++;
					break;
				}
			}
			if (x2==0)
			{
				Player->Msg("The GM Queue is currently full. Contact the shard operator");
				Player->Msg("and ask them to increase the size of the queue.");
			}
			else
			{
				Player->Msg("Call successfully transferred to the GM queue.");
				donewithcall(Player,1);
			}
		}
		else
		{
			Player->Msg("Only Counselors may use this command.");
		}
	}
	else
	{
		Player->Msg("You are not currently on a call");
	}
}

bool cCommands::GetPageReason(P_PLAYER Player, char* comm)
{
	P_CHAR pc;
	AllPlayersIterator api;
	if (Player->pagegm == 1)
	{
		gmpages[Player->playercallnum].reason = comm;
		sprintf(temp, "GM Page from %s [%x]: %s",Player->name, Player->serial, comm);
		int x = 0;
		for (pc = api.First(); pc != NULL; pc = api.Next())
		{
			if (pc->isGM())
			{
				x = 1;
				pc->Msg(temp);
			}
		}
		if (x == 1)
			Player->Msg("Available Game Masters have been notified of your request.");
		else 
			Player->Msg("There was no Game Master available to take your call.");
		Player->pagegm = 0;
		return true;
	}
	if (Player->pagegm == 2) // Counselor page
	{
		counspages[Player->playercallnum].reason = comm;
		sprintf(temp, "Counselor Page from %s [%x]: %s", Player->name, Player->serial, comm);
		int x = 0;
		for (pc = api.First(); pc != NULL; pc = api.Next())
		{
			if (pc->isCounselor())
			{
				x = 1;
				pc->Msg(temp);
			}	
		}
		if (x == 1)
			Player->Msg("Available Counselors have been notified of your request.");
		else 
			Player->Msg("There was no Counselor available to take your call.");
		Player->pagegm = 0;
		return true;
	}
	return false;
}

void cCommands::broadcast(P_PLAYER Player, char* ttx, bool gmonly)
{
	if (now==1)
	{
		Player->Msg("There are no other users connected.");
		return;
	}

	short tl=strlen(ttx)+1;

	UOXSOCKET s = Player->cs;
	unsigned char talk[14];
	
	LongToCharPtr(Player->serial, &talk[3]);
	ShortToCharPtr(Player->id(), &talk[7]);
	talk[9]=1;
	talk[10]=buffer[s][4];
	talk[11]=buffer[s][5];
	talk[12]=buffer[s][6];
	talk[13]=Player->fonttype;
			 
	P_CHAR pc;
	AllPlayersIterator api;
	for (pc = api.First(); pc != NULL; pc = api.Next())
	{
		if (!gmonly || pc->isGM())
		{
			XStartPkt(0x1c,44+tl);
			XAddPkt(&talk[0+3], 14-3);
			XAddPkt((unsigned char*)&Player->name[0], 30);
			XAddPkt((unsigned char*)ttx, tl);   
			XFinishPkt(api.getSocket());
		}
	}
}

