/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//


// Include files

#include "coord.h"

// System Includes
#include <algorithm>
#include <cmath>
using namespace std;
#include "rectangle.h"

#define COORD_ALL_PLANES 255
#define COORD_HIDDEN_PLANE 254


cCoord::cCoord(void)
{
	x = y = z = map  = 0;
}

cCoord::cCoord(const cCoord& clCoord)
{
	(*this) = clCoord ;
}

cCoord::cCoord(UI16 uiX, UI16 uiY, SI08 siZ, UI08 uiMap)
{
	x = uiX;
	y = uiY;
	z = siZ;
	map = uiMap;
}

// temporary c'tor to distinguish map usage (Duke)
cCoord::cCoord(UI16 uiX, UI16 uiY, SI08 siZ)
{
	x = uiX;
	y = uiY;
	z = siZ;
	map = 0;
}

void cCoord::reset()
{
	x = y = z = map  = 0;
}
bool cCoord::isNotSet() const
{
	return !(x || y || z || map);
}

/*!
 * Calculates the distance between 2 coordinates. If they are in different planes and maps, the
 * return value is infinite (0xFFFFFFFF). The exception is if one of the planes is COORD_ALL_PLANES
 *
 * @param &src : Coordinate to compare
 *
 * @return UI32  : distance result
 */
UI32 cCoord::distance(const cCoord &src) const
{
	UI32 uiResult = ~0; // Initialize with *infinite*
	if (map == src.map)
	{
		RF64 rfResult = sqrt(static_cast<RF64>((x - src.x)*(x - src.x) + (y - src.y)*(y - src.y)));
		//		RF64 rfResult = sqrt(static_cast<RF64>((x * src.x + y * src.y + z * src.z)));
		uiResult = static_cast<UI32>(floor(rfResult)); // truncate
	}
	return uiResult;
}

// Operators
cCoord& cCoord::operator=(const cCoord& clCoord)
{
	x = clCoord.x;
	y = clCoord.y;
	z = clCoord.z;
	map = clCoord.map;	
	return (*this);
}

bool cCoord::operator ==(const cCoord& src) const
{
	return (x == src.x && y == src.y && z == src.z && map == src.map);
}

cCoord cCoord::operator+(const cCoord& src) const
{
	return cCoord(this->x + src.x, this->y + src.y, this->z + src.z, this->map);
}

bool cCoord::IsSameXY(const cCoord &other) const
{
	if (other.map!=map) return false;
	return (x==other.x && y==other.y);
}
bool cCoord::IsSameXY(short ox, short oy) const
{	
	return (x==ox && y==oy);
}
bool cCoord::IsIn(const cMapRectangle& mrect) const
{
	return mrect.includes(x,y,map);
}
void cCoord::Add(short dx, short dy, short dz)
{
	x+=dx;
	y+=dy;
	z+=dz;
}

int cCoord::calcDirTo(const cCoord &other) const
{
	int dx = other.x - this->x;
	int dy = other.y - this->y;

	int dir=-1;
	if		(dx==0 && dy<0) dir=0;
	else if (dx>0 && dy<0)	dir=1;
	else if (dx>0 && dy==0) dir=2;
	else if (dx>0 && dy>0)	dir=3;
	else if (dx==0 && dy>0) dir=4;
	else if (dx<0 && dy>0)	dir=5;
	else if (dx<0 && dy==0) dir=6;
	else if (dx<0 && dy<0)	dir=7;

	return dir;
}

