
#include "games.h"
#include "typedefs.h"
#include "globals.h"
#include "mapstuff.h"
#include "utilsys.h"
#include "debug.h"
#include "assert.h"
#include "items.h"
#include "scriptc.h"
#undef DBGFILE
#define DBGFILE "games.cpp"

// games in UO/lonewolf ...

// game #1:
// (actually not really a game in the traditional sense)

////////////////////////////////////////////////
//
// Coningways' Game of Life meets UO :-)
//
//  LB 1'st Nov 2004
//
/////////////////////////////////////////////////


// infos for experts:
// classical rules only -> #B3/S23, but should be fairly simple to adobt to any non classical Bx/Sx ruleset
// not the fastest implementaion on earth, but it has a few nice optimizations.
// (dynamic bounding box of active cells most noteably)
// On my 3GHZ test machine I could calculate 1.000.000 generations in 2 minutes on a 500*500 Universe with this "life-engine"
// Since it's only called once per sec here, this should be sufficienttly fast.
// Life patters are taken from gol.scp, for details about them look at it.

cRangebox::cRangebox(void)
{
  this->resetBox();
}

inline void cRangebox::addPoint(int x, int y)
{
  if      (x<curr_min_x) curr_min_x=x;
  else if (y<curr_min_y) curr_min_y=y;
  else if (x>curr_max_x) curr_max_x=x;
  else if (y>curr_max_y) curr_max_y=y;
}

void cRangebox::resetBox(void)
{
  this->setBox(0,0,0,0);
}

inline void cRangebox::getBox(int &xmin, int &ymin, int&xmax, int &ymax)
{
   xmin=curr_min_x;
   ymin=curr_min_y;
   xmax=curr_max_x;
   ymax=curr_max_y;
}

inline void cRangebox::setBox(int xmin, int  ymin, int xmax, int ymax)
{
  curr_min_x=xmin;
  curr_min_y=ymin;
  curr_max_x=xmax;
  curr_max_y=ymax;
}


bool cLife::timerOk(unsigned long int currtime)
{   
   if (intervall==0) return true;
   if (intervall<0) return false;

   if (lastUpdateTime < currtime)
   {
	  lastUpdateTime = currtime+intervall*(MY_CLOCKS_PER_SEC);
	  return true;
   } else return false;
}

int cLife::createSeeditem(void)
{
  P_ITEM pi = Items->SpawnItem(ITY_GOLSEED,1,"GOLSEED",0x186f,0);
  pi->decayOff();
  pi->setNotMovable();
  pi->visible=2; // only gm ivisible
  pi->MoveTo2(center);

  // yiekes, ugly as hell :/

  pi->hp=size;
  pi->maxhp=scriptnum;
  pi->st=pattern;
  pi->st2=effekt1;
  pi->dx=effekt2;
  pi->dx2=effekt3;
  pi->in=effekt4;
  pi->in2=intervall;

  return pi->serial;
}

cLife::cLife(int Pattern, int universeSize, cCoord cnter, int script, int interv, int e1, int e2, int e3, int e4)
{
  size=universeSize;
  center=cnter;
  generation=0;
  scriptnum=script;
  effekt1=e1;
  effekt2=e2;
  effekt3=e3;
  effekt4=e4;
  intervall=interv;
  lastUpdateTime=getNormalizedTime()-MY_CLOCKS_PER_SEC*interv;
  pattern = Pattern;
  error=false;

  if ( (size>1024) || (size<0)) {error=true; return; } // no technical reason, but takes huge amountes of ram, basicly 2*(size)^2+size+const bytes, limiting to approx max 3MB objects

  if ((size%2)==0) size++; // world size has to be odd

  // size*size grid
  int a,b;

  Universe    = new char*[size];
  tmpUniverse = new char*[size];

  for (a=0; a<size; a++)
  {
	  Universe[a]=    new char[size];
	  tmpUniverse[a]= new char[size];
  }

  for (a=0;a<size;a++)
  {
	  for(b=0;b<size;b++)
	  {
		  Universe[a][b]=NEW_DEAD__OLD_DEAD;
		  tmpUniverse[a][b]=NEW_DEAD__OLD_DEAD;
	  }
  }

  p_ofs = size / 2;

  seedItemserial = this->createSeeditem();

  bool ok  = this->loadFile(pattern);
  if (!ok) error=true;

}

cLife::~cLife(void)
{ 

   /* 2: removing all cell-items from game */ 

   std::map<std::pair<short, short>,int>::iterator itty; 
   for (itty=sers.begin(); itty != sers.end(); ++itty)
   {
	  int serial = (*itty).second;
	  P_ITEM pi = FindItemBySerial(serial);
	  Items->DeleItem( pi );
   }

   /* 3 freeing memory */

   sers.clear();

   int a;
   for (a=0; a<size; a++)
   {
	 delete [] Universe[a];
	 delete [] tmpUniverse[a];
   }

   delete[] Universe;
   delete[] tmpUniverse; 

}

inline bool cLife::coordstoArray(int x, int y, int &array_x, int &array_y)
{
  array_x = x + p_ofs;
  array_y = y + p_ofs;
  if ( array_x >=size || array_y >= size || array_x <0 || array_y <0) return false; else return true;
}

bool cLife::fileToVect(int patternnum, std::vector<std::pair<short,short> >& v)
{    

  Script* Scp = new Script("golpatterns.scp");
  if (!Scp->Open())
  {
	 LogWarning("golpatterns.scp not found");
	 delete Scp;
	 return false;
  }

  char sect[512];
  sprintf(sect, "PATTERN %i", patternnum);
  if (!Scp->find(sect))
  {
	LogWarningVar("Undefined pattern in golpatterns.scp %i", patternnum);
	Scp->Close();
	delete Scp;
	return false;
  } 

  int loopexit=0;
  while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS))
  {	
	 short x,y;
	 if (script1!=NULL && script2!=NULL)
	 {
	   x=atoi(script1);
	   y=atoi(script2);
	 }

	 v.push_back(std::pair<short,short>(x,y));
  } 
    
  Scp->Close();
  delete Scp;
  return true;
}

void cLife::vectToUniverse(std::vector<std::pair<short,short> >& v)
{   
   int cells=v.size();
   int a;

   for (a=0; a < cells; a++)
   {
	  short x,y;
	  int x_array;
	  int y_array;

	  x = v[a].first;
	  y = v[a].second;	 	  	 

      bool ok = coordstoArray(x, y, x_array, y_array);

	  if (ok)
	  {
	    Universe[x_array][y_array]=NEW_ALIFE__OLD_DEAD;
	    box.addPoint(x,y); // updates bounding box of active cells in universe
	  }	 

   }

   int x_min,y_min,x_max,y_max;
   box.getBox(x_min,y_min, x_max, y_max);
   oldbox.setBox(x_min,y_min, x_max, y_max); // initially, oldrangebox=currentbox
}

void cLife::restart(void)
{

  // delete all cell-items from server

  std::map<std::pair<short, short>,int>::iterator itty; 
  for (itty = sers.begin(); itty != sers.end(); ++itty)
  {
	int serial = (*itty).second;
	P_ITEM pi = FindItemBySerial(serial);
	if (pi!=NULL) { Items->DeleItem( pi );}// remove cell item from game							
  }							

  sers.clear(); // delete hash

  // now empty universe

  int a,b;

  for (a=0;a<size;a++)
  {
	  for(b=0;b<size;b++)
	  {
		  Universe[a][b]=NEW_DEAD__OLD_DEAD;
		  tmpUniverse[a][b]=NEW_DEAD__OLD_DEAD;
	  }
  }
	
  generation=0;

  box.resetBox(); 
  oldbox.resetBox();

  lastUpdateTime=getNormalizedTime()-MY_CLOCKS_PER_SEC*intervall;

  this->loadFile(pattern); // reload

}

bool cLife::loadFile(int patternnum)
{
	std::vector<std::pair<short,short> >v;

	bool ok = this->fileToVect(patternnum, v);
	this->vectToUniverse(v);

	return ok;
}

short color(cCoord pos, P_ITEM pi)
{
   int b;
   short color=pi->color();
   //if (rand()%44==0) color+=pi->pos.x-pi->pos.y; else
   color+=pos.x+pos.y;
   color+=rand()%3; // add random "noise"
   // lots of color consistancy checks
   color=color%0x03E9;
   if (color<0x0002 || color>0x03E9)
   {
	  color=0x03E9;
   }
   b=((color&0x4000)>>14)+((color&0x8000)>>15);
   if (b)
   {
	  color=0x100+rand()%255;
   }

	return color;
}

void cLife::print(void)
{
  int x_min,y_min,x_max,y_max;
  int x,y;

  oldbox.getBox(x_min,y_min, x_max, y_max); // we have to use old range, because it could have shrunk by unknown numer of borders. (maximum expansion is one, but maximun collapse is unknown)

  //int total_dead=0;
  //int total_alife=0;
  int new_born=0;
  int new_died=0;
  int unchanged_dead=0;
  int unchanged_alife=0;

  //#define __debuged

  #ifdef __debuged 
    printf("\n");
  #endif

  for (y=y_min-1; y<=y_max+1; y++) // we have to iterate over extender +1 border because we wouldnt fetch freshly died cells otherwise
  {
	  for (x=x_min-1;x<=x_max+1; x++)
	  {
		  int x_array,y_array;
		  bool ok = coordstoArray(x, y, x_array, y_array);
		  if (ok) 
		  {
			  char z=Universe[x_array][y_array];		
			  switch(z)
			  {
			     case NEW_DEAD__OLD_DEAD: 
					 unchanged_dead++;
					 #ifdef __debuged 
                       printf(".");
                     #endif
					 break;

                 case NEW_ALIFE__OLD_DEAD: 
					 {
						 new_born++; 

						  P_ITEM pi = Items->CreateScriptItem(scriptnum); // create new cell-item
                          if (pi!=NULL)
						  {
	                        pi->decayOff();
							pi->setNotMovable();
							pi->setpersistance(false); // dont make them persistent						

							cCoord it=center;
							it.x+=-x;
                            it.y+=y;
							it.z=Map->Height(it, it.z);							
	                        pi->MoveTo(it);

							if (effekt2!=0)
							{
								short clr=color(pi->pos, pi);
								pi->setColor(clr);
							}

	                        pi->refresh();
													
							sers.insert(make_pair(std::pair<short,short>(x,y), pi->serial)); // we have to keep track of serials of cell-items :(, otherwise we couldnt remove them if they die (without cross checking of ALL items in game for each dying cell)

							#ifdef __debuged 
                              printf("+");
                            #endif					
						  }												 
						  break;
					 }
					 
                 case NEW_DEAD__OLD_ALIFE: 
					 {
						 new_died++;

						 std::map<std::pair<short, short>,int>::iterator itty; // find cell-item-serial of dying cell
						 itty = sers.find(std::pair<short, short>(x,y));
                         if (itty != sers.end())
						 {
							int serial = (*itty).second;
							P_ITEM pi = FindItemBySerial(serial);
							if (pi!=NULL) { Items->DeleItem( pi );}// remove cell item from game						
							sers.erase(std::pair<short,short>(x,y)); // remove from item list

							#ifdef __debuged 
                        	  printf("-") ;   
                            #endif						
						 }							
						 						 
						 break;
					 }
                 case NEW_ALIFE__OLD_ALIFE: 
					 unchanged_alife++; 
					  #ifdef __debuged 
                        printf("*") ;   
                      #endif		
					 break;
			  }
		  }
	  }
	  #ifdef __debuged 
         printf("\n") ;   
      #endif			
  }

  //printf("total_a %i total_d %i new_born: %i new_dead :%i unchanged dead:%i unchagend alife:%i\n", total_alife, total_dead, new_born, new_died, unchanged_dead, unchanged_alife);
  //printf("\n");
}

void cLife::copytmpUniversetoUniverse(int x_min, int y_min, int x_max, int y_max)
{
  int x,y;

   // copy newly calculated universe to old one, probably could be speed up by memcpy
   for (y=y_min-1; y<=y_max+1; y++) 
   {
	  for (x=x_min-1;x<=x_max+1; x++)
	  {
		 int x_array, y_array;
		 bool ok = coordstoArray(x, y, x_array, y_array);
		 if (ok)
		 {
			 Universe[x_array][y_array]=tmpUniverse[x_array][y_array];
		 }
	  }
   }
}

void cLife::calcNextGen(void)
{
  generation++;

  int x_min,y_min,x_max,y_max;
  int x,y;

  //printf("generation: %i\n",generation);

  box.getBox(x_min,y_min, x_max, y_max); // get current bounding box and store on x_min etc
  box.resetBox(); // reset bounding box, recaclulate (for next gen) while calculating newxt generation

  //printf("ba: %i/%i %i/%i\n", x_min, y_min, x_max, y_max);

  for (y=y_min-1; y<=y_max+1; y++) // we have to look at bounding area extended by 1, otherwise newly born cells at border cant be found
  {
	  for (x=x_min-1;x<=x_max+1; x++)
	  {
		  int nbs, x_array, y_array;

		  nbs = neightbourStatus(x,y);
		  bool ok = coordstoArray(x, y, x_array, y_array);

		  if ( ok )
		  {
		    if ( (Universe[x_array][y_array] & ALIFEMASK_NEW)  )
			{
			  if (nbs==2 || nbs==3) { tmpUniverse[x_array][y_array]=NEW_ALIFE__OLD_ALIFE; box.addPoint(x,y); }
			  else                  { tmpUniverse[x_array][y_array]=NEW_DEAD__OLD_ALIFE;			         }
			}
		    else
			{
			  if (nbs==3) { tmpUniverse[x_array][y_array]=NEW_ALIFE__OLD_DEAD; box.addPoint(x,y); }
			  else        { tmpUniverse[x_array][y_array]=NEW_DEAD__OLD_DEAD;			            }
			  
			}
		  }
	  }	
  }  

  copytmpUniversetoUniverse(x_min,y_min,x_max,y_max);

  oldbox.setBox(x_min,y_min, x_max, y_max); // set old-range-box  to new one


}

inline bool cLife::neightbourStatusHelper(int x, int y, int dx, int dy)
{
  int nx,ny;

  nx=x+dx;
  ny=y+dy;
  
  int x_array, y_array;
  bool ok = coordstoArray(nx, ny, x_array, y_array);
  if (!ok) return false;
  if ( (Universe[x_array][y_array] & ALIFEMASK_NEW) ) return true; else return false;  

  return false;
}

// returns # neightbours, 4 if >=4 (in that case cell dies no matter what)
 
inline int cLife::neightbourStatus(int x, int y)
{
   int dx,dy;  
   int neightbours=0;

   dx=-1;dy=-1;neightbours+=neightbourStatusHelper(x,y,dx,dy);
   dx=-1;dy=0;neightbours+=neightbourStatusHelper(x,y,dx,dy);
   dx=-1;dy=1;neightbours+=neightbourStatusHelper(x,y,dx,dy);
   dx=0;dy=-1;neightbours+=neightbourStatusHelper(x,y,dx,dy);if (neightbours==4) return neightbours;
   dx=0;dy=1;neightbours+=neightbourStatusHelper(x,y,dx,dy); if (neightbours==4) return neightbours;
   dx=1;dy=-1;neightbours+=neightbourStatusHelper(x,y,dx,dy);if (neightbours==4) return neightbours;
   dx=1;dy=0;neightbours+=neightbourStatusHelper(x,y,dx,dy); if (neightbours==4) return neightbours;
   dx=1;dy=1;neightbours+=neightbourStatusHelper(x,y,dx,dy); if (neightbours==4) return neightbours;

   return neightbours;  
}


////////////////////

// #2:


