/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

//
// -- Items.cpp  Item specific routines (add, delete change) in preperation for
//               going to a pointer based system
//

#include "items.h"

#include "boats.h"
#include "books.h"
#include "games.h"
#include "house.h"
#include "itemid.h"
#include "mapS.h"
#include "mapstuff.h"
#include "multi.h"
#include "regions.h"
#include "scriptc.h"
#include "SndPkg.h"
#include "sregions.h"
#include "targeting.h"
#include "tiledata.h"
#include "trigger.h"
#include "utilsys.h"
#include "weight.h"
#include "worldmain.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "items.cpp"

#define ITEM_RESERVE 300	// minimum of free slots that should be left in the array.
							// otherwise, more memory will be allocated in the mainloop (Duke)

unsigned int itemcount;

cItem **realitems;			// the anchor point for all the item memory

void cItem::SetIndex(int i)
{
	this->index = i;
}
int cItem::GetIndex() const	// this is ONLY for places where we will continue to use an index !
{
	return this->index;
}

///////////////
// Name:	ReduceAmount
// history:	by Duke, 4.06.2000
//			added P_ITEM interface Duke, 3.10.2000
//			made it the first member of cItem Duke, 23.12.2000
//			changed to long amt to fix overflows Duke, 4.8.2002
// Purpose:	reduces the given item's amount by 'amt' and deletes it if
//			necessary and returns 0. If the request could not be fully satisfied,
//			the remainder is returned
//
long cItem::ReduceAmount(const long amt)
{
	long rest=0;
	if ( amount > amt )
	{
		amount-=amt;
		this->refresh();
	}
	else
	{
		Items->DeleItem(this);
		rest=amt-amount;
	}
	return rest;
}

// This method does not change the pointer arrays !!
// should only be used in VERY specific situations like initItem... Duke, 6.4.2001
void cItem::setContSerialOnly(int contser)
{
	contserial=contser;
}

bool cItem::Contains(PC_ITEM pi) const
{
	if (!pi) return false;
	return (this->serial == pi->contserial);
}

// looks if one item contains an other one, no matter if its directly in or in a subpack
bool cItem::ContainsSub(P_ITEM pi)
{
	if (!pi) return false;
	if (this->Contains(pi)) return true; //its in the pack
	if (pi->isInWorld()) return false; //its in the world

	//its in an other pack, so look through the packs
	P_ITEM cont = FindItemBySerial(pi->contserial);
	return this->ContainsSub(cont);
}

void cItem::SetContainer(P_ITEM Cont)
{
	if (Cont) contserial = Cont->serial;
}

bool cItem::SetContSerial(int contser)
{
	// note: negative values !=-1
    // a) crash, due to [-x] (-x%y<0)
	// b) are logically wrong

	if (contser<-1 || contserial <-1)
	{
		sprintf(temp,"Contserial invalid: %i %i\n", contser, contserial);
		LogError(temp);
		return false;
	}

	if (contserial!=-1)
		delFromContSp(contserial,this);

	setContSerialOnly(contser);

	if (contser!=-1)
		addToContSp(contser,this);

	return true;
}

void cItem::setOwnSerialOnly(int ownser)
{
	ownserial=ownser;
}

bool cItem::SetOwnSerial(int ownser)
{

	if (ownser<-1 || ownserial<-1)
	{
		sprintf(temp, "Ownerserial invalid: %i %i\n", ownser, ownserial);
		LogError(temp);
		return false;
	}

	if (ownserial!=-1)	// if it was set, remove the old one
		delFromOwnSp(ownserial,this);
	
	setOwnSerialOnly(ownser);

	if (ownser!=-1)		// if there is an owner, add it
		addToOwnSp(ownser,this);

	return true;
}

bool cItem::SetSpawnSerial(int spawnser)
{
	if (spawnser<-1 || spawnserial<-1)
	{
		sprintf(temp,"Spawnserial invalid: %i %i\n", spawnser, spawnserial);
		LogError(temp);
		return false;
	}

	if (spawnserial!=-1)	// if it was set, remove the old one
		delFromSpawnSp(spawnserial,this);

	spawnserial=spawnser;

	if (spawnser!=-1)		// if there is a spawner, add it
		addToSpawnSp(spawnser,this);

	return true;
}

bool cItem::SetMultiSerial(int mulser)
{
	if (mulser<-1 || serial<-1)
	{
		sprintf(temp, "Multiserial invalid: %i %i\n", mulser, serial);
		LogError(temp);
		return false;
	}

	if (multis!=-1)	// if it was set, remove the old one
		imultisp.remove(multis, serial);

	multis=mulser;

	if (mulser!=-1)		// if there is multi, add it
		imultisp.insert(mulser, serial);

	return true;

}

void cItem::MoveTo(int newx, int newy, signed char newz)
{
	mapRegions->Remove(this);
	pos.x=newx;
	pos.y=newy;
	pos.z=newz;
	mapRegions->Add(this);
}
void cItem::MoveTo(cCoord& npos, int offx, int offy, signed char offz)
{
	mapRegions->Remove(this);
	pos=npos;
	pos.Add(offx,offy,offz);
	mapRegions->Add(this);
}
void cItem::MoveTo(const cCoord& npos)
{
	mapRegions->Remove(this);
	pos=npos;
	mapRegions->Add(this);
}
void cItem::MoveTo2(const cCoord& pos)
{
	this->MoveTo(pos);
	this->refresh();
}

// author: LB purpose: returns the type of pack
// to handle its x,y coord system corretly.
// interpretation of the result:
// valid x,y ranges depending on type:
// type -1 : no pack
// type  1 : y-range 50  .. 100
// type  2 : y-range 30  .. 80
// type  3 : y-range 100 .. 150
// type  4 : y-range 40  .. 140 
// x-range 18 .. 118 for 1,2,3
//         40 .. 140 for 4
//
short cItem::GetContGumpType() const
{
	switch (id())
	{
	case 0x09a8: return 1;
	case 0x09a9: return 2;
	case 0x09aa: return 1;
	case 0x09ab: return 3;	// bankbox
	case 0x09b0: return 1;
	case 0x09b2: return 4;
		
	case 0x0A2C: return 1;	// chest of drawers
	case 0x0A30: return 1;	// chest of drawers
	case 0x0A34: return 1;	// chest of drawers
	case 0x0A38: return 1;	// chest of drawers

	case 0x0A4D: return 1;	// armoire
	case 0x0A4F: return 1;	// armoire
	case 0x0A51: return 1;	// armoire
	case 0x0A53: return 1;	// armoire
		
	case 0x0A97: return 1;	// bookshelf
	case 0x0A98: return 1;	// bookshelf
	case 0x0A99: return 1;	// bookshelf
	case 0x0A9A: return 1;	// bookshelf
	case 0x0A9B: return 1;	// bookshelf
	case 0x0A9C: return 1;	// bookshelf
	case 0x0A9D: return 1;	// bookshelf
	case 0x0A9E: return 1;	// bookshelf
		
	case 0x0e3c: return 2;
	case 0x0e3d: return 2;
	case 0x0e3e: return 2;
	case 0x0e3f: return 2;
	case 0x0e40: return 3;
	case 0x0e41: return 3;
	case 0x0e42: return 3;
	case 0x0e43: return 3;
	case 0x0e75: return 4;	// backpack
		
	case 0x0e76: return 1;
	case 0x0e77: return 4;
	case 0x0e78: return 2;
	case 0x0e79: return 1;
	case 0x0e7a: return 1;	// picnic basket
		
	case 0x0e7c: return 3;
	case 0x0e7d: return 1;
	case 0x0e7e: return 2;
	case 0x0e7f: return 4;
	case 0x0e80: return 1;
	case 0x0e83: return 4;
	
	case 0x0EFA: return 4;	// spellbook. Position shouldn't matter, but as it can be opened like a backpack...(Duke)

    case 0x2AF8: return 1;	// shop packs

		
	case 0x2006: return 5;	// a corpse/coffin
	default: return -1;
	}
}

void cItem::SetRandPosInCont(PC_ITEM pCont)
{
	int k=pCont->GetContGumpType();	
	if (k==-1)
	{
		LogWarningVar("trying to put something INTO a non container, id=0x%X",pCont->id());
		k=1;
	}
	pos.x = RandomNum(18, 118);
	pos.z=9;

	switch (k) 
	{
	case 1: pos.y = RandomNum(50, 100);		break;
	case 2: pos.y = RandomNum(30, 80);		break;
	case 3: pos.y = RandomNum(100, 140);	break;
	case 4: pos.y = RandomNum(60, 140);
			pos.x = RandomNum(60, 140);		break;
	case 5: pos.y = RandomNum(85, 160);
			pos.x = RandomNum(20, 70);		break;
	default: pos.y = RandomNum(30, 80);
	}
}

bool cItem::AddItem(cItem* pItem, short xx, short yy, bool sendit) const // Add Item to container
{
	if (pItem==NULL) return false;
	bool woulddoublesend=false;
	
	if (sendit)
	{
		SndRemoveItemToAll(pItem);		
	}
	
	// resets layer ... if the item was worn and moved to container this is essential
	pItem->oldlayer = pItem->layer;
    pItem->layer = 0;

	if (pItem->isInWorld())   // - remove from mapRegions if in world 
		mapRegions->Remove(pItem);

	pItem->SetContSerial(this->serial);
	if (xx!=-1)	// use the given position
	{
		pItem->pos.x=xx;
		pItem->pos.y=yy;
		pItem->pos.z=9;
	}
	else		// no pos given
	{
		if (!this->ContainerPileItem(pItem))	// try to pile
			pItem->SetRandPosInCont(this);		// not piled, random pos
		    else woulddoublesend=true;
	}

	if (sendit && !woulddoublesend) pItem->refresh();// refresh already done if gone through ContainerPileItem code

	return true;
}

bool cItem::PileItem(cItem* pItem)	// pile two items
{
	if (!(this->pileable && pItem->pileable &&
		this->serial!=pItem->serial &&
		this->id()==pItem->id() &&
		this->color()==pItem->color() ))
		return false;	//cannot stack.
	if (this->isType(ITY_CastingItem) || pItem->isType(ITY_CastingItem) )
		return false;	//cannot stack.

	if (this->amount+pItem->amount>65535)
	{
		pItem->pos.x=this->pos.x;
		pItem->pos.y=this->pos.y;
		pItem->pos.z=9;
		pItem->amount=(this->amount+pItem->amount)-65535;
		this->amount=65535;
		if (!this->isInWorld())
			pItem->SetContSerial(this->contserial);		// add to the same container as the item we dropped upon
		pItem->refresh();
	}
	else
	{
		this->amount+=pItem->amount;
		Items->DeleItem(pItem);
	}
	this->refresh();
	return true;
}

bool cItem::ContainerPileItem(cItem* pItem)	const // try to find an item in the container to stack with
{
	P_ITEM pi;	// item to stack with
	int ci=0,loopexit=0;
	while ( ((pi=ContainerSearchFor(this->serial,&ci,pItem->id(),pItem->color())) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->PileItem(pItem))
			return true;
	}
	return false;
}

P_ITEM cItem::searchForMatch(P_ITEM Other) const
{
	int	ci=0,loopexit=0;
	P_ITEM pi = NULL;

	while ( ((pi=this->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->matches(Other))
			return pi;
	}
	return NULL;
}

int cItem::CountItems(short ID, short col, short ty) const
{
	return ContainerCountItems(serial, ID, col, ty);
}

///////////////////////
// Name:	DeleteAmount
// history:	DeleQuan() by Duke, 16.11.2000
//			moved to cItem (Duke,27.3.2001)
// Purpose:	recurses through this container and deletes items of
//			the given id and color(if given) until the given amount is reached
//
int cItem::DeleteAmount(int amount, short id, short color, short ty)
{
	int rest=amount,loopexit=0;
	int ci=0;
	P_ITEM pi;
	while ( ((pi=this->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->isType(ITY_Container))
			rest=pi->DeleteAmount(rest, id, color, ty);
		if (pi->id()==id
			&& (color==-1 || (pi->color()==color))
			&& (ty==-1 || pi->isType(ty)))
			rest=pi->ReduceAmount(rest);
		if (rest<=0)
			break;
	}
	return rest;
}

///////////////////////
// Name:	LockDown
// history:	by Duke, 10.01.2003
// Purpose:	locks down an item. If it's a container, locks down the items inside too
//
void cItem::LockDown(int ownser)
{
	this->setLockedDown();
	this->setOwnSerialOnly(ownser);
	if (this->isType(ITY_Container))
	{
		int loopexit=0;
		int ci=0;
		P_ITEM pi;
		while ( ((pi=this->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
		{
			pi->LockDown(ownser);
		}
	}
	this->refresh();
}

///////////////////////
// Name:	ReleaseLockedDown
// history:	by Duke, 16.01.2003
// Purpose:	a locked down item. If it's a container, releases the items inside too
//
void cItem::ReleaseLockedDown()
{
	this->ResetLockedDown();
	if (this->isType(ITY_Container))
	{
		int loopexit=0;
		int ci=0;
		P_ITEM pi;
		while ( ((pi=this->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
		{
			pi->ReleaseLockedDown();
		}
	}
	this->secureIt = 0;
	this->refresh();
}

///////////////////////
// Name:	TransferLockDown
// history:	by Duke, 15.01.2003
// Purpose:	changes the owner. If it's a container, changes the items inside too
//
void cItem::TransferLockedDown(int newownser)
{
	this->setOwnSerialOnly(newownser);
	if (this->isType(ITY_Container))
	{
		int loopexit=0;
		int ci=0;
		P_ITEM pi;
		while ( ((pi=this->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
		{
			pi->TransferLockedDown(newownser);
		}
	}
	this->refresh();
}

void cItem::setId(short id)
{
	id1=id>>8;
	id2=id&0x00FF;
}

void cItem::setColor(short col)
{
	_color=col;
}

static int getItemName(short id, unsigned short amount, char* itemname)
{
	const tileData* tile = TileData->SeekTile(id);
	itemname[0]=0;
	if (amount <= 1)
	{
		if (TileData->isAn(tile->flags))   strcpy(itemname, "an ");
		else if (TileData->isA(tile->flags)) strcpy(itemname, "a ");
	}

	int namLen = strlen(itemname );
	int mode=0;
	int used=0;
	int len=strlen((char *) tile->name);
	int j,ok;
	for (j=0;j<len;j++)
	{
		ok=0;
		if		(tile->name[j]=='%' && mode==0) mode=2;	// start of plural "%s%"
		else if (tile->name[j]=='%' && mode!=0) mode=0;	// end of plural 's'
		else if (tile->name[j]=='/' && mode==2) mode=1;	// no clue
		else if (mode==0) ok=1;
		else if (mode==1 && amount==1) ok=1;
		else if (mode==2 && amount >1) ok=1;
		if (ok)
		{
			itemname[namLen++] = tile->name[j];		// copy the character
			itemname[namLen]   = '\0';
			if (mode) used=1;
		}
	}
	if (amount > 1 && !used)	// if no plural s has been added by %s%
		strcat(itemname,"s");

	return strlen(itemname)+1;
}

int cItem::getName(char* itemname)
{
	P_ITEM pi=this;
	if (this->name[0]!='#')
	{
		strcpy(itemname, this->name);
		return strlen(itemname)+1;
	}
	return getItemName(id(),pi->amount,itemname);
}

string cItem::getName(void)
{
	char itemname[256] = {0,};
	cItem::getName(itemname);
	string sItemName(itemname);
	return sItemName;
}

// return the weight of an item. May have to seek it from MUL files
int cItem::getWeight()
{
	unsigned int itemweight=0;

	if (this->weight>0) //weight is defined in scripts for this item
		itemweight=this->weight;
	else
	{
		const tileData *tile;
		tile = TileData->SeekTile(this->id());
		if (tile->weight==0) // can't find weight
		{
			if (!this->isType(ITY_Food))
				itemweight = 2;		// not food weighs .02 stone
			else
				itemweight = 100;	//food weighs 1 stone
		}
		else //found the weight from the tile, set it for next time
		{			
			itemweight = tile->weight*100;
			this->weight=itemweight; // set weight so next time don't have to search
		}
	}
	return itemweight;
}

///////////////////////
// Name:	setMore1_4
// history:	by Duke, 19.02.2002
// Purpose:	sets more1-more4 from the int given by n
//
void cItem::setMore1_4(int n)
{
	more1 = (unsigned char) (n>>24);
	more2 = (unsigned char) (n>>16);
	more3 = (unsigned char) (n>>8);
	more4 = (unsigned char) (n&0xFF);
}

void cItem::setMore1_2(int n)
{
	more1 = (unsigned char) (n>>8);
	more2 = (unsigned char) (n&0xFF);
}

void cItem::setMore3_4(int n)
{
	more3 = (unsigned char) (n>>8);
	more4 = (unsigned char) (n&0xFF);
}

void cItem::setMoreB1_4(int n)
{
	moreb1 = (unsigned char) (n>>24);
	moreb2 = (unsigned char) (n>>16);
	moreb3 = (unsigned char) (n>>8);
	moreb4 = (unsigned char) (n&0xFF);
}

void cItem::setMoreB1_2(int n)
{
	moreb1 = (unsigned char) (n>>8);
	moreb2 = (unsigned char) (n&0xFF);
}

void cItem::setMoreB3_4(int n)
{
	moreb3 = (unsigned char) (n>>8);
	moreb4 = (unsigned char) (n&0xFF);
}

void cItem::setMorePos(const cCoord& pos)
{
	morex = pos.x;
	morey = pos.y;
	morez = pos.z;
	morem = pos.map;
}

int		cItem::getMore1_4()	const	{return (getMore1_2()<<16) | (unsigned short)getMore3_4();}
short	cItem::getMore1_2()	const	{return (more1<<8)+more2;}
short	cItem::getMore3_4()	const	{return (more3<<8)+more4;}

int		cItem::getMoreB1_4() const	{return (getMoreB1_2()<<16) | (unsigned short)getMoreB3_4();}
short	cItem::getMoreB1_2() const	{return (moreb1<<8)+moreb2;}
short	cItem::getMoreB3_4() const	{return (moreb3<<8)+moreb4;}

void cItem::refresh()
{
	RefreshItem(this);
}

void cItem::startDecay()
{
	this->decaytimer.setSecs(SrvParms->decaytimer);
}

P_ITEM cItem::Search(int *ci) const
{
	return ContainerSearch(this->serial, ci);
}


void cItem::setSerialOnly(int ser)
{
	this->serial=ser;
}

void cItem::setSerial(int ser)
{
	setSerialOnly(ser);
	if (ser > -1)
		addToItemSp(this);
}

// -- Initialize an Item in the items array
void cItem::Init(bool mkser)
{
	if (mkser)		// give it a NEW serial #
		this->setSerial(Items->GetNextFreeSerial());
	else
		this->setSerial(-1);

	this->archive = false;
	this->free=0;
	strcpy(this->name,"#");
	strcpy(this->name2,"#");  // Magius(CHE)
	this->creator[0] = '\0';
	this->incognito=false;   // AntiChrist - incognito
	this->madewith=0;        // Added by Magius(CHE)
	this->rank=0;            // Magius(CHE)
	this->good=-1;           // Magius(CHE)
	this->rndvaluerate=0;    // Magius(CHE) (2)

	this->setpersistance(true);     // item gets saved/persistent (to wordfile)

	this->qsf=-1;
	this->multis=-1;         // Multi serial
	this->flags.isBeeingDragged=0;
	this->setId(0x0001);    // Item visuals as stored in the client
	this->setColor(0x00);         // Hue

	this->pos.x=this->oldpos.x=100;
	this->pos.y=this->oldpos.y=100;
	this->pos.z=this->oldpos.z=0;
	this->pos.map=0;  // do not set to server default map (allMaps->getserverdefault())

	this->setContSerialOnly(-1);  // Container that this item is found in
	this->oldcontserial=-1;
	this->layer=this->oldlayer=0; // Layer if equipped on paperdoll
	this->itmhand=0;              // Layer if equipped on paperdoll
	this->setType(ITY_None);	  // For things that do special things on doubleclicking
	this->type2=0;
	this->offspell=0;
	this->weight=0;
	this->setMore1_4(0);	// For various stuff
	this->setMoreB1_4(0);	// For various stuff
	this->morex=0;
	this->morey=0;
	this->morez=0;
	this->morem=0;
	this->amount=1;    // Amount of items in pile
	this->amount2=0;   // Used to track things like number of yards left in a roll of cloth
	this->doordir=0;   // Reserved for doors
	this->dooropen=0;
	this->pileable=0;  // Can item be piled
	this->dye=0;       // Reserved: Can item be dyed by dye kit
	this->corpse=0;    // Is item a corpse
	this->carve=-1;    // AntiChrist-for new carving system
	this->att=0;       // Item attack
	this->def=0;       // Item defense
	this->lodamage=0;  // Minimum Damage weapon inflicts
	this->hidamage=0;  // Maximum damage weapon inflicts
	this->racehate=0;  // race hating weapon -Fraz-
	this->secureIt=0;  // secured chests
	this->hp=0;        // Number of hit points an item has.
	this->maxhp=0;     // Max number of hit points an item can have.
	this->st=0;        // The strength needed to equip the item
	this->st2=0;       // The strength the item gives
	this->dx=0;        // The dexterity needed to equip the item
	this->dx2=0;       // The dexterity the item gives
	this->in=0;        // The intelligence needed to equip the item
	this->in2=0;       // The intelligence the item gives
	this->modskill=-1;		// item modifies this skill #
	this->modskillamt=0;	// amount of skill change to the modified skill
	this->spd=0;       // The speed of the weapon
	this->wipe=0;      // Should this item be wiped with the /wipe command
	this->setMovability(0); // 0=Default as stored in client, 1=Always movable, 2=Never movable, 3=Owner movable.
	this->gatetime=0;
	this->decaytimer.reset();
	this->setOwnSerialOnly(-1);
	this->visible=0;   // 0=Normally Visible, 1=Owner & GM Visible, 2=GM Visible
	this->spawnserial=-1;
	this->dir=0;       // Direction, or light source type.
	this->resetPriv(); // Bit 0, decay off/on.  Bit 1, newbie item off/on.  Bit 2 Dispellable
	this->value=0;     // Price shopkeeper sells item at.
	this->restock=0;   // Number up to which shopkeeper should restock this item
	this->trigger=0;   //Trigger number that item activates
	this->trigtype=0;  //Type of trigger
	this->trigon=0;    // equipped item trigger -Frazurbluu-
	this->disabled=0;  //Item is disabled, cant trigger.
	this->tuses=0;             //Number of uses for trigger
	this->poisoned=0;          //AntiChrist -- for poisoning skill
	this->murderer[0]=0x00;    //AntiChrist -- for corpse -- char's name who kille the char (forensic ev.)
 	this->murdertime=0;        //AntiChrist -- for corpse -- when the people has been killed
	this->glow=0;
	this->glow_effect=0;
	this->glow_color=0;
	this->time_unused=0;
	this->timeused_last=getNormalizedTime();
	this->spawnregion=0;
	this->thi=NULL;
	this->TrigInf=NULL;
	this->desc[0]=0x00;
	this->SetFoodType(0);	// foodtype 0 = no food

	this->lockdownmax=0;
	this->securemax=0;

}

///////////////
// purpose:	check if items are equivalent from a vendor's point of view
// history: created as a q&d fix for 'sell price higher than buy price' bug (Duke, 30.3.2001)
//
bool cItem::matches(const P_ITEM pi) const
{
	bool basic=false;

	if (this->id()==pi->id() &&
		this->isType(pi->type) ) basic=true;

	if (basic)
	{		
		if      (this->id()==0x14F0 && (this->morex!=pi->morex)) return true;		    // house deeds only differ by morex
		else if (IsShield(this->id() && !strcmp(this->name2,pi->name2))) return true;    // magic shields only differ by name2
		else if (IsMetalArmour(this->id() && this->color()==pi->color()) ) return true;	// color checking for armour
				
		if (this->color()==pi->color()) return true;

	}

	return false;
}
 
int cItem::calcBuyPrice(short region)	// the price a player pays if he buys it from a vendor
{
	int price = this->value;
	price = calcValue(this, price);
	if (SrvParms->trade_system==1) price=calcGoodValue(region,this,price,0);
	return price;
}

int cItem::calcSellPrice(short region, P_ITEM PlayersItem)
{
	int price = this->value;	
    price = calcValue(PlayersItem, price);
	if (SrvParms->trade_system==1) price=calcGoodValue(region,PlayersItem,price,1);
	return price;
}

int cItem::CountSpawnedItems() const
{
	int count=0;
	int loopexit=0;
	int ci=0;
	P_ITEM pi;
	while ( ((pi=SearchSpawnedItems(this->serial,&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (!pi->isSameAs(this)
			&& this->pos == pi->pos )
			count++;
	}
	return count;
}

int cItem::CountSpawnedChars() const
{
	int count=0;
	int loopexit=0;
	int ci=0;
	P_CHAR pc;
	while ( ((pc=SearchSpawnedChars(this->serial,&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		count++;
	}
	return count;
}

bool cItem::isContainerType()	//item is another container ?
{
	return (type == ITY_Container
		||	type == ITY_SpawnChest
		||	type == ITY_65
		||	type == ITY_Trashbin
		||	type == ITY_LockedContainer
		||	type == ITY_LockedSpawnChest );
}

bool cItem::IsSpawnerType()
{
	return (type == ITY_ItemSpawner
		||	type == ITY_NpcSpawner
		||	type == ITY_NpcAreaSpawner
		||	type == ITY_QuestSpawn
		||	type == ITY_SpawnChest
		||	type == ITY_LockedSpawnChest
		||	type == ITY_65
		||	type == ITY_66
		||	type == ITY_LockedContainer	);
}

//////////////////
// name:	Item_ToolWearOut (2 interfaces)
// purpose:	apply wear out to item, delete if necessary
// history:	first impl by Ripper in various places
//			collected here by Duke, 7.11.2000
//			finally moved to cItem by Duke, 8.12.2003
//			added variable chance (Duke, 13.5.05)
//
bool cItem::WearOut(int chance)
{
	if (chance < 1) return false;
	if ((rand()%chance)==0)
		this->hp--; //Take off a hit point
	if (this->hp<=0)
	{	 
		Items->DeleItem(this);
		return true;
	}
	else
		return false;
}

int cItem::getBowtype()
{
	short id = this->id();
	if (IsBow(id)) return 1;
	if (IsCrossbow(id)) return 2;
	if (IsHeavyCrossbow(id)) return 3;
	return 0;
}

short cItem::GetSound(bool well)
{
	short snd = 0x38;
	switch(this->id())
	{
	case 0x0E9C: snd=0x0038;	break;
	case 0x0E9D:
	case 0x0E9E: snd=0x0052;	break;
	case 0x0EB1:
	case 0x0EB2: snd=0x0045;	break;
	case 0x0EB3:
	case 0x0EB4: snd=0x004C;	break;
	default:
		LogError("switch reached default");
	}
	if (!well) snd++;		// sound for bad playing is one above the 'good' sound

	return snd;
}


cItem::cTriggerInfo* cItem::GetTrigInf()
{
	if (TrigInf == NULL)
	{
		TrigInf = new cTriggerInfo;
		TrigInf->disabledmsg[0]=0;
	}
	return TrigInf;
}


cAllItems::cAllItems() {imem=0; itemcount=0; moreItemMemoryRequested=false; itemcount2=0x40000000;}
int cAllItems::GetCount()		{ return itemcount; }
int cAllItems::GetCapacity()	{ return imem; }

P_ITEM cAllItems::MakeRef(int i)
{
	errorIA=0; // clear previous state, 0=no error
	
	if ( i<0 || i>=(int)itemcount || realitems[i] == NULL)
	{
		errorIA=1;
		return NULL;
	}
	
	return realitems[i];
}

void cAllItems::CheckMemoryRequest()
{
	if (moreItemMemoryRequested)
		ResizeMemory();
}

bool cAllItems::AllocateMemory(int NumberOfItems)
{
	imem=NumberOfItems;
	imem+=(2*ITEM_RESERVE);	// some reserve so realloc doesn't occur immediately after serverstart(Duke)
	if (imem<400000) imem=400000;	// as we now only allocate item *pointers* that's only 1.6 MB (Duke)

	int sz = imem*sizeof(cItem*);
	if ((realitems= (cItem **) malloc(sz))==NULL)
	{
		Console.send("\nERROR: Could not Allocate item memory!\n");
		error=1;
		return false;
	}

	memset(realitems,0,sz);		// make it a bunch of NULL pointers

	Console.send("Done\n");
	return true;
}

bool cAllItems::ResizeMemory()
{
	const int slots=4000;   // bugfix for crashes under w95/98, LB never ever touch this number ...
	// it has major influence on performance !! if it too low -> slow and can cause crashes under w95/98.
	// this is because w95/98 can only handle 8196 subsequent realloc calls ( crappy thing...)
	// free() calls DONT help !!!, btw so we have to make that number real big
	// under nt and unix this limit doesnt exist

	P_ITEM *tp = (cItem **)realloc(realitems, (imem + slots)*sizeof(cItem*));
	if (tp)
		realitems=tp;	// change realitems only if realloc was successfull !! (Duke)
	else
	{
		LogCriticalVar("Could not reallocate item memory after %i. No more items will be created.\nLoneWolf may become unstable.",imem);
		LogMessage("LoneWolf shutdown due to item memory allocation problems");
		keeprun = 0;		// schedule shutdown, using the regular save in main()
		return false;
	}
	
	for (unsigned int i=imem;i<(imem+slots);i++)
	{
		realitems[i] = NULL;
	}
	imem+=slots;
	moreItemMemoryRequested=false;
	return true;
}

///////////////////////
// Name:	GetReusableSlot
// history:	by Duke, 17.05.2001
// Purpose:	searches for an empty slot in the items array
//			It does this by simply searching from the *last* position where it found a slot.
//			If it reaches the end, it will search from the start. A timer prevents too many wrap-arounds.
//
int cAllItems::GetReusableSlot()
{
	static unsigned int NextMemCheck=0;
	static unsigned int LastFree=0;	// remember this point in the array to continue search from here next time

	unsigned int ctime=uiCurrentTime;
	if (NextMemCheck<=ctime || (overflow))	// didn't wrap around in the past minute
	{
		unsigned int started=LastFree;		// remember where we started the search from here next time
		for (  ;LastFree<itemcount;LastFree++)
		{
			P_ITEM iLastfree = MAKE_ITEM_REF(LastFree);
			if (iLastfree==NULL)
				return LastFree;	
		}
		// we have reached the end so start again from the beginning of the array
		for (LastFree=0;LastFree<started;LastFree++)
		{
			P_ITEM iLastfree = MAKE_ITEM_REF(LastFree);
			if (iLastfree==NULL)
				return LastFree;	
		}
		NextMemCheck=ctime+60*MY_CLOCKS_PER_SEC;	// nothing found, set time for next try
	}
	return -1;
}

int cAllItems::MemItemFree()// -- Find a free item slot
{
	if (itemcount+ITEM_RESERVE >=  imem)	//less than 'Reserve' free slots left, so get more memory
		moreItemMemoryRequested=true;

	int nItem=GetReusableSlot();		// try to use the space of a deleted item
	if (nItem==-1)
	{
		if (itemcount+1>=imem)	//theres no more free sluts.. er slots
		{
			LogCritical("couldn't resize item memory in time");
			return -1;
		}
		nItem=itemcount;
		itemcount++;
	}
	realitems[nItem] = new cItem;
	return nItem;
}

int cAllItems::GetNextFreeSerial()
{
	return itemcount2++;
}

void cAllItems::DetermineHighestSerial(int ser)
{
	if (itemcount2<=ser) itemcount2=ser+1;
}

P_ITEM cAllItems::SplitPile(P_ITEM Pile, short amount)
{
	if (amount >= Pile->amount) return Pile;
	
	P_ITEM Part = Items->CloneItem(Pile);
	if (!Part) return NULL;	// bounce etc
		
	Part->amount = amount;
	Pile->amount -= amount;
		
	Part->SetContSerial(Part->contserial);
	Part->SetOwnSerial(Part->ownserial);
	Part->SetSpawnSerial(Part->spawnserial);

	Pile->refresh();	// the Part is expected to be moved elsewhere, NO refresh here
	return Part;
}

P_ITEM cAllItems::CloneItem(P_ITEM Source)
{
	int i = MemItemFree();
	P_ITEM pi = MAKE_ITEM_REF(i);
	if (!pi) return NULL;
	memcpy(pi, Source, sizeof(cItem));
	pi->SetIndex(i);
	pi->setSerial(Items->GetNextFreeSerial());
	if (Source->TrigInf)
		pi->GetTrigInf();		// will actually allocate and init a separate instance
	return pi;
}

P_ITEM cAllItems::MakeBaseItem(bool newserial)
{
	int i = MemItemFree();
	P_ITEM pi = MAKE_ITEM_REF(i);
	if (!pi) return NULL;
	pi->SetIndex(i);
	pi->Init(newserial);
	return pi;
}

// debug verison (tracing deletes)
// gets actived if #define debugDelete in lwmisc.h is defined
void cAllItems::dbg_DeleItem(P_ITEM pi, int src, char*srcfile)
{
	if (!pi) return;

	char tmp[768];
	sprintf(tmp,"DELEITEM called from: %s line %i\nDELETED-Item Infos: serial:%i, contserial:%i, id:%i, name:%s, pos:(%i/%i/%i/%i)\n", srcfile, src, pi->serial,  pi->contserial,  pi->id(), pi->name, pi->pos.x, pi->pos.y, pi->pos.z, pi->pos.map);
	LogWarning(tmp);
	Console.send(tmp);

	this->_DeleItem(pi);
}


// -- delete an item (Actually just mark it is free)
void cAllItems::_DeleItem(P_ITEM pi)
{	
	if (!pi) return;

	SndRemoveItemToAll(pi);	// tell the client

	if (pi->serial >= 0)	// invalid serial is a problem ... because it crashes all hash tables
	{
		if (pi->isType(ITY_Container) || pi->isType(ITY_SpellBook))	// a simple container or a spellbook ?
		{
			int ci=0,loopexit=0;
			P_ITEM pDel;
			while ( ((pDel=pi->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
			{
				Items->DeleItem(pDel);	// delete the contents too
			}
		}
		
		DecSpawnCount(pi->spawnregion);
		
		if (pi->glow>0) 
		{  
			P_ITEM Halo = FindItemBySerial( pi->glow );
			if (Halo) Items->DeleItem(Halo); // lb glow stuff, deletes the glower of an glowing stuff automatically
		}
		
		// - remove from pointer arrays
		delFromItemSp(pi);
		pi->SetSpawnSerial(-1);
		pi->SetOwnSerial(-1);
		
		if (pi->isInWorld())	// - remove from mapRegions if a world item
			mapRegions->Remove(pi);
		else
			pi->SetContSerial(-1);
		
		if (pi->isType(ITY_Book) && (pi->morex==666 || pi->morey==999))	// if a new book gets deleted also delete the corresponding bok file
			Books->delete_bokfile(pi);
	}
	if (pi->TrigInf)
		delete pi->TrigInf;

	if (pi->getType()==ITY_GOLSEED)
	{
		vector<cLife*>::iterator it;
		for (it=golSeeds.begin(); it!=golSeeds.end(); ++it)
		{
			if (pi->serial==(*it)->getSeedSerial())
			{	
				cLife* d=(*it);
                golSeeds.erase(it); 
				delete d;
				break;
			}
		}	

	}
	
//	int i = pi->GetIndex();
//	delete realitems[i];
//	realitems[i] = NULL;	// we can't do that here because it invalidated existing pointers
	pi->free=1;				// mark it as 'free' and delete it in the mainloop (Duke)
}

void cAllItems::FreeMemory()
{
	for (unsigned int i=0; i<itemcount; i++)
	{
		if (realitems[i])
		{
			if (realitems[i]->free)
			{
				delete realitems[i];
				realitems[i] = NULL;
			}
		}
	}
}

int cAllItems::applyCommonTags(P_ITEM pi)
{
	switch(script1[0])
	{
	case 'A':
		if (!strcmp(script1,"AMOUNT"))			pi->amount = str2num(script2);
		else return 0;
		break;
	case 'C':
		if (!strcmp(script1, "CREATOR"))		strcpy(pi->creator,script2);
		else return 0;
		break;
	case 'D':
		if (!strcmp(script1, "DEF"))			pi->def=str2num(script2);
		else if (!strcmp(script1, "DISABLEMSG"))strcpy((pi->GetTrigInf())->disabledmsg,script2);
		else if (!strcmp(script1, "DISABLED"))	pi->disabled=uiCurrentTime+(str2num(script2)*MY_CLOCKS_PER_SEC);
		else if (!strcmp(script1, "DIR"))		pi->dir=str2num(script2);
		else return 0;
		break;
	case 'F':
		if (!strcmp("FOODTYPE",script1)) // tags for new food system (thrawn)
		{
			if (!strlen(script2)) // be sure min. one food flag exists
			{
				LogWarningVar("Tag '%s' without subtags found in items.scp",script1);
				break; 
			}

			pi->setType(ITY_Food); // automatically setting food type when food tags are found

			if (strstr(script2,"BAKED"))		pi->AddFoodType((unsigned char)FOOD_BAKED);
			if (strstr(script2,"CHEESE"))		pi->AddFoodType((unsigned char)FOOD_CHEESE);
			if (strstr(script2,"FISH"))			pi->AddFoodType((unsigned char)FOOD_FISH);
			if (strstr(script2,"FRUITS"))		pi->AddFoodType((unsigned char)FOOD_FRUIT);
			if (strstr(script2,"MAGIC"))		pi->AddFoodType((unsigned char)FOOD_MAGIC);
			if (strstr(script2,"MEAT"))			pi->AddFoodType((unsigned char)FOOD_MEAT);
			if (strstr(script2,"RAW"))			pi->AddFoodType((unsigned char)FOOD_RAW);
			if (strstr(script2,"VEGETABLES"))	pi->AddFoodType((unsigned char)FOOD_VEGETABLE);
		}
		else return 0;
		break;
	case 'G':
		if (!strcmp(script1, "GOOD"))			pi->good=str2num(script2);
		else return 0;
		break;
	case 'H':
		if (!strcmp(script1, "HIDAMAGE"))		pi->hidamage=str2num(script2);
		else if (!strcmp(script1, "HP"))		pi->hp=str2num(script2);
		else return 0;
		break;
	case 'I':
		if (!strcmp(script1, "ITEMHAND"))		pi->itmhand=str2num(script2);
		else return 0;
		break;
	case 'L':
		if (!strcmp(script1, "LODAMAGE"))		pi->lodamage=str2num(script2);
		else return 0;
		break;
	case 'M':
		if (!strcmp(script1, "MORE"))			pi->setMore1_4(str2num(script2));
		else if (!strcmp(script1, "MORE2"))		pi->setMoreB1_4(str2num(script2));
		else if (!strcmp(script1, "MOVABLE"))	pi->setMovability(str2num(script2));
		else if (!strcmp(script1, "MAXHP"))		pi->maxhp=str2num(script2);
		else if (!strcmp(script1, "MOREX"))		pi->morex=str2num(script2);
		else if (!strcmp(script1, "MOREY"))		pi->morey=str2num(script2);
		else if (!strcmp(script1, "MOREZ"))		pi->morez=str2num(script2);
		else if (!strcmp(script1, "MOREM"))		pi->morem=str2num(script2);
		else if (!strcmp(script1, "MODSKL"))	pi->modskill=str2num(script2);
		else if (!strcmp(script1, "MODSKLAMT"))	pi->modskillamt=str2num(script2);
		else return 0;
		break;
	case 'N':
		if (!strcmp(script1, "NAME"))			strcpy(pi->name,script2);
		else if (!strcmp(script1, "NAME2"))		strcpy(pi->name2,script2);
		else return 0;
		break;
	case 'O':
		if (!strcmp(script1, "OFFSPELL"))		pi->offspell=str2num(script2);
		else return 0;
		break;
	case 'P':
		if (!strcmp(script1, "POISONED"))
		{
			pi->poisoned=str2num(script2);
			if (pi->Poisoned() < 0 || pi->Poisoned() > 4)
			{
				LogWarning("Invalid POISONED value loaded from sripts or wsc.");
				pi->poisoned = 0;
			}
		}
		else return 0;
		break;
	case 'R':
		if (!strcmp(script1, "RACEHATE"))		pi->racehate=str2num(script2);
		else if (!strcmp(script1, "RESTOCK"))	pi->restock=str2num(script2);
		else if (!strcmp(script1, "RANK"))
		{
			pi->rank=str2num(script2);	if (pi->rank<=0) { pi->rank=10; } // By Magius(CHE)
		}
		else return 0;
		break;
	case 'S':
		if (!strcmp(script1, "SK_MADE"))		pi->madewith=str2num(script2);// by Magius(CHE)
		else if (!strcmp(script1, "SPD"))		pi->spd=str2num(script2);
		else return 0;
		break;
	case 'T':
		if (!strcmp(script1, "TYPE"))			pi->setType(str2num(script2));
		else if (!strcmp(script1, "TRIGGER"))	pi->trigger=str2num(script2);
		else if (!strcmp(script1, "TRIGTYPE"))	pi->trigtype=str2num(script2);
		else if (!strcmp(script1, "TRIGON"))	pi->trigon=str2num(script2);//equiiped trigger setting -Frazurbluu-
		else return 0;
		break;
	case 'U':
		if (!strcmp(script1, "USES"))			pi->tuses=str2num(script2);
		else return 0;
		break;
	case 'V':
		if (!strcmp(script1, "VISIBLE"))		pi->visible=str2num(script2);
		else if (!strcmp(script1, "VALUE"))		pi->value=str2num(script2);
		else return 0;
		break;
	case 'W':
		if (!strcmp(script1, "WEIGHT"))			pi->weight=str2num(script2); // Ison 2-20-99
		else return 0;
		break;
	
	default:
		return 0;
	}
	return 1;	// we have handled it :)
}

int cAllItems::applyWscTags(P_ITEM pi)
{
	switch (script1[0])
	{
	case 'A':
		if (!strcmp(script1, "ATT")) { pi->att=str2num(script2); }
		else return 0;
		break;
		
	case 'C':
		if (!strcmp(script1, "COLOR"))
		{
			unsigned long i = str2num(script2);
			
			unsigned long b=((i&0x4000)>>14)+((i&0x8000)>>15);
			if (i == 32767)
			{ 
				b = 0; // Corrects the etheral color so horses you are rideing maintain their color.(Aldur)
			} 
			if (!b)
				pi->setColor(i);
			else
			{
				pi->setColor(0); // bugged color found, leave it undyed
				Console.send("item# %i with problematic hue corrected\n",pi->serial);
			}
		}
		else if (!strcmp(script1, "CORPSE"))	{ pi->corpse=str2num(script2); }
		else if (!strcmp(script1, "CARVE"))		{ pi->carve=str2num(script2); }
		else return 0;
		break;
		
	case 'D':
		if (!strcmp(script1, "DOORFLAG"))		{ pi->doordir=str2num(script2); }
		else if (!strcmp(script1, "DYEABLE"))	{ pi->dye=str2num(script2); }
		else if (!strcmp(script1, "DESC"))		{ strcpy(pi->desc,script2); }
		else if (!strcmp(script1, "DX"))		{ pi->dx=str2num(script2);}
		else if (!strcmp(script1, "DX2"))		{ pi->dx2=str2num(script2);}
		else if (!strcmp(script1, "DECAYTIME")) {int si = str2num(script2);pi->decaytimer.setMilliSecs((si<0) ? 1 : si);}
		else return 0;
		break;
		
	case 'G':
		if (!strcmp(script1, "GLOW"))			{ pi->glow=str2num(script2); }
		else if (!strcmp(script1, "GLOWBC"))	{ pi->glow_color=str2num(script2);	}
		else if (!strcmp(script1, "GLOWTYPE"))	{ pi->glow_effect=str2num(script2);}
		else return 0;
		break;
		
	case 'I':
		if (!strcmp(script1, "IN"))				{ pi->in=str2num(script2); }
		else if (!strcmp(script1, "IN2"))		{ pi->in2=str2num(script2); }
		else if (!strcmp(script1, "ID"))
		{
			short i=str2num(script2);
			pi->setId(i);		
			if (i>=0x4000)
			{						               
				if (!Multi->multiIteratorStart(i-0x4000))
				{
					LogErrorVar("bad item not loaded, serial: %i",pi->serial);
					return 2;
				}
			}
			if (i<0)
			{
			  LogErrorVar("bad item not loaded, serial: %i",pi->serial);
			  return 2;
			}
		}
		else return 0;
		break;
		
	case 'L':
		if (!strcmp(script1, "LAYER"))		{ pi->layer=str2num(script2); }
		else return 0;
		break;
		
	case 'M':
		if (!strcmp(script1, "MAP"))		{ pi->pos.map=str2num(script2); }
		else return 0;
		break;
		
	case 'P':
		if (!strcmp(script1, "PILEABLE"))	{ pi->pileable=str2num(script2); }
		else if (!strcmp(script1, "PRIV"))	{ pi->setPriv(str2num(script2)); }
		else return 0;
		break;
		
	case 'S':
		if (!strcmp(script1, "SECUREIT"))	{ pi->secureIt=str2num(script2); }
		else if (!strcmp(script1, "ST"))	{ pi->st=str2num(script2); }
		else if (!strcmp(script1, "ST2"))	{ pi->st2=str2num(script2); }
		else return 0;
		break;
		
	case 'T':
		if (!strcmp(script1, "TYPE2"))		{ pi->type2=str2num(script2); }
		else if (!strcmp(script1, "TIME_UNUSED"))	{ pi->time_unused=str2num(script2); } // house decay etc, LB
		else return 0;
		break;
		
	case 'W':
		if (!strcmp(script1, "WIPE")) { pi->wipe=str2num(script2); }
		else return 0;
		break;
		
	case 'X':
		if (!strcmp(script1, "X")) { pi->pos.x=str2num(script2); }
		else return 0;
		break;
		
	case 'Y':
		if (!strcmp(script1, "Y")) { pi->pos.y=str2num(script2); }
		else return 0;
		break;
		
	case 'Z':
		if (!strcmp(script1, "Z")) { pi->pos.z=str2num(script2); }
		else return 0;
		break;
	default:
		return 0;
	}
	return 1;	// we have handled it :)
}

int cItem::applyScriptTags(int itemnum)
{
	Script* Scp = findSection(items_script,"ITEM", itemnum);
	if (!Scp) return 0;

	scpMark m;
	int loopexit=0;
	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS))
	{
		int rc = Items->applyCommonTags(this);				// tags used in both scp and wsc
		if (rc) {if (rc==1) continue; else return 0;}

		P_ITEM pi = this;

		switch (script1[0])
		{
		case 'B':
			if (!Scp->cmpTag("BASE"))		// this item is a modification of another item
			{
				Scp->Suspend(m);
				this->applyScriptTags(str2num(script2));
				Scp->Resume(m);
			}
			break;

		case 'C':
			if (!Scp->cmpTag("COLOR"))	pi->setColor(hstr2num(script2));
			else if (!Scp->cmpTag("COLORLIST"))
			{
				Scp->Suspend(m);
				pi->setColor(addrandomcolor(str2num(script2)));
				Scp->Resume(m);
			}
			else if (!Scp->cmpTag("CONTAIN") && (pi->type==ITY_Container || pi->type==ITY_LockedContainer)) 
			{ 
				Scp->Suspend(m); 
				P_ITEM ItemInside=Items->CreateScriptItem(str2num(script2)); 
				pi->AddItem(ItemInside); 
				Scp->Resume(m); 
			}
			break;

		case 'D':
			if (!Scp->cmpTag("DEX"))				pi->dx = str2num(script2);
			else if (!Scp->cmpTag("DISPELLABLE"))	pi->dispellableOn();
			else if (!Scp->cmpTag("DECAY"))			pi->decayOn();
			else if (!Scp->cmpTag("DYE"))			pi->dye = str2num(script2);
			else if (!Scp->cmpTag("DEXADD"))		pi->dx2 = str2num(script2);
			break;

		case 'I':
			if (!Scp->cmpTag("ID"))				pi->setId(hstr2num(script2));
			else if (!Scp->cmpTag("INT"))		pi->in = str2num(script2);
			else if (!Scp->cmpTag("INTADD"))	pi->in2 = str2num(script2);
			break;

		case 'N':
			if (!Scp->cmpTag("NEWBIE"))			pi->newbieOn();
			break;

		case 'S':
			if (!Scp->cmpTag("STR"))			pi->st = str2num(script2);
			else if (!Scp->cmpTag("STRADD"))	pi->st2 = str2num(script2);
			break;
		}
	}
	Scp->Close();
	return 1;	// success
}

P_ITEM cAllItems::CreateFromScript(int itemnum, cCoord& pos, int recurse)
{
	P_ITEM pi = CreateFromScript(itemnum,recurse);
	if (pi)
		pi->MoveTo2(pos);
	return pi;
}

P_ITEM cAllItems::CreateFromScript(int itemnum, int recurse)
{
	if (recurse-- <= 0)
	{
		LogWarningVar("Too many recursions for ITEM/ITEMLIST %i",itemnum);
		return NULL;
	}

	Script* Scp = findSection(items_script,"ITEM", itemnum);
	if (!Scp)
	{
		LogWarningVar("ITEM %i not found in the scripts",itemnum);
		return NULL;
	}

	// check if the item is actually an ITEMLIST
	int loopexit=0;
	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS) )
	{
		if (!strcmp("ITEMLIST", script1))
		{
			Scp->Close();
			
			itemnum=Items->PickFromItemlist(str2num(script2));
			if (itemnum==-1) 
			{ 
				return NULL;
			}
			return CreateFromScript(itemnum,recurse);
		}
	}
	Scp->Close();

	P_ITEM pi = Items->MakeBaseItem();
	if (!pi)
	{
		Scp->Close();	// make sure script is closed before exiting (Duke, 27.4.01)
		return NULL;
	}
	
	if (!pi->applyScriptTags(itemnum))
	{
		Items->DeleItem(pi);
		return NULL;
	}
	
	if (isTruelyPileable(pi->id())) pi->pileable = 1;
	
	if (!pi->maxhp && pi->hp)
		pi->maxhp = pi->hp; // Magius(CHE)
	
	if (pi->isType(ITY_TownHouseDeed))
	{
		PC_ITEM pD;
		int deeds4house = 0;
		AllItemsIterator aii;
		for ( aii.Begin(); (pD=aii.GetData())!=aii.End(); aii++)
		{
			if (pD->isType(ITY_TownHouseDeed) ||
				pD->isType(ITY_TownHouse))
			{
				if (pD->morex == pi->morex)
					deeds4house++;
			}
		}
		if (deeds4house > 1)
		{
			LogWarningVar("more than one deed for townhouse %i",pi->morex);
			Items->DeleItem(pi);
			return NULL;
		}
		if (!initTownHouseArea(pi))
			LogWarningVar("can't initialize townhouse %i",pi->morex);
		
	}
	return pi;	
}

P_ITEM cAllItems::CreateScriptItem(int itemnum)
{
	return CreateFromScript(itemnum);
}

P_ITEM cAllItems::CreateScriptItem(int s, int itemnum, int nSpawned)
{
	P_ITEM pi = Items->CreateFromScript(itemnum);
	if (!pi)
	{
		LogWarningVar("Couldn't create ITEM %i",itemnum);
		return NULL;
	}

	if (s!=-1 && !nSpawned)
	{
		short xx,yy;
		signed char zz;
		xx=(buffer[s][11]<<8)+buffer[s][12];
		yy=(buffer[s][13]<<8)+buffer[s][14];
		zz=buffer[s][16]+Map->TileHeight((buffer[s][17]<<8)+buffer[s][18]);
		pi->MoveTo(xx,yy,zz);
	}
	else
	{
		if (pi->isInWorld())
			mapRegions->Add(pi);
	}

	return pi;
}

int cAllItems::PickFromItemlist(int ilist)
{
	Script* Scp = findSection(items_script,"ITEMLIST", ilist);
	if (!Scp)
	{
		LogWarningVar("ITEMLIST %i not found",ilist);
		return -1;
	}

	return Scp->PickFromList();
}

////////////////
// Name:	SpawnItem (3 interfaces)
// Purpose:	Will create an item based on item id.
// History:	by unknown,
//			revamped by Duke,24.04.2000
// Remarks:	I took the idea (to overload) from the 'official' source. So thanks to fur!
//

P_ITEM cAllItems::SpawnItem(enum eItemTypes ity, int Amount, char* Name, short id, short color)
{
	P_ITEM pi=Items->MakeBaseItem();
	if (!pi) return NULL;

	if (Name!=NULL)
		strcpy(pi->name,Name);
	pi->setId(id);
	pi->setColor(color);
	pi->setType(ity);
	pi->amount=Amount;
	pi->decayOn();
	GetScriptItemSetting(pi); // Added by Magius(CHE) (2)
	return pi;
}

P_ITEM cAllItems::SpawnItem(enum eItemTypes ity, int Amount, char* Name, short id, short color, P_ITEM Cont)
{
	if (!Cont) return NULL;
	bool pileable = false;
	if (isTruelyPileable(id)) 
	{
		pileable=true;
		P_ITEM pSt;	// item to stack with
		int ci=0,loopexit=0;
		while ( ((pSt=ContainerSearchFor(Cont->serial,&ci,id,color)) != NULL) && (++loopexit < MAXLOOPS) )
		{
			if (pSt->amount+Amount > 65535)	// if it would create an overflow (amount is ushort!),
				continue;						// let's search for another pile to add to
			pSt->amount += Amount;
			pSt->refresh();
			return pSt;
		}
	}
	// either the item is not pileable or we didin't find anything to pile with
	P_ITEM pi = SpawnItem(ity,Amount,Name,id,color);
	if (!pi) return NULL;

	pi->pileable=pileable;
	pi->SetContSerial(Cont->serial);
	pi->SetRandPosInCont(Cont);
	pi->refresh();
	return pi;
}

P_ITEM cAllItems::SpawnItem(enum eItemTypes ity, int Amount, char* Name, short id, short color, cCoord& pos)
{
	P_ITEM pi = SpawnItem(ity,Amount,Name,id,color);
	if (!pi) return NULL;
	pi->MoveTo2(pos);
	return pi;
}

P_ITEM cAllItems::SpawnItem(enum eItemTypes ity, int Amount, char* Name, short id, short color, P_CHAR pc)
{
	if (!pc) return NULL;
	P_ITEM Pack=pc->getBackpack();
		
	P_ITEM pi = NULL;
	if (Pack)
	{
		pi = SpawnItem(ity,Amount,Name,id,color,Pack);
		if (!pi) return NULL;
		UOXSOCKET s = pc->getSlot();
		if (s > -1)
		{
			Weight->NewCalc(pc);
			statwindow(s,pc);
		}
	}
	else
	{// LB place it at players feet if he hasn't got backpack
		pi = SpawnItem(ity,Amount,Name,id,color,pc->pos);
	}
	return pi;
}

P_ITEM cAllItems::SpawnGoldInBank(P_PLAYER Player, int amount) // Fallens baby to handle bank checks and new banker ai
{
	if (!Player) return NULL;

	P_ITEM Bankbox = Player->GetBankBox();
	if (Bankbox == NULL)
	{
		LogWarning("Bank box not found in SpawnItemBank()");
		return NULL;
	}

	int loopexit=0;
	while ( (amount > 0) && (++loopexit<MAXLOOPS*10))
	{
		if (amount > 65535) 
		{
			P_ITEM pi=Items->SpawnItem(ITY_None,65535,"#",0x0EED,0);
			if (!pi) return NULL;
			pi->pileable=1;
			pi->SetContSerial(Bankbox->serial);
			pi->SetRandPosInCont(Bankbox);
			pi->refresh();
			//bankbox->AddItem(pi);
			amount -= 65535;
		}
		else
		{
			P_ITEM pSt;	// item to stack with
			int ci=0,loopexit=0;
			while ( ((pSt=ContainerSearchFor(Bankbox->serial,&ci,0x0EED,0)) != NULL) && (++loopexit < MAXLOOPS) )
			{
				if (pSt->amount+amount > 65000)	// if it would create an overflow (amount is ushort!),
					continue;						// let's search for another pile to add to
				pSt->amount += amount;
				amount = 0;
				pSt->refresh();
				break;
			}
			if (amount > 0)
			{
				P_ITEM pi=Items->SpawnItem(ITY_None,amount,"#",0x0EED,0);
				if (!pi) return NULL;
				pi->pileable=1;
				pi->SetContSerial(Bankbox->serial);
				pi->SetRandPosInCont(Bankbox);
				pi->refresh();
				amount = 0;
			}
		}
		statwindow(Player->cs, Player);
	}
	return false;
}

void cAllItems::GetScriptItemSetting(P_ITEM pi)
{// by Magius(CHE) //bugfixed by AntiChrist

	unsigned char tscript1[512];//AntiChrist - important bug/crash fix!!!
	unsigned char tscript2[512];//these have to be backed up before using them!!!
	int loopexit=0;

	strcpy((char*)tscript1,script1);//AntiChrist
	strcpy((char*)tscript2,script2);

	if (pi == NULL) 
		return;

	bool sectfound=false;
	char buff[512];   

	Script *Scp;
	Scp = getScript(harditems_script);
	if (!Scp) return;
	Scp->Open();

	buff[0] = '\0'; // Fix by Magius(CHE)

	sprintf(buff,"x%x",pi->id());

	do
	{
		Scp->NextLineSplitted();
		if (!Scp->EndSect())
		{
			if (!strcmp("SECTION",script1))
			{
				if (strstr(script2,buff)) sectfound = true;//AntiChrist bugfix
			}
			
			if (sectfound)//AntiChrist bugfix
			{
				int rc = Items->applyCommonTags(pi);				// tags used in both scp and wsc
				if (rc) {if (rc==1) continue; else return;}
				
				switch (script1[0])
				{
				case 'C':
					if (!strcmp("COLOR",script1))	pi->setColor(hstr2num(script2));
					break;
					
				case 'D':
					if (!strcmp("DEX", script1)) pi->dx=str2num(script2);
					else if (!strcmp("DEXADD", script1)) pi->dx2=str2num(script2);
					else if (!strcmp("DYE",script1))			pi->dye=str2num(script2);
					else if (!strcmp("DECAY",script1))			pi->decayOn();
					else if (!strcmp("DISPELLABLE",script1))	pi->dispellableOn();
					break;
					
				case 'I':
				case 'L':
					if (!strcmp("INT", script1))				pi->in=str2num(script2);
					else if (!strcmp("INTADD", script1))		pi->in2=str2num(script2);
					else if (!strcmp("LAYER",script1))			pi->layer=str2num(script2);
					break;
					
				case 'N':
					if (!strcmp("NEWBIE",script1))				pi->newbieOn();
					break;
					
				case 'S':
					if (!strcmp("STR", script1))				pi->st=str2num(script2);
					else if (!strcmp("STRADD", script1))		pi->st2=str2num(script2);
					break;
				}
			}
		}
		if (Scp->EndSect() && sectfound) break;
	}
	while( (strcmp(script1, "EOF")) && (++loopexit < MAXLOOPS) );
    //while ((script1[0]!='}' && !sectfound) && strcmp(script1, "EOF"));

    Scp->Close();

	strcpy(script1,(char*)tscript1);//AntiChrist
	strcpy(script2,(char*)tscript2);
}

P_ITEM cAllItems::SpawnItemBackpack(P_PLAYER Player, int nItem, int nDigging)
{
	return SpawnItemBackpack2(Player->cs,nItem,nDigging);
}

P_ITEM cAllItems::SpawnItemBackpack2(UOXSOCKET s, int nItem, int nDigging) // Added by Genesis 11-5-98
{
	if (s==-1)
		return NULL;
	P_CHAR Player = MAKE_CHARREF_LRV(currchar(s), NULL);
	P_ITEM Pack = Player->getBackpack();
	if (!Pack) return NULL;
	
	P_ITEM pi = CreateScriptItem(s, nItem, 1);
	if (!pi)
		return NULL;

	if (nDigging) 
	{
		if (pi->value!=0) pi->value=1+(rand()%(pi->value)); 
		if (pi->hp!=0) pi->hp=1+(rand()%(pi->hp));
		// blackwinds fix 
		if (pi->maxhp != 0)
		{
			pi->maxhp=1+(rand()%pi->maxhp); 
			pi->hp=1+(rand()%pi->maxhp);
		}
	}

	Pack->AddItem(pi);
	
	Weight->NewCalc(Player);
	statwindow(s,Player);
	return pi;
}

char cAllItems::isFieldSpellItem(P_ITEM pi) //LB
{
	if (!pi) return 0;
	int a=0;
	short id=pi->id();
	if (id==0x3996 || id==0x398C) a=1;  // fire field
	if (id==0x3915 || id==0x3920) a=2;  // poison field
	if (id==0x3979 || id==0x3967) a=3;  // paralyse field
	if (id==0x3956 || id==0x3946) a=4;  // energy field;
	if (id==0x0080) a=5;                // wall of stone
	if (id>=0x122a && id <=0x122e) a=6; // blood
	
	return a;
}
//taken from 6904t2(5/10/99) - AntiChrist
void cAllItems::DecayItem(P_ITEM pi) 
{
	if (!pi) return;
	if (!pi->decays() ||	// set to nodecay
		!pi->isInWorld() ||	// in cont or on a char
		pi->isLockedDown() ||
		pi->spawnregion)	// part of an item spawn region
	{
		pi->decaytimer.reset();
		return;
	}
	if (ishouse(pi)) return;	// decay of house is handled elsewhere

	if (pi->decaytimer.hasExpired())
	{
		//Multis --Boats ->
		
		if (!Items->isFieldSpellItem(pi) // Gives fieldspells a chance to decay in multis, LB
			&& !pi->corpse)
		{
			if (pi->multis < 1)
			{
				// JustMichael -- Added a check to see if item is in a house
				PC_ITEM House = findHouseAt(pi->pos);
				if (House!=NULL)
				{
					if (House->more4==0) //JustMichael -- set more to 1 and stuff can decay in the building
					{
						pi->startDecay();
						pi->SetMultiSerial(House->serial);
						return;
					}
				}
			} 
			else
			{					
				pi->startDecay();
				return;
			}
		}
		//End Boats/Mutlis
		
		//JustMichael--Keep player's corpse as long as it has more than 1 item on it
		//up to playercorpsedecaymultiplier times the decay rate
		if (pi->corpse == 1 && pi->GetOwnSerial()!=-1)
		{
			int preservebody=0;
			int ci=0,loopexit=0;
			P_ITEM pi2;
			while ( ((pi2=pi->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
			{
				preservebody++;
				break; //lagfix - AntiChrist - not necessary to check ALL the items!!!
			}
			
			if ( preservebody > 1 && pi->more4 )
			{
				pi->more4--;
				pi->startDecay();
				return;
			}
		}
		if ( ( pi->isType(ITY_Container) && pi->corpse != 1 )
			|| (pi->GetOwnSerial() != -1 && pi->corpse)
			|| (!SrvParms->lootdecayswithcorpse && pi->corpse ))
		{
			int ci=0,loopexit=0;
			P_ITEM pi2;
			while ( ((pi2=pi->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
			{
				pi2->startDecay();
				pi2->SetContSerial(-1);
				pi2->MoveTo2(pi->pos);
			}
			Items->DeleItem(pi);
		} 
		else 
		{
			if (pi->isInWorld())
			{
				Items->DeleItem(pi);
			}
		}
	}
}
//END FUNCTION

//NEW RESPAWNITEM FUNCTION STARTS HERE -- AntiChrist merging codes -- (24/6/99)
void cAllItems::RespawnItem(unsigned int currenttime, PC_ITEM pi)
{
	if (!pi) return;
	
	if (pi->disabled>0 && (pi->disabled<=currenttime || overflow))
	{
		pi->unconst()->disabled=0;
	}

//	if (!pi->IsSpawnerType()) return;

	int k,c;
	for (c=0;c<pi->amount;c++)
	{
		if (pi->gatetime+(c*pi->morez*MY_CLOCKS_PER_SEC)<=currenttime)
		{
			if (pi->isType(ITY_ItemSpawner))
			{
				k=pi->CountSpawnedItems();
				if (k==0)
				{
					if (pi->gatetime==0)
					{
						pi->unconst()->gatetime=(rand()%((int)(1+((pi->morez-pi->morey)*(MY_CLOCKS_PER_SEC*60))))) +
							(pi->morey*MY_CLOCKS_PER_SEC*60)+currenttime;
					}
					if ((pi->gatetime<=currenttime ||(overflow)) && pi->morex!=0)
					{
						Items->AddRespawnItem(pi,pi->morex, 0);
						pi->unconst()->gatetime=0;
					}
				}
			}
			if (pi->isType(ITY_NpcSpawner) || pi->isType(ITY_NpcAreaSpawner) || pi->isType(ITY_QuestSpawn))
			{
				k=pi->CountSpawnedChars();			

				if (k<pi->amount)
				{
					if (pi->gatetime==0)
					{
						pi->unconst()->gatetime=(rand()%((int)(1+((pi->morez-pi->morey)*(MY_CLOCKS_PER_SEC*60))))) +
							(pi->morey*MY_CLOCKS_PER_SEC*60)+currenttime;
					}
					if ((pi->gatetime<=currenttime || (overflow)) && pi->morex!=0)
					{
						Npcs->AddRespawnNPC(pi,pi->morex);//If you are trying to spawn NPCs you must call an
						pi->unconst()->gatetime=0;					//NPC spawn not an item spawn. Fixed 9-3-99 Just Michael
					}
				}
			}
			if (pi->isType(ITY_SpawnChest)||pi->isType(ITY_LockedSpawnChest)||pi->isType(ITY_65)
				||pi->isType(ITY_66)||pi->isType(ITY_LockedContainer))
			{
				int m=0;
				int ci=0,loopexit=0;
				P_ITEM pi2;
				while ( ((pi2=pi->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
				{
					m++;
				}
				if (m<pi->amount)
				{
					if (pi->gatetime==0)
					{
						pi->unconst()->gatetime=(rand()%((int)(1+((pi->morez-pi->morey)*(MY_CLOCKS_PER_SEC*60))))) +
							(pi->morey*MY_CLOCKS_PER_SEC*60)+uiCurrentTime;
					}
					if ((pi->gatetime<=currenttime ||(overflow)) && pi->morex!=0)
					{
						if (pi->isType(ITY_SpawnChest)) pi->unconst()->setType(ITY_LockedSpawnChest); //Lock the container 
						
						if (pi->morex)
							Items->AddRespawnItem(pi,pi->morex,1);//If the item contains an item list then it will randomly choose one from the list, JM
						else
						{						
							ci=Items->PickFromItemlist(70); //default chest list
							Items->AddRespawnItem(pi,ci, 1);
							
						}
						pi->unconst()->gatetime=0;	
					}
				}
			}
		}//If time
	}//for 
}

void cAllItems::AddRespawnItem(PC_ITEM Spawner, int inum, bool inChest)
{
	if (!Spawner) return;
	P_ITEM pi=CreateFromScript(inum);
	if (!pi) return;
	
	pi->SetSpawnSerial(Spawner->serial);
	if (!inChest)
	{
		pi->MoveTo2(Spawner->pos); //add spawned item to map cell if not in a container
	}
	else
	{
		Spawner->AddItem(pi);
	}
}

///////////////////////////////////////////////////////////////////
//// currently only a helper method for tempdupeNpc.
///  overwrites all data members from source with values from source (except serial)
///  use with extreme caution
///  target has to be already existing ! (source too of course)
///////////////////////////////////////////////////////////////////
bool cAllItems::copyItemRawData(const cItem* const source,cItem* target)
{
	if (source==NULL || target==NULL) return false;

	int ser = target->serial; // don't copy serial	
	int index = target->GetIndex(); // copying indices also fatal

	*target = *source;

	target->setSerialOnly(ser);
	target->SetIndex(index);

	if (source->TrigInf)
		target->GetTrigInf();		// will actually allocate and init a separate instance

	return true;

	// note: we DO have a pointers in item class, namely inside townhouse struct, but this can be neglacted, cause ONLY used
	// for townhouse-deeds. 
	// in current context it aint used to dupe townhouse deeds
}

/////////////
// name:	AllItemsIterator
// purpose:	intended to replace the itemcount loops
// history:	by Duke, 27.6.01
//			changed ++ operator to skip free entries (Duke, 11.06.03)
//			changed GetData to skip free entries too (Duke, 28.07.03)
//
AllItemsIterator::AllItemsIterator()				{ pos = 0; }
AllItemsIterator::~AllItemsIterator()				{ ; }
void AllItemsIterator::Begin()						{ pos = 0; }
P_ITEM AllItemsIterator::End()						{ return NULL; }
PC_ITEM AllItemsIterator::GetData(void)
{
	PC_ITEM pi = MAKE_ITEM_REF(pos);
	while (pi == NULL || (pi && pi->free))
	{
		pos++;
		if (pos >= itemcount)
			return NULL;
		pi = MAKE_ITEM_REF(pos);
	}
	return pi;
}

AllItemsIterator& AllItemsIterator::operator++(int)
{
	P_ITEM pi = NULL;
	while (pi == NULL || (pi && pi->free))
	{
		pos++;
		if (pos >= itemcount)
			break;
		pi = MAKE_ITEM_REF(pos);
	}
	return *this;
}

bool isTruelyPileable(short itemid)
{
	const tileData* tile;
	tile = TileData->SeekTile(itemid);
	return TileData->isStackable(tile->flags);
}

class cSmallBulkOrderDeed : public cItemController
{
public:
	cSmallBulkOrderDeed(P_ITEM pi);
	virtual int onSingleClick(P_PLAYER Player);
	virtual int onDoubleClick(P_PLAYER Player);
	virtual int onAcceptDrop(P_PLAYER Player, P_ITEM Drop)	{return 0;}
	virtual int onProcessTarget(P_PLAYER Player, short targtype, P_ITEM Target);
	virtual int onDroppedOnChar(P_PLAYER Player, P_CHAR pc, P_ITEM Drop);
};

cSmallBulkOrderDeed::cSmallBulkOrderDeed(P_ITEM pi)
{
	_pi = pi;
}

int cSmallBulkOrderDeed::onSingleClick(P_PLAYER Player)
{
	char tmpn[500];
	getItemName(_pi->getMore1_2(),2,tmpn);

	char tmp[500];
	sprintf(tmp,"a bulk order with %i of %i %s", _pi->morez, _pi->morey, tmpn);
	itemmessage(Player->cs,_pi,tmp);
	return 1;
}

int cSmallBulkOrderDeed::onDoubleClick(P_PLAYER Player)
{
	target(Player,T_AddToBulkOrder,"Select item to add to the bulk order");
	return 1;
}

int cSmallBulkOrderDeed::onProcessTarget(P_PLAYER Player, short targtype, P_ITEM Target)
{
	if (targtype != T_AddToBulkOrder) return 0;

	if (_pi->morez >= _pi->morey)
	{
		Player->Msg("This order is already complete.");
		return 2;	// handled, but bounce !
	}
	if (_pi->getMore1_2() != Target->id()
		|| _pi->getMore3_4() != Target->color())
	{
		Player->Msg("This item doesn't match the criteria of the order.");
		return 2;	// handled, but bounce !
	}

	_pi->morez++;			// ok, add it

	if (_pi->morez == 1)	// first drop ?
		_pi->value = Target->value;		// value of a single item
	else
	{
		if (_pi->value > Target->value)	// adding items with minor value
			_pi->value = Target->value;	// reduces the value of the whole order (exploit protection)
	}
	Items->DeleItem(Target);
	return 1;
}

int cSmallBulkOrderDeed::onDroppedOnChar(P_PLAYER Player, P_CHAR Vendor, P_ITEM Drop)
{
	if (!Vendor->shop) return 0;

	int subAI = Vendor->getSubAi();
//	if (subAI != BLACKSMITH && subAI != WEAPONSMITH) return 0;
	if (subAI != _pi->moreb1) return 0;
	if (_pi->morex != Vendor->serial)	// vendor is not the issuer of this deed
	{
		npctalkall(Vendor, "I have not ordered that.");
		return 2;	// handled & bounce
	}

	if (_pi->morez < _pi->morey)
	{
		npctalkall(Vendor, "That is less than I have ordered.");
		return 2;	// handled & bounce
	}

	int g = _pi->value * _pi->morey;	// value * # of items
	addgold(Player,g);
	Fame(Player,Drop->getMoreB3_4());
	Items->SpawnItemBackpack(Player, _pi->morem);
	npctalkall(Vendor, "I thank thee. Here is thy reward.");
	return 1;
}

// a free constructor
// note: this is called from speech. Returning false means 'this vendor is not affected by the speech'
//		thus, we return true even on script errors
bool giveBulkOrder(P_PLAYER Player, P_CHAR Vendor)
{
	int subAI = Vendor->getSubAi();
//	if (subAI != BLACKSMITH && subAI != WEAPONSMITH) return false;

	if (rand()%2)
	{
		npctalkall(Vendor, "Right now, I have everything I need. Come back later.");
		return true;
	}

	int bod = PickFromList(bod_script,"BODLIST",subAI,"BOD");
	if (bod == -1)
	{
		npctalkall(Vendor, "Sorry, I never give bulk orders.");
		return true;
	}

	Script* Scp = findSection(bod_script,"BOD",bod);
	if (!Scp)
	{
		LogWarningVar("section BOD %i not found",bod);
		return true;
	}
	int reward=0, idlist=0, amtlist=0, matlist=0, fame=0;
	int id=0, amount=0, material=0;
	while (Scp->NextLineSplitted())
	{
		if (Scp->getKeyVal("ID",&id)) continue;
		if (Scp->getKeyVal("AMT",&amount)) continue;
		if (Scp->getKeyVal("MAT",&material)) continue;
		if (Scp->getKeyVal("IDLIST",&idlist)) continue;
		if (Scp->getKeyVal("AMTLIST",&amtlist)) continue;
		if (Scp->getKeyVal("MATLIST",&matlist)) continue;
		if (Scp->getKeyVal("REWARD",&reward)) continue;
		if (Scp->getKeyVal("FAME",&fame)) continue;
		Scp->invalidTag();
	}
	Scp->Close();

	if (!id)
	{
		id = PickFromList(bod_script,"IDLIST",idlist,"ID",true);
		if (id == -1)
		{
			LogWarningVar("invalid ID/IDLIST %i",idlist);
			return true;
		}
	}

	if (!amount)
	{
		amount = PickFromList(bod_script,"AMTLIST",amtlist,"AMT",false);
		if (amount == -1)
		{
			LogWarningVar("invalid AMT/AMTLIST %i",amtlist);
			return true;
		}
	}

	if (!material)
	{
		material = 0;
		if (matlist)
		{
			material = PickFromList(bod_script,"MATLIST",matlist,"MAT",true);
			if (material == -1)
			{
				LogWarningVar("invalid MAT/MATLIST %i",matlist);
				return true;
			}
		}
	}

	P_ITEM BoD = Items->SpawnItem(ITY_SmallBulkOrder,1,"small bulk order",0x14EF,0,Player);
	if (!bod) return true;
	BoD->morex = Vendor->serial;	// vendor to return it to
	BoD->morey = amount;			// amount to deliver
	BoD->morez = 0;					// amount added
	BoD->morem = reward;			// script # of item

	BoD->setMore1_2(id);
	BoD->setMore3_4(material);		// standard color / material type
	BoD->moreb1 = subAI;			// merchant type
	BoD->moreb2 = 0;				// exceptional ?
	BoD->value = 0;					// value of a single item (set when items are dropped on the deed)
	BoD->setMoreB3_4(fame);			// fame gain (or loss)

	npctalkall(Vendor, "Yes, I need some of these items.");
	return true;
}

cItemController* cAllItems::getController(P_ITEM pi)
{
	if (!pi) return NULL;
	switch (pi->getType())
	{
	case ITY_SmallBulkOrder:	return new cSmallBulkOrderDeed(pi);
	default:					return NULL;
	}
}

