/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//
#ifndef __MSGBOARD_H__
#define __MSGBOARD_H__

// msgboard.h
//

#if defined(__unix__)
#include <dirent.h>

#include <string>
#include <vector>
using namespace std ;
#endif

#include "typedefs.h"

// Maximum number of posts per board
// BUFFER SIZE = 2560 
// Therefore 0x3c max size = 2560 - 5 ( 0x3c header info ) 
//                         = 2550 
//
//                           2550 / 19 ( item segment size per msg ) 
//                         = 134 
// Round down to 128 Messages allowable on a message board (better safe than sorry)
// MAXPOSTS = 128 
#define MAXPOSTS        128

// Maximum number of entries in a ESCORTS list in the MSGBOARD.SCP file
#define MAXENTRIES      256

// Different types of user posts
#define LOCALPOST       0
#define REGIONALPOST    1
#define GLOBALPOST      2

// Different types of QUESTS (nQuestType)
// Added for Quests (ESCORTS)
// Reason for starting high and counting down, is that I store the entire user
// posted message as is (which includes the message type).  The message type
// for a user post is 0x05 and I also use this field to determine whether the
// post is marked for deletion (0x00).  In order to allow for the maximum number
// of different quest types, I opted to start high and count down.
#define ESCORTQUEST 0xFF
#define BOUNTYQUEST 0xFE
#define ITEMQUEST   0xFD


// Function Prototypes 
void    MsgBoardEvent(P_PLAYER Player);
void    MsgBoardSetPostType(P_PLAYER Player, int Type );
void    MsgBoardGetPostType(P_PLAYER Player);
int     MsgBoardPostQuest( int nSerial, int nQuestType );
bool    MsgBoardQuestEscortCreate( P_CHAR pc );
void    MsgBoardQuestEscortArrive( P_CHAR pc_npc, P_PLAYER Escort);
void    MsgBoardQuestEscortDelete( P_CHAR pc );
void    MsgBoardQuestEscortRemovePost( P_CHAR pc );
void    MsgBoardMaintenance( void );
bool    MsgBoardRemoveGlobalPostBySerial( int nPostSerial );
#if defined(__unix__)
std::vector<std::string> MsgBoardGetFile( char* pattern, char* path) ;
#endif

#endif
