/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "gumps.h"
#include "house.h"
#include "itemid.h"
#include "LoS.h"
#include "magic.h"
#include "regions.h"
#include "skills.h"
#include "SndPkg.h"
#include "targeting.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "p_ai.cpp"

class cBrain
{
public:
			void doAI(P_CHAR pci);
protected:
	virtual void doWarAI(P_CHAR pci) {;}
	virtual void doPeaceAI(P_CHAR pci) = 0;
			void iterItems(P_CHAR pci,unsigned int range);
			void iterBystanders(P_CHAR pci,unsigned int range,int prio=1);
			bool iterClosestBystanders(P_CHAR pci,unsigned int range,int prio=1);
			bool allwaysSkip(P_CHAR pci,P_CHAR pc);
			bool isNotAttackable(P_CHAR pc);
	virtual bool skip(P_CHAR pci,P_CHAR pc,int prio) {return false;}	// NO skip by default
	virtual bool actOn(P_CHAR pci,P_ITEM pi) {return true;}	// break loop by default
	virtual bool actOn(P_CHAR pci,P_CHAR pc,int prio) {return true;}	// break loop by default
};

bool cBrain::isNotAttackable(P_CHAR pc)
{
	return (pc->isInvul() || pc->isHidden() || pc->dead) ? true : false;
};

bool cBrain::allwaysSkip(P_CHAR pci,P_CHAR pc)
{
	return (pci->isSameAs(pc) ||		// we don't loop just to find ourselves
			pc->isDummy() ||			// NEVER touch dummies
			(!pc->npc && !pc->InGame()))// NEVER touch offline players
		? true : false;
}

void cBrain::iterItems(P_CHAR pci,unsigned int range)
{
	P_ITEM pi;
	int loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(pci->pos);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (pci->dist(pi) > range) continue;

		if (actOn(pci,pi))					// do what you have to do on this item and break the loop OR
			return;							// actOn can return false to iter through ALL the pcs
	}
};

void cBrain::iterBystanders(P_CHAR pci,unsigned int range,int prio)
{
	P_CHAR pc;
	int loopexit=0;
	RegGrid3x3Iterator4Chars rg3ic(pci->pos);
	while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (allwaysSkip(pci,pc)) continue;	// ourselves, dummies and offline players
		if (skip(pci,pc,prio)) continue;	// no interest in this kind of char

		if (pci->dist(pc) > range) continue;

		if (actOn(pci,pc,prio))				// do what you have to do on this pc and break the loop OR
			return;							// actOn can return false to iter through ALL the pcs
	}
};

bool cBrain::iterClosestBystanders(P_CHAR pci,unsigned int range,int prio)
{
	P_CHAR pc;
	P_CHAR Victim=NULL;
	unsigned int closest=999;
	int loopexit=0;
	RegGrid3x3Iterator4Chars rg3ic(pci->pos);
	while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (allwaysSkip(pci,pc)) continue;	// ourselves, dummies and offline players
		if (skip(pci,pc,prio)) continue;	// no interest in this kind of char

		unsigned int d = pci->dist(pc);
		if (d > range)
			continue;
		if ( closest > d)
		{
			closest = d;
			Victim = pc;
		}
	}
	if (Victim)
	{
		actOn(pci,Victim,prio);				// do what you have to do on the victim chosen
		return true;
	}
	return false;
};

void cBrain::doAI(P_CHAR pci)
{
	if (pci->war)
		doWarAI(pci);
	else
		doPeaceAI(pci);
}

class cMonsterBrain : public cBrain
{
	virtual void doPeaceAI(P_CHAR pci);
	virtual bool skip(P_CHAR pci,P_CHAR pc,int prio);
	virtual bool actOn(P_CHAR pci,P_CHAR pc,int prio);
};

bool cMonsterBrain::skip(P_CHAR Npc,P_CHAR pc,int prio)
{
	if (isNotAttackable(pc))
		return true;
	if (pc->Owns(Npc))		// don't attack your master
		return true;
//	if (pc->npc && (pc->npcaitype == 2 || pc->npcaitype == 1))
//		continue;
//	if (SrvParms->monsters_vs_animals == 0 && (strlen(pc->title) > 0 || !onl))
//		continue;
//	if (SrvParms->monsters_vs_animals == 1 && RandomNum(1, 100) > SrvParms->animals_attack_chance)
//		continue;

	int onl = pc->InGame();
	if (onl || (pc->npc && !pc->isAiMonster() && pc->npcaitype!=1) &&
		((SrvParms->monsters_vs_animals == 0 && (strlen(pc->title)>0 || onl))||
		((SrvParms->monsters_vs_animals == 1)&&((unsigned)RandomNum(1,100)<=SrvParms->animals_attack_chance))))
	{
		return false;
	}
	return true;
};

bool cMonsterBrain::actOn(P_CHAR Npc,P_CHAR pc,int prio)
{
	Npc->attack(pc);
	if (Npc->getRealSkill(MAGERY)>400)
	{
		if (Npc->Hp() < (Npc->getHitMax()/2))
		{
			npctalkall(Npc, "In Vas Mani");
			OldMagic->NPCHeal(Npc);
		}
		if (Npc->Poisoned())
		{
			npctalkall(Npc, "An Nox");
			OldMagic->NPCCure(Npc);
		}
		if (pc->IsDispellable())
		{
			npctalkall(Npc, "An Ort");
			OldMagic->NPCDispel(Npc, pc);
		}
	}
	return true;
};

void cMonsterBrain::doPeaceAI(P_CHAR pci)
{
	P_NPC Npc = MakeNpcRef(pci);
	if (Npc)
	{
		if (Npc->peacemaking.hasExpired())	Npc->peacemaking.reset();
		if (Npc->peacemaking.IsRunning())	return;
	}

	iterBystanders(pci,SrvParms->attack_distance);
}

class cVortexBrain : public cBrain
{
	virtual void doPeaceAI(P_CHAR pci);
	virtual bool skip(P_CHAR pci,P_CHAR pc,int prio);
	virtual bool actOn(P_CHAR pci,P_CHAR pc,int prio);
};

void cVortexBrain::doPeaceAI(P_CHAR pci)
{
	iterClosestBystanders(pci,SrvParms->attack_distance);
}

bool cVortexBrain::skip(P_CHAR pci,P_CHAR pc,int prio)
{
	return isNotAttackable(pc);
}
bool cVortexBrain::actOn(P_CHAR pci,P_CHAR pc,int prio)
{
	pci->attack(pci);
	return false;
}

class cPixieBrain : public cBrain
{
	virtual void doPeaceAI(P_CHAR pci);
	virtual bool skip(P_CHAR pci,P_CHAR pc,int prio);
	virtual bool actOn(P_CHAR pci,P_CHAR pc,int prio);
};

void cPixieBrain::doPeaceAI(P_CHAR pci)
{
	P_NPC Npc = MakeNpcRef(pci);
	if (Npc)
	{
		if (Npc->peacemaking.hasExpired())	Npc->peacemaking.reset();
		if (Npc->peacemaking.IsRunning())	return;
	}

	iterBystanders(pci,SrvParms->attack_distance);
}
bool cPixieBrain::skip(P_CHAR pci,P_CHAR pc,int prio)
{
	if (!pc->isHuman())	return true;
	if (isNotAttackable(pc)) return true;
	if (!(pc->isAiMonster() || pc->isCriminal() || pc->isMurderer() || pc->karma < 0))
		return true;
	return false;
}
bool cPixieBrain::actOn(P_CHAR pci,P_CHAR pc,int prio)
{
	pci->attack(pc);
	return false;	// right now we'll attack ALL of them (bug ? suboptimal ? (Duke))
}

class cMercenaryBrain : public cBrain
{
	virtual void doPeaceAI(P_CHAR pci);
	virtual bool actOn(P_CHAR pci,P_ITEM pi);
};

void cMercenaryBrain::doPeaceAI(P_CHAR pci)
{
	iterItems(pci,1);		// pick up arrows and auto-carve

	if (pci->Hunger() < 6)	// eat when hungry
	{
		P_ITEM Pack = pci->getBackpack();
		if (Pack)
		{
			int ci=0,loopexit=0;
			P_ITEM pi;
			while ( ((pi=Pack->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
			{
				if (pi->getType()==ITY_Food)
				{
					if(pci->CanEat(pi))
					{
						pi->ReduceAmount(1);	// Remove a food item
						pci->chgHunger(1);
						break;
					}
				}
			}
		}
	}

	if (!pci->dead)
	{
		pci->checkCraftAI();
	}
}

bool cMercenaryBrain::actOn(P_CHAR pci,P_ITEM pi)
{
	if (IsArrowOrBolt(pi->id()))
	{
		if (!pci->canPickUp(pi)) return false;

		P_ITEM Pack = pci->getBackpack();
		if (!Pack) return true;	// no need for searching, so break loop
		Pack->AddItem(pi);
		pci->DoAction(0x20);
		return true;
	}
	if (IsCorpse(pi->id()) && pi->corpse				// a true corpse
		&& pi->amount != 0x0190 && pi->amount != 0x0191	// not human
		&& !pi->more1)									// and not already carved
	{
		P_ITEM Pack = pci->getBackpack();
		if (!Pack) return true;		// no need for searching, so break loop
		P_ITEM Blade = pci->getCarvingTool();
		if (!Blade) return true;	// no need for searching, so break loop

		pci->dclickedserial = Blade->serial;
		Targ->CorpseTarget(pci,pi);
	}
	return false;
}

class cInWorldGuardBrain : public cBrain
{
	virtual void doPeaceAI(P_CHAR pci);
	virtual bool skip(P_CHAR pci,P_CHAR pc,int prio);
	virtual bool actOn(P_CHAR pci,P_CHAR pc,int prio);
};

void cInWorldGuardBrain::doPeaceAI(P_CHAR pci)
{
	if (!pci->inGuardedArea()) return;

	iterClosestBystanders(pci,SrvParms->attack_distance,1);	// priority=1=attack

	iterBystanders(pci,3,2);	// range=3, priority=2=greet
}
bool cInWorldGuardBrain::skip(P_CHAR pci,P_CHAR pc,int prio)
{
	if (this->isNotAttackable(pc)) return true;

	if (prio == 1)	// attacking evil
	{
		if ((pc->npc && pc->isAiMonster())	// evil npc
			|| (!pc->npc && pc->isMurderer()))	// a player, not grey or blue
			return false;
	}
	else if (prio == 2)	// greeting
	{
		if (!pc->npc) return false;
	}
	return true;
}
bool cInWorldGuardBrain::actOn(P_CHAR pci,P_CHAR pc,int prio)
{
	if (prio == 1)
	{
		pci->pos = pc->pos; //Ripper..guards teleport to enemies.
		soundeffect2(pci, 0x01FE);
		staticeffect(pc, 0x372A, 0x09, 0x06);
		pci->attack(pc);
		npctalkall(pci, "Thou shalt regret thine actions, swine!",10);
	}
	else if (prio == 2)
	{
		if (pc->crimflag > 0 )
		{
			sprintf(temp,"You better watch your step %s, I am watching thee!!", pc->name);
			npctalkall(pci,temp,30);
		}
		else if (pc->isInnocent() && !pc->dead )
		{
			sprintf(temp,"%s is an upstanding citizen, I will protect thee in %s.", pc->name, region[pc->region].name);
			npctalkall(pci,temp,30);
		}
	}
	return false;
}


void cCharStuff::initBrains()
{
	MercenaryBrain = new cMercenaryBrain();
	MonsterBrain = new cMonsterBrain();
	VortexBrain = new cVortexBrain();
	PixieBrain = new cPixieBrain();
	InWorldGuardBrain = new cInWorldGuardBrain();
}

void cCharStuff::CheckAI(P_CHAR pc_i) //Lag Fix -- Zippy
{
	if (!pc_i) return;
	int onl;

    // in npc.scp add script # for npc`s ai
	// case - script - case - script -   case - script -  case - script
	//   0   -  0     -  4    -  4     -  8    -  8     -  12   -  C
	//   1   -  1     -  5    -  5     -  9    -  9     -  13   -  D
	//   2   -  2     -  6    -  6     -  10   -  A     -  14   -  E
	//   3   -  3     -  7    -  7     -  11   -  B     -  15   -  F
	// case - script - case - script
	//   16  -  10    -  20   -  14
	//   17  -  11    -  21   -  15
	//   18  -  12    -  22   -  16
	//   19  -  13    -  23   -  17  ...this is just a guide...Ripper
	cChar* pc;
	int loopexit=0;

	if (pc_i->isScriptedwalkingAi()) return;

	switch(pc_i->getAi())
	{
	case 0: // Shopkeepers greet players..Ripper
		if (SrvParms->VendorGreet==1 && pc_i->npc && pc_i->shop && pc_i->isHuman())
		{
			RegGrid3x3Iterator4Chars rg3ic(pc_i->pos);
			while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
			{
				if (pc_i->dist(pc) > 3)
					continue;
				if (pc->isInvul() || pc->npc || pc->dead || !pc->isInnocent() || !pc->InGame() || pc->isHidden())
					continue;
				if (!line_of_sight(pc_i->pos, pc->pos))
					continue;
				sprintf(temp,"Hello %s, Welcome to my shop, How may i help thee?.", pc->name);
				npctalkall(pc_i,temp,30);
				break;
			}
		}
		break;
	case 1: // good healers
		if (!pc_i->war)
		{
			RegGrid3x3Iterator4Chars rg3ic(pc_i->pos);
			while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
			{
				onl = pc->InGame();
				if (!pc->dead || pc_i->dist(pc) > 3 || pc->npc || !onl)
					continue;
				if (!line_of_sight(pc_i->pos, pc->pos))
					continue;
				if (pc->isMurderer()) 
				{
					npctalkall(pc_i, "I will nay give life to a scoundrel like thee!",10);
					continue;
				}
				if (pc->isCriminal()) 
				{
					npctalkall(pc_i, "I will nay give life to thee for thou art a criminal!",10);
					continue;
				}
				if (pc->isInnocent())
				{
					// prevent house break-ins by Uber 07-25-2002
					PC_ITEM House = findHouseAt( pc->pos );
					if (House && !pc->Owns(House))
					{
						npctalkall(pc_i, "I will nay give life to thee for thou art not in thine own abode!",10);
						continue;
					}
					if (pc_i->timeused_last + MY_CLOCKS_PER_SEC*10 >= uiCurrentTime) return;
					pc_i->timeused_last = uiCurrentTime;
					Gumps->Menu(pc->getSlot(),7);
					pc_i->DoAction(0x10);
					
					switch(RandomNum(0, 4)) 
					{
					case 0: npctalkall(pc_i, "Thou art dead, but 'tis within my power to resurrect thee.  Live!",10);			break;
					case 1: npctalkall(pc_i, "Allow me to resurrect thee ghost.  Thy time of true death has not yet come.",10);	break;
					case 2: npctalkall(pc_i, "Perhaps thou shouldst be more careful.  Here, I shall resurrect thee.",10);		break;
					case 3: npctalkall(pc_i, "Live again, ghost!  Thy time in this world is not yet done.",10);					break;
					case 4: npctalkall(pc_i, "I shall attempt to resurrect thee.",10);											break;
					}
				}
			}
		}
		break;
	case 2 : // Monsters, PK's - (stupid NPCs)
		MonsterBrain->doAI(pc_i);
		break;
	case 3 : //Evil Healers
		if (!pc_i->war)
		{
			RegGrid3x3Iterator4Chars rg3ic(pc_i->pos);
			while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
			{
				onl = pc->InGame();
				if (!pc->dead || pc_i->dist(pc) > 3 || pc->npc || !onl)
					continue;
				if (!line_of_sight(pc_i->pos, pc->pos))
					continue;
				if (pc->isInnocent())
				{
					npctalkall(pc_i, "I dispise all things good. I shall not give thee another chance!",10);
					continue;
				}
				PC_ITEM House = findHouseAt( pc->pos );	// prevent house break-ins by Uber 07-25-2002
				if (House && !pc->Owns(House))
				{
					npctalkall(pc_i, "I will nay give life to thee for thou art not in thine own abode!",10);
					continue;
				}
				if (pc_i->timeused_last + MY_CLOCKS_PER_SEC*10 >= uiCurrentTime) return;
				pc_i->timeused_last = uiCurrentTime;
				Gumps->Menu(pc->getSlot(),7);
				pc_i->DoAction(0x10);

				switch(RandomNum(0, 4)) 
				{
				case 0: npctalkall(pc_i, "Fellow minion of Mondain, Live!!",10); break;
				case 1: npctalkall(pc_i, "Thou has evil flowing through your vains, so I will bring you back to life.",10); break;
				case 2: npctalkall(pc_i, "If I res thee, promise to raise more hell!.",10); break;
				case 3: npctalkall(pc_i, "From hell to Britannia, come alive!.",10); break;
				case 4: npctalkall(pc_i, "Since you are Evil, I will bring you back to consciouness.",10); break;
				}
			}
		}
		break;
	case 4 : // Guards
		if (!pc_i->war	// guard isnt busy 
			&& pc_i->inGuardedArea())	// this region is guarded
		{
			cChar* Victim=NULL;
			unsigned int closest=999;
			RegGrid3x3Iterator4Chars rg3ic(pc_i->pos);
			while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
			{
				if (!pc->npc && !pc->InGame())
					continue;					// logged out player
				if (pc_i->isSameAs(pc))
					continue;					// the guard himself
				if (!(pc->isAiMonster() || /*pc->isCriminal() ||*/ pc->isMurderer()))
					continue;
				unsigned int d = pc_i->dist(pc);
				if (d > SrvParms->attack_distance || pc->isInvul() || pc->dead || pc->isDummy())
					continue;
				if ( closest > d)
				{
					closest = d;
					Victim = pc;
				}
			}
			if (Victim)
			{
				pc_i->attack(Victim);
				//npctalkall(pc_i, "Thou shalt regret thine actions, swine!",1); // ANTISPAM !!! LB
			}
		}
		break;
	case 5: // npc beggars
		if (!pc_i->war)
		{
			RegGrid3x3Iterator4Chars rg3ic(pc_i->pos);
			while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
			{
				if (pc_i->dist(pc) > 3 || pc->isInvul() || pc->npc || pc->dead || !pc->InGame() || !pc->isInnocent() || pc->isHidden())
					continue;
				int beg= RandomNum (0,2);
				{
					switch (beg)
					{
					case 0: npctalkall(pc_i,"Could thou spare a few coins?",30);						break;
					case 1: npctalkall(pc_i,"Hey buddy can you spare some gold?",30);					break;
					case 2: npctalkall(pc_i,"I have a family to feed, think of the children.",30);		break;
					}
				}
			}
		}
		break;
	case 6: break; // Ripper- chaos guards.
	case 7: break; // Ripper- order guards.
	case 8: break; //morrolan - old banker
	case 9 : // in world guards, they dont teleport out...Ripper
		InWorldGuardBrain->doAI(pc_i);
#if 0
		if (!pc_i->war	// guard isnt busy 
			&& pc_i->inGuardedArea())	// this region is guarded
		{	// this bracket just to keep compiler happy
			RegGrid3x3Iterator4Chars rg3ic(pc_i->pos);
			while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
			{
				onl = pc->InGame();
				unsigned int d = pc_i->dist(pc);
				if (pc_i->isSameAs(pc) || d > 3 || pc->isInvul() || pc->dead || !onl || pc->isHidden() || pc->isDummy())
					continue;
				if ( !pc->npc && pc->crimflag > 0 )
				{
					sprintf(temp,"You better watch your step %s, I am watching thee!!", pc->name);
					npctalkall(pc_i,temp,10);
					pc_i->antispamtimer=uiCurrentTime+MY_CLOCKS_PER_SEC*30;
				}
				else if ( !pc->npc && pc->isInnocent() && !pc->dead )
				{
					sprintf(temp,"%s is an upstanding citizen, I will protect thee in %s.", pc->name, region[pc->region].name);
					npctalkall(pc_i,temp,10);
					pc_i->antispamtimer=uiCurrentTime+MY_CLOCKS_PER_SEC*30;
				}
				else if ( d <= 10 && (
					(pc->npc && pc->isAiMonster())	// evil npc
					|| (!pc->npc && !(pc->isInnocent()) && !(pc->isCriminal()))	// a player, not grey or blue
					|| (pc->attackfirst==1)))	// any agressor
				{
					pc_i->pos = pc->pos; //Ripper..guards teleport to enemies.
					soundeffect2(pc_i, 0x01FE);
					staticeffect(pc, 0x372A, 0x09, 0x06);
					npcattacktarget(pc_i, pc);
					//npctalkall(pc_i, "Thou shalt regret thine actions, swine!",1); // ANTISPAM !!! LB
					return;
				}
			}
		}
#endif
		break;
	case 10: // Tamed Dragons ..not white wyrm..Ripper
		// so regular dragons attack reds on sight while tamed.
		if (pc_i->npc && pc_i->tamed)
		{
			RegGrid3x3Iterator4Chars rg3ic(pc_i->pos);
			while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
			{
				if (pc_i->dist(pc) > 10 || !pc->npc || !pc->isAiMonster())
					continue;
				pc_i->attack(pc);
				return;
			}
		}
		break;
	case 11 : // add NPCAI B in scripts to make them attack reds. (Ripper)
		if (!pc_i->war)
		{
			RegGrid3x3Iterator4Chars rg3ic(pc_i->pos);
			while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
			{
				onl = pc->InGame();
				if (pc_i->dist(pc) > 10 || pc->isInvul() || pc->dead || pc->isDummy() || pc->isHidden())
					continue;
				if (!(pc->isAiMonster() || pc->isCriminal() || pc->isMurderer()))
					continue;
				if (!pc->npc && !onl)
					continue;
				pc_i->attack(pc);
			}
		}
		break;
	case 12: break;	// blue decorative npcs. 
	case 13: // Pixie brain by Fizz, 5.12.2002
		PixieBrain->doAI(pc_i);
		break;
	case 14:
		MercenaryBrain->doAI(pc_i);
		break;

	case 17: break; //Zippy Player Vendors.
	case 30: // why is this the same as case 50???..Ripper
		if (!pc_i->war)
		{
			RegGrid3x3Iterator4Chars rg3ic(pc_i->pos);
			while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
			{
				onl = pc->InGame();
				if (pc_i->dist(pc) > 10 || pc->isInvul() || pc->dead || !pc->isAiMonster() || !onl || pc->isDummy())
					continue;
				pc_i->attack(pc);
				return;
			}
		}
		break;
	case 32: // Pets Guarding..Ripper
		if (pc_i->npc && pc_i->tamed)
		{
			RegGrid3x3Iterator4Chars rg3ic(pc_i->pos);
			while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
			{
				onl = pc->InGame();
				if (pc_i->dist(pc) > 10 || pc->npc || pc->dead || !pc->guarded || !onl)
					continue;
				if (pc->Owns(pc_i))
				{
					pc_i->FightBack(pc->getAttacker());
					return;
				}
			}
		}
		break;
	case 50:// Energy Vortex / Blade Spirit
		VortexBrain->doAI(pc_i);
		break;
		// Case 60-70 is Skyfires new AI
	case 96:
	case 60: //Skyfire - Dragon AI
		if (pc_i->war)
		{
			RegGrid3x3Iterator4Chars rg3ic(pc_i->pos);
			while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
			{
				onl = pc->InGame();
				if (pc->npc || pc->dead || !onl)
					continue;
				if (pc_i->dist(pc) > 4 && pc->isSameAs(pc_i->getAttacker()))
				{
					if (pc->getBowType()!=0)
					{
						pc_i->dir=pc->dir;
						pc_i->teleTo(pc->pos);
						npctalkall(pc_i,"Foolish Mortal!");
					}
				}
				else
				{
					switch(RandomNum(0, 4))
					{
					case 0:	OldMagic->NPCEBoltTarget(pc_i,pc);			break;
					case 1:	OldMagic->NPCFlameStrikeTarget(pc_i,pc);	break;
					case 2:	OldMagic->ParalyzeSpell(pc_i,pc);			break;
					case 3:	OldMagic->NPCLightningTarget(pc_i,pc);		break;
					case 4:	OldMagic->ParalyzeSpell(pc_i,pc);			break;
					}
				}
				pc_i->attack(pc);
				if (pc->IsDispellable())
					OldMagic->NPCDispel(pc_i,pc);
			}
			if (pc_i->Hp() < pc_i->getHitMax()/2)
				OldMagic->NPCHeal(pc_i);
			if (pc_i->Poisoned())
				OldMagic->NPCCure(pc_i);
			return;
		}
		else
		{
			if (pc_i->Hp() < pc_i->getHitMax()/2)
				OldMagic->NPCHeal(pc_i);
			if (pc_i->Poisoned())
				OldMagic->NPCCure(pc_i);
		}
		break;

	case 127: // doppelgaenger AI		

		break;

	default:	
		{ 
			Console.send("ERROR: cCharStuff::CheckAI-> Error npc %i has invalid AI type %i\n",pc_i->serial,pc_i->getAi()); 
		}
		return;
	}
}

PC_ITEM InRange(P_CHAR Player,bool (*IsIdFunction)(short), const int distance)
{
	if (!Player || !IsIdFunction) return NULL;
	PC_ITEM pi;
	int loopexit=0;

	RegGrid3x3Iterator4Items rg3ii(Player->pos);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if ((*IsIdFunction)(pi->id()) && inRange(Player->pos,pi->pos,distance))
			return pi;
	}
	return NULL;
}

void cChar::setCraftAI(int ai)
{
	this->craftAI = ai;
	this->nextCraft.setSecs(1);
}

void cChar::resetCraftAI()
{
	this->craftAI = 0;
	this->nextCraft.reset();
}

void cChar::checkCraftAI()
{
	if (!this->craftAI || !this->nextCraft.hasExpired())
		return;

	if (this->war)
	{
		this->resetCraftAI();
		return;
	}

	switch (this->craftAI)
	{
	case 1:
		{
		PC_ITEM Pack = this->getBackpack();
		if (!Pack)
		{
			this->resetCraftAI();
			return;
		}
		PC_ITEM Heat = InRange(this,&IsCookingPlace,3);
		if (!Heat)
		{
			this->resetCraftAI();
			return;
		}
		
		int loopexit=0;
		int ci=0;
		P_ITEM Raw = NULL;
		PC_ITEM pi;
		while ( ((pi=Pack->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
		{
			if (IsRawFood(pi->id()))
			{
				Raw = Items->SplitPile(pi->unconst(),1);	// take one from the pile
				break;
			}
		}
		if (!Raw)
		{
			this->resetCraftAI();
			return;
		}

		Skills->CookOnFire(this,Raw);

		this->nextCraft.setSecs(10*SrvParms->objectdelay);
		}
		break;
	case 2:
		{
		PC_ITEM Dummy = InRange(this,&IsTrainingDummy,2);
		if (!Dummy)
			this->resetCraftAI();
		else
		{
			if (!IsSwingingDummy(Dummy->id()))
			{
				if (Skills->TDummy(this,Dummy->unconst()))
					this->nextCraft.setSecs(15);
				else
					this->resetCraftAI();
			}
		}
		}
		break;
	case 3:
		{
		PC_ITEM Butte = InRange(this,&IsArcheryButte,8);
		if (!Butte)
		{
			this->resetCraftAI();
			return;
		}
		if (!Skills->AButte(this,Butte->unconst()))
		{
			this->resetCraftAI();
			return;
		}
		this->nextCraft.setSecs(15);
		}
		break;
	}
}


