/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

// - Tauriel experimental pointer array *crap*
//
// -- pointer.cpp		Item specific routines (add, delete change) in preperation for
//						going to a pointer based system (compiled under level 3 no errors!
//
#include "pointer.h"

#include "wolfpack.h"

#include <algorithm>

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "pointer.cpp"

class cPoF
{
public:
	int gameSer;
	int trueSerLeader;	//leader
	int trueSerCurr;
	void reset()
	{
		gameSer = -1;
		trueSerLeader = -1;
		trueSerCurr = -1;
	}
};

static cPoF PoF;

void setGameSerial(P_PLAYER Player)
{
	PoF.gameSer = Player->serial | 0x20000000;
	PoF.trueSerLeader = Player->serial;
	PoF.trueSerCurr = Player->serial;
}
void setCurrSerial(int gameSer,P_PLAYER Player)
{
	if (PoF.gameSer == gameSer)
		PoF.trueSerCurr = Player->serial;
}
P_CHAR getByGameSerial(void)
{
	return FindCharBySerial(PoF.trueSerCurr);
}
#if EXP1
int gameSerial(int ser)
{
	return (ser==PoF.trueSerCurr) ? PoF.gameSer : ser;
}
int trueSerial(int ser)
{
	return (ser & 0x20000000) ? PoF.trueSerCurr : ser;
}
#endif

// - the below structure is for looking up items based on serial #
// - item's serial, owner's serial, char's serial, and container's serial

/*defines for debugging only, to remove for official releases!
#define itemsp itemsp_.prepare(DBGFILE,__LINE__)
#define charsp charsp_.prepare(DBGFILE,__LINE__)
*/

#define HASHMAX 4973 // lord binary, hashmax must be a prime for maximum performce (see Knuth, Sedgewick et al).
//#define HASHMAX 149111 // Correa
// Comment about HASHMAX:
// The size of the hash array is very important. In order to get good performance, you should 
// use a suitably large prime number. Suitable means equal to or larger than the maximum expected 
// number of items/chars.

// 149111 is far too high for small systems :( keep in mind how much additional memory will be allocated
// for each number: nearly 1k! So with 149111 we have 140 MB of memory for the hashtable (Duke, 17.9.01)

CArray<HASHMAX, lookuptr_st>itemsp("itemsp");
CArray<HASHMAX, lookuptr_st>charsp("charsp");
CArray<HASHMAX, lookuptr_st>contsp("contsp");
CArray<HASHMAX, lookuptr_st>spawnsp("spawnsp");
CArray<HASHMAX, lookuptr_st>cspawnsp("cspawnsp");
CArray<HASHMAX, lookuptr_st>ownsp("ownsp");
CArray<HASHMAX, lookuptr_st>cownsp("cownsp");
CArray<HASHMAX, lookuptr_st>glowsp("glowsp");

bool BuildPointerArray()
{
	int memerrflg=0;
	for (int i=0;i<HASHMAX;i++)
	{
		// null them first
		itemsp[i].pointer = NULL;
		charsp[i].pointer = NULL;
		cownsp[i].pointer = NULL;
		spawnsp[i].pointer = NULL;
		contsp[i].pointer = NULL;
		cspawnsp[i].pointer = NULL;
		ownsp[i].pointer = NULL;
		glowsp[i].pointer = NULL;


		// init them
		/*if (( itemsp[i].pointer = new int[25]) == NULL) memerrflg=1;
		if (( charsp[i].pointer = new int[25]) == NULL) memerrflg=1;
		if (( cownsp[i].pointer = new int[25]) == NULL) memerrflg=1;
		if (( spawnsp[i].pointer = new int[25]) == NULL) memerrflg=1;
		if (( contsp[i].pointer = new int[25]) == NULL) memerrflg=1;
		if (( cspawnsp[i].pointer = new int[25]) == NULL) memerrflg=1;
		if (( ownsp[i].pointer = new int[25]) == NULL) memerrflg=1;
		if (( glowsp[i].pointer = new int[25]) == NULL) memerrflg=1;
		*/

		// mixing new and realloc/free is bad, LB
		// usually no noticeably problems, because global new operator usually just calls malloc
		// but not true anywhere

		int bytesalloc=sizeof(int)*25;
		if (( itemsp[i].pointer   = (int*) malloc(bytesalloc)) == NULL)  memerrflg=1;
		if (( charsp[i].pointer   = (int*) malloc(bytesalloc)) == NULL)  memerrflg=1;
		if (( cownsp[i].pointer   = (int*) malloc(bytesalloc)) == NULL)  memerrflg=1;
		if (( spawnsp[i].pointer  = (int*) malloc(bytesalloc)) == NULL)  memerrflg=1;
		if (( contsp[i].pointer   = (int*) malloc(bytesalloc)) == NULL)  memerrflg=1;
		if (( cspawnsp[i].pointer = (int*) malloc(bytesalloc)) == NULL)  memerrflg=1;
		if (( ownsp[i].pointer    = (int*) malloc(bytesalloc)) == NULL)  memerrflg=1;
		if (( glowsp[i].pointer   = (int*) malloc(bytesalloc)) == NULL)  memerrflg=1;
		
		if (memerrflg)
		{
			return false;
		}
		itemsp[i].max=ownsp[i].max=spawnsp[i].max=contsp[i].max=cownsp[i].max=cspawnsp[i].max=charsp[i].max=glowsp[i].max=25;
		for (int j=0;j<25;j++)
			itemsp[i].pointer[j]=ownsp[i].pointer[j]=cownsp[i].pointer[j]=spawnsp[i].pointer[j]=contsp[i].pointer[j]=cspawnsp[i].pointer[j]=charsp[i].pointer[j]=glowsp[i].pointer[j]=-1;
	}
	return true;
}

//
// - Sets an item into the array, reallocating space if needed
//   Usage: setptr(&itemsp[serial%256], item#);
//          setptr(&charsp[serial%256], char#);
//   NOTE: can be used to set other pointer arrays too (like regions)
void setptr(lookuptr_st *ptr, P_CHAR pc) //set char in pointer array
{
	if (pc->serial <0) return; // crash avoidance
	setptr(ptr, pc->GetIndex());
}
void setptr(lookuptr_st *ptr, PC_ITEM pi) //set item in pointer array
{
	if (pi->serial <0) return; // crash avoidance
	setptr(ptr, pi->GetIndex());
}
void setptr(lookuptr_st *ptr, int item) //set item in pointer array
{
	int i;
	if (ptr->max == 0)	// allow for deferred memory allocation ie. just when needed (Duke, 18.12.01)
	{
		if (ptr->pointer)
		{
			LogCritical("Corrupted pointer memory!");
			return;
		}
		else	// both are zero, so allocate
		{
			int* ti = (int *)malloc(25*sizeof(int));
			if (!ti)
			{
				LogCritical("Error allocating pointer memory!");
				return;
			}
			else
			{
				ptr->pointer = ti;
				ptr->max = 25;
				memset(ptr->pointer,0xFF,25*sizeof(int));	// initialize to -1
			}
		}
	}

	for (i=0;i<(ptr->max);i++)
	{
		if (ptr->pointer[i]==-1)
		{
			ptr->pointer[i]=item;
			return;
		} else if (ptr->pointer[i]==item) return;
	}

	//resize(ptr->pointer, ptr->max, ptr->max+25);
	// Must be out of slots, so reallocate some and set item
//	if ((ptr->pointer = (int *)realloc(ptr->pointer, (ptr->max+25)*sizeof(int)))==NULL)
	int* ti = (int *)realloc(ptr->pointer, (ptr->max+25)*sizeof(int));
	if (!ti)
	{
		LogCritical("Error reallocating pointer memory!");
		error=1;
		//keeprun=0;  //shutdown
		return;
	}
	else
		ptr->pointer = ti;

	for (i=ptr->max;i<(ptr->max+25);i++) ptr->pointer[i]=-1;
	ptr->pointer[ptr->max]=item;
	ptr->max+=25;
	return;
}

void addToItemSp(const P_ITEM pi)				{setptr(&itemsp[pi->serial%HASHMAX], pi);}
void addToCharSp(const P_CHAR pc)				{setptr(&charsp[pc->serial%HASHMAX], pc);}
void addToContSp(int serial,const P_ITEM pi)	{setptr(&contsp[serial%HASHMAX], pi);}
void addToSpawnSp(int serial,const P_ITEM pi)	{setptr(&spawnsp[serial%HASHMAX], pi);}
void addToCspawnSp(int serial,const P_CHAR pc)	{setptr(&cspawnsp[serial%HASHMAX], pc);}
void addToOwnSp(int serial,const P_ITEM pi)		{setptr(&ownsp[serial%HASHMAX], pi);}
void addToCownSp(int serial,const P_CHAR pc)	{setptr(&cownsp[serial%HASHMAX], pc);}
void addToGlowSp(const P_CHAR pc,PC_ITEM pi)	{setptr(&glowsp[pc->serial%HASHMAX], pi);}

// - Find a specific item/char by serial number.
//   Usage: item=findbyserial(&itemsp[serial%HASHMAX], serial, 0);
//          char=findbyserial(&charsp[serial%HASHMAX], serial, 1);
//          if (item==-1) Console.send("Couldn't find by serial: %d\n", serial);
int findbyserial(lookuptr_st *ptr, int nSerial, int nType)
{ 
	if (nSerial < 0) return -1;  //prevent errors from some clients being slower than the server clicking on nolonger valid items
	
	if (ptr == NULL || ptr->pointer == NULL) // Blackwind / Crashfix 
		return -1;
	for (int i=0;i<(ptr->max);i++)
	{ 
		if (nType==0 && (ptr->pointer[i]> -1 && ptr->pointer[i] < Items->GetCount()) )
		{
			P_ITEM TheItem = MAKE_ITEM_REF(ptr->pointer[i]);
			if (TheItem!=NULL)
			{
				if (TheItem->serial==nSerial)
					return ptr->pointer[i];
			}
		}
		if (nType==1 && (ptr->pointer[i]>-1 && ptr->pointer[i] < Npcs->GetCount()) )
		{
			P_CHAR TheChar = MAKE_CHAR_REF(ptr->pointer[i]);
			if (TheChar!=NULL)
			{
				if (TheChar->serial==nSerial)
					return ptr->pointer[i];
			}
		}
	}
	return -1;
}

// - Remove an item/char from a pointer array
//   (Ok, just mark it as a free slot ;P )
void removefromptr(lookuptr_st *ptr, P_CHAR pc)
{
	if (pc->serial <0) return; // crash avoidance
	removefromptr(ptr, pc->GetIndex());
}
void removefromptr(lookuptr_st *ptr, PC_ITEM pi)
{
	if (pi->serial <0) return; // crash avoidance
	removefromptr(ptr, pi->GetIndex());
}
void removefromptr(lookuptr_st *ptr, int nItem)
{
	int i;
	for (i=0;i<(ptr->max);i++)
	{
		if (ptr->pointer[i]==nItem)
		{
			ptr->pointer[i]=-1;
		//	return;		// don't return here, let's remove ALL the occurences (Duke)
		}
	}
}

void delFromItemSp(const P_ITEM pi)				{removefromptr(&itemsp[pi->serial%HASHMAX], pi);}
void delFromCharSp(const P_CHAR pc)				{removefromptr(&charsp[pc->serial%HASHMAX], pc);}
void delFromContSp(int serial,const P_ITEM pi)	{removefromptr(&contsp[serial%HASHMAX], pi);}
void delFromSpawnSp(int serial,const P_ITEM pi) {removefromptr(&spawnsp[serial%HASHMAX], pi);}
void delFromCspawnSp(int serial,const P_CHAR pc){removefromptr(&cspawnsp[serial%HASHMAX], pc);}
void delFromOwnSp(int serial,const P_ITEM pi)	{removefromptr(&ownsp[serial%HASHMAX], pi);}
void delFromCownSp(int serial,const P_CHAR pc)	{removefromptr(&cownsp[serial%HASHMAX], pc);}
void delFromGlowSp(const P_CHAR pc,const P_ITEM pi) {removefromptr(&glowsp[pc->serial%HASHMAX], pi);}

///////////////////////////
// Name:	SearchOwnedPets
// history:	by Duke, 24.12.2002
// Purpose:	simplyfies the access via cownsp and creates CHAR pointers
//			IMPORTANT: ci should point to a 0 at the start of the search
//
P_CHAR SearchOwnedPets(const int serial, int *ci)
{
	if (serial <0) return NULL; // crash avoidance
	int loopexit=0;
	while ( (*ci<cownsp[serial%HASHMAX].max) && (++loopexit < MAXLOOPS) ) 
	{
		int i=cownsp[serial%HASHMAX].pointer[*ci];
		(*ci)++;
		if (i>-1)
		{
			P_CHAR pc = MAKE_CHAR_REF(i);
			if (pc)
			{
				if (pc->ownserial==serial)
					return pc;
			}
		}
	}
	return NULL;
}

///////////////////////////
// Name:	SearchOwnedItems
// history:	by Duke, 3.3.2002
// Purpose:	simplyfies the access via ownsp and creates ITEM pointers
//			IMPORTANT: ci should point to a 0 at the start of the search
//
P_ITEM SearchOwnedItems(const int serial, int *ci)
{
	if (serial <0) return NULL; // crash avoidance
	int loopexit=0;
	while ( (*ci<ownsp[serial%HASHMAX].max) && (++loopexit < MAXLOOPS) ) 
	{
		int i=ownsp[serial%HASHMAX].pointer[*ci];
		(*ci)++;
		if (i>-1)
		{
			P_ITEM iItem = MAKE_ITEM_REF(i);
			if (iItem!=NULL)
			{
				if (iItem->ownserial==serial)
					return iItem;
			}
		}
	}
	return NULL;
}

///////////////////////////
// Name:	ContainerSearch
// history:	by Duke, 3.10.2000
// Purpose:	simplyfies the access via contsp and creates ITEM pointers
//			IMPORTANT: ci should point to a 0 at the start of the search
//
P_ITEM ContainerSearch(const int serial, int *ci)
{
	if (serial <0) return NULL; // crash avoidance
	int loopexit=0;
	while ( (*ci<contsp[serial%HASHMAX].max) && (++loopexit < MAXLOOPS) ) 
	{
		int i=contsp[serial%HASHMAX].pointer[*ci];
		(*ci)++;
		if (i>-1)
		{
			P_ITEM iItem = MAKE_ITEM_REF(i);
			if (iItem!=NULL)
			{
				if (iItem->contserial==serial)
					return iItem;
			}
		}
	}
	return NULL;
}

void AllContainerItemsInStableOrderHelper(const int serial, int *ci, vector<int> & sers)
{	
	if (serial <0) return; // crash avoidance
	int loopexit=0;
	while ( (*ci<contsp[serial%HASHMAX].max) && (++loopexit < MAXLOOPS) ) 
	{
		int i=contsp[serial%HASHMAX].pointer[*ci];
		(*ci)++;
		if (i>-1)
		{
			P_ITEM iItem = MAKE_ITEM_REF(i);
			if (iItem!=NULL)
			{
				if(iItem->contserial==serial) sers.push_back(iItem->serial);
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////
// returns all items inside a container in a defined/stable order 
// (always same order regardless of server shutdowns, additions & removals of items etc)
// input: container serial
// output: all items of the container (vector of P_ITEMS)
/////////////////////////////////////////////////////////////////////////

void AllContainerItemsInStableOrder(const int contserial, vector<P_ITEM> & pitems)
{ 
   int ci=0;
   unsigned int a;
   vector<int> serials;

   pitems.clear();
   serials.clear();
   AllContainerItemsInStableOrderHelper(contserial, &ci, serials);
   std::sort(serials.begin(), serials.end());

   for (a=0; a<serials.size(); a++) // inelegant & suboptimal but it does its job ;) 
	                                // better: define an ordering relation (<) of the items-class to make std:sort work for items (< -> < of items->serial) & storing items in helper instead of serials)
								    // -> marked as to do ;)
   {
	   P_ITEM item = FindItemBySerial(serials[a]);
	   if (item!=NULL)
	   {
		   pitems.push_back(item);
	   }
   }
}

///////////////////////////
// Name:	ContainerSearchFor
// history:	by Duke, 14.12.2000
// Purpose:	simplyfies the access via contsp and creates ITEM pointers.
//			searches the container given by serial for items with the given id and (if given) color.
//			IMPORTANT: ci should point to a 0 at the start of the search
//
P_ITEM ContainerSearchFor(const int serial, int *ci, short id, short color)
{
	P_ITEM pi;
	int loopexit=0;
	while ( ((pi=ContainerSearch(serial,ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->id()==id &&	(color==-1 || pi->color()==color))
		return pi;
	}
	return NULL;
}

///////////////////////////
// Name:	ContainerCountItems
// history:	by Duke, 26.03.2001
// Purpose:	searches the container given by serial AND the subcontainers
//			for items with the given id and (if given) color.
//			Returns the total number of items found
//
int ContainerCountItems(const int serial, short id, short color, short type)
{
	int ci=0; 
	int total=0;
	P_ITEM pi;
	int loopexit=0;
	while ( ((pi=ContainerSearch(serial,&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->isType(ITY_Container))		// a subcontainer ?
		{
			total += ContainerCountItems(pi->serial, id, color, type);
			continue;
		}
		if ((id==-1 || pi->id()==id) &&
			(color==-1 || pi->color()==color) &&
			(type==-1 || pi->isType(type))) // added count by type support (thrawn)
			total += pi->amount;
	}
	return total;
}

///////////////////////////
// Name:	SearchSpawnedItems
// history:	by Duke, 18.4.2003
// Purpose:	simplyfies the access via spawnsp and creates ITEM pointers
//			IMPORTANT: ci should point to a 0 at the start of the search
//
P_ITEM SearchSpawnedItems(const int serial, int *ci)
{
	if (serial <0) return NULL; // crash avoidance
	int loopexit=0;
	while ( (*ci<spawnsp[serial%HASHMAX].max) && (++loopexit < MAXLOOPS) ) 
	{
		int i=spawnsp[serial%HASHMAX].pointer[*ci];
		(*ci)++;
		if (i>-1)
		{
			P_ITEM iItem = MAKE_ITEM_REF(i);
			if (iItem!=NULL)
			{
				if (iItem->spawnserial==serial)
					return iItem;
			}
		}
	}
	return NULL;
}

///////////////////////////
// Name:	SearchSpawnedChars
// history:	by Duke, 18.4.2003
// Purpose:	simplyfies the access via cspawnsp and creates P_CHAR pointers
//			IMPORTANT: ci should point to a 0 at the start of the search
//
P_CHAR SearchSpawnedChars(const int serial, int *ci)
{
	if (serial <0) return NULL; // crash avoidance
	int loopexit=0;
	while ( (*ci<cspawnsp[serial%HASHMAX].max) && (++loopexit < MAXLOOPS) ) 
	{
		int c=cspawnsp[serial%HASHMAX].pointer[*ci];
		(*ci)++;
		if (c>-1)
		{
			P_CHAR pc = MAKE_CHAR_REF(c);
			if (pc!=NULL)
			{
				if (pc->spawnserial==serial)
					return pc;
			}
		}
	}
	return NULL;
}

///////////////////////////
// Name:	SearchGlowers
// history:	by Duke, 23.7.2005
// Purpose:	simplyfies the access via glowsp and creates ITEM pointers
//			IMPORTANT: ci should point to a 0 at the start of the search
//
P_ITEM SearchGlowers(const int serial, int *ci)
{
	if (serial <0) return NULL; // crash avoidance
	int loopexit=0;
	while ( (*ci<glowsp[serial%HASHMAX].max) && (++loopexit < MAXLOOPS) ) 
	{
		int i=glowsp[serial%HASHMAX].pointer[*ci];
		(*ci)++;
		if (i>-1)
		{
			P_ITEM iItem = MAKE_ITEM_REF(i);
			if (iItem!=NULL)
			{
				return iItem;
			}
		}
	}
	return NULL;
}

/////////////////////
// Name:	FindItemBySerial
// Purpose:	creates an item pointer from the given serial, returns NULL if not found
// History:	by Duke, 5.11.2000
//
P_ITEM FindItemBySerial(int serial)
{
	if (!isItemSerial(serial))
		return NULL;
	int i=findbyserial(&itemsp[serial%HASHMAX], serial, 0);
	if (i==-1) return NULL;		// legal rc from findbyserial, don't log
	return MAKE_ITEM_REF(i);
}
P_ITEM FindItemBySerPtr(unsigned char *p)
{
	int serial=LongFromCharPtr(p);
	if (!isItemSerial(serial)) return NULL;
	return FindItemBySerial(serial);
}

/////////////////////
// Name:	FindCharBySerial
// Purpose:	creates an chars pointer from the given serial, returns NULL if not found
// History:	by Duke, 11.11.2000
//			added BySerPtr Duke, 24.2.2001
//
P_CHAR FindCharBySerial(int serial)
{
	if (!isCharSerial(serial))
		return NULL;
	int i=findbyserial(&charsp[serial%HASHMAX], serial, 1);
	if (i==-1) return NULL;		// legal rc from findbyserial, don't log
	else return MAKE_CHARREF_LRV(i,NULL);
}
P_CHAR FindCharBySerPtr(unsigned char *p)
{
	int serial=LongFromCharPtr(p);
	if (!isCharSerial(serial)) return NULL;
	return FindCharBySerial(serial);
}

P_CHAR FindWearerOf(PC_ITEM pi)
{
	return FindCharBySerial(pi->contserial);
}

