/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

/*
 Cache script section locations by <erwin@andreasen.com>
 Reads through the contents of the file and saves location of each
 SECTION XYZ entry

 Calling Script::find() will then seek to that location directly rather
 than having to parse through all of the script
 */

#include "scriptc.h"

#include "rectangle.h"
#include "mapS.h"
#include "speech.h"
#include "globals.h"

#include <fstream>
#include <iostream>
#include <string>
#include <sstream>

using namespace std;
#include <sys/stat.h>
#include <errno.h>

#include "mstring.h"
#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "scriptc.cpp"

// Script files that need to be cached
// Crackerjack Jul 31/99
Script* i_scripts[NUM_SCRIPTS]; // array of script references


bool get_modification_date(const char *filename, time_t* mod_time, bool musthave = true) 
{
    struct stat stat_buf;
    
    if ((stat(filename, &stat_buf)))
	{
		if (musthave)	// don't abort on optional scripts (xxx2.scp)
		{
			char tmperr[1024];
			sprintf(tmperr, "Cannot stat %s: %s", filename, strerror(errno));
			LogWarning(tmperr);
			fprintf(stderr, tmperr);
			exit(1);
		}
		return false;
    }
	else
	{
		*mod_time = stat_buf.st_mtime;
		return true;
	}
}

// Parse a string removing comments, CAT keywords, and simpligying white space
#define SCRIPTWHITESPACE " \t\v\f\0 "
#define SCRIPTCOMMENTTAG "//"
void Script::parsestring(std::string &s)
{
	if (s.empty()) 
		return;
	std::string comment(SCRIPTCOMMENTTAG);
	std::string whitespace(SCRIPTWHITESPACE);
	std::string::size_type index = s.find(comment) ;
	if (index != std::string::npos)
	{
		s=s.substr(0,index);
	}
	//Comments are now out of the string, we need to delete all the white space
	// First elimainate the leading white space
	index = s.find_first_not_of(whitespace) ;
	std::string::size_type index2 = s.find_last_not_of(whitespace);
	if (index != std::string::npos && index2 != std::string::npos)
	{
		s = s.substr(index,index2-index+1);
	}
	else if (index == std::string::npos)
	{
		// The entire line is white space
		s ="";

	}
#if 0
	// Now we just need to turn all the other white space to spaces, and trim them as we go.
	//std::string working = s ;
	index = s.find_first_of(whitespace) ;
	while (index != std::string::npos)
	{
		
		if (index != std::string::npos)
		{
			index2 = s.find_first_not_of (whitespace,index);
			if (index2!=std::string::npos)
			{

				s = s.substr(0,index) + " " + s.substr(index2) ;
				// Move paste the space we just added
				++index ;
				index = s.find_first_of(whitespace,index);
			}

			else
				index = std::string::npos ;
		
		}
	}
#endif

	// Last thing is strip off the CAT keywords
	index = s.find_first_of(whitespace);
	if (index != std::string::npos)
	{
		std::string temp = s.substr(0,index);
		if ((temp=="CAT0") || (temp=="CAT1") || (temp=="CAT2"))
		{
			s="";
		}
	}

}
bool Script::preload()	// load the complete script into memory (Duke,6.1.2001)
{
    FILE *fp;
    struct stat stat_buf;
    
    if (stat(sf->filename, &stat_buf)) return false;

    sf->scpsize = stat_buf.st_size;

	if (sf->img!=NULL) delete [] sf->img; // ??

	sf->img = new char[sf->scpsize];
	sf->curmempos = sf->img;
    if (!(fp = fopen(sf->filename, "rb")))
	{
        fprintf(stderr, "Cannot open %s: %s", sf->filename, strerror(errno));
        exit(1);
	}

	int r=fread(sf->img, sizeof( char ), sf->scpsize, fp);	// read in the whole file
	if (r != sf->scpsize)
	{
		LogErrorVar("problem preloading %s",sf->filename);
	}

	fclose(fp);
	

	this->unicode_hack();
	return true;
}



// LB 1'st Feb 2005
//
// Actually we don't support real unicode.
// Clean unicode support is not feasable because it would trigger a major rewrite to change all string handling to unicode.
// This big ugly hack works as follows:
// Scripts are asumed to be in utf-8 format. 
// Since 7-bit ascii chars don't get altered in utf8-encoding, scripts saved in text format (7 bit ascii) still work without any change.
// So it's necassairy to save scripts in utf-8 format only if chars outside 7-bit ascii are used.
// Conversion works as follows:
// a) The text from a script is first interpreted as utf-8 stream.
// b) Then the utf-8 stream is converted to unicode (16).
// c) 2-Byte unicode is mapped to one byte (c-style strings).
// First of all it is obvious that this can only work with small alphabets.
// Actually it has to be <127 signs, because 0..127 of our sinlge byte is reserved for 7-bit ASCII, 
// 128..256 for our uni-code to single byte mapping . (similar to but not exacltly SCU)
// It's also easily seen that we can't mix alphabets because some signs might map to the same number.
// Steb b) might seem redundant, that's purely technical, simplifies step c) greatly. 
// It it definityl ugly, but better than no unicode-like support at all.

void Script::unicode_hack()
{
	int actualsz;
	wchar_t* out;
	out = new wchar_t[sf->scpsize+4];
	char* tmp=new char[sf->scpsize+2];

	memcpy(tmp, sf->img, sf->scpsize);
	tmp[sf->scpsize]=0;
	tmp[sf->scpsize+1]=0; // utf8 routine needs 0 terminated strings, so we place two 0's after "EOF", ugly, yes.

	delete [] sf->img;	

	sf->scpsize+=2;

	utf8_to_unicode_string((unsigned char*)tmp, sf->scpsize, actualsz, out);

	delete[] tmp;

	sf->scpsize=actualsz;

    sf->img = new char[actualsz];

	int a; // memorize the (few) real uncide chars extra
	char* p = sf->img;
	for (a=0; a<actualsz;a++)
	{
		if (out[a]>0x7f)
		{
		   sf->uc.insert(make_pair<char*, wchar_t>(p, out[a]));				 
		}
		p++;
	}

	unicode_to_char(out, actualsz, true, (unsigned char*)(sf->img));

	delete [] out;

	sf->curmempos = sf->img;	
}

void Script::reload()
{
	preload();
	MakeIndexForMem();
}


void Script::MakeIndexForMem()
{
    char section_name[257];
    
    sf->entries.clear();

	Console.send("Reloading %-15s: (memory mapped)", sf->filename); fflush (stdout);

		int loopexit=0;
		while( NextLine() && (++loopexit < MAXLOOPS) )
		{
			if (sscanf((char*)temp, "SECTION %256[^\n]", section_name) == 1)
			{
				if (strlen(temp) > 260)
				{
					LogWarningVar("Too long section in '%s', skipped !", sf->filename);
					continue;
				}
				if (section_name[0] == 'x') // currently for harditems.scp and wtrigers.scp (all sections that start like SECTION xff6xff7)
				{
					mstring sScript1(section_name);
					vector<mstring> Tokens = sScript1.split("x");
					unsigned int i;
					for (i = 0; i < Tokens.size(); i++)
					{
						if (Tokens[i].size()>0) // the split routine always returns an redundant empty string as first vector element
						{
						  //sprintf(temp, "\n\nWT:%s(%i/%i) from sect:%s\n\n", Tokens[i].c_str(), i+1, Tokens.size(), section_name);
						  //LogWarning(temp);

						  ScriptEntry section;
						  section.name = Tokens[i];					
						  section.offset = (unsigned long) (sf->curmempos);
						  sf->entries.insert(make_pair(section.name, section));
						}
					}
				} 
				else 
				{
					ScriptEntry section;
					section.name = section_name;
					section.offset =  (unsigned long) (sf->curmempos);
					sf->entries.insert(make_pair(section.name, section));
				}
			}
		}

	Console.send("%6d sections found.\n", sf->entries.size());
}

// Parse this script, caching section positions
Script::Script(const char *_filename)
{
	sf = &_sf;
	//flags = _flags;
	sf->filename = NULL;
    sf->filename = strdup(_filename);
    
    get_modification_date(sf->filename, &(sf->last_modification));

	sf->img = NULL;

    reload();
}

Script::~Script()
{
	if ( sf->filename != NULL ) delete [] sf->filename;
	if ( sf->img !=NULL) delete [] sf->img;
}

void Script::checkModification()
{
    time_t current;
    
    if (get_modification_date(sf->filename, &current, false))
	{
		if (current > sf->last_modification)
		{
			reload();
			sf->last_modification = current;
		}
	}
	else	// didn't find the file
	{
		//if (sf->img)	// we once had it...
		//	;			// we could issue a warning here
	}
}

// Look for that section in this previously parsed script file
bool Script::find(const char *secname, const int secID)
{
	char sect[512];
	sprintf(sect, "%s %i", secname, secID);
	return this->find(sect);
}

// Look for that section in this previously parsed script file
bool Script::find(const char *section)
{
	checkModification();

	std::string dummy = section;
	std::map<std::string, ScriptEntry>::iterator iter_entry = sf->entries.find(dummy);

    //lp = iter_entry->second.offset; // save last position, for debugging only

    if (iter_entry == sf->entries.end())
        return 0;

	sf->curmempos=(char*)(iter_entry->second.offset);

    return 1;
}

bool Script::Open(int scriptnum)
{
	sf->curmempos = sf->img;
	return true;
}

void Script::Close()
{
	return;
}

void Script::Suspend(scpMark& mark)
{
	mark.fnum = (sf == &_sf) ? 1 : 2;
	mark.cmp = sf->curmempos;
	this->Close();
}

void Script::Resume(scpMark mark)
{
	this->Open();
	sf->curmempos = mark.cmp;
}

inline bool Script::ReadMemLine(void) 
{ 
	bool rv, comment; 
	do // skip over commented out lines
	{
	  rv=this->__ReadMemLine(false, comment); 
	}  while (rv && comment);

	return rv;
}

// rewrite LB 27-03-2005
// note: the concept to scan on each ReadMemLine call for end of line markers and comments is a bit suboptimal 
// basic idea for optimization: if we'd store the lines as vectors of strings (on each reload) -> Nextline/ReadMemeLine just iterates the vector -> better
// (adding or not adding commneted lines to the vector would be a property of the script)
// but currently no time for this ... (LB)

bool Script::__ReadMemLine(bool raw, bool &comment) // basicly reads a line from script
{
	char *p = sf->curmempos;	

	temp[0]=0;temp[1]=0;
	comment=false;

	if (!isValidScpPtr(p)) { return false; }

	int i=0, loopexit=0;
	sf->line_start = sf->curmempos;
	
	while ( isValidScpPtr(p) && !(isNewLine(*p)) && (++loopexit<MAXLOOPS*17) )
	{		
		temp[i]=*p;
		p++;	
		if (!raw && (isSingleCharComment(temp[0]) || isDoubleCharComment(temp[0],temp[1])) ) // note: we cant just skip the loop in case of a comment because we need to move the pointer correctly to the end of line
		{
			temp[0]=0;
			comment=true;
		} else i++;
	}

	if (i>=1023) LogError("overlength line in some script file");

	if (isNewLine(*p)) // linux useses 0x0a, windows 0x0a and 0x0d as end of lien marker, isNewline returns true for both markers, so we need to check twice to get it correctly for windows
	{
		sf->curmempos=p+1;
		if (isNewLine(*(sf->curmempos))) sf->curmempos++;
	}
	else sf->curmempos=p; 
	temp[i]=0;

	return true;
}


bool Script::NextLineInSection()
{
	if (!this->NextLine()) return false;
	if (EndSect()) return false;
	return true;
}

bool Script::NextLine()
{	
   if (!ReadMemLine()) return false;

   strcpy(script1,(char*)temp);
   sf->last_scpt1_start_pos = sf->line_start;
   return true;
}

bool Script::NextLineSplitted()
{	
	ReadMemLine();
	int i=0;
	script1[0]=0;
	script2[0]=0;
	
	while(temp[i]!=0 && temp[i]!=' ' && temp[i]!='=' && i<1024 )
	{
		i++;
	}
	
	strncpy(script1,temp, i);
	script1[i]=0;
	if (!EndSect() && temp[i]!=0) strcpy(script2, (temp+i+1));
	
	sf->last_scpt1_start_pos = sf->line_start;
	sf->last_scpt2_start_pos = sf->line_start+i+1;
	
	if (EndSect()) return false;
	
	return true;
}


// Look for that section in this previously parsed script file
char Script::isin(const char *section)
{
    checkModification();
    
	std::map<std::string, ScriptEntry>::iterator iter_entry = sf->entries.find(section);

	if (iter_entry == sf->entries.end()) return 0; 
	else 
	{
		sf->curmempos = (char*)iter_entry->second.offset;
		return 1;
	}
}

/////////////////
// name:	getKeyVal
// purpose:	if the current line has the given key, read the values
// history:	by Duke, 24.03.2002
//			added support for int values. Duke, 6.01.2003
//
bool Script::getKeyVal(const char* key, int* num)
{
	if (strcmp(script1,key)) return false;
	sscanf(script2,"%i",num); return true;
}
bool Script::getKeyVal(const char* key, cCoord** pos)
{
	if (strcmp(script1,key))
		return false;

	int x,y,z,m=0,n;
	n = sscanf(script2,"%i %i %i %i",&x,&y,&z,&m);
	if (n < 3 || n > 4)
		LogWarning("Invalid position given");
	else
		*pos = new cCoord(x,y,z,m);

	return true;
}
bool Script::getKeyVal(const char* key, cCoord& pos)
{
	if (strcmp(script1,key))
		return false;

	int tz, tm = 0;	// z & map are only one byte; sscanf will place FOUR bytes at the adress given (Duke)
	unsigned int tx,ty;	// much better for big-endian machines 
	int n = sscanf(script2,"%i %i %i %i",&tx,&ty,&tz,&tm);
	if (n < 3 || n > 4)
	{
		LogWarning("Invalid position given");
		return false;
	}
	pos.x = tx;
	pos.y = ty;
	pos.z = tz;
	pos.map = tm;
	return true;
}
bool Script::getKeyVal(const char* key, cMapRectangle* rect)
{
	if (strcmp(script1,key))
		return false;

	int x1,y1,x2,y2,map=AllMaps->getdefaultmap();
	int p = sscanf(script2,"%i %i %i %i %i",&x1,&y1,&x2,&y2,&map);
	if (p<4 || p>5)
		LogWarning("Invalid rectangle given");
	else
		rect->init(x1,y1,x2,y2,map);

	return true;
}
bool Script::getKeyVal(const char* key, char* name)
{
	if (strcmp(script1,key))
		return false;

	sscanf(script2,"%s", name);
	return true;
}
bool Script::getKeyVal(const char* key, short* Value)
{
	if (strcmp(script1,key))
		return false;

	sscanf(script2,"%hi",Value);
	return true;
}

void Script::invalidTag()
{
	char wmsg[1000];
	sprintf(wmsg,"Invalid tag '%s' in script '%s', ignored !",script1,sf->filename);
	LogWarning(wmsg);
}


/////////////////
// name:	findSection
// purpose:	a *free* constructor for Script, positioning to the given section
// history:	by Duke, 25.03.2002
//
Script* findSection(enScripts scpID, const char* sectName, int sectID)
{
	char sect[512];
	sprintf(sect, "%s %i", sectName, sectID);
	return findSection(scpID, sect);
}

Script* findSection(enScripts scpID, const char* sectName)
{
	Script* Scp=getScript(scpID);
	if (!Scp) return NULL;
	if (scpID <= dual_scripts)
	{
		if (Scp->Open(2))
		{
			if (Scp->find(sectName))
				return Scp;
			else
				Scp->Close();
		}
		if (!Scp->Open(1)) return NULL;
		if (!Scp->find(sectName))
		{
			Scp->Close();
			return NULL;
		}
	}
	else
	{
		if (!Scp->Open()) return NULL;
		if (!Scp->find(sectName))
		{
			Scp->Close();
			return NULL;
		}
	}
	return Scp;
}

Script* getScript(enScripts scpID)
{
	return i_scripts[scpID];
}
/////////////////
// name:	PickFromList
// purpose:	randomly picks a line from a list of numbers, weightened by the second numbers
// history:	by Duke, 03.01.2003
//			added support for tags in the list (Duke, 22.12.05)
//
int PickFromList(enScripts scpID, const char* sectName, int sectID, char* tag, bool hexval)
{
	Script* Scp = findSection(scpID,sectName,sectID);
	if (!Scp) return -1;

	return Scp->PickFromList(tag,hexval);
}
int Script::PickFromList(char* tag, bool hexval)
{
	char fc[9];
	strcpy(fc, hexval ? "%x" : "%i");
	char form[99];
	if (!tag)
		sprintf(form,"%s %%i",fc);
	else
		sprintf(form,"%s %s %%i",tag,fc);

	int i=0, loopexit=0;
	int iList[512];		//-- no more than 512 items in a single item list (changed by Magius(CHE))
	int draws[512];		// a multiplier for that entry (works as if they had repeated this line n times)
	int totaldraws=0;
	while ( this->NextLine() && i<512 && (++loopexit < MAXLOOPS) )  // -- count items storing item #'s in iList[]
	{
		if (this->EndSect()) break;
		draws[i]=-1;
		sscanf(script1,form,&iList[i],&draws[i]);
		if (draws[i] == -1) draws[i] = 1;
		totaldraws += draws[i];				// summarize all draw chances
		i++;
	}
	this->Close();
	if (i==0) 
		return -1;

	int picked = rand()%totaldraws;
	totaldraws=0;
	for (int j=0;j<i;j++)
	{
		totaldraws += draws[j];
		if (totaldraws > picked)
			return(iList[j]);
	}
	LogError("should not happen");
	return -1;
}


//////////////////////////////////////////////////////
// (Pseudo) Unicode stuff                            //
/////////////////////////////////////////////////////

void Script::translate_scpt1_to_Unicode(void)
{
   int a,l1;
   l1=strlen(script1)+1;
   char*p;
   map<char*, wchar_t>::iterator uci;

   p = sf->last_scpt1_start_pos;
   for (a=0; a<l1;a++)
   {
	 if (*p >0x7f)
	 {
		uci = sf->uc.find(p);
		if (uci == sf->uc.end())
		{
		   LogError("Fehler in translate_lastlinetoUnicode1\n");
		   Console.send("Fehler in translate_lastlinetoUnicode1\n");
		}
		wscript1[a]=uci->second;
	 }
	 else wscript1[a]=*p;
	 p++;
   }  
}

void Script::translate_scpt2_to_Unicode(void)
{
	int a,l2;   
	l2=strlen(script2)+1;
	char*p;
	map<char*, wchar_t>::iterator uci;
	
	p = sf->last_scpt2_start_pos;
	for (a=0; a<l2;a++)
	{
		if (*p >0x7f)
		{
			uci = sf->uc.find(p);
			if (uci == sf->uc.end())
			{
				LogError("Fehler in translate_lastlinetoUnicod2");
			}
			wscript2[a]=uci->second;
		}
		else wscript2[a]=*p;
		p++;
	}  
	wscript2[a]=0;
}

// basically like NextLine, just doesn't skip // and  {} 's
bool Script::NextLineRaw(void)
{
	bool dummy;
	if (!(__ReadMemLine(true,dummy))) return false;
	strcpy(script1,(char*)temp);
	sf->last_scpt1_start_pos = sf->line_start;
	return true;
}

void LoadScripts()
{
	i_scripts[items_script]		= new cDualScript("items.scp");
	i_scripts[npc_script]		= new cDualScript("npc.scp");
	i_scripts[create_script]	= new cDualScript("create.scp");
	i_scripts[regions_script]	= new cDualScript("regions.scp");
	i_scripts[misc_script]		= new cDualScript("misc.scp");
	i_scripts[skills_script]	= new cDualScript("skills.scp");
	// when changing to cDualScript, don't forget to move them in enScripts !!
	i_scripts[location_script]	= new Script("location.scp");
	i_scripts[menus_script]		= new Script("menus.scp");
	i_scripts[spells_script]	= new Script("spells_new.scp");
	i_scripts[speech_script]	= new Script("speech.scp");
	i_scripts[tracking_script]	= new Script("tracking.scp");
	i_scripts[newbie_script]	= new Script("newbie.scp");
	i_scripts[titles_script]	= new Script("titles.scp");
	i_scripts[advance_script]	= new Script("advance.scp");
	i_scripts[trigger_script]	= new Script("triggers.scp");
	i_scripts[ntrigger_script]	= new Script("ntrigrs.scp");
	i_scripts[wtrigger_script]	= new Script("wtrigrs.scp");
	i_scripts[necro_script]		= new Script("necro.scp");
	i_scripts[house_script]		= new Script("house.scp");
	i_scripts[colors_script]	= new Script("colors.scp");
	i_scripts[spawn_script]		= new Script("spawn.scp");
	i_scripts[html_script]		= new Script("htmlstrm.scp");
	i_scripts[metagm_script]	= new Script("metagm.scp");
	i_scripts[polymorph_script]	= new Script("polymorph.scp");
	i_scripts[harditems_script]	= new Script("harditems.scp");
	i_scripts[menupriv_script]	= new Script("menupriv.scp");
	i_scripts[carve_script]		= new Script("carve.scp");
	i_scripts[msgboard_script]	= new Script("msgboard.scp");
	i_scripts[fishing_script]	= new Script("fishing.scp");
	i_scripts[gump_script]		= new Script("gumps.scp");
	i_scripts[startlocs_scp]	= new Script("startlocs.scp"); 
	i_scripts[maps_script]		= new Script("maps.scp");
	i_scripts[townhouse_script]	= new Script("townhouse.scp");
	i_scripts[reserve_script]	= new Script("reserve.scp");
	i_scripts[resources_script]	= new Script("resources.scp");
	i_scripts[races_script]		= new Script("races.scp"); 
	i_scripts[teleport_script]	= new Script("teleport.scp");
	i_scripts[envoke_script]	= new Script("envoke.scp");
	i_scripts[hostsdeny_script]	= new Script("hosts_deny.scp");
	i_scripts[oldspells_script]	= new Script("spells.scp");
	i_scripts[accounts_script]	= new Script("accounts.adm");
	i_scripts[itemmenu_script]	= new Script("itemmenu.scp");
	i_scripts[bod_script]		= new Script("bod.scp");
}

/////////////
// name:	class DualScript
// purpose:	support a 'custom' script alternative for each selected script
//			
// history:	by Duke, 11.04.2006
//
cDualScript::cDualScript(const char *_filename) : Script(_filename)
{
	sf = &_sf;
    sf->filename = strdup(_filename);
    get_modification_date(sf->filename, &(sf->last_modification));
	sf->img = NULL;
    reload();

	char fn2[266];							// alternate filename
	char* p=strchr(_filename,'.');			// find start of extension
	if (!p) LogError("Bad script filename");// a scriptname from n_scripts *really* should have an extension ;)
	int len = p - _filename;
	strncpy(fn2,_filename, len);			// copy base name (warning: pointer arithmetiks )
	*(fn2+len) = 0;							// strncpy doesn't add teminating zero
	strcat(fn2,"2");
	strcat(fn2,p);							// add extension

	sf = &_sf2;
    sf->filename = strdup(fn2);
    get_modification_date(sf->filename, &(sf->last_modification), false);
	sf->img = NULL;
    reload();
}

cDualScript::~cDualScript()
{
	if (_sf.filename) delete [] _sf.filename;
	if (_sf.img) delete [] _sf.img;
	if (_sf2.filename) delete [] _sf2.filename;
	if (_sf2.img) delete [] _sf2.img;
}

bool cDualScript::Open(int scriptnum)
{
	if (scriptnum == 1)			// activate primary script
		sf = &_sf;
	else if (scriptnum == 2)	// activate alternate script
	{
		if (!_sf2.img)
			return false;
		sf = &_sf2;
	}
	sf->curmempos = sf->img;
	return true;
}

void cDualScript::Resume(scpMark mark)
{
	this->Open(mark.fnum);
	sf->curmempos = mark.cmp;
}


