/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#ifndef __SCRIPTC_H__
#define __SCRIPTC_H__

#include "platform.h"
#include <sys/types.h>
#include <map>
#include <string>
#include <vector>
using namespace std ;

class cCoord;
class cMapRectangle;

//typedef unsigned long scpMark;
class scpMark
{
public:
	char* cmp;
	int fnum;
};

// One entry in the cache
struct ScriptEntry
{
	string name;
    unsigned long offset;
};


extern char script1[512];

enum  enScripts { items_script = 0, npc_script, create_script, regions_script, misc_script, skills_script,
			dual_scripts,
			location_script, menus_script, spells_script, speech_script, tracking_script,
			newbie_script, titles_script, advance_script, trigger_script, ntrigger_script,
			wtrigger_script, necro_script, house_script, colors_script, spawn_script,
			html_script, metagm_script, polymorph_script, harditems_script,
			menupriv_script, carve_script, msgboard_script,
			fishing_script, gump_script, startlocs_scp, maps_script, townhouse_script, reserve_script, 
			resources_script, races_script, teleport_script, envoke_script, hostsdeny_script, oldspells_script, 
			accounts_script, itemmenu_script, bod_script, NUM_SCRIPTS} ;

#ifndef _MSC_VER
typedef long int _off_t ;
#endif


class Script
{
public:
    Script(const char *_filename);
	virtual ~Script();
	virtual bool Open(int scriptnum = 0);
	void Close(void);
	void checkModification();
	void Suspend(scpMark& mark);
	virtual void Resume(scpMark mark);
	bool NextLine(void);
	bool NextLineInSection(void);
	bool NextLineSplitted(void);
	bool NextLineRaw(void);
	int  cmpTag(char* comp) {return strcmp(script1,comp);}
	void translate_scpt1_to_Unicode(void);
	void translate_scpt2_to_Unicode(void);
	
	bool EndSect() {return (script1[0]=='}');}
	bool find(const char *secname, const int secID);
	bool find(const char *section);
    char isin(const char *section);
	bool getKeyVal(const char* key, int* num);
	bool getKeyVal(const char* key, cCoord** pos);
	bool getKeyVal(const char* key, cCoord& pos);
	bool getKeyVal(const char* key, cMapRectangle* mrect);
	bool getKeyVal(const char* key, char* name);
	bool getKeyVal(const char* key, short* Value);
	void invalidTag();
	int  PickFromList(char* tag=NULL, bool hexval=false);
    
protected:
	bool preload();
    void reload();
	void unicode_hack();
	void MakeIndexForMem();
	void parsestring(std::string &s);

	bool __ReadMemLine(bool raw, bool &comment);		
	inline bool ReadMemLine(void);                      
	inline bool isNewLine(char p)                   { if (p==0x0A || p==0x0D) return true; else return false; }
	inline bool isValidScpPtr(char *p)              { if ((p<(sf->img+sf->scpsize)) && (p>=sf->img)) return true; else return false;}
	inline bool isSingleCharComment(char f)         { if ((f=='{')) return true; else return false;} // '}' not being treated as comment (and '{' being a comment) IS very important 
	inline bool isDoubleCharComment(char f, char s) { if ((f=='/') && (s=='/')) return true; else return false;}
    
	//short flags;

	struct ScriptFile
	{
		char* filename;
		time_t last_modification;
		char* img;
		_off_t scpsize;
		char* curmempos;			// current read position for preloaded files
		map<string, ScriptEntry> entries;
		// unicode hack related stuff
		map<char*, wchar_t> uc;     // memorize only those chars as real 2-byte unicode for UC>0x80
		char* line_start;
		char* last_scpt1_start_pos; // for each script1/script2 remember the start position of it, otherwise we have no channce to find the uc chars saved in above map / for script1/script2 (scp1/2 itself aint unicode)
		char* last_scpt2_start_pos;
	} _sf;
	ScriptFile* sf;
};

class cDualScript : public Script
{
public:
    cDualScript(const char* _filename);
	virtual ~cDualScript();
	virtual bool Open(int scriptnum = 0);
	virtual void Resume(scpMark mark);
protected:
	struct ScriptFile _sf2;
};

// free functions
Script* findSection(enScripts scpID, const char* sectName, int sectID);
Script* findSection(enScripts scpID, const char* sectName);	// For loading sections that don't have numbers
Script* getScript(enScripts scpID);
int PickFromList(enScripts scpID, const char* sectName, int sectID, char* tag=NULL, bool hexval=false);

void LoadScripts();
#endif
