/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "spells.h"

#include "chars.h"
#include "items.h"
#include "SndPkg.h"
#include "globals.h"
#include "magic_new.h"
#include "sregions.h"
#include "skills.h"
#include "TmpEff.h"
#include "trigger.h"

cSpell::cSpell(unsigned char num) : spellnum(num)
{
	// some initialisations
	int i;
	for (i=0;i<3;++i)
	{
		mana[i] = 0;	// mana requirements
		tithing[i] = 0;	// required tithing points
		loskill[i] = 0;		// low magery skill req.
		hiskill[i] = 1000;	// high magery skill req.
		delay[i] = 10;		// spell delay
	}

	targtype = 0;

	action = 10;			// character action
	
	strcpy(mantra,"");	// words of power
	strcpy(targmsg,"");	// string visualized with targ. system

	
	for (i=0;i<3;++i)
	{
		trig[i] = 0;
	}

	flags = 0x00;
}

cSpell::~cSpell()
{
	reags.reagamount.clear();
	reags.reagid.clear();
}

cSpell& cSpell::operator= (cSpell& b)
{
	int i;
	for (i=0;i<3;++i)
	{
		mana[i]		= b.mana[i];	// mana requirements
		tithing[i]	= b.tithing[i];	// tithing points required
		delay[i]	= b.delay[i];	// spell delay
		loskill[i]	= b.loskill[i];	// low magery skill req.
		hiskill[i]	= b.hiskill[i];	// high magery skill req.
	}

	action	= b.action;			// character action
	reags	= b.reags;			// holds the all required reagents
	
	strcpy(mantra,b.mantra);	// words of power
	strcpy(targmsg,b.targmsg);	// string visualized with targ. system

	for (i=0;i<3;++i)
	{
		trig[i] = b.trig[i];
	}
	
	flags	= b.flags;
	return *this;
}

char cSpell::GetTargetType()
{
	return targtype;
}

bool cSpell::IsInBook(P_ITEM Book)
{
	if (!Book) return false;

	unsigned int more=0;
	unsigned int spellid = 1;

	if (this->spellnum<=32) //first 4 circles in regual spellbook
	{
		if (!Book->isType(ITY_SpellBook)) return false;
		spellid <<= (spellnum-1); // calc the spell in binary position
		more = Book->getMore1_4();
	}
	else if (this->spellnum<=64) // second 4 circles in regular spellbook
	{
		if (!Book->isType(ITY_SpellBook)) return false;
		spellid <<= (spellnum-33); // calc the spell in binary position
		more = Book->getMoreB1_4();
	}
	else if (this->spellnum>100 && this->spellnum<=116 ) // 16 spells in necro book
	{
		if (!Book->isType(ITY_NecroBook)) return false;
		spellid <<= (spellnum-101); // calc the spell in binary position
		more = Book->getMore1_4();
	}
	else if (this->spellnum>200 && this->spellnum<=210 ) // 10 spells in paladin book
	{
		if (!Book->isType(ITY_PaladinBook)) return false;
		spellid <<= (spellnum-201); // calc the spell in binary position
		more = Book->getMore1_4();
	}

	return ((more & spellid) == spellid) ? true : false;
}

void cSpell::AddToBook(P_ITEM Book)
{
	unsigned int more;
	unsigned int spellid = 1;

	if (this->spellnum<=32) //first 4 circles in regual spellbook
	{
		if (!Book->isType(ITY_SpellBook)) return;
		spellid <<= (spellnum-1); // calc the spell in binary position
		more = Book->getMore1_4();
		Book->setMore1_4(more | spellid);
	}
	else if (this->spellnum<=64) // second 4 circles in regular spellbook
	{
		if (!Book->isType(ITY_SpellBook)) return;
		spellid <<= (spellnum-33); // calc the spell in binary position
		more = Book->getMoreB1_4();
		Book->setMoreB1_4(more | spellid);
	}
	else if (this->spellnum>100 && this->spellnum<=116 ) // 16 spells in necro book
	{
		if (!Book->isType(ITY_NecroBook)) return;
		spellid <<= (spellnum-101); // calc the spell in binary position
		more = Book->getMore1_4();
		Book->setMore1_4(more | spellid);
	}
	else if (this->spellnum>200 && this->spellnum<=210 ) // 10 spells in paladin book
	{
		if (!Book->isType(ITY_PaladinBook)) return;
		spellid <<= (spellnum-201); // calc the spell in binary position
		more = Book->getMore1_4();
		Book->setMore1_4(more | spellid);
	}
}

void cSpell::AddReag(char amount, short id)
{
	short*ID = (short*)malloc(sizeof(short));
		 *ID = id;
	char*amt = (char*)malloc(sizeof(char));
		*amt = amount;

	this->reags.reagid.push_back(*ID);
	this->reags.reagamount.push_back(*amt);
}


bool cSpell::CheckRequirements(P_PLAYER Player, const cCoord* targpos, char type)
{
	if (Player->isGM())									return true;	// gms shouldn't have any requirements

	if (Player->dead==1)								return false;
	if (!this->CheckAvailible(Player,type))				return false;
	if (!this->CheckReags(Player,type))					return false;
	if (!this->CheckMana(Player,type))					return false;
	if (!this->CheckTithing(Player,type))				return false;
	if (!this->CheckRegion(Player,targpos))				return false;
	if (!this->CheckPlayersItems(Player))				return false;

	return true;
}

void cSpell::AccomplishRequirements(P_CHAR Player, char type, bool succeeded)
{
	if (Player->isGM())	return;	// gms shouldn't have ane requirements

	if (succeeded)	// don't delete mana and tithing when casting failed due to skillcheck
	{
		this->DeleteMana	(Player,type);
		this->DeleteTithing	(Player,type);
	}
	this->DeleteReags(Player,type);
}

bool cSpell::CheckAvailible(P_PLAYER Player, char type)
{
	if (this->IsDisabled())
	{
		Player->Msg("This spell is currently disabled!");	// give the sysmessage
		return false;
	}
	if (type == STY_BOOK)
	{
		P_ITEM Book = NULL;

		//get the right spellbook
			 if ((this->GetSpellnum()>=  1) && (this->GetSpellnum()<= 64)) Book = Magic->GetSpellBook(Player,SBT_REGULAR);
		else if ((this->GetSpellnum()>=101) && (this->GetSpellnum()<=116)) Book = Magic->GetSpellBook(Player,SBT_NECRO);
		else if ((this->GetSpellnum()>=201) && (this->GetSpellnum()<=210)) Book = Magic->GetSpellBook(Player,SBT_PALADIN);
		else return false;

		if (!this->IsInBook(Book)) 
		{
			Player->Msg("You don't have that spell");	// give the sysmessage
			return false;
		}
	}
	else if (type == STY_SCROLL)
	{
	}
	else if (type == STY_ITEM)
	{
	}
	else return false;
	
	return true;
}


// checks if player has all reagents he needs
bool cSpell::CheckReags(P_CHAR Player, char type)		
{
	if (Player->DoesNotNeedRegs())	return true; // check if player needs reagents
	if (type == STY_SCROLL)			return true; // scrolls don't need reags
	if (type == STY_ITEM)			return true; // casting items don't need reags

	unsigned int i;
	for (i=0;i<this->reags.reagid.size();++i)
	{
		if (Player->CountItems(this->reags.reagid[i],-1,ITY_Reagent) < this->reags.reagamount[i])
		{
			Player->Msg("You lack some reagents for this spell");
			return false;
		}
	}
	return true;
}

// removes the needed reagents..
void cSpell::DeleteReags(P_CHAR Player, char type)
{
	if (Player->DoesNotNeedRegs())	return; // check if player needs reagents
	if (type == STY_SCROLL)			return; // scrolls don't need reags
	if (type == STY_ITEM)			return; // casting items don't need reags

	unsigned int i;
	for (i=0;i<this->reags.reagid.size();++i)
	{
		Player->deleteItems(this->reags.reagamount[i],this->reags.reagid[i],-1,ITY_Reagent);
	}
}

bool cSpell::CheckMana(P_CHAR Player, char type)
{
	if (Player->DoesNotNeedMana()) return true; // check if player needs mana

	if (this->GetMana(type) > Player->Mana())
	{

		char c[128];
		sprintf(c,"You need at least %d mana to cast this spell!",this->GetMana(type));
		Player->Msg(c);
		return false;
	}
	return true;
}

void cSpell::DeleteMana(P_CHAR Player, char type)
{
	if (Player->DoesNotNeedMana()) return; // check if player needs mana

	Player->reduceMana(this->GetMana(type));
}

bool cSpell::CheckTithing(P_CHAR Player, char type)
{
	if (Player->DoesNotNeedMana()) return true; // if the player doesn't need mana, he shouldn't need tithing points either

	if (this->GetTithing(type) > Player->GetTithingPoints())
	{

		char c[128];
		sprintf(c,"You need at least %d tithing points to cast this spell!",this->GetTithing(type));
		Player->Msg(c);
		return false;
	}
	return true;
}

void cSpell::DeleteTithing(P_CHAR Player, char type)
{
	if (Player->DoesNotNeedMana()) return; // if the player doesn't need mane, he shouldn't need tithing points either

	Player->AddTithingPoints(-1*this->GetTithing(type));
}

bool cSpell::CheckRegion(P_CHAR Player, const cCoord* targpos)
{
	if (Player->cell)
	{
		Player->Msg("You are in jail and cannot cast spells");
		return false;
	}

	short r = calcRegionFromXY(Player->pos);
	if (!region[r].isSpellAllowed(this->GetSpellnum())==1)
	{
		Player->Msg("You can't cast this spell here");
		return false;
	}

	if (!targpos) return true; // if we have no target, we don't need to check it :)
	r = calcRegionFromXY(*targpos);
	if (!region[r].isSpellAllowed(this->GetSpellnum())==1)
	{
		Player->Msg("You can't cast this spell here");
		return false;
	}
	return true;
}

bool cSpell::CheckPlayersItems(P_CHAR Player)
{
	if (Player->WieldsAntiMagicalItem())
	{
		Player->Msg("You cannot cast while carrying something in your hands.");
		return false;
	}
	return true;
}

char cSpell::CheckTrigger(P_PLAYER Player, short pos) // checks for trigger to fire with the spell
{
	if (!this->trig[pos]) return 0;

	int tmptrg = Player->trigger; // can't think of a situation where players trigges is used for longer times... but its saver not to overwrite (Thrawn)
	Player->trigger = this->trig[pos];
	triggernpc(Player,Player,1);
	Player->trigger = tmptrg;
	return 1;
}


// some constructors

// START REGULAR SPELLS
// LVL 1 SPELLS
cClumsySpell::			cClumsySpell():			cSpell(1)	{targtype=CHAR_TARGET;}
cCreateFoodSpell::		cCreateFoodSpell():		cSpell(2)	{targtype=  NO_TARGET;}
cFeeblemindSpell::		cFeeblemindSpell():		cSpell(3)	{targtype=CHAR_TARGET;}
cHealSpell::			cHealSpell():			cSpell(4)	{targtype=CHAR_TARGET;}
cMagicArrowSpell::		cMagicArrowSpell():		cSpell(5)	{targtype=CHAR_TARGET;}
cNightsightSpell::		cNightsightSpell():		cSpell(6)	{targtype=CHAR_TARGET;}
cReactiveArmorSpell::	cReactiveArmorSpell():	cSpell(7)	{targtype=CHAR_TARGET;} // NEED TO BE CHANGED TO NO_TARGET LATER !!
cWeakenSpell::			cWeakenSpell():			cSpell(8)	{targtype=CHAR_TARGET;}
// LVL 2 SPELLS
cAgilitySpell::			cAgilitySpell():		cSpell(9)	{targtype=CHAR_TARGET;}
cCunningSpell::			cCunningSpell():		cSpell(10)	{targtype=CHAR_TARGET;}
cCureSpell::			cCureSpell():			cSpell(11)	{targtype=CHAR_TARGET;}
cHarmSpell::			cHarmSpell():			cSpell(12)	{targtype=CHAR_TARGET;}
cMagicTrapSpell::		cMagicTrapSpell():		cSpell(13)	{targtype=ITEM_TARGET;}
cMagicUntrapSpell::		cMagicUntrapSpell():	cSpell(14)	{targtype=ITEM_TARGET;}
cProtectionSpell::		cProtectionSpell():		cSpell(15)	{targtype=CHAR_TARGET;} // NEED TO BE CHANGED TO NO_TARGET LATER !!
cStrengthSpell::		cStrengthSpell():		cSpell(16)	{targtype=CHAR_TARGET;}
// LVL 3 SPELLS
cBlessSpell::			cBlessSpell():			cSpell(17)	{targtype=CHAR_TARGET;}
cFireballSpell::		cFireballSpell():		cSpell(18)	{targtype=CHAR_TARGET;}
cMagicLockSpell::		cMagicLockSpell():		cSpell(19)	{targtype=ITEM_TARGET;}
cPoisonSpell::			cPoisonSpell():			cSpell(20)	{targtype=CHAR_TARGET;}
cTelekinesisSpell::		cTelekinesisSpell():	cSpell(21)	{targtype=ITEM_TARGET;}
cTeleportSpell::		cTeleportSpell():		cSpell(22)	{targtype= MAP_TARGET;}
cUnlockSpell::			cUnlockSpell():			cSpell(23)	{targtype=ITEM_TARGET;}
cWallOfStoneSpell::		cWallOfStoneSpell():	cSpell(24)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
// LVL 4 SPELLS
cArchCureSpell::		cArchCureSpell():		cSpell(25)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
cArchProtectionSpell::	cArchProtectionSpell():	cSpell(26)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
cCurseSpell::			cCurseSpell():			cSpell(27)	{targtype=CHAR_TARGET;}
cFireFieldSpell::		cFireFieldSpell():		cSpell(28)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
cGreaterHealSpell::		cGreaterHealSpell():	cSpell(29)	{targtype=CHAR_TARGET;}
cLightningSpell::		cLightningSpell():		cSpell(30)	{targtype=CHAR_TARGET;}
cManaDrainSpell::		cManaDrainSpell():		cSpell(31)	{targtype=CHAR_TARGET;}
cRecallSpell::			cRecallSpell():			cSpell(32)	{targtype=ITEM_TARGET;}
// LVL 5 SPELLS
cBladeSpiritSpell::		cBladeSpiritSpell():	cSpell(33)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
cDispelFieldSpell::		cDispelFieldSpell():	cSpell(34)	{targtype=CHAR_TARGET+ITEM_TARGET;}
cIncognitoSpell::		cIncognitoSpell():		cSpell(35)	{targtype=  NO_TARGET;}
cMagicReflectSpell::	cMagicReflectSpell():	cSpell(36)	{targtype=  NO_TARGET;}
cMindBlastSpell::		cMindBlastSpell():		cSpell(37)	{targtype=CHAR_TARGET;}
cParalyzeSpell::		cParalyzeSpell():		cSpell(38)	{targtype=CHAR_TARGET;}
cPoisonFieldSpell::		cPoisonFieldSpell():	cSpell(39)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
cSummonCreatureSpell::	cSummonCreatureSpell():	cSpell(40)	{targtype=  NO_TARGET;}
// LVL 6 SPELLS
cDispelSpell::			cDispelSpell():			cSpell(41)	{targtype=CHAR_TARGET+ITEM_TARGET;}
cEnergyBoltSpell::		cEnergyBoltSpell():		cSpell(42)	{targtype=CHAR_TARGET;}
cExplosionSpell::		cExplosionSpell():		cSpell(43)	{targtype=CHAR_TARGET;}
cInvisibilitySpell::	cInvisibilitySpell():	cSpell(44)	{targtype=CHAR_TARGET;}
cMarkSpell::			cMarkSpell():			cSpell(45)	{targtype=ITEM_TARGET;}
cMassCurseSpell::		cMassCurseSpell():		cSpell(46)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
cParalyzeFieldSpell::	cParalyzeFieldSpell():	cSpell(47)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
cRevealSpell::			cRevealSpell():			cSpell(48)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
// LVL 7 SPELLS
cChainLightningSpell::	cChainLightningSpell():	cSpell(49)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
cEnergyFieldSpell::		cEnergyFieldSpell():	cSpell(50)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
cFlamestrikeSpell::		cFlamestrikeSpell():	cSpell(51)	{targtype=CHAR_TARGET;}
cGateTravelSpell::		cGateTravelSpell():		cSpell(52)	{targtype=ITEM_TARGET;}
cManaVampireSpell::		cManaVampireSpell():	cSpell(53)	{targtype=CHAR_TARGET;}
cMassDispelSpell::		cMassDispelSpell():		cSpell(54)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
cMeteorSwarmSpell::		cMeteorSwarmSpell():	cSpell(55)	{targtype=CHAR_TARGET+ITEM_TARGET+MAP_TARGET;}
cPolymorphSpell::		cPolymorphSpell():		cSpell(56)	{targtype=  NO_TARGET;}
// LVL 8 SPELLS
cEarthquakeSpell::		cEarthquakeSpell():		cSpell(57)	{targtype=  NO_TARGET;}
cEnergyVortexSpell::	cEnergyVortexSpell():	cSpell(58)	{targtype= MAP_TARGET;}
cResurrectionSpell::	cResurrectionSpell():	cSpell(59)	{targtype=CHAR_TARGET;}
cAirElementalSpell::	cAirElementalSpell():	cSpell(60)	{targtype=  NO_TARGET;}
cEarthElementalSpell::	cEarthElementalSpell():	cSpell(61)	{targtype=  NO_TARGET;}
cWaterElementalSpell::	cWaterElementalSpell():	cSpell(62)	{targtype=  NO_TARGET;}
cFireElementalSpell::	cFireElementalSpell():	cSpell(63)	{targtype=  NO_TARGET;}
cSummonDaemonSpell::	cSummonDaemonSpell():	cSpell(64)	{targtype=  NO_TARGET;}
// END REGULAR SPELLS

// START NECRO SPELLS
cAnimateDeadSpell::		cAnimateDeadSpell():	cSpell(101) {targtype=ITEM_TARGET;}
cBloodOathSpell::		cBloodOathSpell():		cSpell(102) {targtype=CHAR_TARGET;}
cCorpseSkinSpell::		cCorpseSkinSpell():		cSpell(103) {targtype=CHAR_TARGET;}
cCurseWeaponSpell::		cCurseWeaponSpell():	cSpell(104)	{targtype=  NO_TARGET;}
cEvilOmenSpell::		cEvilOmenSpell():		cSpell(105) {targtype=CHAR_TARGET;}
cHorrificBeastSpell::	cHorrificBeastSpell():	cSpell(106) {targtype=  NO_TARGET;}
cLichFormSpell::		cLichFormSpell():		cSpell(107) {targtype=  NO_TARGET;}
cMindRotSpell::			cMindRotSpell():		cSpell(108)	{targtype=CHAR_TARGET;}
cPainSpikeSpell::		cPainSpikeSpell():		cSpell(109) {targtype=CHAR_TARGET;}
cPoisonStrikeSpell::	cPoisonStrikeSpell():	cSpell(110) {targtype=CHAR_TARGET;}
cStrangleSpell::		cStrangleSpell():		cSpell(111) {targtype=CHAR_TARGET;}
cSummonFamiliarSpell::	cSummonFamiliarSpell():	cSpell(112) {targtype=  NO_TARGET;}
cVampiricEmbraceSpell::	cVampiricEmbraceSpell():cSpell(113) {}
cVengefulSpiritSpell::	cVengefulSpiritSpell():	cSpell(114) {targtype=CHAR_TARGET;}
cWitherSpell::			cWitherSpell():			cSpell(115) {targtype=  NO_TARGET;}
cWraithFormSpell::		cWraithFormSpell():		cSpell(116) {targtype=  NO_TARGET;}
// END NECRO SPELLS

// START PALADIN SPELLS
cCleanseByFireSpell::	cCleanseByFireSpell():	cSpell(201)	{action=0;targtype=CHAR_TARGET;}
cCloseWoundsSpell::		cCloseWoundsSpell():	cSpell(202)	{action=0;targtype=CHAR_TARGET;}
cConsecrateWeaponSpell::cConsecrateWeaponSpell():cSpell(203){action=0;targtype=  NO_TARGET;}
cDispelEvilSpell::		cDispelEvilSpell():		cSpell(204)	{action=0;targtype=  NO_TARGET;}
cDivineFurySpell::		cDivineFurySpell():		cSpell(205)	{action=0;targtype=  NO_TARGET;}
cEnemyOfOneSpell::		cEnemyOfOneSpell():		cSpell(206)	{action=0;targtype=  NO_TARGET;}
cHolyLightSpell::		cHolyLightSpell():		cSpell(207)	{action=0;targtype=  NO_TARGET;}
cNobleSacrificeSpell::	cNobleSacrificeSpell():	cSpell(208)	{action=0;targtype=  NO_TARGET;}
cRemoveCurseSpell::		cRemoveCurseSpell():	cSpell(209)	{action=0;targtype=CHAR_TARGET;}
cSacredJourneySpell::	cSacredJourneySpell():	cSpell(210)	{action=0;}
// END PALADIN SPELLS





// and now the spell funktions:
void cSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cSpell::doEffect() without Targ");
}
void cSpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg)
{
	Caster->Msg("cSpell::doEffect() with CharTarg");
}
void cSpell::DoEffect(P_CHAR Caster, P_ITEM ItemTarg)
{
	Caster->Msg("cSpell::doEffect() with ItemTarg");
}
void cSpell::DoEffect(P_CHAR Caster, cCoord* MapTarg)
{
	Caster->Msg("cSpell::doEffect() with MapTarg");
}

// LVL 1 SPELLS
void cClumsySpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg)
{
	float percent	= ((Caster->Skill(EVALUATINGINTEL)/100)+8) - (CharTarg->Skill(MAGICRESISTANCE)/100);
	float points	= (CharTarg->realDex() * percent) / 100;
	float dur		= ((Caster->Skill(EVALUATINGINTEL)/5)+1)*6;
	// the above calculations are taken directly from uo stratics (Thrawn)

	tempeffect(Caster, CharTarg, TEF_Clumsy,(int)points, 0, 0,(int)dur);
 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,CharTarg,NULL); // some visual effects on target and caster
}

void cCreateFoodSpell::DoEffect(P_CHAR Caster)
{
	P_ITEM pi = Items->SpawnItem(ITY_Food, 1, "#", 0x09D3, 0x00, Caster);
	if (pi)
	{
		pi->AddFoodType((unsigned char)FOOD_MAGIC); // flag for new food system
	}

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}

void cFeeblemindSpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg)
{
	float percent	= ((Caster->Skill(EVALUATINGINTEL)/100)+8) - (CharTarg->Skill(MAGICRESISTANCE)/100);
	float points	= (CharTarg->realInt() * percent) / 100;
	float dur		= ((Caster->Skill(EVALUATINGINTEL)/5)+1)*6;
	// the above calculations are taken directly from uo stratics (Thrawn)

	tempeffect(Caster, CharTarg, TEF_FeebleMind,(int)points, 0, 0,(int)dur);
 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,CharTarg,NULL); // some visual effects on target and caster
}

void cHealSpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg)
{
	if (Caster->isInnocent() && !Caster->isSameAs(CharTarg))
	{
		if (CharTarg->crimflag>0 || CharTarg->isMurderer()) criminal(Caster);
	}

	float points = (Caster->Skill(MAGERY)/100) + rand()%3;
	CharTarg->increaseHp((int)points);
 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,CharTarg,NULL); // some visual effects on target and caster
}

void cMagicArrowSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cMagicArrowSpell::doEffect()");
}

void cNightsightSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cNightsightSpell::doEffect()");
}

void cReactiveArmorSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cReactiveArmorSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}

void cWeakenSpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg)
{
	float percent	= ((Caster->Skill(EVALUATINGINTEL)/100)+8) - (CharTarg->Skill(MAGICRESISTANCE)/100);
	float points	= (CharTarg->realStr() * percent) / 100;
	float dur		= ((Caster->Skill(EVALUATINGINTEL)/5)+1)*6;
	// the above calculations are taken directly from uo stratics (Thrawn)

	tempeffect(Caster, CharTarg, TEF_Weaken,(int)points, 0, 0,(int)dur);
 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,CharTarg,NULL); // some visual effects on target and caster
}
// LVL 2 SPELLS
void cAgilitySpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg)
{
	float percent	= ((Caster->Skill(EVALUATINGINTEL)/100)+1);
	float dur		= ((Caster->Skill(EVALUATINGINTEL)/5)+1)*6;
	float points	= (CharTarg->realInt() * percent) / 100;
	// the above calculations are taken directly from uo stratics (Thrawn)

	tempeffect(Caster, CharTarg, TEF_Agility,(int)points, 0, 0,(int)dur);
 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,CharTarg,NULL); // some visual effects on target and caster
}
void cCunningSpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg)
{
	float percent	= ((Caster->Skill(EVALUATINGINTEL)/100)+1);
	float dur		= ((Caster->Skill(EVALUATINGINTEL)/5)+1)*6;
	float points	= (CharTarg->realInt() * percent) / 100;
	// the above calculations are taken directly from uo stratics (Thrawn)

	tempeffect(Caster, CharTarg, TEF_Cunning,(int)points, 0, 0,(int)dur);
 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,CharTarg,NULL); // some visual effects on target and caster
}
void cCureSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cCureSpell::doEffect()");
}
void cHarmSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cHarmSpell::doEffect()");
}
void cMagicTrapSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cMagicTrapSpell::doEffect()");
}
void cMagicUntrapSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cMagicUntrapSpell::doEffect()");
}
void cProtectionSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cProtectionSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cStrengthSpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg)
{
	float percent	= ((Caster->Skill(EVALUATINGINTEL)/100)+1);
	float dur		= ((Caster->Skill(EVALUATINGINTEL)/5)+1)*6;
	float points	= (CharTarg->realStr() * percent) / 100;
	// the above calculations are taken directly from uo stratics (Thrawn)

	tempeffect(Caster, CharTarg, TEF_Strength,(int)points, 0, 0,(int)dur);
 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,CharTarg,NULL); // some visual effects on target and caster
}
// LVL 3 SPELLS
void cBlessSpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg)
{
	float percent	= ((Caster->Skill(EVALUATINGINTEL)/100)+1);
	float dur		= ((Caster->Skill(EVALUATINGINTEL)/5)+1)*6;

	float points;
	
	points = (CharTarg->realStr() * percent) / 100;
	tempeffect(Caster, CharTarg, TEF_Strength,(int)points, 0, 0,(int)dur);

	points = (CharTarg->realDex() * percent) / 100;
	tempeffect(Caster, CharTarg, TEF_Agility,(int)points, 0, 0,(int)dur);

	points = (CharTarg->realInt() * percent) / 100;
	tempeffect(Caster, CharTarg, TEF_Cunning,(int)points, 0, 0,(int)dur);
	// the above calculations are taken directly from uo stratics (Thrawn)

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,CharTarg,NULL); // some visual effects on target and caster
}
void cFireballSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cFireballSpell::doEffect()");
}
void cMagicLockSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cMagicLockSpell::doEffect()");
}
void cPoisonSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cPoisonSpell::doEffect()");
}
void cTelekinesisSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cTelekinesisSpell::doEffect()");
}
void cTeleportSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cTeleportSpell::doEffect()");
}
void cUnlockSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cUnlockSpell::doEffect()");
}
void cWallOfStoneSpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg) { DoEffect(Caster,&CharTarg->pos); }
void cWallOfStoneSpell::DoEffect(P_CHAR Caster, P_ITEM ItemTarg) { DoEffect(Caster,&ItemTarg->pos); }
void cWallOfStoneSpell::DoEffect(P_CHAR Caster, cCoord* MapTarg)
{
	Magic->AddMagicField(Caster,MapTarg,0x0080,0x0080,3,10,GetSpellnum());
}
// LVL 4 SPELLS
void cArchCureSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cArchCureSpell::doEffect()");
}
void cArchProtectionSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cArchProtectionSpell::doEffect()");
}
void cCurseSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cCurseSpell::doEffect()");
}

void cFireFieldSpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg) { DoEffect(Caster,&CharTarg->pos); }
void cFireFieldSpell::DoEffect(P_CHAR Caster, P_ITEM ItemTarg) { DoEffect(Caster,&ItemTarg->pos); }
void cFireFieldSpell::DoEffect(P_CHAR Caster, cCoord* MapTarg)
{
	Magic->AddMagicField(Caster,MapTarg,0x398c,0x3996,5,4+(Caster->Skill(MAGERY)/2),GetSpellnum());
}
void cGreaterHealSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cGreaterHealSpell::doEffect()");
}
void cLightningSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cLightningSpell::doEffect()");
}
void cManaDrainSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cManaDrainSpell::doEffect()");
}
void cRecallSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cRecallSpell::doEffect()");
}
// LVL 5 SPELLS
void cBladeSpiritSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cBladeSpiritSpell::doEffect()");
}
void cDispelFieldSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cDispelFieldSpell::doEffect()");
}
void cIncognitoSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cIncognitoSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cMagicReflectSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cMagicReflectSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cMindBlastSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cMindBlastSpell::doEffect()");
}
void cParalyzeSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cParalyzeSpell::doEffect()");
}
void cPoisonFieldSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cPoisonFieldSpell::doEffect()");
}
void cSummonCreatureSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cSummonCreatureSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
// LVL 6 SPELLS
void cDispelSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cDispelSpell::doEffect()");
}
void cEnergyBoltSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cEnergyBoltSpell::doEffect()");
}
void cExplosionSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cExplosionSpell::doEffect()");
}
void cInvisibilitySpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cInvisibilitySpell::doEffect()");
}
void cMarkSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cMarkSpell::doEffect()");
}
void cMassCurseSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cMassCurseSpell::doEffect()");
}
void cParalyzeFieldSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cParalyzeFieldSpell::doEffect()");
}
void cRevealSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cRevealSpell::doEffect()");
}
// LVL 7 SPELLS
void cChainLightningSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cChainLightningSpell::doEffect()");
}
void cEnergyFieldSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cEnergyFieldSpell::doEffect()");
}
void cFlamestrikeSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cFlamestrikeSpell::doEffect()");
}
void cGateTravelSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cGateTravelSpell::doEffect()");
}
void cManaVampireSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cManaVampireSpell::doEffect()");
}
void cMassDispelSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cMassDispelSpell::doEffect()");
}
void cMeteorSwarmSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cMeteorSwarmSpell::doEffect()");
}
void cPolymorphSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cPolymorphSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
// LVL 8 SPELLS
void cEarthquakeSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cEarthquakeSpell::doEffect()");

	// MASS SPELL: EFFECT WILL BE CHANGED WHEN SPELL FUNCTION FINNISHED
 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cEnergyVortexSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cEnergyVortexSpell::doEffect()");
}
void cResurrectionSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cResurrectionSpell::doEffect()");
}
void cAirElementalSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cAirElementalSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cEarthElementalSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cEarthElementalSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cWaterElementalSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cWaterElementalSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cFireElementalSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cFireElementalSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cSummonDaemonSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cSummonDaemonSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
// END REGULAR SPELLS

// START NECRO SPELLS
void cAnimateDeadSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cAnimateDeadSpell::doEffect()");
}
void cBloodOathSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cBloodOathSpell::doEffect()");
}
void cCorpseSkinSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cCorpseSkinSpell::doEffect()");
}
void cCurseWeaponSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cCurseWeaponSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cEvilOmenSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cEvilOmenSpell::doEffect()");
}
void cHorrificBeastSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cHorrificBeastSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cLichFormSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cLichFormSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cMindRotSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cMindRotSpell::doEffect()");
}
void cPainSpikeSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cPainSpikeSpell::doEffect()");
}
void cPoisonStrikeSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cPoisonStrikeSpell::doEffect()");
}
void cStrangleSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cStrangleSpell::doEffect()");
}
void cSummonFamiliarSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cSummonFamiliarSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cVampiricEmbraceSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cVampiricEmbraceSpell::doEffect()");
}
void cVengefulSpiritSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cVengefulSpiritSpell::doEffect()");
}
void cWitherSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cWitherSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cWraithFormSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cWraithFormSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
// END NECRO SPELLS

// START PALADIN SPELLS
void cCleanseByFireSpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg)
{
	//Heals the target for 7 to 39 points of damage. The caster's Karma affects the amount of damage healed.
	if (Caster->isInnocent() && !Caster->isSameAs(CharTarg))
	{
		if (CharTarg->crimflag>0 || CharTarg->isMurderer()) criminal(Caster);
	}
	float hp = (32.0f*((float)Caster->karma/10000.0f))+7.0f;
	CharTarg->increaseHp((int)hp);

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,CharTarg,NULL); // some visual effects on target and caster
}

void cCloseWoundsSpell::DoEffect(P_CHAR Caster, P_CHAR CharTarg)
{
	//Cures the target of poisons, but causes the caster to be burned by fire damage for 13-55 hit points.
	//The amount of fire damage is lessened if the caster has high Karma.
	//The Paladin casting this spell cannot be killed by it, but at most be reduced to 0 Hit Points.
	if (Caster->isInnocent() && !Caster->isSameAs(CharTarg))
	{
		if (CharTarg->crimflag>0 || CharTarg->isMurderer()) criminal(Caster);
	}

	CharTarg->PoisonWearOff(Caster->getSlot()); // cure target

	float damage = (42.0f*((float)Caster->karma/10000.0f))+13.0f; // calc damage done to caster
	if (damage>=(float)Caster->Hp()) damage = (float)Caster->Hp()-1.0f; // don't kill caster

	Caster->reduceHp((int)damage); // do damage to caster
	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,CharTarg,NULL); // some visual effects on target and caster
}

void cConsecrateWeaponSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cConsecrateWeaponSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}

void cDispelEvilSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cDispelEvilSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}

void cDivineFurySpell::DoEffect(P_CHAR Caster)
{
	//Temporarily increases the Paladin's swing speed, chance to hit, and damage dealt,
	//while lowering the Paladin's defenses. Upon casting, the Paladin's Stamina is also refreshed.
	//Duration of the spell is affected by caster's Karma. The effect lasts for 7 to 22 seconds.

	Caster->fullStm();	// refresh stamina

	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}

void cEnemyOfOneSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cEnemyOfOneSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}

void cHolyLightSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cHolyLightSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}

void cNobleSacrificeSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cNobleSacrificeSpell::doEffect()");

 	ParticleEffects->SpellCastingDestination(this->GetSpellnum(),Caster,Caster,NULL); // some visual effects on target and caster
}
void cRemoveCurseSpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cRemoveCurseSpell::doEffect()");
}
void cSacredJourneySpell::DoEffect(P_CHAR Caster)
{
	Caster->Msg("cSacredJourneySpell::doEffect()");
}
// END PALADIN SPELLS
