/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

/////////////////////
// Name:	sregions.cpp
// Purpose: functions that handle the spawn regions
// History:	cut from wolfpack.cpp by Duke, 26.10.00
// Remarks:	not necessarily ALL those functions
//
#include "sregions.h"

#include "mapS.h"
#include "mapstuff.h"
#include "npcinfo.h"
#include "scriptc.h"
#include "SndPkg.h"
#include "walking.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "sregions.cpp"

int locationcount;
unsigned int logoutcount;//Instalog

struct location_st
{
	cMapRectangle rect;
	unsigned short region;
};

struct logout_st// Instalog
{
	cMapRectangle rect;
};

CArray<4000, location_st>location("location");
CArray<1024, logout_st>logout("logout");	//Instalog areas

struct spawnregion_st// Regionspawns
{
	int srid;	// ID, section number in spawn.scp
	unsigned char name[512];// Any Name to show up when this region is spawned [512]
	int npclists[512];// NPC lists [512]
	int npcs[512];// Individual npcs [512]
	int itemlists[512];// item Lists [512]
	int totalitemlists;// Total number of item lists
	int totalnpcs;// Total Number of indiviual npcs
	int totalnpclists;// Number of NPC lists
	int max;// Max amount of spawned characters
	int current;// Current amount of spawned chars
	int mintime;// Minimum spawn time
	int maxtime;// Maximum spawn time
	unsigned int nexttime;// Nextspawn time for this region
	cMapRectangle rect;
};

cRespawn	*Respawn;
std::vector<spawnregion_st> spawnregion;


spawnregion_st* GetSpawnRegion(int r)
{
	if (r < 1) return NULL;
	unsigned int i;
	for (i=0;i<spawnregion.size();i++)
	{
		if (spawnregion[i].srid == r)
			return &spawnregion[i];
	}
	return NULL;
}

int NumberOfSpawnRegions()
{
	return spawnregion.size();
}

////////////
// History:	by Duke, 16.12.2002
// Purpose:	count the number of spawned at world load time
// Return:	0 if the given region was invalid.
// Remark:	it's unusual to have a return code for a function like this.
//			But this little trick avoids checking the validity of the region twice ;)
//
int IncSpawnCount(int r)
{
	spawnregion_st *SR = GetSpawnRegion(r);
	if (!SR) return 0;
	SR->current++;
	return r;
}

void DecSpawnCount(int r)
{
	if (r > 0)
	{
		spawnregion_st *SR = GetSpawnRegion(r);
		if (!SR) return;
		SR->current--;
	}
}

int GetSpawnDelta(spawnregion_st *SR)
{
	if (!SR) return 0;
	return SR->max - SR->current;
}

bool DoTimedSpawn()
{
	unsigned int i;
	for (i=0;i<spawnregion.size();i++)
	{
		if (spawnregion[i].nexttime<=uiCurrentTime)
		{
			spawnregion[i].nexttime=uiCurrentTime+(MY_CLOCKS_PER_SEC*60*RandomNum(spawnregion[i].mintime,spawnregion[i].maxtime));
			if (doregionspawn(&spawnregion[i])) return true;	// need to stop here to avoid allocation problems (Duke)
		}
	}
	return false;	// nothing to spawn now
}

int AllSpawnMax()
{
	int spawned=0;
	unsigned int i;
	for (i=0;i<spawnregion.size();i++)
	{
		int k = GetSpawnDelta(&spawnregion[i]);
		spawned += k;
		SpawnSomeMore(&spawnregion[i], k);
	}
	return spawned;
}

void SpawnSomeMore(spawnregion_st *SR, int amount)
{
	if (!SR) return;

	int j;
	for (j = 0; j < amount; j++)
	{
		doregionspawn(SR);
	}
	SR->nexttime = uiCurrentTime+(MY_CLOCKS_PER_SEC*60*RandomNum(SR->mintime,SR->maxtime));
}


static bool FindSpawnSpot(spawnregion_st *SR, cCoord& pos, P_CHAR Npc = NULL)
{
	if (!SR) return false;

	int a;
	for (a=0; a<100; a++)
	{
		pos.x=RandomNum(SR->rect.rect.lowerLeftX,SR->rect.rect.upperRightX);
		pos.y=RandomNum(SR->rect.rect.lowerLeftY,SR->rect.rect.upperRightY);
		pos.map=SR->rect.mapNo;
		pos.z=Map->MapElevation(pos);
		
		if (Npc)
		{
			if (validNPCMove(pos, pos.z, Npc))
			{
				return true;
			}
		}
		else
		{
			if (Map->CanMonsterMoveHere(pos, pos.z))
			{
				return true;
			}
		}
	}
	LogWarningVar("Problem regionspawn [%i] found. Nothing will be spawned.",SR->srid);
	return false;
}

// this is used to adjust the rectangle/circle for npcwander 2+3
// must be called before the teleport !! (Duke, 8.1.05)
void adjustFXY(P_CHAR pc, const cCoord& newpos)
{
	if (pc->npcWander == 2)		// circle
	{
		pc->fx1 += newpos.x - pc->pos.x;		// adjust center
		pc->fy1 += newpos.y - pc->pos.y;
		// but leave fx2 (radius) unchanged. fy2 irrelevant for circle
	}
	if (pc->npcWander == 3)		// rectangle
	{
		pc->fx1 += newpos.x - pc->pos.x;
		pc->fy1 += newpos.y - pc->pos.y;
		pc->fx2 += newpos.x - pc->pos.x;
		pc->fy2 += newpos.y - pc->pos.y;
	}
}

// Total rewrite by LB 21'th September 2000
// changed to bool. breaks after one spawn to allow for reallocation of memory (Duke, 15.4.2001)
// added 'balanced spawning' for mixed regions (Duke, 15.12.2002)
bool doregionspawn(spawnregion_st *SR)
{
	if (!SR) return false;
	if (SR->current<=0) SR->current=0;
	if (SR->current >= SR->max || SR->max == 0) return false;
	
	cCoord spos(110,110,5,0);	// spawn position
	int counter;
	
	int totaldraws = SR->totalnpclists + SR->totalitemlists + SR->totalnpcs;
	if (!totaldraws)
	{
		LogWarningVar("Nothing to spawn in region %d",SR->srid);
		return false;
	}
	int picked = rand()%totaldraws;		// decide what to spawn from
	totaldraws=0;

	totaldraws += SR->totalnpclists;
	if (totaldraws > picked)
	{
		if ( SR->totalnpclists > 0 )
		{
			counter = rand()%SR->totalnpclists;
			P_CHAR pc = Npcs->AddNPC(-1, SR->npclists[counter], spos);
			if (!pc) return false;
			if (FindSpawnSpot(SR, spos, pc))
			{
				adjustFXY(pc,spos);
				pc->teleTo(spos);
				SR->current++;
				pc->spawnregion=SR->srid;
				return true;
			}
			else
			{
				Npcs->DeleteChar(pc);
				return false;
			}
		}
	}
	
	totaldraws += SR->totalitemlists;
	if (totaldraws > picked)
	{
		if ( SR->totalitemlists > 0 )
		{
			counter = rand()%SR->totalitemlists;
			if (FindSpawnSpot(SR, spos))
			{
				int num = Items->PickFromItemlist(SR->itemlists[counter]);
				const P_ITEM pi = Items->CreateScriptItem(num);
				if (!pi) return false;
				pi->MoveTo(spos);
				pi->spawnregion=SR->srid;
				pi->refresh();
				SR->current++;
				return true;
			}
		}
	}
	
	totaldraws += SR->totalnpcs;
	if (totaldraws > picked)
	{
		if ( SR->totalnpcs>0 )
		{
			counter = rand()%SR->totalnpcs;
			P_CHAR pc = Npcs->AddNPC(-1, SR->npcs[counter], spos);
			if (!pc) return false;
			if (FindSpawnSpot(SR, spos, pc))
			{
				adjustFXY(pc,spos);
				pc->teleTo(spos);
				SR->current++;
				pc->spawnregion=SR->srid;
				return true;
			}
			else
			{
				Npcs->DeleteChar(pc);
				return false;
			}
		}
	}
	return false;
}

void loadspawnregions()//Regionspawns
{
	spawnregion_st dummy;

	spawnregion.clear();

	Script* Scp = getScript(spawn_script);
	Scp->checkModification();	// reload if necessary

	if (!Scp->Open())
	{
		LogWarning("spawn.scp not found");
		return;
	}
	int loopexit=0 ;
	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS))
	{
		if (!strcmp(script1, "SECTION") && script2[0]=='R')
		{
			int num=0;
			if (sscanf(script2,"REGIONSPAWN %i",&num) != 1)
				continue;
			memset(&dummy, 0, sizeof(dummy));
			dummy.srid = num;

			int loopexit2=0;
			while (Scp->NextLineSplitted() && (++loopexit2 < MAXLOOPS))
			{
				if (!strcmp("NLIST", script1))
				{
					dummy.npclists[dummy.totalnpclists]=str2num(script2);
					dummy.totalnpclists++;
				}
				else if (!strcmp("NPC",script1))
				{
					dummy.npcs[dummy.totalnpcs]=str2num(script2);
					dummy.totalnpcs++;
				}
				else if (!strcmp("ILIST",script1))
				{
					dummy.itemlists[dummy.totalitemlists]=str2num(script2);
					dummy.totalitemlists++;
				}
				else if (!strcmp("MAX", script1)) dummy.max=str2num(script2);
				else if (!strcmp("MINTIME", script1)) dummy.mintime=str2num(script2);
				else if (!strcmp("MAXTIME", script1)) dummy.maxtime=str2num(script2);
				else if (Scp->getKeyVal("MRECT",&dummy.rect)) continue;
				else if (!strcmp("X1", script1)) dummy.rect.rect.lowerLeftX=str2num(script2);
				else if (!strcmp("X2", script1)) dummy.rect.rect.upperRightX=str2num(script2);
				else if (!strcmp("Y1", script1)) dummy.rect.rect.lowerLeftY=str2num(script2);
				else if (!strcmp("Y2", script1)) dummy.rect.rect.upperRightY=str2num(script2);			
			}
			if (num != 0)
				spawnregion.push_back(dummy);
		}
	}
	Scp->Close();
	LogMessageVar("%i spawn regions loaded from script.",spawnregion.size());


	// in case of reloading the script, we need to count the chars/items that are already spawned (Duke, 30.8.03)
	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++)
	{
		IncSpawnCount(pc->spawnregion);
	}

	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
	{
		IncSpawnCount(pi->spawnregion);
	}
}

static void loadpredefspawnregion(char *name)//Load predefined spawnregion //Regionspawns
{
	Script* Scp = getScript(spawn_script);
	if (!Scp->Open())
	{
		LogWarning("spawn.scp not found");
		return;
	}
	char sect[512];
	sprintf(sect, "PREDEFINED_SPAWN %s", name);
	if (!Scp->find(sect))
	{
		LogWarningVar("Undefined region spawn %s, check your regions.scp and spawn.scp files.",name);
		return;
	}
	spawnregion_st dummy;
	memset(&dummy, 0, sizeof(spawnregion_st));
	int loopexit=0;
	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS))
	{
		if (!strcmp(script1, "NLIST"))
		{
			dummy.npclists[dummy.totalnpclists]=str2num(script2);
			dummy.totalnpclists++;
		}
		if (!strcmp("NPC",script1))
		{
			dummy.npcs[dummy.totalnpcs]=str2num(script2);
			dummy.totalnpcs++;
		}
		if (!strcmp("ILIST",script1))
		{
			dummy.itemlists[dummy.totalitemlists]=str2num(script2);
			dummy.totalitemlists++;
		}
		if (!strcmp("MINTIME", script1)) dummy.mintime=str2num(script2);
		if (!strcmp("MAXTIME", script1)) dummy.maxtime=str2num(script2);
		if (!strcmp("MAX", script1)) dummy.max=str2num(script2);
	}
	spawnregion.push_back(dummy);
	Scp->Close();
	Console.send("LoneWolf: %s loaded into spawn region #%i.\n",sect,NumberOfSpawnRegions());
}

void applyGuardset(int r, int set)
{
	Script* Scp = findSection(regions_script,"GUARDSET",set);
	if (!Scp) {LogWarningVar("GUARDSET %i not found !",set); return;}

	int n = 0;
	int	loopexit=0;
	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS) )
	{
		int gn;
		if (Scp->getKeyVal("GUARDNUM",&gn)) // script # of guards
		{
			if (n < 10)
			{
				region[r].guardnum[n] = gn;
				n++;
			}
			else
			{
				LogWarningVar("Region %i has more than 10 'GUARDNUM', The ones after 10 will not be used",r);
			}
		}
	}
	Scp->Close();
}

void applySpellset(int r, int set)
{
	Script* Scp = findSection(regions_script,"SPELLSET",set);
	if (!Scp) {LogWarningVar("SPELLSET %i not found !",set); return;}

	int	loopexit=0;
	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS) )
	{
		int sn;
		if (Scp->getKeyVal("SPELLALLOWED",&sn)) // individual spell # enabled
		{
			region[r].setSpellAllowed(sn);
		}
		else
			Scp->invalidTag();
	}
	Scp->Close();
}

void loadregions()//New -- Zippy spawn regions
{
	int i, noregion, l=0, a=0,loopexit=0;
	//int rnd1=0,rnd2=0,rnd3=0;
	char sect[512];
	int actgood=-1; // Magius(CHE)

	Script* Scp=getScript(regions_script);

	//cMapRectangle theRectangle;

	bool rl;
	for (i=0;i<MAXREGIONS;i++)
	{
		rl=0;
		region[i].midilist=0;
		region[i].ResetPriv();
		region[i].snowchance=0;
		region[i].rainchance=0;
		region[i].name[0]=0;
		region[i].brightlevel=-2;
		region[i].darklevel=-2;
		region[i].fixedlight=-2;
		region[i].dungeonlevel=-2;
		region[i].curlight=-2;
		region[i].dungeon=255;	
		
		region[i].arena=false;		// by default: no region is an arena

		noregion=0;
		for (a=0;a<10;a++)
		{
			region[i].guardnum[a]=RandomNum(1000,1001);
		}
		for (a=0;a<256;a++)		// added by Magius(CHE)
		{
			region[i].goodsell[a]=0;
			region[i].goodbuy[a]=0;
			region[i].goodrnd1[a]=0;
			region[i].goodrnd2[a]=0;
		}
		a=0;		// end added by Magius(CHE)

		if (!Scp->Open()) return;
		sprintf(sect, "REGION %i", i);
		if (!Scp->find(sect)) 
		{
			noregion=1;
		}

		loopexit=0;
		while (Scp->NextLineSplitted() && !noregion && (++loopexit < MAXLOOPS) )
		{
			if (!strcmp("GUARDNUM",script1))
			{
				if (a<10)
				{
					region[i].guardnum[a]=str2num(script2);
					a++;
				}
				else
				{
					LogWarningVar("Region %i has more than 10 'GUARDNUM', The ones after 10 will not be used",i);
				}
			}
			if (!strcmp("GUARDSET",script1))
			{
				scpMark m; Scp->Suspend(m);
				applyGuardset(i,str2num(script2));
				Scp->Resume(m);
			}
			if (!strcmp("NAME",script1))
			{
				strcpy(region[i].name,script2);
				actgood=-1; // Magius(CHE)
			}
			// Dupois - Added April 5, 1999
			// To identify whether this region is escortable or not.
			else if (!strcmp("ESCORTS",script1))
			{
				// Load the region number in the global array of valid escortable regions
				if ( str2num(script2) == 1 )
				{
					// Store the region index into the valid escort region array
					validEscortRegion[escortRegions] = i;
					escortRegions++;
				}
			} // End - Dupois
			if (!strcmp("GUARDOWNER",script1))		strcpy(region[i].guardowner,script2);
			if (!strcmp("MIDILIST",script1))		region[i].midilist=str2num(script2);
			if (!strcmp("GUARDED",script1))
			{
				if (str2num(script2)) region[i].setGuarded();
			}
			
			////////// MAGERY RELATED STUFF //////////////
			
			
			if (!strcmp("MAGICDAMAGE",script1)) // no damage done by any spell.
			{
				if (str2num(script2)) region[i].setMagicDamage(); 	
			}
			
			if (!strcmp("RUNEBOOK",script1)) // runebooks enabled/disabled
			{
				if (str2num(script2)) region[i].setRuneBookAllowed();
			}
			
			if (!strcmp("SPELLALLOWED",script1)) // individual spell # enabled/disabled
			{
				char sn = (str2num(script2));
				region[i].setSpellAllowed(sn);
			}
			
			if (!strcmp("SPELLSET",script1))
			{
				scpMark m; Scp->Suspend(m);
				applySpellset(i,str2num(script2));
				Scp->Resume(m);
			}
			
			if (!strcmp("ALLSPELLSALLOWED",script1)) // all spells allowed
			{
				int a;
				for (a=1;a<MAXSPELLS+1;++a) region[i].setSpellAllowed(a);
			}
			
			if (!strcmp("ALLREGULARSPELLSALLOWED",script1)) // all spells allowed
			{
				int a;
				for (a=1;a<65;++a) region[i].setSpellAllowed(a);
			}
			
			if (!strcmp("ALLNECROSPELLSALLOWED",script1)) // all spells allowed
			{
				int a;
				for (a=101;a<117;++a) region[i].setSpellAllowed(a);
			}
			
			if (!strcmp("ALLPALADINSPELLSALLOWED",script1)) // all spells allowed
			{
				int a;
				for (a=201;a<211;++a) region[i].setSpellAllowed(a);
			}
			
			///////////////////////////////////////////////
			
			///////////////////////////////////////////////
			////////     ARENA STUFF (THRAWN)     /////////
			///////////////////////////////////////////////
			if (!strcmp("ARENA",script1)) // make the region an arena
			{
				region[i].setArena();
			}
			
			if (!strcmp("ARENAEXIT",script1)) // define the exit coords
			{
				cline = &script2[0];
				splitline();
				region[i].arenaexit.x = makenumber(0);
				region[i].arenaexit.y = makenumber(1);
				region[i].arenaexit.z = makenumber(2);
				region[i].arenaexit.map = makenumber(3);
			}
			///////////////////////////////////////////////
			
#if 0
			if (!strcmp("SNOWCHANCE", script1))
			{
				gettokennum(script2, 0);
				rnd1=str2num(gettokenstr);
				gettokennum(script2, 1);
				rnd2=str2num(gettokenstr);
				rnd3=RandomNum(rnd1,rnd2);
				
				region[i].snowchance=str2num(script2);
			}
			if (!strcmp("RAINCHANCE", script1))
			{
				gettokennum(script2, 0);
				rnd1=str2num(gettokenstr);
				gettokennum(script2, 1);
				rnd2=str2num(gettokenstr);
				rnd3=RandomNum(rnd1,rnd2);
				
				region[i].rainchance=str2num(script2);
			}
			if (!strcmp("WEATHDURATION",script1)))
			{
				gettokennum(script2, 0);
				rnd1=str2num(gettokenstr);
				gettokennum(script2, 1);
				rnd2=str2num(gettokenstr);
				rnd3=RandomNum(rnd1,rnd2);
			}
			if (!strcmp("WEATHSTART",script1)))
			{
				if (str2num(script2)>0)
				{
					gettokennum(script2, 0);
					rnd1=str2num(gettokenstr);
					gettokennum(script2, 1);
					rnd2=str2num(gettokenstr);
					rnd3=RandomNum(rnd1,rnd2);
				}
				else
				{
					
				}
			}
#endif
			
			if (!strcmp("GOOD", script1)) // Magius(CHE)
			{			
				actgood=str2num(script2);
				if (actgood>=256) 
				{
					LogWarning("regions.scp: Actgood value >=256. Correct it, might cause crashes!");
				}
			}
			if (!strcmp("BUYABLE", script1)) // Magius(CHE)
			{
				if (actgood>-1) region[i].goodbuy[actgood]=str2num(script2);
				else Console.send("ERROR in regions.scp. You must write BUYABLE after GOOD <num>!\n");
			}
			if (!strcmp("SELLABLE", script1)) // Magius(CHE)
			{
				if (actgood>-1) region[i].goodsell[actgood]=str2num(script2);
				else Console.send("ERROR in regions.scp. You must write SELLABLE after GOOD <num>!\n");
			}
			if (!strcmp("RANDOMVALUE", script1)) // Magius(CHE) (2)
			{
				if (actgood>-1)
				{
					gettokennum(script2, 0);
					region[i].goodrnd1[actgood]=str2num(gettokenstr);
					gettokennum(script2, 1);
					region[i].goodrnd2[actgood]=str2num(gettokenstr);
					if (region[i].goodrnd2[actgood]<region[i].goodrnd1[actgood])
					{
						Console.send("ERROR in regions.scp. You must write RANDOMVALUE NUM2[%i] grater than NUM1[%i].\n",region[i].goodrnd2[actgood],region[i].goodrnd1[actgood]);
						region[i].goodrnd2[actgood]=region[i].goodrnd1[actgood]=0;
					}
				}
				else Console.send("ERROR in regions.scp. You must write RANDOMVALUE after GOOD <num>!\n");
			}
		

			bool newscpt=false;
			if (Scp->getKeyVal("MRECT",&location[l].rect)) 
			{				
				location[l].region=i;
				l++;	
				newscpt=true;				

			} 

			if (!newscpt)
			{
			  if (!strcmp("X1", script1))
			  {
				location[l].rect.rect.lowerLeftX=str2num(script2);
			  }
			  if (!strcmp("X2", script1))
			  {
				location[l].rect.rect.upperRightX=str2num(script2);
			  }
			  if (!strcmp("Y1", script1))
			  {
				location[l].rect.rect.lowerLeftY=str2num(script2);
			  }
			  if (!strcmp("Y2", script1))
			  {
				location[l].rect.rect.upperRightY=str2num(script2);
				location[l].region=i;
				l++;
				LogWarning("X1-Y2 tags are depricated in section REGION. Use MRECT instead !");	// Duke, 30.3.2006
			  }
			}

			if (!strcmp("SPAWN", script1))
			{
				scpMark m; Scp->Suspend(m);
				loadpredefspawnregion(script2);
				Scp->Resume(m);
				
				spawnregion_st *SR = GetSpawnRegion(NumberOfSpawnRegions());

				SR->rect=location[l-1].rect;		
				
				strcpy(script1, "SOMECRAZYREGION"); // To prevent accidental exit of loop.
			}
			
			
			if (!strcmp("REGIONBRIGHTLEVEL", script1))
			{
				region[i].brightlevel=str2num(script2);
				rl=1;
			}
			
			if (!strcmp("REGIONDARKLEVEL", script1))
			{
				region[i].darklevel=str2num(script2);
				rl=1;
			}
			
			if (!strcmp("REGIONFIXEDLEVEL", script1))
			{
				region[i].fixedlight=str2num(script2);
				rl=1;
			}
			
			if (!strcmp("REGIONDUNGEONLIGHTLEVEL", script1))
			{
				region[i].dungeonlevel=str2num(script2);
				rl=1;
			}
			
			if (!strcmp("DUNGEON", script1))
			{
				region[i].dungeon=str2num(script2);				  
			}
		}
		Scp->Close();

		if ( (region[i].brightlevel==-2 || region[i].darklevel==-2 || region[i].fixedlight==-2 || region[i].dungeonlevel==-2) && (rl==1))
		{
			sprintf(temp,"Not All region light values set for region:%s, global values will be used\n",region[i].name);
			LogWarning(temp);
		}		
	}

	locationcount=l;
	logoutcount=0;
	//Instalog
	if (!Scp->Open()) return;
	strcpy(sect, "INSTALOG");
	if (!Scp->find(sect)) 
	{
		Scp->Close();
		return;
	}

	loopexit=0;
	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS))
	{
		if (Scp->getKeyVal("MRECT",&logout[logoutcount].rect)) 
		{	
			logoutcount++; 
			continue;		
		}
		else
		{		
			if (!strcmp(script1,"X1")) logout[logoutcount].rect.rect.lowerLeftX=str2num(script2);
			if (!strcmp(script1,"Y1")) logout[logoutcount].rect.rect.lowerLeftY=str2num(script2);
			if (!strcmp(script1,"X2")) logout[logoutcount].rect.rect.upperRightX=str2num(script2);
			if (!strcmp(script1,"Y2"))
			{
				logout[logoutcount].rect.rect.upperRightY=str2num(script2);
				logoutcount++;
				LogWarning("X1-Y2 tags are depricated in section INSTALOG. Use MRECT instead !");	// Duke, 30.3.2006
			}
		}
	}
	Scp->Close();
}

short calcRegionFromXY(const cCoord& pos)
{
	int i;

	for (i=0;i<locationcount;i++)
	{
		if (location[i].rect.includes(pos))
			return location[i].region;
/*		if (pos.map == location[i].rect.mapNo)
		{
			//printf("cr:p:%i %i %i r: %i %i %i %i  %i\n", pos.x, pos.y, pos.map, location[i].rect.rect.lowerLeftX, location[i].rect.rect.lowerLeftY, location[i].rect.rect.upperRightX, location[i].rect.rect.upperRightY, location[i].rect.mapNo);
		    if (location[i].rect.rect.lowerLeftX<=pos.x && location[i].rect.rect.lowerLeftY<=pos.y && location[i].rect.rect.upperRightX>=pos.x &&
			location[i].rect.rect.upperRightY>=pos.y)
			{			   
			   return location[i].region;
			}
		}*/
	}
	
	int dummy=MAXREGIONS;
	return dummy-1;
}

bool IsInInstantLogoutArea(const cCoord& pos)
{
	unsigned int a;
	for(a=0;a<logoutcount;a++)
	{
		if (pos.IsIn(logout[a].rect))
			return true;
	}
	return false;
}

void checkregion(P_CHAR pc)
{
	////////////////////////////////////////
	/// "UNKOWNDIMENSION sysmessages"
	//// using color (!=0) is essential within entire function 
	///  if not 3d clients >=3.0.6g die
	///  pc->Msg("xxx") does use color 0 !
	//// you can change 0x37 to your liking, but not to 0
	/////////////////////////////////////////////////////////////////////
	if (!pc) return;
	int calcreg=calcRegionFromXY(pc->pos);
	if (calcreg!=pc->region)
	{
		int s = pc->getSlot();
		if (s!=-1)
		{
			if (strcmp(region[pc->region].name,region[calcreg].name))	// only if the names are different
			{
				if (region[pc->region].name[0]!=0)
				{
					sprintf(temp, "You have left %s.", region[pc->region].name);
					pc->Msg(0x37,  temp);
				}
				if (region[calcreg].name[0]!=0)
				{
					sprintf(temp, "You have entered %s.", region[calcreg].name);
					pc->Msg(0x37, temp);
				}
			}
			int j=strcmp(region[calcreg].guardowner, region[pc->region].guardowner);
			if ( (region[calcreg].isGuarded())!=(region[pc->region].isGuarded()) ||
				(region[calcreg].isGuarded() && j))
			{
				if (region[calcreg].isGuarded())
				{
					if (region[calcreg].guardowner[0]==0)
					{
						pc->Msg(0x37, "You are now under the protection of the guards.");
					}
					else
					{
						sprintf(temp, "You are now under the protection of %s guards.", region[calcreg].guardowner);
						pc->Msg(0x37, (char*) temp);
					}
				}
				else
				{
					if (region[pc->region].guardowner[0]==0)
					{
						pc->Msg(0x37, "You are no longer under the protection of the guards.");
					}
					else
					{
						sprintf(temp, "You are no longer under the protection of %s guards.", region[pc->region].guardowner);
						pc->Msg(0x37, temp);
					}
				}
			}
		}
		pc->region=calcreg;
		if (s!=-1) 
		{ 
			dosocketmidi(MakePlayerRef(pc));
			dolight(s);
		}
	}
}

///////////////////
// Name:	Continue()
// Purpose:	handles one region/spawner per call, then takes a break
//			to allow for memory reallocation and to avoid lag
// History:	by Duke, 17.12.2000
//
void cRespawn::Continue()
{
	unsigned int i;
	for( i = currentSpawnRegion; i < (unsigned) NumberOfSpawnRegions(); i++) //New -- Zippy region spawner
	{
		if (doregionspawn(&spawnregion[i]) == false)	// no more things to spawn in this region
			currentSpawnRegion++;
		return;			// take a break after each region 
	}

	PC_ITEM pi;
	for ( ; (pi = currentSpawnItem.GetData()) != currentSpawnItem.End(); currentSpawnItem++)
	{
		if (!pi->isInWorld()) continue;

		int k;
		if (pi->isType(ITY_ItemSpawner))
		{
			k = pi->CountSpawnedItems();
			if (k==0)
			{
				Items->AddRespawnItem(pi,pi->morex,0);
				currentSpawnItem++;	// item spawners can only spawn *one* item anyway, so proceed
				return;			// take a break
			}
		}

		if (pi->isType(ITY_NpcSpawner) || pi->isType(ITY_NpcAreaSpawner) || pi->isType(ITY_QuestSpawn))	// NPC Spawner / Escort Quest spawner(125)
		{
			k=pi->CountSpawnedChars();

			if (k<pi->amount)	// lord binary, adds spawn amount checking
			{
				int olditemcount=Items->GetCount();
				if (!Npcs->AddRespawnNPC(pi,pi->morex))
				{
					currentSpawnItem++;		// if we fail to spawn, don't try again ;)
					continue;
				}
				int delta = Items->GetCount() - olditemcount;
				if (delta>300)
				{
					char ttt[512];
					sprintf(ttt,"script npc %d produced %d items. Current limit is 300 !", pi->morex,delta);
					LogWarning(ttt);
				}
				pi->unconst()->gatetime=0;
				return;			// take a break
			}
		}
	}
	sysbroadcast("Respawn now complete.");
	respawning=false;	// if we get here, the respawning has finished
}

bool findLocation(int i, cCoord& pos)
{
	int x = -1, y = 0, z = 0, loopexit = 0;

	Script* Scp = findSection(location_script,"LOCATION",i);
	if (!Scp)
	{
		return false;
	}

	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS) )
	{
		if (Scp->getKeyVal("POS", pos))	continue;
		if (Scp->cmpTag("NAME"))	continue;
		if (Scp->cmpTag("CAT0"))	continue;
		Scp->invalidTag();	// the other tags are deprecated, log warning
		if (Scp->getKeyVal("X",	&x))	continue;
		if (Scp->getKeyVal("Y",	&y))	continue;
		if (Scp->getKeyVal("Z",	&z))	continue;
	}

	if (x!=-1)	// they did not use the POS tag
	{
		pos.x = x;
		pos.y = y;
		pos.z = z;
		pos.map = AllMaps->getdefaultmap();
	}

	Scp->Close();
	return true;
}

void InitJails()
{
/*	jails[1].pos = cCoord(5276, 1164, 0); // Jail1
	jails[1].occupied = false;

	jails[2].pos = cCoord(5286, 1164, 0); // Jail2
	jails[2].occupied = false;

	jails[3].pos = cCoord(5296, 1164, 0); // Jail3
	jails[3].occupied = false;

	jails[4].pos = cCoord(5306, 1164, 0); // Jail4
	jails[4].occupied = false;

	jails[5].pos = cCoord(5276, 1174, 0); // Jail5
	jails[5].occupied = false;

	jails[6].pos = cCoord(5286, 1174, 0); // Jail6
	jails[6].occupied = false;

	jails[7].pos = cCoord(5296, 1174, 0); // Jail7
	jails[7].occupied = false;

	jails[8].pos = cCoord(5306, 1174, 0); // Jail8
	jails[8].occupied = false;

	jails[9].pos = cCoord(5283, 1184, 0); // Jail9
	jails[9].occupied = false;

	jails[10].pos = cCoord(5304, 1184, 0); // Jail10
	jails[10].occupied = false;*/

	int i;
	for (i = 1; i < 11; i++) 
	{
		if (findLocation(400+i, jails[i].pos))
			jails[i].occupied = false;
		else
		{
			jails[i].pos = cCoord(5276, 1164, 0, 0); // Jail1 in Britannia
			jails[i].occupied = true;
		}
	}
}

void region_st::DeathInArena(P_PLAYER Player)
{
	// fully refresh player
	Player->fullMana();
	Player->fullHp();
	Player->fullStm();

	// do some effects and teleport the player to arenaexit
	staticeffect(Player,0x375A,0x09,0x1E,0);
	soundeffect2(Player,0x01EE);
	Player->travelTo(arenaexit);
	staticeffect(Player,0x375A,0x09,0x1E,0);
}

