/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "targeting.h"

#include "basics.h"
#include "boats.h"
#include "Client.h"
#include "commands.h"
#include "guilds.h"
#include "gumps.h"
#include "house.h"
#include "itemid.h"
#include "LoS.h"
#include "magic.h"
#include "magic_new.h"
#include "mapstuff.h"
#include "necro.h"
#include "network.h"
#include "regions.h"
#include "scriptc.h"
#include "skills.h"
#include "SndPkg.h"
#include "TmpEff.h"
#include "trigger.h"
#include "weight.h"
#include "globals.h"
#include "mapS.h"
#include "tiledata.h"
#include "Partysystem.h"
#include "npcinfo.h"
#include "LwMisc.h"
#include "spells.h"


#include "wolfpack.h"

#include "debug.h"
#undef DBGFILE
#define DBGFILE "targeting.cpp"

class cPacket6C
{
public:
//Clicking Commands (19 bytes) 
//* BYTE cmd 
//* BYTE type 
//* 0x00 = server set cursor to pointer 
//* 0x01 = client return pointer position 
//* BYTE[4] charID
	short Tnum;		// we only use the low byte anyway, so short
//* BYTE Cursor Type 
//* 0x00 - Select Object 
//* 0x01 - Choose x, y, z 
//* The following are always sent but are only valid if sent by client 
//* BYTE[4] Clicked On ID
	long Tserial;
//* BYTE[2] click xLoc
	short TxLoc;
//* BYTE[2] click yLoc 
	short TyLoc;
//* BYTE unknown2 (0x00) 
//* BYTE click zLoc
	signed char TzLoc;
//* BYTE[2] model # (if a static tile, 0 if a map/landscape tile)
	short model;
//Note: the model # and charID should NEVER be trusted.

	cPacket6C(unsigned char *pbuf)
	{
		Tnum=ShortFromCharPtr(pbuf+4); // changed from byte to short ... actually we have 4 bytes, but 65kb ought to be enough ... LB
		Tserial=LongFromCharPtr(pbuf+7);
		TxLoc=ShortFromCharPtr(pbuf+11);
		TyLoc=ShortFromCharPtr(pbuf+13);
		TzLoc=*(pbuf+16);
		model=ShortFromCharPtr(pbuf+17);
	}
};
	
///////////////////
// class:	cTarget
// purpose:	base class of all Target classes
//			encapsulates the basic functions of target processing
// remarks:	intended to become a member of the 'transaction' class in the future
//			the cTargets::multitarget shall become the ctarget::factory
//			Duke, 07/16/00
//			Changed my mind. These classes are now considered 'experimental'
//			Duke, 7.11.2000
//
class cTarget
{
protected:
	P_PLAYER _Player;
	int serial;
	P_ITEM pi;
	P_CHAR pc;
	void makeSerial()		{serial=LongFromCharPtr(&buffer[_Player->cs][7]);}
	void FindTargetChar()	{pc=FindCharBySerial(serial);}
	void FindTargetItem()	{pi=FindItemBySerial(serial);}
public:
	cTarget(P_PLAYER Player)	{_Player = Player;}
	virtual void process() = 0;
};

class cCharTarget : public cTarget
{
public:
	cCharTarget(P_PLAYER Player) : cTarget(Player) {}
	virtual void CharSpecific(P_CHAR pc) = 0;
	virtual void process()
	{
		makeSerial();
		FindTargetChar();
		if (pc)
			CharSpecific(pc);
		else
			_Player->Msg("That is not a character.");
	}
};

class cItemTarget : public cTarget
{
public:
	cItemTarget(P_PLAYER Player) : cTarget(Player) {}
	virtual void ItemSpecific() = 0;
	virtual void process()
	{
		makeSerial();
		FindTargetItem();
		if (pi)
			ItemSpecific();
		else
			_Player->Msg("That is not an item.");
	}
};

class cWpObjTarget : public cTarget
{
public:
	cWpObjTarget(P_PLAYER Player) : cTarget(Player) {}
	virtual void CharSpecific(P_CHAR pc) = 0;
	virtual void ItemSpecific() = 0;
	virtual void process()
	{
		makeSerial();
		if (isItemSerial(serial)) // an item's serial ?
		{
			FindTargetItem();
			if (pi)
				ItemSpecific();
			else
				_Player->Msg("That is not a valid item.");
		}
		else
		{
			FindTargetChar();
			if (pc)
				CharSpecific(pc);
			else
				_Player->Msg("That is not a valid character.");
		}
	}
};


void cTargets::PlVBuy(P_PLAYER Player, P_ITEM pi)//PlayerVendors
{
	if (!Player) return;
	UOXSOCKET s = Player->cs;

	P_CHAR Vendor = FindCharBySerial(addx[s]);
	if (!Vendor) return;

	P_ITEM Pack = Player->getBackpack();
	if (!Pack) {Player->Msg("Time to buy a backpack"); return; } //LB

	if (!pi) return;
	if (pi->isInWorld()) return;

	P_ITEM SellCont=FindItemBySerial(pi->contserial);	// the pack
	P_CHAR Vendor2=GetPackOwner(SellCont);				// the vendor
	if (!Vendor2) return;
	if (!Vendor2->isSameAs(Vendor) || !Vendor->isAiVendor()) return;

	if (Player->Owns(Vendor))
	{
		npctalk(s, Vendor, "I work for you, you need not buy things from me!");
		return;
	}

	if (!Player->canAfford(pi->value))
	{
		npctalk(s, Vendor, "You cannot afford that.");
		return;
	}
	Player->pay(pi->value);	// take gold from player

	npctalk(s, Vendor, "Thank you.");
	Vendor->holdg+=pi->value; // putting the gold to the vendor's "pocket"

	// sends item to the proud new owner's pack
	pi->SetContSerial(Pack->serial);
	pi->refresh();

}

void cTargets::BanTarg(int s,P_CHAR pc)
{
	if (!pc) return;
	if (!pc->npc)
		Accounts->SetBan(pc->account, true);
}

void cTargets::UnBanTarg(int s,P_CHAR pc)
{
	if (!pc) return;
	if (!pc->npc)
		Accounts->SetBan(pc->account, false);
}

////////////////
// name:		addtarget
// history:		UnKnown (Touched tabstops by Tauriel Dec 29, 1998)
// Purpose:		Adds an item when using /add # # .
//
static void AddTarget(P_PLAYER Player, cPacket6C *pp)
{
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;
	UOXSOCKET s = Player->cs;

	if (addid1[s]>=0x40) // LB 25-dec-199, changed == to >= for chest multis, probably not really necassairy
	{
		switch (addid2[s])
		{
		case 100:
		case 102:
		case 104:
		case 106:
		case 108:
		case 110:
		case 112:
		case 114:
		case 116:
		case 118:
		case 120:
		case 122:
		case 124:
		case 126:
		case 140:
			buildHousePass2(Player,addx[s]);//If its a valid house, send it to buildhouse!
			return; // Morrolan, here we WANT fall-thru, don't mess with this switch
		}
	}

	short id=(addid1[s]<<8)+addid2[s];

	cCoord tpos(pp->TxLoc,pp->TyLoc,pp->TzLoc+Map->TileHeight(pp->model),Player->pos.map);
	P_ITEM pi=Items->SpawnItem(ITY_None, 1, "#", id, 0,tpos);
	if (!pi) return;
	pi->resetPriv();	//Make them not decay

	addid1[s]=0;
	addid2[s]=0;
}

class cRenameTarget : public cWpObjTarget
{
public:
	cRenameTarget(P_PLAYER Player) : cWpObjTarget(Player) {}
	void CharSpecific(P_CHAR pc)
	{
		strcpy(pc->name,xtext[_Player->cs]);
	}
	void ItemSpecific()
	{
		UOXSOCKET s = _Player->cs;
		if (addx[s]==1) //rename2 //New -- Zippy
			strcpy(pi->name2,xtext[s]);
		else
			strcpy(pi->name,xtext[s]);
	}
};

//we don't need this anymore - AntiChrist (9/99)
// hehe, yes we do
// what about the /tele command ???
// modified for command only purpose and uncommented by LB...

// Changed to have a gmmove effect at origin and at target point 
// Aldur
// 
static void TeleTarget(P_PLAYER Player, cPacket6C *pp) 
{ 
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;

	int x=pp->TxLoc; 
	int y=pp->TyLoc; 
	signed char z=pp->TzLoc; 
	cCoord newPos(x,y,z+Map->TileHeight(pp->model),Player->pos.map);	

	if (x<0 || y<0) return;

	UOXSOCKET s = Player->cs;
	if (line_of_sight(Player->pos, (cCoord&) newPos) || Player->isGM()) 
	{ 
		doGmMoveEff(s); 

		Player->teleTo(newPos); 

		doGmMoveEff(s); 
	} 
}

class cRemoveTarget : public cWpObjTarget
{
public:
	cRemoveTarget(P_PLAYER Player) : cWpObjTarget(Player) {}
	void CharSpecific(P_CHAR pc)
	{
		if (pc->account>-1 && !pc->npc) // player check added by LB
		{
			_Player->Msg("You cant delete players");
			return;
		}
		_Player->Msg("Removing character.");
		Npcs->DeleteChar(pc);
	}
	void ItemSpecific()
	{
		_Player->Msg("Removing item.");
		Items->DeleItem(pi);
	}
};

void DyeTarget(P_PLAYER Player, int serial)
{
	UOXSOCKET s = Player->cs;
	unsigned short color = ((addid1[s])<<8)+addid2[s];

	if (color == 0xFFFF)	// color wasn't give with the cmd, so send menu
	{
		P_ITEM pi=FindItemBySerial(serial);
		if (pi!=NULL)
		{
			SndDyevat(s,pi->serial, pi->id());
			pi->refresh();
		}
		P_CHAR pc = FindCharBySerial(serial);
		if (pc)
		{
			SndDyevat(s,pc->serial,0x2106);
		}
	}
	else
	{
		Commands->DyeItem(Player,serial,color);
	}
}

class cNewzTarget : public cWpObjTarget
{
public:
	cNewzTarget(P_PLAYER Player) : cWpObjTarget(Player) {}
	void CharSpecific(P_CHAR pc)
	{
		pc->dispz=pc->pos.z=addx[_Player->cs];
		updatechar(pc);
	}
	void ItemSpecific()
	{
		pi->pos.z=addx[_Player->cs];
		pi->refresh();
	}
};

class cInczTarget : public cWpObjTarget
{
public:
	cInczTarget(P_PLAYER Player) : cWpObjTarget(Player) {}
	void CharSpecific(P_CHAR pc)
	{
		pc->dispz=pc->pos.z=addx[_Player->cs]+pc->pos.z;
		updatechar(pc);
	}
	void ItemSpecific()
	{
		pi->pos.z=addx[_Player->cs]+pi->pos.z;
		pi->refresh();
	}
};


void cTargets::IDtarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;
	short id=(addx[s]<<8)+addy[s];
	if (pi)
	{
		pi->setId(id);
		pi->refresh();
	}
	else if (pc)
	{
		pc->setId(id);
		pc->setXid(id);
		updatechar(pc);
	}
}

//public !!
void cTargets::XTeleport(P_PLAYER Player, int x)
{
	if (!Player) return;

	UOXSOCKET s = Player->cs;
	int serial = -1;
	P_CHAR pc = NULL;
	switch (x)
	{
	case 0:	// get by target
		serial = LongFromCharPtr(&buffer[s][7]);
		pc = FindCharBySerial(serial);
		break;
	case 2:	// get by socket
		if (perm[makenumber(1)])
		{
			pc = MAKE_CHAR_REF(currchar(makenumber(1)));
		}
		else 
			return;
		break;
	case 5:	// get by serial
		serial = hexnumber(1,2,3,4);
		pc = FindCharBySerial(serial);
		break;
	}
	
	if (pc)
	{
		pc->teleTo(Player->pos);
		return;// Zippy
	}

	P_ITEM pi = FindItemBySerial(serial);
	if (pi)
	{
		pi->MoveTo2(Player->pos);
	}
}

void XgoTarget(P_PLAYER Player, P_CHAR pc)
{
	if (pc)
	{
		P_PLAYER Player2 = MakePlayerRef(pc);
		if (Player2)
			Player2->travelTo(Player->TD._Target1Pos);
		else
			pc->teleTo(Player->TD._Target1Pos); //teleTo automatically updates the other clients
	}
}

static void PrivTarget(P_PLAYER Player, P_CHAR pc)
{
	if (!Player) return;
	UOXSOCKET s = Player->cs;
	if (SrvParms->gm_log)	//Logging
	{
		sprintf(temp, "%s.gm_log",Player->name);
		sprintf(temp2, "%s as given %s Priv [%x][%x]\n",Player->name,pc->name,addid1[s],addid2[s]);
		savelog(temp2, temp);
	}
	pc->setPriv(addid1[s]);
	pc->SetPriv2(addid2[s]);
}

static void KeyTarget(P_PLAYER Player, P_ITEM pi) // new keytarget by Morollan
{
	if (!Player || !pi) return;

	UOXSOCKET s = Player->cs;
	if (pi->getMore1_4()==0)
	{
		if ( pi->isType(ITY_Key) && (iteminrange(Player,pi,2) || (!pi->isInWorld()) ) )
		{
			if (!Player->checkSkill(TINKERING, 400))
			{
				Player->Msg("You fail and destroy the key blank.");
				Items->DeleItem(pi);
			}
			else
			{
				pi->setMore1_4(getAddid1_4(s));
				Player->Msg("You copy the key."); //Morrolan can copy keys
			}
		}
		return;
	}
	
	else if (pi->getMore1_4()==getAddid1_4(s) || addid1[s]==0xFF)
	{
		if (IsSign(pi->id()))
		{
			Player->Msg("What do you wish the sign to say?");
			Player->setInput(6,pi);
			return;
		}
		
		//Boats ->
		if (pi->isType(ITY_Boat) && pi->type2==3)
		{
			Boats->OpenPlank(pi);
			return;
		}
		//End Boats --^

		if (!iteminrange(Player,pi,2)) // check if item is in range
		{
			Player->Msg("You can't reach the item!");
			return; // return if not
		}

		if (pi->isType(ITY_Container) || pi->isType(ITY_SpawnChest))
		{
			if (pi->isType(ITY_Container)) pi->setType(ITY_LockedContainer);
			if (pi->isType(ITY_SpawnChest)) pi->setType(ITY_LockedSpawnChest);
			Player->Msg("You lock the container.");
			SndRemoveItemToAll(pi);	// close any open gumps
			pi->refresh();
			return;
		}
		if (pi->isType(ITY_Key))
		{
			Player->setInput(6,pi);
			Player->Msg("Enter new name for key.");//morrolan rename keys
			return;
		}
		if (pi->isType(ITY_LockedContainer) || pi->isType(ITY_LockedSpawnChest))
		{
			if (pi->isType(ITY_LockedContainer)) pi->setType(ITY_Container);
			if (pi->isType(ITY_LockedSpawnChest)) pi->setType(ITY_SpawnChest);
			Player->Msg("You unlock the container.");
			return;
		}
		if (pi->isType(ITY_Door))
		{
			pi->setType(ITY_LockedDoor);
			Player->Msg("You lock the door.");
			return;
		}
		if (pi->isType(ITY_LockedDoor))
		{
			pi->setType(ITY_Door);
			Player->Msg("You unlock the door.");
			return;
		}
	}//else if
	else
	{
		if (pi->isType(ITY_Key)) Player->Msg("That key is not blank!");
		else if (pi->more1=='\x00') Player->Msg("That does not have a lock.");
		else Player->Msg("The key does not fit into that lock.");
		return;
	}
	return;
}//keytarget()

void cTargets::IstatsTarget(P_PLAYER Player, cPacket6C *pt)
{
	if (pt->Tserial==0)
	{
		const tileData *tile = TileData->SeekTile(pt->model);
		sprintf(temp, "Item [Static] ID [%x]",pt->model);
		Player->Msg(temp);
		sprintf(temp, "ID2 [%i], Height [%i]",pt->model, tile->height);
		Player->Msg(temp);
	}
	else
	{
		P_ITEM pi=FindItemBySerial(pt->Tserial);
		if (pi!=NULL)
		{
			char serstr[20];
			LongToHexChunks(pi->serial,serstr);
			char contstr[20];
			LongToHexChunks(pi->contserial,contstr);
			sprintf(temp, "Item [Dynamic] Ser %s ID [%x] Name [%s] Name2 [%s] Color [%x] Cont %s Layer [%x] Type [%d] Magic [%x] More [%x %x %x %x] Position [%i %i %i] Amount [%i] Movable [%i] Priv [%x]",
				serstr,pi->id(),
				pi->name,pi->name2,pi->color(),
				contstr,
				pi->layer,pi->type,pi->getMovability(),
				pi->more1,pi->more2,pi->more3,pi->more4,
				pi->pos.x,pi->pos.y,pi->pos.z,pi->amount,pi->getMovability(),pi->getPriv());
			Player->Msg(temp);
			sprintf(temp,"STR [%d] HP/MAX [%d/%d] Damage [%d-%d] Defence [%d] Rank [%d] SecureIt [%d] MoreXYZ [%i %i %i] Poisoned [%i] RaceHate [%i] Weight [%d] Owner [%x] Creator [%s] MadeValue [%i] Value [%i] Decaytime[%i] Decay [%i] GoodType[%i] RandomValueRate[%i]",
				pi->st, pi->hp,pi->maxhp, pi->lodamage, pi->hidamage,pi->def,pi->rank,pi->secureIt,
				pi->morex, pi->morey, pi->morez,pi->Poisoned(),pi->racehate,
				pi->weight, pi->ownserial, // Ison 2-20-99
				pi->creator,pi->madewith,pi->value,
				pi->decaytimer.getRemainingSecs(),pi->decays(),
				pi->good,pi->rndvaluerate); // Magius(CHE) (2)
			Player->Msg(temp); // Ison 2-20-99
		}
	}
}

void cTargets::TargIdTarget(P_PLAYER Player, P_ITEM pi) // Fraz
{
	if (pi)
	{
		if (!pi->isLockedDown())
		{
			if (pi->name2 &&(strcmp(pi->name2, "#")))
				strcpy(pi->name, pi->name2);
			if (pi->name[0] == '#')
				pi->getName(temp2);
			else 
				strcpy(temp2, pi->name);
			sprintf(temp, "You found that this item appears to be called: %s", temp2);
			Player->Msg(temp);
		}
		if (!pi->isType(ITY_CastingItem))
		{
			if (!pi->isType(ITY_IDwand))
			{
				Player->Msg("This item has no hidden magical properties.");
			}
			else
			{
				sprintf(temp, "It is enchanted with item identification, and has %d charges remaining.", pi->morex);
				Player->Msg(temp);
			}
		}
		else
		{
			sprintf(temp, "It is enchanted with the spell %s, and has %d charges remaining.", spellname[(8*(pi->morex - 1)) + pi->morey - 1], pi->morez);
			Player->Msg(temp);
		}
	}
}

static void CstatsTarget(P_PLAYER Player, P_CHAR pc)
{
	char serstr[20];
	LongToHexChunks(pc->serial,serstr);
	sprintf(temp, "Ser %s ID [%x] Name [%s] Skin [%x] Account [%x] Priv [%x %x] Position [%i %i %i] CTimeout [%i] Fame [%i] Karma [%i] Deaths [%i] Kills [%i] NPCAI [%x] NPCWANDER [%d] WEIGHT [%.2f]",
		serstr,pc->id(),pc->name,pc->skin,
		pc->account,pc->getPriv(),pc->GetPriv2(),
		pc->pos.x,pc->pos.y,pc->pos.z, pc->timeout,
		pc->fame,pc->karma,pc->deaths,pc->kills,
		pc->npcaitype, pc->npcWander, (float)pc->weight);
	Player->Msg(temp);

	char rname[50];
	if (!pc->getRaceName(rname,50)) strcpy(rname,"None");

	sprintf(temp, "Other Info: Poisoned [%i] Poison [%i] Hunger [%i] Carve[%i] Race[%s]",
		pc->Poisoned(),pc->Poison(),pc->Hunger(),pc->carve,rname);
	Player->Msg(temp);

	//Gumps->Open(s, c, 0, 8); // sending statuswindows agian if already open crash 3.0.7 clients
	statwindow(Player, pc);
}

static void MoveBelongingsToBp(P_CHAR pc)
{
	if (!pc) return;
	P_ITEM Pack = pc->getBackpack();
	if (!Pack)
	{
		Pack = pc->addBackpack();
		if (!Pack) return;
	}

	int ci=0, loopexit=0;
	P_ITEM pi;
	while ( (( pi=pc->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->layer!=0x15 && pi->layer!=0x1D &&
			pi->layer!=0x10 && pi->layer!=0x0B)
		{
			if (pi->trigon==1 && pi->trigtype==2 && pi->layer<19)// -Frazurbluu- Trigger Type 2 is my new trigger type *-
			{
				triggerItem(pc, pi);	// trigger is fired
			}
			pi->SetRandPosInCont(Pack);
			pi->SetContSerial(Pack->serial);
			pi->layer=0x00;
			SndRemoveItemToAll(pi);
			pi->refresh();
		}
		else if (pc->Wears(pi) &&
			(pi->layer==0x0B || pi->layer==0x10))	// hair & beard (Duke)
		{
			Items->DeleItem(pi);
		}
	}
	updatechar(pc);
}

static void GMTarget(P_PLAYER Player, P_CHAR pc)
{
	if (SrvParms->gm_log)
	{
		sprintf(temp, "%s.gm_log",Player->name);
		sprintf(temp2, "%s as made %s a GM.\n",Player->name,pc->name);
		savelog(temp2, temp);
	}
	unmounthorse(pc);
	
	pc->setId(0x03DB);
	pc->skin = 0x8021;
	pc->setXid(0x03DB);
	pc->xskin = 0x8021;
	pc->setPriv(0xF7);
	pc->SetPriv2(0xD9);
	pc->gmrestrict = 0; // By default, let's not restrict them.
	
	pc->cmds.set(Npcs->metagm[0]); // GM defaults. this overwrites all previous settings !
	if (pc->isAdmin()) pc->cmds.setAll();

	int sk=SkillManager->getAllSkillsValue();
	int i;	
	for (i = 0; i < sk; i++)
	{
		pc->setRealSkill(i,1000);
		pc->setSkill(i,1000);
	}
	
	// All stats to 100
	pc->setStr(100);
	pc->fullHp();
	pc->setDex(100);
	pc->fullStm();
	pc->setInt(100);
	pc->fullMana();

	short off = 0;
	if (!strncmp(pc->name, "GM ",3)) off=3;
	if (!strncmp(pc->name, "Counselor ",10)) off=10;
	sprintf(temp, "GM %s", pc->name+off);
	strcpy(pc->name,temp);

	MoveBelongingsToBp(pc);
}

static void CnsTarget(P_PLAYER Player, P_CHAR pc)
{
	if (SrvParms->gm_log)
	{
		// logging
		sprintf(temp, "%s.gm_log",Player->name);
		sprintf(temp2, "%s as made %s a Counselor.\n",Player->name,pc->name);
		savelog(temp2, temp);
	}
	pc->setId(0x03DB);
	pc->skin=0x8003;
	pc->setXid(0x03DB);
	pc->xskin=0x8002;
	pc->setPriv(0xB6);
	pc->SetPriv2(0x8D);

	short off = 0;
	if (!strncmp(pc->name, "GM ",3)) off=3;
	if (!strncmp(pc->name, "Counselor ",10)) off=10;
	sprintf(temp, "Counselor %s", pc->name+off);
	strcpy(pc->name,temp);

	pc->cmds.set(Npcs->metagm[1]); // counselor defaults. this overwrites all previous settings !
	if (pc->isAdmin()) pc->cmds.setAll();

	MoveBelongingsToBp(pc);
}

void cTargets::GhostTarget(P_PLAYER Player, P_CHAR pc)
{
	if (!pc->dead)
	{
		pc->setAttacker(Player); // for forensics ev
		bolteffect(pc, true);
		soundeffect2(pc, 0x0029);
		deathstuff(pc);
	}
	else
		Player->Msg("That player is already dead.");
}

class cBoltTarget : public cCharTarget
{
public:
	cBoltTarget(P_PLAYER Player) : cCharTarget(Player) {}
	void CharSpecific(P_CHAR pc)
	{
		UOXSOCKET s = _Player->cs;
		if (addid1[s]!=3) // no param or more than 2 given ? default behaviour
		{
			if (w_anim[0]==0 && w_anim[1]==0)
			{
				bolteffect(pc, true);
				soundeffect2(pc, 0x0029);
			}
			else
			{
				for (int j=0;j<=333;j++) 
				{
					bolteffect2(pc,w_anim[0],w_anim[1]);
				}
			}
		}
		else // 2 params given ... "play" individual animation NOT global
		{
			for (int j=0;j<=333;j++) 
			{
				bolteffect2(pc,addx[s],addy[s]);
			}
		}
	}
};

class cSetAmountTarget : public cItemTarget
{
public:
	cSetAmountTarget(P_PLAYER Player) : cItemTarget(Player) {}
	void ItemSpecific()
	{
		UOXSOCKET s = _Player->cs;
		if (addx[s] > 64000) //Ripper..to fix a client bug for over 64k.
		{
			_Player->Msg("No amounts over 64k in a pile!");
			return;
		}
		this->pi->amount=addx[s];
		pi->refresh();
	}
};

void cTargets::KickTarget(P_PLAYER Player, P_CHAR pc)
{
	if (!Player || !pc) return;
	UOXSOCKET sk=pc->getSlot();
	if (sk>-1)
	{
		Player->Msg("Kicking player");
		pc->Msg("You have been kicked!"); //New -- Zippy
		Network->Disconnect(sk);
	}
}

void cTargets::AddMenuTarget(int s, int addmitem, cCoord& tpos)
{
	LogAssert(addmitem!=-1);
	P_ITEM pi=Items->CreateFromScript(addmitem);
	if (!pi) return;
	pi->MoveTo2(tpos);
}

void cTargets::VisibleTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;
	
	if (pi)
	{
		pi->visible=addx[s];
		pi->refresh();
	}
	else if (pc)
	{
		pc->hidden=addx[s];
		updatechar(pc);
	}
}

///////////////
// Name:	ContainerEmptyTarget1&2
// Purpose:	used by the '#empty' user-command
// history: by Duke, 29.12.2000
//
static void ContainerEmptyTarget1(P_PLAYER Player, P_ITEM pi)
{
	UOXSOCKET s = Player->cs;
	if (isItemSerial(pi->serial) && pi->isType(ITY_Container))
	{
		addx[s]=pi->serial;
		target(Player, T_72, "Select container to fill:");
	}
	else
		Player->Msg("That is not a valid container!");
}
static void ContainerEmptyTarget2(P_PLAYER Player, P_ITEM pNewCont)
{
	UOXSOCKET s = Player->cs;
	if (pNewCont->isType(ITY_Container))
	{
		P_ITEM pi;	// item to move from old container
		int ci=0,loopexit=0;
		while ( ((pi=ContainerSearch(addx[s],&ci)) != NULL) && (++loopexit < MAXLOOPS) )
			pNewCont->AddItem(pi);
	}
	else
		Player->Msg("That is not a valid container!");
}

static void OwnerTarget(P_PLAYER Player, P_CHAR pc)
{
	UOXSOCKET s = Player->cs;
	
	int addser=getAddid1_4(s);
	pc->SetOwnSerial(addser);
	if (addser==-1)
	{
		pc->tamed=false;
	}
	else
	{
		pc->tamed=true;
	}
}

static void OwnerTarget(P_PLAYER Player, P_ITEM pi)
{
	UOXSOCKET s = Player->cs;
	
	int os=getAddid1_4(s);
	pi->SetOwnSerial(os);

	if (pi->visible==1)
	{
		for (unsigned int j=0;j<now;j++) if (perm[j]) senditem(j,pi); // necassairy for items with visible value of 1
	}
}

void cTargets::DvatTarget(P_PLAYER Player, P_ITEM pi)
{
	if (!Player) return;
	if (pi && pi->dye==1)//if dyeable
	{
		P_CHAR pc=GetPackOwner(pi);
		if (Player->isSameAs(pc) || pi->isInWorld())
		{//if on ground or currchar is owning the item - AntiChrist
			UOXSOCKET s = Player->cs;
			pi->setColor((addid1[s]<<8)+addid2[s]);
			pi->refresh();
			soundeffect2(Player,0x023E); // plays the dye sound, LB
		} else
		{
			Player->Msg("That is not yours!!");
		}
	}
	else
	{
		Player->Msg("You can only dye cloth with this.");
	}
}

static void AddNpcByIdTarget(UOXSOCKET s, const cCoord& targpos)
{
	P_CHAR pc = Npcs->MakeBaseNpc();
	if (!pc) return;
	strcpy(pc->name, "Dummy");
	short id=(addid1[s]<<8)+addid2[s];
	pc->setId(id);
	pc->setXid(id);
	pc->setPriv(0x10);	// don't show skill titles
	pc->teleTo(targpos);
	pc->npc=1;
}

void cTargets::AllSetTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	
	int j;
	int k=pc->getSlot();

	int sk=SkillManager->getAllSkillsValue();
	if (addx[s]<sk)
	{
		pc->setRealSkill(addx[s],addy[s]);
		SkillManager->updateSkillLevel(pc, addx[s]);
		if (k!=-1) updateskillPacket(k, addx[s]);
	}
	else if (addx[s]==ALL)
	{
		for (j=0;j<sk;j++)
		{
			pc->setRealSkill(j,addy[s]);
			SkillManager->updateSkillLevel(pc, j);
			if (k!=-1) updateskillPacket(k,j);
		}
	}
	else if (addx[s]==STR)
	{
		pc->setStr(addy[s]);
		for (j=0;j<sk;j++)
		{
			SkillManager->updateSkillLevel(pc,j);
			if (k!=-1) updateskillPacket(k,j);
		}
		if (k!=-1) statwindow(k,pc);
	}
	else if (addx[s]==DEX)
	{
		pc->setDex(addy[s]);
		for (j=0;j<sk;j++)
		{
			SkillManager->updateSkillLevel(pc,j);
			if (k!=-1) updateskillPacket(k,j);
		}
		if (k!=-1) statwindow(k,pc);
	}
	else if (addx[s]==INTELL)
	{
		pc->setInt(addy[s]);
		for (j=0;j<sk;j++)
		{
			SkillManager->updateSkillLevel(pc,j);
			if (k!=-1) updateskillPacket(k,j);
		}
		if (k!=-1) statwindow(k,pc);
	}
	else if (addx[s]==FAME)
	{
		pc->fame=addy[s];
	}
	else if (addx[s]==KARMA)
	{
		pc->karma=addy[s];
	}
}

#include "walking.h" // only for validNPCMove

static void DevtestTarget(P_PLAYER Player, cPacket6C *pp,cCoord& TargPos)
{
		
//	return;	// do nothing in production


#if 0
	if (validNPCMove(TargPos.x,TargPos.y,TargPos.z,Player))
		Player->Msg("valid for Npc !!");
	else
		Player->Msg("No GO !!");
#endif
#if 0
	if (Boats->IsFreeWater2(TargPos))
		Player->Msg("Doesn't block boats !!");
	else
		Player->Msg("Blocks boats !!");
#endif
#if 0
	if (line_of_sight(s, Player->pos, TargPos, LOS_STD_BLOCKERS))
		Player->Msg("I see you !!");
	else
		Player->Msg("No LoS !!");
#endif
#if 1
	P_CHAR pc = FindCharBySerial(pp->Tserial);
	if (pc)
	{
		pc->craftAI = 2;
		pc->nextCraft.setSecs(1);
	}
#endif


}

static void InfoTarget(P_PLAYER Player, cPacket6C *pp) // rewritten to work also with map-tiles, not only static ones by LB
{
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;

	if (pp->model==0)	// damn osi not me why the tilenum is only send for static tiles, LB
	{	// calculating the ID's if it's a maptype

		cCoord c(pp->TxLoc,pp->TyLoc,pp->TzLoc,Player->pos.map); // note: needs to be changed when multiple maps get implemented
	
		cellData map1;
		AllMaps->SeekMap(c, map1);

		const landData* land = TileData->SeekLand(map1.TID);

		Console.send("type: map-tile\n");
		Console.send("tilenum: %i\n",map1.TID);
		Console.send("Flags:%08x\n", land->flags);	
		Console.send("TextureId:%04x\n", land->textureId);		
		Console.send("Name:%s\n", land->name);
	}
	else
	{
		int tilenum=pp->model; // lb, bugfix
		const tileData* tile = TileData->SeekTile(tilenum);
		Console.send("type: static-tile\n");
		Console.send("tilenum: %i\n",tilenum);
		Console.send("Flags:%08x\n", tile->flags);	
		Console.send("Weight:%x\n", tile->weight);
		Console.send("Layer:%x\n", tile->quality);
		Console.send("Anim:%lx\n", tile->appearance);
		Console.send("Unknown1:%x\n", tile->uk1);
		Console.send("Unknown2:%x\n", tile->uk2);
		Console.send("Unknown3:%x\n", tile->uk3);
		Console.send("Unknown4:%x\n", tile->uk4);
		Console.send("Height:%x\n", tile->height);
		Console.send("Name:%s\n", tile->name);
	}
	Player->Msg("Item info has been dumped to the console.");
	Console.send("\n");
}

void cTargets::TweakTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;

	if (pc)
		tweakmenu(s, pc->serial);
	else
		tweakmenu(s, pi->serial);
}

static void SetInvulFlag(P_PLAYER Player, P_CHAR pc)
{
	if (addx[Player->cs]==1)
		pc->makeInvulnerable();
	else
		pc->makeVulnerable();
}

static void Tiling(P_PLAYER Player, cPacket6C *pp) // Clicking the corners of tiling calls this function - Crwth 01/11/1999
{
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;
	UOXSOCKET s = Player->cs;

	if (clickx[s]==-1 && clicky[s]==-1)
	{
		clickx[s]=pp->TxLoc;
		clicky[s]=pp->TyLoc;
		target(Player,T_198, "Select second corner of bounding box.");
		return;
	}

	int c;
	int x1=clickx[s],x2=pp->TxLoc;
	int y1=clicky[s],y2=pp->TyLoc;

	clickx[s]=-1;clicky[s]=-1;

	if (x1>x2) {c=x1;x1=x2;x2=c;}
	if (y1>y2) {c=y1;y1=y2;y2=c;}

	if (addid1[s]==0x40)
	{
		switch (addid2[s])
		{
		case 100:
		case 102:
		case 104:
		case 106:
		case 108:
		case 110:
		case 112:
		case 114:
		case 116:
		case 118:
		case 120:
		case 122:
		case 124:
		case 126:
		case 140:
			AddTarget(Player,pp);
			return;
		}
	}

	
	int x,y;
	unsigned short id=0;
	P_ITEM pi;

	id=(addid1[s]<<8)+addid2[s];

	for (x=x1;x<=x2;x++)
	{
		for (y=y1;y<=y2;y++)
		{
			cCoord tpos(x,y,pp->TzLoc+Map->TileHeight(pp->model),Player->pos.map);
			pi=Items->SpawnItem(ITY_None,1,"#",id,0,tpos);		
			if (!pi) return;
			pi->decayOff();	//Make them not decay
		}
	}

	addid1[s]=0;
	addid2[s]=0;
}

static void TilingScript(P_PLAYER Player, cPacket6C *pp, cCoord& tpos) // Clicking the corners of tiling calls this function - Crwth 01/11/1999
{
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;
	UOXSOCKET s = Player->cs;
	if (clickx[s]==-1 && clicky[s]==-1)
	{
		clickx[s]=pp->TxLoc;
		clicky[s]=pp->TyLoc;
		target(Player,T_299, "Select second corner of bounding box for tilescript.");
		return;
	}

	int c; //pileable=0;
	int x1=clickx[s],x2=pp->TxLoc;
	int y1=clicky[s],y2=pp->TyLoc;

	clickx[s]=-1;clicky[s]=-1;

	if (x1>x2) {c=x1;x1=x2;x2=c;}
	if (y1>y2) {c=y1;y1=y2;y2=c;}
	
	int x,y;	
	P_ITEM pi;

	for (x=x1;x<=x2;x++)
	{
		for (y=y1;y<=y2;y++)
		{		
			pi = Items->CreateFromScript(addx2[s]);
			if (!pi) return;
			pi->resetPriv();	//Make them not decay
			cCoord ipos(tpos);
			ipos.x=x;
			ipos.y=y;
			pi->MoveTo(ipos);
			pi->refresh();
		}
	}
	addid1[s]=0;
	addid2[s]=0;
}

static void KillNpcs(P_PLAYER Player, const cCoord& tpos)
{
	if (Player->TD._Target1Pos.isNotSet())
	{
		Player->TD._Target1Pos = tpos;
		target(Player,T_300,"Select second corner of bounding box for killnpcs.");
		return;
	}

	cMapRectangle rect(Player->TD._Target1Pos, tpos);
	Player->TD._Target1Pos.reset();

	// charcount loop cauz they are not guaranteed to be in regions.
	// and it won't be executed frequently.
	int cnt=0;
	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc = aci.GetData()) != aci.End(); aci++)
	{
		if ( pc->npc && pc->account<0 )
		{
			if (pc->pos.IsIn(rect))
			{
				cnt++;						
				Npcs->DeleteChar(pc);
			}
			
		}
	}
	
	sprintf(temp, "%i npcs killed\n",cnt);
	Player->Msg(temp);
}

void MakeNpcItem(P_PLAYER Player, P_CHAR Npc)
{
	if (!Player) return;
	
	P_ITEM NpcItem = Player->getDclickedItem();
	if (NpcItem==NULL) return;
	
	if (Npc==NULL) return;
	
	if (!Npc->npc) return; // don't put players into packs ;)
	assert (Npc->account==-1);
	
	/////////
	/// make npc invisble (to regular players), but don't delete it
	/////////
	
	Npc->doppelgaenger_serial=-2;	// -2: marker for npc in pack status
	
	Npc->makeInvulnerable();		// they should be neither killable nor visible. without them no later restoring possible	
	Npc->hide();
	cCoord NpcPos(Player->pos);
	NpcPos.Add(1,1);
	Npc->MoveTo(NpcPos);			// get it out of old mapregions !
	updatechar(Npc);
	
	///////
	/// set npc-item values
	///////
	
	NpcItem->morez=NpcItem->id();	// save old item id
	short icon = NpcInfo_getIcon( Npc->id() ); // set it to icon-item best fitting to npc-id
	NpcItem->setId(icon); 
	NpcItem->morex=Npc->serial;		// save serial for retrival (remeber: it still exists, just hidden)
	NpcItem->morey=1;				// enable it
	NpcItem->refresh();
}

static void SwapNpcsTarget(P_PLAYER Player, cPacket6C *pp) 
{
	// first run = second targeting cursor
	if (pp->Tserial==-1) return;
	UOXSOCKET s = Player->cs;
	if (clickx[s]==-1)
	{
		clickx[s]=pp->Tserial;
		target(Player,T_301, "Select source creature");
		return;
	}

	// second run, now we have all data
	
	int targetSerial=clickx[s];
	int sourceSerial=pp->Tserial;
	int duration = addx2[s];

	P_CHAR targetNPC, sourceNPC;
	targetNPC = FindCharBySerial(targetSerial);
	sourceNPC = FindCharBySerial(sourceSerial);
	
	if (targetNPC == NULL || sourceNPC ==NULL) // extra care/check, shouldn't happen
	{
		Player->Msg("error, invalid npc data, command not executed");
		return;
	}

	// actual code coming here
	// using it for testing doppelgaenger AI (helper) code
	// but might be a usefull command as well, let's see


	AllTmpEff->Add(sourceNPC, targetNPC, TEF_Doppelgaenger, 1, 0, 0, duration); //1: skill & stats multiplier
		
	// clear temp storage memory
	clickx[s]=-1;
	addid1[s]=0;
	addid2[s]=0;

}

void cTargets::Wiping(P_PLAYER Player, cPacket6C *pt) // Clicking the corners of wiping calls this function - Crwth 01/11/1999
{
	UOXSOCKET s = Player->cs;

	if (clickx[s]==-1 && clicky[s]==-1)
	{
		clickx[s]=pt->TxLoc;
		clicky[s]=pt->TyLoc;
		if (addid1[s])
			target(Player,T_199,"Select second corner of inverse wiping box.");
		else
			target(Player,T_199,"Select second corner of wiping box.");
		return;
	}

	cMapRectangle rect(clickx[s],clicky[s],pt->TxLoc,pt->TyLoc,Player->pos.map);
	clickx[s]=-1;clicky[s]=-1;

	if (addid1[s]==1) // addid1[s]==1 means to inverse wipe
		Commands->PartialWipe(Player,rect,true);
	else
		Commands->PartialWipe(Player,rect,false);
}

static void ExpPotionTarget(P_PLAYER Player, const cCoord& targpos) //Throws the potion and places it (unmovable) at that spot
{
	if (!Player) return;

	// ANTICHRIST -- CHECKS LINE OF SIGHT!
	if (line_of_sight(Player->pos, targpos))
	{
		P_ITEM pi = FindItemBySerial(getAddid1_4(Player->cs));
		if (pi) // crashfix LB
		{
			pi->MoveTo(targpos);
			pi->SetContSerial(-1);
			pi->setNotMovable(); //make item unmovable once thrown
			movingeffect2(Player, pi, 0x0F, 0x0D, 0x11, 0x00, 0x00);
			pi->refresh();
		}
	}
	else Player->Msg("You cannot throw the potion there!");
}

static void Priv3Target(UOXSOCKET s, P_CHAR pc)
{
	Console.send("setpriv3target: %s\n", pc->name);
	pc->cmds.setValue(0,priv3a[s]);
	pc->cmds.setValue(1,priv3b[s]);
	pc->cmds.setValue(2,priv3c[s]);
	pc->cmds.setValue(3,priv3d[s]);
	pc->cmds.setValue(4,priv3e[s]);
	pc->cmds.setValue(5,priv3f[s]);
	pc->cmds.setValue(6,priv3g[s]);
}

void cTargets::SquelchTarg(P_PLAYER Player, P_CHAR pc)
{
	if (!Player || !pc) return;

	if (pc->isGM())
	{
		Player->Msg("You cannot squelch GMs.");
		return;
	}

	if (pc->squelched)
	{
		pc->squelched=0;
		Player->Msg("Un-squelching...");
		pc->Msg("You have been unsquelched!");
		pc->mutetime=-1;
	}
	else
	{
		pc->mutetime=-1;
		pc->squelched=1;
		Player->Msg("Squelching...");
		pc->Msg("You have been squelched!");
		
		UOXSOCKET s = Player->cs;
		if (addid1[s]!=255 || addid1[s]!=0)
		{
			pc->mutetime=(unsigned int) (uiCurrentTime+(addid1[s]*MY_CLOCKS_PER_SEC));
			addid1[s]=255;
			pc->squelched=2;
		}
	}
}

static void TeleStuff1(P_PLAYER Player, cPacket6C *pp)
{
	if (!IsValidSerial(pp->Tserial)) return;

	if (isCharSerial(pp->Tserial))
		target(Player,T_215,"Select location to put this character.");
	else
		target(Player,T_215,"Select location to put this item.");
}

static void TeleStuff2(P_PLAYER Player, cPacket6C *pp, const cCoord& Tpos)
{
	int serial = Player->target1serial;
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;
	if (!IsValidSerial(serial)) return;

	if (isCharSerial(serial))
	{
		P_CHAR pc = FindCharBySerial(serial);
		if (!pc) return;
		Player->Msg("Moving character...");
		pc->teleTo(Tpos);
	}
	else
	{
		P_ITEM pi = FindItemBySerial(serial);
		if (!pi) return;
		pi->MoveTo(Tpos);
		Player->Msg("Moving item...");
		pi->refresh();
	}
}

// Note: this can be called for both Players and Npcs (Duke, 19.07.05)
void CarveTarget(P_CHAR pc, P_ITEM Corpse, int feat, int ribs, int hides, int fur, int wool, int bird)
{
	P_ITEM Blood=Items->SpawnItem(ITY_None,1,"#",0x122A,0,Corpse->pos);	//add the blood puddle
	if (!Blood) return;
	Blood->setNotMovable();
	Blood->startDecay();
	Blood->refresh();

	P_ITEM pi;
	if (feat)
	{
		pi=Items->SpawnItem(ITY_None,feat,"feather",0x1BD1,0,pc);
		if (!pi) return;
		pc->Msg("You pluck the bird and get some feathers.");
	}
	if (ribs)
	{
		pi=Items->SpawnItem(ITY_Food,ribs,"raw rib",0x09F1,0,pc);
		if (!pi) return;
		pi->AddFoodType(FOOD_RAW+FOOD_MEAT);
		pc->Msg("You carve away some meat.");
	}

	if (hides)
	{
		pi=Items->SpawnItem(ITY_None,hides,"hide",0x1078,0,pc);
		if (!pi) return;
		pc->Msg("You skin the corpse and get the hides.");
	}

	if (fur)
	{	// animals with fur now yield hides (OSI). Duke, 7/17/00
		pi=Items->SpawnItem(ITY_None,fur,"hide",0x1078,0,pc);
		if (!pi) return;
		pc->Msg("You skin the corpse and get the hides.");
	}

	if (wool)
	{
		pi=Items->SpawnItem(ITY_None,wool,"unspun wool",0x0DF8,0,pc);
		if (!pi) return;
		pc->Msg( "You skin the corpse and get some unspun wool.");
	}
	if (bird)
	{
		pi=Items->SpawnItem(ITY_Food,bird,"raw bird",0x09B9,0,pc);
		if (!pi) return;
		pi->AddFoodType(FOOD_RAW+FOOD_MEAT);
		pc->Msg( "You carve away some raw bird.");
	}
}

//AntiChrist - new carving system - 3/11/99
//Human-corpse carving code added
//Scriptable carving product added
// Note: this can be called for both Players and Npcs (Duke, 19.07.05)
static void newCarveTarget(P_CHAR pc, P_ITEM Corpse)
{
	if (!pc || !Corpse) return;

	bool deletecorpse=false;

	P_ITEM Blood=Items->SpawnItem(ITY_None,1,"#",0x122A,0);	//add the blood puddle
	if (!Blood) return;
	Blood->setNotMovable();
	Blood->startDecay();
	Blood->MoveTo2(Corpse->pos);


	if (Corpse->morey)	//if it's a human corpse
	{
		pc->karma-=100;
		pc->Msg("You lost some karma!");
		criminal(pc);
		//create the Head
		sprintf(temp,"the head of %s",Corpse->name2);
		P_ITEM pi=Items->SpawnItem(ITY_None,1,temp,0x1DA0,0);
		if (!pi) return;
		pi->SetContSerial(Corpse->serial);
		pi->layer=0x01;

		//AntiChrist & Magius(CHE) - store item's owner, so that lately
		//if we want to know the owner we can do it
		pi->setOwnSerialOnly(Corpse->ownserial);

		//create the Body
		sprintf(temp,"the heart of %s",Corpse->name2);
		P_ITEM pi4=Items->SpawnItem(ITY_None,1,temp,0x1CED,0);
		if (!pi4) return;
		pi4->SetContSerial(Corpse->serial);
		pi4->layer=0x01;
		pi4->setOwnSerialOnly(Corpse->ownserial);	// see above

		//create the Heart
		sprintf(temp,"the body of %s",Corpse->name2);
		P_ITEM pi5=Items->SpawnItem(ITY_None,1,temp,0x1DAD,0);
		if (!pi5) return;
		pi5->SetContSerial(Corpse->serial);
		pi5->layer=0x01;
		pi5->setOwnSerialOnly(Corpse->ownserial);	// see above

		//create the Left Arm
		sprintf(temp,"the left arm of %s",Corpse->name2);
		P_ITEM pi6=Items->SpawnItem(ITY_None,1,temp,0x1DA1,0);
		if (!pi6) return;
		pi6->SetContSerial(Corpse->serial);
		pi6->layer=0x01;
		pi6->setOwnSerialOnly(Corpse->ownserial);	// see above

		//create the Right Arm
		sprintf(temp,"the right arm of %s",Corpse->name2);
		P_ITEM pi7=Items->SpawnItem(ITY_None,1,temp,0x1DA2,0);
		if (!pi7) return;
		pi7->SetContSerial(Corpse->serial);
		pi7->layer=0x01;
		pi7->setOwnSerialOnly(Corpse->ownserial);	// see above

		//create the Left Leg
		sprintf(temp,"the left leg of %s",Corpse->name2);
		P_ITEM pi8=Items->SpawnItem(ITY_None,1,temp,0x1DA3,0);
		if (!pi8) return;
		pi8->SetContSerial(Corpse->serial);
		pi8->layer=0x01;
		pi8->setOwnSerialOnly(Corpse->ownserial);	// see above

		//create the Rigth Leg
		sprintf(temp,"the right leg of %s",Corpse->name2);
		P_ITEM pi9=Items->SpawnItem(ITY_None,1,temp,0x1DA4,0);
		if (!pi9) return;
		pi9->SetContSerial(Corpse->serial);
		pi9->layer=0x01;
		pi9->setOwnSerialOnly(Corpse->ownserial);	// see above

		//human: always delete corpse!
		deletecorpse=true;
	}
	else
	{
		Script* Scp = findSection(carve_script,"CARVE",Corpse->carve);
		if (!Scp) return;

		int loopexit=0;
		while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS))
		{
			int inum = 0;
			if (Scp->getKeyVal("ADDITEM",&inum))
			{
				scpMark m; Scp->Suspend(m);
				P_ITEM pi10=Items->CreateScriptItem(inum);
				if (pi10)
				{
					pi10->layer=0;
					Corpse->AddItem(pi10);
				}
				Scp->Resume(m);
			}
			else
				Scp->invalidTag();	// the other tags are invalid
		}
		Scp->Close();
	}

	if (deletecorpse)//if corpse has to be deleted
	{
		//let's empty it
		int ci=0, loopexit=0;
		P_ITEM pj;
		while ( (( pj=Corpse->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
		{
			pj->startDecay();
			pj->SetContSerial(-1);
			pj->MoveTo2(Corpse->pos);
		}
		Items->DeleItem(Corpse);	//and then delete the corpse
	}
}

void cTargets::CorpseTarget(P_CHAR pc, P_ITEM pi)
{
	if (!pi) return;

	if (!iteminrange(pc,pi,1))
	{
		pc->Msg("That is too far away.");
		return;
	}
	
	pc->DoAction(0x20);

	if (pi->more1)
	{
		pc->Msg("You carve the corpse but find nothing usefull.");
		return;
	}
	
	P_ITEM Tool = pc->getDclickedItem();
	if (!Tool) return;
	if (pc->ToolWearOut(Tool))
		return;

	pi->more1=1;//corpse being carved...can't carve it anymore

	if (pi->morey || pi->carve>-1)
	{//if specified, use enhanced carving system!
		newCarveTarget(pc,pi);
	}
	else
	{//else use standard carving
		short id = pi->amount;
		CarveTarget(pc, pi,
			NpcInfo_GetFeathers(id),
			NpcInfo_GetRibs(id),
			NpcInfo_GetHides(id),
			NpcInfo_GetFur(id),
			NpcInfo_GetWool(id),
			NpcInfo_GetBird(id) );
	}
}

static void BladeTarget(P_PLAYER Player, cPacket6C *pp)
{
	short id=pp->model;

	if (isCharSerial(pp->Tserial))
	{
		const P_CHAR Sheep=FindCharBySerial(pp->Tserial);
		if (!Sheep) return;
		if (Sheep->id() == 0xDF)	// a shorn sheep ?
		{
			Player->Msg("This sheep doesn't have enough wool.");
			return;
		}
		if (Sheep->id() == 0xCF)	// an ushorn sheep ?
		{
			if (Sheep->summoned) // summoned animals shouldn't give wool (Thrawn)
			{
				Player->Msg("You are unable to shear this sheep.");
				return;
			}
			P_ITEM pi=Items->SpawnItem(ITY_None,1,"#",0x0DF8,0,Player);
			if (!pi) return;
			tempeffect(Sheep, TEF_Sheep, 0, 0, 0); // change to 'naked'
			Player->Msg("You shear the sheep and get some unspun wool.");
			return;
		}
	}
	else if (IsCorpse(id))
	{
		const P_ITEM pi=FindItemBySerial(pp->Tserial);
		if (!pi) return;
		Targ->CorpseTarget(Player,pi);
	}
	else if (IsFish(id))
	{
		const P_ITEM rpi=FindItemBySerial(pp->Tserial);
		if (!rpi) return;
		short amt=0;
		if (rpi->amount>1)
			amt = rpi->amount*4;
		else
			amt=4; 
		soundeffect(Player,0x0056);
		P_ITEM pi=Items->SpawnItem(ITY_Food,amt,"#",0x097A,0,Player);
		if (!pi) return;
		pi->AddFoodType(FOOD_RAW+FOOD_FISH);
		Items->DeleItem(rpi);
	}
	else
		Player->Msg("You can't think of a way to use your blade on that.");
}

void cTargets::SwordTarget(P_PLAYER Player, cPacket6C *pp)
{
	UOXSOCKET s = Player->cs;

	if (IsTree(pp->model))
	{
		Player->DoAction((!Player->onhorse) ? 0x0D : 0x1D);
		soundeffect2(Player,0x013E);
		P_ITEM pi=Items->SpawnItem(ITY_None,1,"#",0x0DE1,0,Player->pos); //Kindling
		if (!pi) return;
		Player->Msg("You hack at the tree and produce some kindling.");
	}
	else if (IsLog(pp->model) || IsBoard(pp->model))
	{
		const P_ITEM pi=FindItemBySerial(pp->Tserial);
		Skills->BowCraft(s,Player,pi);
	}
	else
		BladeTarget(Player,pp);
}

static void AxeTarget(P_PLAYER Player, cPacket6C *pp, const cCoord& Tpos)
{
	if (IsTree(pp->model))
	{
		Skills->TreeTarget(Player,Tpos);
	}
	else
		BladeTarget(Player,pp);
}

void cTargets::NpcTarget(P_PLAYER Player, P_CHAR pc)
{
	if (!pc) return;
	setAddid1_4(Player->cs,pc->serial);
	target(Player, T_57, "Select NPC to follow this player.");
}

void cTargets::NpcTarget2(int s, P_CHAR pc)
{
	if (!pc) return;
	if (pc->npc)
	{
		P_CHAR ToFollow = FindCharBySerial(getAddid1_4(s));
		if (!ToFollow) return;
		pc->SetFollowTarget(ToFollow);
		pc->npcWander=1;
	}
}

void cTargets::NpcRectTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	if (pc->npc)
	{
		pc->fx1=addx[s];
		pc->fy1=addy[s];
		pc->fz1=-1;
		pc->fx2=addx2[s];
		pc->fy2=addy2[s];
		pc->npcWander=3;
	}
}

void cTargets::NpcCircleTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	if (pc->npc)
	{
		pc->fx1=addx[s];
		pc->fy1=addy[s];
		pc->fz1=-1;
		pc->fx2=addx2[s];
		pc->npcWander=2; // bugfix, LB
	}
}

void cTargets::NpcWanderTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	if (pc->npc)
	{
		pc->npcWander=addx[s];
	}
}

//taken from 6904t2(5/10/99) - AntiChrist
void cTargets::NpcAITarget(P_PLAYER Player, P_CHAR pc)
{
	if (!pc) return;
	if (pc->npc)
	{
		pc->setAi(addx[Player->cs]);
		Player->Msg("Npc AI changed.");//AntiChrist
	}
}

void cTargets::ReleaseTarget(P_PLAYER Player, P_CHAR pc) 
{ 
	if (pc) 
	{ 
		if (pc->cell == 0) 
		{ 
			Player->Msg("That player is not in jail!"); 
		} 
		else 
		{ 
			UOXSOCKET s2=pc->getSlot();
			jails[pc->cell].occupied = false; 
			pc->travelTo(jails[pc->cell].oldpos, s2);
			pc->cell = 0; 
			pc->SetPriv2(0);
			pc->jailtimer.reset(); 
			soundeffect2(pc, 0x01FD); // Play sound effect for player 
			pc->Msg("You are released.."); 
			sprintf(temp,"Player %s released.", pc->name);
			Player->Msg(temp); 
		} 
	} 
}

void cTargets::GmOpenTarget(P_PLAYER Player, P_CHAR pc)
{
	UOXSOCKET s = Player->cs;
	int ci=0, loopexit=0;
	P_ITEM pi;
	while ( (( pi=ContainerSearch(pc->serial,&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->layer==addmitem[s])
		{
			backpack(s,pi);
			return;
		}
	}
	Player->Msg("No object was found at that layer on that character");
}

void cTargets::StaminaTarget(P_PLAYER Player, P_CHAR pc)
{
	if (pc)
	{
		soundeffect2(pc, 0x01F2);
		staticeffect(pc, 0x376A, 0x09, 0x06);
		pc->fullStm();
		updatestats(pc, 2);
		return;
	}
	Player->Msg("That is not a person.");
}

void cTargets::ManaTarget(P_PLAYER Player, P_CHAR pc)
{
	if (pc)
	{
		soundeffect2(pc, 0x01F2);
		staticeffect(pc, 0x376A, 0x09, 0x06);
		pc->fullMana();
		updatestats(pc, 1);
		return;
	}
	Player->Msg("That is not a person.");
}

void cTargets::MakeShopTarget(P_PLAYER Player, P_CHAR pc)
{
	Commands->MakeShop(pc,1);
	updatechar(pc);
	Player->Msg("The buy containers have been added.");
}

void cTargets::JailTarget(P_PLAYER Player, P_CHAR pc) 
{ 
	if (!pc) return;
	
	if (pc->cell>0) 
	{ 
		Player->Msg("That player is already in jail!"); 
		return; 
	} 
	
	int i, x = 0; 
	for (i = 1; i < 11; i++) 
	{ 
		if (!jails[i].occupied) 
		{ 
			UOXSOCKET prisoner = pc->getSlot(); 
			jails[i].oldpos = pc->pos;
			pc->travelTo(jails[i].pos,prisoner); 
			pc->cell = i; 
			pc->Freeze(); // freeze them  Ripper 
			

			UOXSOCKET s = Player->cs;
			// blackwinds jail
			pc->jailtimer.setSecs(addmitem[s]); // Additem array used for jail time here.. 
			addmitem[s] = 0; // clear it 
			jails[i].occupied = 1; 
			pc->Msg("You are jailed !"); 
			pc->Msg("You notice you just got something new at your backpack.."); 
			sprintf(temp,"Player %s has been jailed in cell %i.", pc->name, i);
			Player->Msg(temp); 

			Items->SpawnItemBackpack2(prisoner, SrvParms->hci.BallOfJustice); // spawn it to prisoner. 

			// end blackwinds jail 
			
			x++; 
			break; 
		} 
	} 
	if (x == 0) 
		Player->Msg("All jails are currently full!"); 
}

void cTargets::AttackTarget(P_PLAYER Player, P_CHAR Target2)
{
	P_CHAR Pet = FindCharBySerial(addx[Player->cs]);
	if (!Pet) return;

	if (Pet->inGuardedArea()) // Ripper..No pet attacking in town.
	{
		Player->Msg("You cant have pets attack in town!");
		return;
	}
	if (addz[Player->cs] == 0)	// they did NOT say ALL KILL
	{
		Pet->attack(Target2);
	}
	else
	{
		P_CHAR pc;
		int loopexit=0;
		RegGrid3x3Iterator4Chars rg3ic(Player->pos);
		while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
		{
			// 1. general speech restrictions
			if (!pc->npc && !pc->automode)	// only npcs will respond automagically, players still have to do that themselves ;)
				continue;
			if (Player->dist(pc) > 16)	// at least they should be on the screen
				continue;
			if (Player->isSameAs(pc))	// not talking to ourselves
				continue;
			
			// 2. pet command restrictions
			if (pc->isDummy())			// dont command "dummy" npcs
				continue;
			if (!(Player->Owns(pc) || Player->isGM()))		// owner of the char || a GM
				continue;
			if (pc->isAiVendor())		// player vendor
				continue;
			if (Player->dist(pc) > 7)	// too far away to hear us
				continue;
	
			pc->attack(Target2);
		}
	}
}

void cTargets::FollowTarget(int s, P_CHAR pc2)
{
	P_CHAR pc = FindCharBySerial(addx[s]);
	if (!pc) return;

	pc->SetFollowTarget(pc2);
	pc->npcWander=1;
}

void cTargets::TransferTarget(int s, P_CHAR NewOwner)
{
	P_CHAR Pet = FindCharBySerial(addx[s]);
	if (!Pet) return;

	bool controlOk=false;
	if (Pet->isMount()) controlOk=true;
	else if (Pet->totame<300) controlOk=true;
	else
	{
		int skillhave =((NewOwner->Skill(TAMING))*4 + NewOwner->Skill(ANIMALLORE))/5;
		if (skillhave > Pet->totame) controlOk=true;
	}

	char t[120];
	if (!controlOk)
	{
		sprintf(t,"* %s does not accept %s as his master *",Pet->name,NewOwner->name);
		npctalkall(Pet,t);
		return;
	}

	sprintf(t,"* %s will now take %s as his master *",Pet->name,NewOwner->name);
	npctalkall(Pet,t);

	Pet->SetOwnSerial(NewOwner->serial);
	Pet->npcWander=1;

	Pet->ResetFollowTarget();
	Pet->npcWander=0;
}

int cTargets::BuyShop(P_PLAYER Player, P_CHAR pc)
{
	if (!Player || !pc) return 0;
	P_ITEM Cont1A=NULL, Cont1B=NULL;

	int ci=0, loopexit=0;
	P_ITEM pi;
	while ( (( pi=pc->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->layer==0x1A)
		{
			Cont1A=pi;
		}
		else if (pi->layer==0x1B)
		{
			Cont1B=pi;
		}
	}

	if (!Cont1A || !Cont1B)
	{
		return 0;
	}

	UOXSOCKET s = Player->cs;
	impowncreate(s, pc, 0);	// Send the NPC again to make sure info is current. (OSI does this we might not have to)

	bool i1 = sendshopinfo(s, pc, Cont1A); // Send normal shop items
	bool i2 = sendshopinfo(s, pc, Cont1B); // Send items sold to shop by players

	if (!i1 && i2)	// workaround for client side hardcoded behaviour we don't want
					// namly sending nothing when 0x1a layer container empty but 0x1b not
	{
		senddummyshopinfo(s,pc,Cont1A);
	}


	if (i1 || i2)  // only open buy gump if vendor actually has something to sell 
	{
		SndShopgumpopen(s,pc->serial);
		statwindow(s,Player); // Make sure the gold total has been sent.
		return 1;
	}

	npctalk(s, pc, "I have nothing to sell");
	return 0;
}

////////////////////////////////// 
// Changed hideing to make flamestrike and hide work better 
// (Aldur) 
// 
void cTargets::permHideTarget(P_PLAYER Player, P_CHAR pc) 
{ 
	if (!pc) return;
	
	if (pc->isHiddenBySkill()) 
	{ 
		if (pc->isSameAs(Player))
			Player->Msg("You are already hiding."); 
		else 
			Player->Msg("He is already hiding."); 
		return; 
	} 
	pc->MakePermaHidden(); 
	cCoord effPos(pc->pos);
	effPos.Add(1,1,10);
	staticeffect3(effPos, 0x37, 0x09, 0x09, 0x19, 0); 
	soundeffect2(pc, 0x0208); 
	tempeffect(pc, TEF_HideGM, 1, 0, 0); 
}

////////////////////////////////// 
// Changed unhideing to make flamestrike and unhide work better 
// (Aldur) 
// 
void cTargets::unHideTarget(P_PLAYER Player, P_CHAR pc) 
{ 
	if (!pc) return;
	
	if (pc->hidden == 0) 
	{ 
		if (pc->isSameAs(Player))
			Player->Msg("You are not hiding."); 
		else 
			Player->Msg("He is not hiding."); 
		return; 
	} 
	pc->RemovePermaHidden(); // unhide - AntiChrist 
	// we cant use staticeffect cause the char is invis and 
	// regular chars could see it. Instead we will use the staticeffect3 
	// which takes the char coords. 
	// staticeffect(i, 0x37, 0x09, 0x09, 0x19); 
	cCoord effPos(pc->pos);
	effPos.Add(1,1,10);
	staticeffect3(effPos, 0x37, 0x09, 0x09, 0x19, 0); 
	soundeffect2(pc, 0x0208); 
	tempeffect(pc, TEF_UnHideGM, 1, 0, 0); 
} 

void cTargets::SetSpeechTarget(P_PLAYER Player, P_CHAR pc)
{
	if (!pc) return;

	if (!pc->npc)
		Player->Msg("You can only change speech for npcs.");
	else
		pc->speech=addx[Player->cs];
}

void cTargets::FullStatsTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	
	soundeffect2(pc, 0x01F2);
	staticeffect(pc, 0x376A, 0x09, 0x06);
	pc->fullMana();
	pc->fullHp();
	pc->fullStm();
	
	updatestats(pc, 0);
	updatestats(pc, 1);
	updatestats(pc, 2);
}

////////////////
// name:		CanTrainTarget
// history:		by Antrhacks 1-3-99
// Purpose:		Used for training by NPC's
//
void cTargets::CanTrainTarget(P_PLAYER Player, P_CHAR pc)
{
	if (!pc) return;
	
	if (!pc->npc)
		Player->Msg("Only NPC's may train.");
	else
		pc->cantrain = !pc->cantrain;	//turn on if off, off if on
}

void cTargets::SetDirTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;
	
	if (pi)
	{
		pi->dir=addx[s];
		pi->refresh();
	}
	else if (pc)
	{
		pc->dir=addx[s];
		updatechar(pc);
	}
}

void cTargets::NewXTarget(int s, P_CHAR pc, P_ITEM pi) // Notice a high similarity to th function above? Wonder why. - Gandalf
{
	if (!pc && !pi) return;

	if (pi)
	{
		pi->MoveTo(addx[s],pi->pos.y,pi->pos.z);
		pi->refresh();
	}
	else if (pc)
	{
		cCoord newPos(addx[s],pc->pos.y,pc->pos.z, pc->pos.map);
		pc->teleTo(newPos);
	}
}

void cTargets::NewYTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;

	if (pi)
	{
		pi->MoveTo(pi->pos.x,addx[s],pi->pos.z);
		pi->refresh();
	}
	else if (pc)
	{
		cCoord newPos(pc->pos.x,addx[s],pc->pos.z, pc->pos.map);
		pc->teleTo(newPos);
	}
}

void cTargets::IncXTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;

	if (pi)
	{
		pi->MoveTo(pi->pos, addx[s]);
		pi->refresh();
	}
	else if (pc)
	{
		cCoord newPos(pc->pos.x + addx[s],pc->pos.y,pc->pos.z, pc->pos.map);
		pc->teleTo(newPos);
	}
}

void cTargets::IncYTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;

	if (pi)
	{
		pi->MoveTo(pi->pos, 0, addx[s]);
		pi->refresh();
	}
	if (pc)
	{
		cCoord newPos(pc->pos.x, pc->pos.y + addx[s], pc->pos.z, pc->pos.map);
		pc->teleTo(newPos);
	}
}

void cTargets::Priv3XTarget(P_PLAYER Player, P_CHAR pc) // crackerjack's addition, jul.24/99
{
	if (!pc) return;
	
	UOXSOCKET s = Player->cs;
	struct cmdtable_s *pct=&Commands->command_table[addx[s]];
	if (addy[s])
	{
		pc->cmds.set(pct->cmd_priv_m, pct->cmd_priv_b);
		sprintf(temp,"%s has been granted access to the %s command.",pc->name, pct->cmd_name);
		Player->Msg(temp);
	}
	else
	{
		pc->cmds.reset(pct->cmd_priv_m, pct->cmd_priv_b);
		sprintf(temp,"%s has been revoked access to the %s command.",pc->name, pct->cmd_name);
		Player->Msg(temp);
	}
}

void cTargets::ShowPriv3Target(int s, P_CHAR pc) // crackerjack, jul 25/99
{
	if (!pc) return;
	
	char priv_info[10248];
	unsigned int i;
	sprintf(priv_info, "%s can execute the following commands:\n", pc->name);
	for (i=0; i < Commands->command_table.size(); i++)
	{
		cmdtable_s& Cmd = Commands->command_table[i];
		if (Cmd.cmd_priv_m==255||
			(pc->cmds.get(Cmd.cmd_priv_m)&
			(1<<Cmd.cmd_priv_b)))
		{
			sprintf(temp, " %s", Cmd.cmd_name);
			strcpy(&priv_info[strlen(priv_info)], temp);
		}
	}
	SndUpdscroll(s, strlen(priv_info), priv_info);
}

///////////
// name:	HouseOwnerTarget
// purpose:	transfer ownership of a house
// history:	by crackerjack 8/10/99
//			more or less rewritten by Duke, 16.1.2003
//
void cTargets::HouseOwnerTarget(P_PLAYER Player, P_CHAR NewOwnerCand) //  - change house owner
{
	if (!Player || !NewOwnerCand) return;

	P_PLAYER NewOwner = MakePlayerRef(NewOwnerCand);
	UOXSOCKET sno = NewOwner->cs;
	if (sno==-1) return;

	if (NewOwner->isSameAs(Player))
	{
		Player->Msg("you already own this house!");
		return;
	}
	UOXSOCKET s = Player->cs;
	P_ITEM Sign = FindItemBySerial(getAddid1_4(s));
	if (!Sign) return;

	P_ITEM House = FindItemBySerial(Sign->getMore1_4());
	if (!House) return;

	killkeys(House);
	Sign->SetOwnSerial(NewOwner->serial);
	House->SetOwnSerial(NewOwner->serial);
	makeKeysForHouse(NewOwner, House, House->id());
	
	cMapRectangle *Estate, rect(0,0,0,0,0);
	Estate = GetEstate(House,rect);

	PC_ITEM pi;
	int loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(House->pos);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (   Player->Owns(pi)
			&& pi->isLockedDown()
			&& pi->pos.IsIn(*(Estate)))
		{
			pi->unconst()->TransferLockedDown(NewOwner->serial);
		}
	}

	sprintf(temp,"You have transferred your house to %s.", NewOwner->name);
	Player->Msg(temp);

	sprintf(temp,"%s has transferred the house to you.", Player->name);
	NewOwner->Msg(temp);

	sprintf(temp, "%s has transferred a house to %s.", Player->name, NewOwner->name);
	P_CHAR pc;
	AllPlayersOnScreenIterator api(Player->pos);
	for (pc = api.First(); pc != NULL; pc = api.Next())
	{
		UOXSOCKET si = api.getSocket();
		if (si!=s && si!=sno)
			pc->Msg(temp);
	}
}

void cTargets::HouseEjectTarget(P_PLAYER Player, P_CHAR pc) // crackerjack 8/11/99 - kick someone out of house
{
	if (!Player || !pc) return;
	
	if (Player->isSameAs(pc))
	{
		Player->Msg("Do you really want to do that?!");
		return;
	}

	P_ITEM House = FindItemBySerial(getAddid1_4(Player->cs));
	if (ejectChar(House,pc))
		Player->Msg("Player ejected.");
	else
		Player->Msg("That is not inside the house.");
}

void cTargets::HouseBanTarget(P_PLAYER Player, P_CHAR pc) // crackerjack 8/12/99 - ban someobdy from the house
{
	if (!Player || !pc) return;

	P_ITEM House = FindItemBySerial(getAddid1_4(Player->cs));
	if (!House) return;

	if (pc->isSameAs(Player)) return;
	int r=add_hlist(pc, House, H_BAN);
	if (r==1)
	{
		sprintf(temp,"%s has been banned from this house.", pc->name);
		Player->Msg(temp);
		Targ->HouseEjectTarget(Player,pc);
	}
	else if (r==2)
	{
		Player->Msg("That player is already on a house register.");
	}
	else
		Player->Msg("That player is not on the property.");
}

void cTargets::HouseFriendTarget(P_PLAYER Player, P_CHAR Friend) // crackerjack 8/12/99 - add somebody to friends list
{
	if (!Player || !Friend) return;
	
	P_ITEM House = FindItemBySerial(getAddid1_4(Player->cs));
	if (!House) return;
	
	if (Friend->isSameAs(Player))
	{
		Player->Msg("You cant do that!");
		return;
	}
	int r=add_hlist(Friend, House, H_FRIEND);
	if (r==1)
	{
		sprintf(temp,"%s has been made a friend of the house.", Friend->name);
		Player->Msg(temp);
	}
	else if (r==2)
	{
		Player->Msg("That player is already on a house register.");
	}
	else 
	{
		Player->Msg("That player is not on the property.");
	}
}

void cTargets::HouseUnlistTarget(P_PLAYER Player,P_CHAR pc) // crackerjack 8/12/99 - remove somebody from a list
{
	if (!pc) return;
	
	P_ITEM House = FindItemBySerial(getAddid1_4(Player->cs));
	if (!House) return;
	
	if (del_hlist(pc, House))
	{
		sprintf(temp,"%s has been removed from the house registry.", pc->name);
		Player->Msg(temp);
	}
	else
		Player->Msg("That player is not on the house registry.");
}

// a little helper for HouseLockdown() & HouseSecuredown()
//	by Duke, 5.5.2004
static int countLockedDowns(P_CHAR Player, PC_ITEM House, int& secure)
{
	cMapRectangle *Estate, rect(0,0,0,0,0);
	Estate = GetEstate(House,rect);
	
	PC_ITEM pj;
	int loopexit=0, amt=0, secu=0;
	RegGrid3x3Iterator4Items rg3ii(House->pos);
	while((pj = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (   Player->Owns(pj)
			&& pj->isLockedDown()
			&& pj->pos.IsIn(*(Estate)))
		{
			amt++;
			if (pj->secureIt)
				secu++;
		}
	}
	secure = secu;
	return amt;
}

void cTargets::HouseLockdown(P_PLAYER Player, P_ITEM pi) // Abaddon
// PRE:		S is the socket of a valid owner/coowner and is in a valid house
// POST:	either locks down the item, or puts a message to the owner saying he's a moron
// CODER:	Abaddon
// DATE:	17th December, 1999
// revamped by Duke, 25.3.2002
{
	if (!Player) return;
	if (!pi)
	{
		Player->Msg("Invalid item!" );
		return;
	}

	if (Items->isFieldSpellItem(pi) || pi->corpse)
	{
		Player->Msg("you cannot lock this down!");
		return;
	}
	if (pi->isType(ITY_Door) || pi->isType(ITY_LockedDoor) || pi->isType(ITY_Sign))
	{
		Player->Msg("You cant lockdown doors or signs!");
		return;
	}
	short id = pi->id();
	if ( IsAnvil(id) )
	{
		Player->Msg("You cant lockdown anvils!");
		return;
	}
	if ( IsForge(id) )
	{
		Player->Msg("You cant lockdown forges!");
		return;
	}
	if (pi->isLockedDown())
	{
		Player->Msg("That item is already locked down, release it first!");
		return;
	}

	PC_ITEM House = findHouseAt( pi->pos );
	if (House && Player->Owns(House))
	{
		int secu=0;
		int amt = countLockedDowns(Player,House,secu);
		// time to lock it down!
		if (amt < House->lockdownmax)
		{
			pi->LockDown(Player->serial);
		}
		else
		{
			sprintf(temp,"this house has a %i lockdown limit.", House->lockdownmax);
			Player->Msg(temp);
		}
	}
	else
		Player->Msg("That item is not in your house!" );
}

void cTargets::HouseSecureDown(P_PLAYER Player, P_ITEM pi) // Ripper
// For locked down and secure chests
{
	if (!Player) return;
	if (!pi)
	{
		Player->Msg("Invalid item!" );
		return;
	}
	
	if (Items->isFieldSpellItem(pi) || pi->corpse)
	{
		Player->Msg("you cannot lock this down!");
		return;
	}
	if (pi->isType(ITY_Door) || pi->isType(ITY_LockedDoor) || pi->isType(ITY_Sign))
	{
		Player->Msg("You cant lockdown doors or signs!");
		return;
	}
	if (pi->isLockedDown())
	{
		Player->Msg("That item is already locked down, release it first!");
		return;
	}
	if (!pi->isType(ITY_Container))
	{
		Player->Msg("You can only secure chests!");
		return;
	}
	
	PC_ITEM House = findHouseAt( pi->pos );
	if (!(House && Player->Owns(House)))
	{
		Player->Msg("That item is not in your house!" );
		return;
	}

	int secu=0;
	countLockedDowns(Player,House,secu);
	// time to lock it down!
	if (secu < House->securemax)
	{
		pi->LockDown(Player->serial);
		pi->secureIt = 1;
	}
	else
	{
		sprintf(temp,"this house has a %i lockdown limit.", House->securemax);
		Player->Msg(temp);
	}
}


void cTargets::HouseRelease(P_PLAYER Player, P_ITEM pi) // Abaddon & Ripper
// PRE:		S is the socket of a valid owner/coowner and is in a valid house, the item is locked down
// POST:	either releases the item from lockdown, or puts a message to the owner saying he's a moron
// CODER:	Abaddon
// DATE:	17th December, 1999
// update: 5-8-00
// revamped by Duke, 25.3.2002
{
	if (!Player) return;
	if (!pi)
	{
		Player->Msg("Invalid item!" );
		return;
	}
	
	if (!Player->Owns(pi))
	{
		Player->Msg("This is not your item!");
		return;
	}
	if (Items->isFieldSpellItem(pi))
	{
		Player->Msg("you cannot release this!");
		return;
	}
	if (pi->isType(ITY_Door) || pi->isType(ITY_LockedDoor) || pi->isType(ITY_Sign))
	{
		Player->Msg("You cant release doors or signs!");
		return;
	}

	PC_ITEM House = findHouseAt( pi->pos );
	if (!(House && Player->Owns(House)))
	{
		Player->Msg("That item is not in your house!" );
		return;
	}

	// ok, release it
	if ( pi->isLockedDown() || pi->isType(ITY_Container))
	{
		pi->ReleaseLockedDown();
	}
}

void cTargets::SetMurderCount(int s, P_CHAR pc)
{
	if (!pc) return;
	
	pc->kills = addmitem[s];
	setcharflag(pc);
}

void cTargets::GlowTarget(P_PLAYER Player, P_ITEM pi1) // LB 4/9/99, makes items glow
{
	if (!Player) return;
	if (!pi1)
	{
		Player->Msg("Item not found.");
		return;
	}

	if (!pi1->isInWorld())
	{
		P_CHAR pc = GetPackOwner(pi1);
		if (!Player->isSameAs(pc))	// creation only allowed in the creators pack/char otherwise things could go wrong
		{
			Player->Msg("you can't create glowing items in other persons packs or hands");
			return;
		}
	}

	if (pi1->glow!=0)
	{
		Player->Msg("that object already glows!\n");
		return;
	}

	pi1->glow_color=pi1->color(); // backup old color
	pi1->setColor(0x99); // set new color to yellow
	cCoord hpos;
	if (pi1->layer==0) // if not equipped -> coords of the light-object = coords of the
	{
		hpos=pi1->pos;
	}
	else // if equipped -> place lightsource at player ( height= approx hand level )
	{
		hpos=Player->pos;
		hpos.z+=4;
	}

	P_ITEM Halo = Items->SpawnItem(ITY_None,1,"glower",0x1647,0,hpos); // new client 1.26.2 glower object
	if (!Halo) return;
	Halo->dir=29; // set light radius maximal
	Halo->visible=0;
	Halo->layer=pi1->layer;
	Halo->resetPriv(); // doesnt decay
	pi1->glow=Halo->serial; // set glow-identifier

	pi1->refresh();
	Halo->refresh();

	impowncreate(Player->cs,Player,0); // if equipped send new color too
}

void cTargets::UnglowTarget(P_PLAYER Player, P_ITEM pi) // LB 4/9/99, removes the glow-effect from items
{
	if (!Player) return;
	if (!pi)
	{
		Player->Msg("no item found");
		return;
	}

	if (!pi->isInWorld())
	{
		P_CHAR pc = GetPackOwner(pi);
		if (!Player->isSameAs(pc))	// creation only allowed in the creators pack/char otherwise things could go wrong
		{
			Player->Msg("you can't unglow items in other perons packs or hands");
			return;
		}
	}

	int c=pi->glow;
	if (c==-1) return;
	P_ITEM Halo=FindItemBySerial(c);

	if (pi->glow==0 || !Halo )
	{
		Player->Msg("that object doesnt glow!\n");
		return;
	}

	pi->setColor(pi->glow_color);	// restore old color

	Items->DeleItem(Halo); // delete glowing object

	pi->glow=0; // remove glow-identifier
	pi->refresh();

	impowncreate(Player->cs,Player,0); // if equipped send new old color too

	Player->removeHalo(pi);
}

void cTargets::ShowSkillTarget(P_PLAYER Player, P_CHAR pc) // LB's showskills
{
	// the gump doesn't support '\n', so we must use blank padding
	int a,j,k,b=0,c,z,zz,ges=0;
	char skill_info[(ALLSKILLSMAX+1)*40];
	
	if (!pc) return;
	if (!Player->isGMorCounselor() && !(pc->tamed && Player->Owns(pc)))
	{
		Player->Msg("You can only use this on your own creatures !");
		return;
	}
	
	z=addx[Player->cs];
	if (z<0 || z>3) z=0;
	if (z==2 || z==3)
		sprintf(skill_info, "%s's skills:", pc->name);
	else
		sprintf(skill_info, "%s's baseskills:", pc->name);
	
	b=strlen(pc->name)+11;
	if (b>23) b=23;
	
	for (c=b;c<=30;c++)
		strcpy(&skill_info[strlen(skill_info)], " ");
	
	char ttt[100];
	int sk=SkillManager->getAllSkillsValue();
	for (a=0;a<sk;a++)
	{
		if (z==0 || z==1) k=pc->getRealSkill(a); else k=pc->Skill(a);
		if (z==0 || z==2) zz=9; else zz=-1;
		
		if (k>zz) // show only if skills >=1
		{
			j=k/10;
			ges+=j;
			sprintf(temp, "%s %i", SkillManager->getSkillName(a),j);
			sprintf(ttt, "%-30.30s", temp);
			strcat(skill_info,ttt);
		}
	}
	sprintf(temp,"sum: %i  ",ges);
	strcat(skill_info,temp);
	
	SndUpdscroll(Player->cs, strlen(skill_info), skill_info);
}

void cTargets::FetchTarget(P_PLAYER Player) // Ripper
{
	Player->Msg("Fetch is not available at this time.");
}

void cTargets::GuardTarget(P_PLAYER Player, P_CHAR pToGuard)
{
	P_CHAR Pet = FindCharBySerial(addx[Player->cs]);
	if (Pet == NULL)
	{
		Player->Msg("Currently can't guard anyone but yourself.."); 
		LogError("Lost pet serial");
		return;
	}

	if ( !pToGuard || !pToGuard->Owns(Pet) )
	{
		Player->Msg("Currently can't guard anyone but yourself!" );
		return;
	}
	Pet->setAi(32); // 32 is guard mode
	Pet->SetFollowTarget(pToGuard);
	Pet->npcWander=1;
	Player->Msg("Your pet is now guarding you.");
	pToGuard->guarded = true;
}

void cTargets::ResurrectionTarget(P_PLAYER Player, P_CHAR pc)
{
	if (!pc) return;
	
	if (pc->dead)
		pc->resurrect();
	else
		Player->Msg("That person isn't dead");
}


static void ItemTarget(P_PLAYER Player, cPacket6C *pt)
{
	UOXSOCKET s = Player->cs;
	P_ITEM pi=FindItemBySerial(pt->Tserial);
	if (pi==NULL) return;
	switch(pt->Tnum)
	{
	case 10://MoreTarget
		pi->setMore1_4(getAddid1_4(s));
		pi->refresh();
		break;
	case 90://MoreTarget
		pi->setMoreB1_4(getAddid1_4(s));
		pi->refresh();
		break;
	case  28://MovableTarget
		pi->setMovability(addx[s]);
		pi->refresh();
		break;
	case 31://ColorsTarget
		if (pi->id()==0x0FAB ||						//dye vat
			pi->id()==0x0EFF || pi->id()==0x0E27 )	//hair dye
			SndDyevat(s,pi->serial,pi->id());
		else
			Player->Msg("You can only use this item on a dye vat.");
		break;
	case 63://MoreXTarget
		pi->morex=addx[s];
		break;
	case 64://MoreYTarget
		pi->morey=addx[s];
		break;
	case 65://MoreZTarget
		pi->morez=addx[s];
		break;
	case 66://MoreXYZTarget
		pi->morex=addx[s];
		pi->morey=addy[s];
		pi->morez=addz[s];
		break;
	case 89://ObjPrivTarget
		if (addid1[s]==0) pi->decayOff();
		if (addid1[s]==1) pi->decayOn();
		if (addid1[s]==3) pi->setPriv(addid2[s]);
		break;
	case 122://SetValueTarget
		pi->value=addx[s];
		break;
	case 123://SetRestockTarget
		pi->restock=addx[s];
		break;
	case 129://SetAmount2Target
		if (addx[s] > 64000) //Ripper..to fix a client bug for over 64k.
		{
			Player->Msg("No amounts over 64k in a pile!");
			return;
		}
		pi->amount2=addx[s];
		pi->refresh();
		break;
	case 133://SetWipeTarget
		pi->wipe=addid1[s];
		pi->refresh();
		break;
	}
}

void cTargets::LoadCannon(P_PLAYER Player, P_ITEM pi)
{
	if (!pi || !Player) return;
	
	//if ((pi->id1==0x0E && pi->id2==0x91) && pi->morez==0)
	if ((pi->getMore1_4()==getAddid1_4(Player->cs))||
		(addid1[Player->cs]==0xFF))
	{
		if (pi->morez==0 && iteminrange(Player,pi,2))
		{
			if (pi->morez==0)
				pi->setType(ITY_CastingItem);
			pi->morex=8;
			pi->morey=10;
			pi->morez=1;
			Player->Msg("You load the cannon.");
		}
		else
		{
			if (pi->more1=='\x00') Player->Msg("That doesn't work in cannon.");
			else Player->Msg("That object doesn't fit into cannon.");
		}
	}
}

void cTargets::DupeTarget(P_PLAYER Player, P_ITEM pi)
{
	int s = Player->cs;
	if (addid1[s]>=1)
	{
		if (pi)
		{
			for (int j=0;j<addid1[s];j++)
			{
				Commands->DupeItem(Player, pi, pi->amount); // lb bugfix
				Player->Msg("DupeItem done.");//AntiChrist
			}
		}
	}
}

static bool SetCharValFromKey(char* p, P_CHAR pc)
{
	char *pv = strpbrk(p,"=");
	if (pv==NULL) return false; // no "=" = NULL=crash
	*pv=0;		// split string here
	pv++;		// skip the equal sign

	bool ok=false;
	if (!strcmp(p,"RACE"))		{ pc->setRace(str2num(pv)); ok=true; }

	return ok;
}

static void SetcTarget(P_PLAYER Player, P_CHAR pc)
{
	char para[150];
	strcpy(para,xtext[Player->cs]);	// get the parm string
	char *pk=para,*pnk;	// ptr to keywords
	
	if (pk==NULL) return;

	int loopexit=0;
	while ( (pk && *pk && pk <= para+120) && ++loopexit<MAXLOOPS )
	{
		pnk = strpbrk(pk,",");
		if (pnk)
		{
			*pnk=0;		// split string here
			pnk++;		// skip the comma
		}
		if (! SetCharValFromKey(pk,pc) )
			Player->Msg("Malformed setc command error");
		pk=pnk;			// proceed to next key
	}
	updatechar(pc);
}

static bool SetItemValFromKey(char* p, P_ITEM pi)
{
	char *pv = strpbrk(p,"=");
	if (pv==NULL) return false; // no "=" = NULL=crash
	*pv=0;		// split string here
	pv++;		// skip the equal sign

	bool ok=false;
	if (!strcmp(p,"AMOUNT"))	{ pi->amount=str2num(pv); ok=true; }
	if (!strcmp(p,"DECAY"))		{ if (str2num(pv)) pi->decayOn(); else pi->decayOff(); ok=true; }
	if (!strcmp(p,"ID"))		{ pi->setId(str2num(pv)); ok=true; }
	if (!strcmp(p,"MOREX"))		{ pi->morex=str2num(pv); ok=true; }
	if (!strcmp(p,"MOREY"))		{ pi->morey=str2num(pv); ok=true; }
	if (!strcmp(p,"MOREZ"))		{ pi->morez=str2num(pv); ok=true; }
	if (!strcmp(p,"MOREM"))		{ pi->morem=str2num(pv); ok=true; }
	if (!strcmp(p,"MOVABLE"))	{ pi->setMovability(str2num(pv)); ok=true; }
	if (!strcmp(p,"NAME"))		{ strcpy(pi->name,pv); ok=true; }
	if (!strcmp(p,"TYPE"))		{ pi->setType(str2num(pv));ok=true; }

	return ok;
}

static void SetiTarget(P_PLAYER Player, P_ITEM pi)
{
	char para[150];
	strcpy(para,xtext[Player->cs]);	// get the parm string
	char *pk=para,*pnk;	// ptr to keywords
	
	if (pk==NULL) return;

	int loopexit=0;
	while ( (pk && *pk && pk <= para+120) && ++loopexit<MAXLOOPS )
	{
		pnk = strpbrk(pk,",");
		if (pnk)
		{
			*pnk=0;		// split string here
			pnk++;		// skip the comma
		}
		if (! SetItemValFromKey(pk,pi) )
			Player->Msg("Malformed seti command error");
		pk=pnk;			// proceed to next key
	}
	pi->refresh();
}

void CloneNPCTarget(P_PLAYER Player, P_CHAR source)
{
	if (source==NULL) return;
	
	if (!source->npc) 
	{
		Player->Msg("No clone babies allowed in Britannia");
		return;
	}
	
	P_CHAR clone=NULL;
	
	int a,b,c,d;
	int dx=0,dy=0,e,f;
	signed char nz=0;
	cCoord p(source->pos);
	bool o=false,ct;
	
	b=1;
	e=6;
	ct=true;

	UOXSOCKET s = Player->cs;
	if (addid1[s]>=2) b=addx[s]; else b=1;
	if (addid1[s]>=3) if (!addy[s]) ct=false; else ct=true;
	if (addid1[s]>=4) e = addz[s];
	
	f = e*e;
	if (f>6666) f = 6666;
	
	for (c=0; c<b;c++)
	{
		clone = Npcs->CloneNpc(source,ct); // create clone
		if (clone==NULL)
		{
			Player->Msg("Sorry, cloning didn't work");
			return;
		}
		
		for (a=0; a<f; a++) // try to place it in a random +-4*+-4 area around original
		{
			
			dy = rand()%e;
			dx = rand()%e;
			d  = rand()%5;
			
			switch(d)
			{
			case 0: dy=dy*-1; break;
			case 1: dx=dx*-1; break;
			case 2: dx=dx*-1; dy=dy*-1; break;
			}
			cCoord tryPos(source->pos);
			tryPos.Add(dx,dy);
			nz = Map->Height(tryPos, p.z);
			o = Map->CanMonsterMoveHere(tryPos, nz);
			if (o) break;
		}
		
		if (!o) 
		{  
			dx=0; 
			dy=0;
			nz = Map->Height(p, p.z);
		}
		cCoord nPos(p.x+dx, p.y+dy, nz,source->pos.map);
		clone->MoveTo(nPos);
		updatechar(clone);
	}
}

void PermActTarget(P_PLAYER Player, P_CHAR pc)
{
	unsigned short actId;
	unsigned short repeat;
	unsigned char speed;
	int params;
	unsigned char direction;

	if (pc==NULL) return;

	UOXSOCKET s = Player->cs;
	params=addid1[s];
	if (params!=5)
	{
	  Player->Msg("usage: permact actionId(0-0xff) repetations(0=inf,0-xffff) speed(0-xff) direction(0-7)" );
	  return;
	}

	actId=addx[s];
	repeat=addy[s];
	speed=addz[s];
	direction=tempint[s];

	SendPermActSinglePlayer(s, pc, actId, repeat, speed, direction);

}

void DclickedTriggerTarget(P_PLAYER Player, int serial)
{
	if (!Player) return;

	P_ITEM Ti = FindItemBySerial(serial);
	if (Ti)
	{
		triggerItemId(Player,Ti);
		return;
	}

	P_CHAR Tc = FindCharBySerial(serial);
	if (Tc)
	{
		triggernpc(Player,Tc,0);
		return;
	}

	triggerItemId(Player,NULL);
}

void UnequipTarget(P_PLAYER Player,P_ITEM pi)
{
	if (pi->contserial == Player->serial)
	{
		Player->Msg("Just drag it.");
		return;
	}
	if (!isCharSerial(pi->contserial))
	{
		Player->Msg("That item is not equipped.");
		return;
	}
	if (pi->layer==0x0B || pi->layer==0x10 || pi->layer==0x15)	// HAIRS, BEARD or BP
	{
		Player->Msg("That item can not be unequipped.");
		return;
	}
	P_CHAR pc = FindCharBySerial(pi->contserial);
	if (pc)
	{
		if (Player->isGM() ||		// players cant unequip items on other players or npc`s paperdolls.
			(pc->tamed && Player->Owns(pc)))	// unless it's a mercenary
		{
			P_ITEM Pack = pc->getBackpack();
			if (Pack)
				Pack->AddItem(pi);
		}
	}
}

void IncZTarget(P_PLAYER Player, P_CHAR pc, P_ITEM pi, int zDelta)
{
	if (Player->isGM())
	{
		if (pc)
		{
			pc->dispz=pc->pos.z = zDelta+pc->pos.z;
			updatechar(pc);
		}
		if (pi)
		{
			pi->pos.z = zDelta+pi->pos.z;
			pi->refresh();
		}
	}
	else
	{
		if (pi)
		{
			PC_ITEM House = findHouseAt( pi->pos );
			if (House && Player->Owns(House))
			{
				if (pi->pos.z + zDelta + Map->TileHeight(pi->id()) <= Player->pos.z + 20	// don't scratch the ceiling
					&& pi->pos.z + zDelta >= Player->pos.z)		// not below floor level
				{
					pi->pos.z = zDelta+pi->pos.z;
					pi->refresh();
				}
				else
					Player->Msg("That would be too much.");
			}
			else
				Player->Msg("You can only use that in your own house.");
		}
	}
}

void cTargets::MultiTarget(P_PLAYER Player) // If player clicks on something with the targetting cursor
{
	UOXSOCKET s = Player->cs;
	targetok[s]=0;
	
	
	cPacket6C tbuf(&buffer[s][0]), *pt=&tbuf;
	
	if (pt->TxLoc==-1 && pt->TyLoc==-1) // do nothing if user cancelled
	{
		if (pt->Tserial==0 && pt->model==0) // this seems to be the complete 'cancel'-criteria (Duke)
			return;
	}
	if (Player->dead && !Player->isGM())
	{
		Player->Msg("You can't target while you are dead");
		return;
	}
	bool Iready=false, Cready=false;
	P_ITEM pi = NULL;
	P_CHAR pc = NULL;
	if (isItemSerial(pt->Tserial))
	{
		pi=FindItemBySerial(pt->Tserial);
		if (pi)
		{
			Iready=true;
			// prototype of new item-interface (Duke, 10.11.2005)
			P_ITEM Origin = FindItemBySerial(Player->dclickedserial);
			cItemController* ic = Items->getController(Origin);
			if (ic)
			{
				ic->onProcessTarget(Player,pt->Tnum,pi);
				delete ic;
				return;
			}
		}
	}
	else
	{
		pc=FindCharBySerial(pt->Tserial);
		if (pc!=NULL)
		{
			Cready=true;
			if (pc->stablemaster_serial)
				return;		// can't target stabled/mounted pets
		}
	}
	cCoord targpos(pt->TxLoc,pt->TyLoc,pt->TzLoc+Map->TileHeight(pt->model),Player->pos.map);
	
	switch(pt->Tnum)
	{
	case T_0: AddTarget(Player,pt); break;
	case T_1: { cRenameTarget		T(Player);		T.process();} break;
	case T_2: TeleTarget(Player,pt); break; // LB, bugfix, we need it for the /tele command
	case T_3: { cRemoveTarget		T(Player);		T.process();} break;
	case T_4: DyeTarget(Player, pt->Tserial); break;
	case T_5: { cNewzTarget			T(Player);		T.process();} break;
	case T_6: if (Iready) pi->setType(addx[s]); break; //Typetarget
	case T_7: Targ->IDtarget(s,pc,pi); break;
	case T_8:	if (Cready) XgoTarget(Player,pc); break;
	case T_9: if (Cready) PrivTarget(Player,pc); break;
	case T_10: ItemTarget(Player,pt); break;//MoreTarget
	case T_11: if (Iready) KeyTarget(Player,pi); break;
	case T_12: Targ->IstatsTarget(Player,pt); break;
	case T_13: if (Cready) CstatsTarget(Player,pc); break;
	case T_14: if (Cready) GMTarget(Player,pc); break;
	case T_15: if (Cready) CnsTarget(Player,pc); break;
	case T_16: if (Cready) DeleteItemOnLayer(pc, 0x0b); break;
	case T_17: if (Cready) DeleteItemOnLayer(pc, 0x10); break;
	case T_18: if (Cready) DeleteItemOnLayer(pc, 0x15); break;
	case T_19: if (Cready) pc->fonttype=addid1[s]; break;
	case T_20: if (Cready) Targ->GhostTarget(Player,pc); break;
	case T_21: if (Cready) Targ->ResurrectionTarget(Player,pc); break; // needed for /resurrect command
	case T_22: { cBoltTarget		T(Player);	T.process();} break;
	case T_23: { cSetAmountTarget	T(Player);	T.process();} break;
	case T_24: DclickedTriggerTarget(Player, pt->Tserial);	break;
	case T_25: if (Cready) Targ->KickTarget(Player,pc); break;
	case T_26: Targ->AddMenuTarget(s, addmitem[s],targpos); break;
	case T_27: Npcs->AddRespawnNPC(s,addmitem[s]); break;
	case T_28: ItemTarget(Player,pt); break;//MovableTarget
	case T_29: Skills->ArmsLoreTarget(Player,pi); break;
	case T_30: if (Cready) OwnerTarget(Player,pc); else if (Iready) OwnerTarget(Player,pi); break;
	case T_31: ItemTarget(Player,pt); break;//ColorsTarget
	case T_32: Targ->DvatTarget(Player,pi); break;
	case T_33: AddNpcByIdTarget(s,targpos); break;
	case T_34: if (Cready) pc->Freeze(); break;
	case T_35: if (Cready) pc->Unfreeze(); break; // unfreeze, AntiChris used LB bugfix
	case T_36: if (Cready) Targ->AllSetTarget(s,pc); break;
	case T_37: Skills->AnatomyTarget(Player,pc); break;
	case T_38: OldMagic->Recall(Player,pi); break;
	case T_39: OldMagic->Mark(Player,pi); break;
	case T_40: Skills->ItemIdTarget(Player,pi); break;
	case T_41: Skills->EvaluateIntTarget(Player,pc); break;
	case T_42: Skills->TameTarget(Player,pc); break;
	case T_43: OldMagic->makeGates(Player,pi); break;
	case T_44: OldMagic->Heal(Player,pc); break; // we need this for /heal command
	case T_45: Skills->FishTarget(Player,targpos,pt->model); break;
	case T_46: InfoTarget(Player,pt); break;
	case T_47: if (Cready) strcpy(pc->title,xtext[s]); break;//TitleTarget
	case T_48: if (Iready) Skills->CookOnFire(Player,pi); break;
//	case T_49: Skills->CookOnFire(s,Player,pi,0x097B,"fish steaks"); break;
	case T_50: Skills->Smith(Player,pi); break;
	case T_51: Skills->Mine(Player,targpos,pt->model); break;
	case T_52: Skills->SmeltOre(Player,pi); break;
	case T_53: if (Cready) pc->DoAction(addid1[s]); break;
//	case T_54: Skills->CookOnFire(s,Player,pi,0x09B7,"bird"); break;
//	case T_55: Skills->CookOnFire(s,Player,pi,0x160A,"lamb"); break;
	case T_56: if (Cready) Targ->NpcTarget(Player,pc); break;
	case T_57: if (Cready) Targ->NpcTarget2(s,pc); break;
	case T_58: DevtestTarget(Player,pt,targpos); break;
	case T_59: if (Cready) Targ->NpcCircleTarget(s,pc); break;
	case T_60: if (Cready) Targ->NpcWanderTarget(s,pc); break;
	case T_61: Targ->VisibleTarget(s,pc,pi); break;
	case T_62: Targ->TweakTarget(s,pc,pi); break;
	case T_63: //MoreXTarget
	case T_64: //MoreYTarget
	case T_65: //MoreZTarget
	case T_66: ItemTarget(Player,pt); break;//MoreXYZTarget
	case T_67: if (Cready) Targ->NpcRectTarget(s,pc); break;
//	case T_68: Skills->CookOnFire(s,Player,pi,0x09F2,"ribs"); break;
//	case T_69: Skills->CookOnFire(s,Player,pi,0x1608,"chicken legs"); break;
	case T_70: Skills->TasteIDTarget(Player,pi); break;
	case T_71: if (Iready) ContainerEmptyTarget1(Player,pi); break;
	case T_72: if (Iready) ContainerEmptyTarget2(Player,pi); break;
	case T_75: if (Iready) Targ->TargIdTarget(Player,pi); break;
	case T_76: AxeTarget(Player,pt,targpos); break;
	case T_77: Skills->DetectHidden(Player,targpos); break;
	case T_78: placeTownHouse(Player,targpos); break;
	case T_79: Skills->ProvocationTarget1(Player,pc); break;
	case T_80: Skills->ProvocationTarget2(s,Player,pc); break;
	case T_81: Skills->EnticementTarget1(Player,pc); break;
	case T_82: Skills->EnticementTarget2(Player,pc); break;
	case T_83: Boats->PlaceShipYard(s,Player,targpos); break;
	case T_84: if (Iready) Boats->AddShipPart(Player,pi); break;
	case T_85: if (Iready) UnequipTarget(Player,pi); break;
	case T_86: Targ->SwordTarget(Player,pt); break;
//	case T_87: if (Iready) OldMagic->SbOpenContainer(s,pi); break; // command doesnt make sens with new spell-storing system
	case T_88: Targ->SetDirTarget(s,pc,pi); break;
	case T_89: ItemTarget(Player,pt); break;//ObjPrivTarget
	case T_90: ItemTarget(Player,pt); break;//More2Target
		
	case T_100: OldMagic->NewCastSpell(Player,pt->Tserial); break;	// we now have this as our new spell targeting location

	// the following is for new magic (thrawn)
	case T_101:
		cSpell* spell;
		spell = Player->GetCastingSpell();
		if (!spell) return;

		if ((spell->GetTargetType()&MAP_TARGET)==MAP_TARGET) // FIRST see whether we need a map target
		{
			Magic->FinishCasting(Player,&targpos,MAP_TARGET); // targeted an item
		}
		else if (Cready && ((spell->GetTargetType()&CHAR_TARGET)==CHAR_TARGET) ) 
		{
			Magic->FinishCasting(Player,pc,CHAR_TARGET); // targeted a char and need a char
		}
		else if (Iready && ((spell->GetTargetType()&ITEM_TARGET)==ITEM_TARGET) )
		{
			Magic->FinishCasting(Player,pi,ITEM_TARGET); // targeted an item and need an item
		}
		else Player->Msg("That is not a valid target for this spell!"); // invalid target for the spell

		break;
	// end magic stuff

	case T_105: if (Cready) openspecialbank(Player,pc); break;//AntiChrist
	case T_106: if (Cready) Targ->NpcAITarget(Player,pc); break;
	case T_107: if (Cready) openbank(s, pc); break;
	case T_108: Skills->AlchemyTarget(Player,pi); break;
	case T_109: Skills->BottleTarget(Player,pi); break;
	case T_110: if (Iready) Targ->DupeTarget(Player,pi); break;
	case T_111: ItemTarget(Player,pt); break;//MovableTarget
	case T_112: if (Cready) sellstuff(Player, pc); break;
	case T_113: Targ->ManaTarget(Player,pc); break;
	case T_114: Targ->StaminaTarget(Player,pc); break;
	case T_115: if (Cready) Targ->GmOpenTarget(Player,pc); break;
	case T_116: if (Cready) Targ->MakeShopTarget(Player,pc); break;
	case T_117: if (Cready) Targ->FollowTarget(s,pc); break;
	case T_118: if (Cready) Targ->AttackTarget(Player,pc); break;
	case T_119: if (Cready) Targ->TransferTarget(s,pc); break;
	case T_120: if (Cready) Targ->GuardTarget(Player,pc); break;
	case T_121: if (Cready) Targ->BuyShop(Player, pc); break;
	case T_122: ItemTarget(Player,pt); break;//SetValueTarget
	case T_123: ItemTarget(Player,pt); break;//SetRestockTarget
	case T_124: Targ->FetchTarget(Player); break;
		
	case T_126: if (Cready) Targ->JailTarget(Player,pc); break;
	case T_127: Targ->ReleaseTarget(Player,pc); break;
	case T_128: Skills->CreateBandageTarget(Player,pi); break;
	case T_129: ItemTarget(Player,pt); break;//SetAmount2Target
	case T_130: Skills->HealingSkillTarget(Player,pc); break;
	case T_131: if (Cready) Targ->permHideTarget(Player,pc); break;
	case T_132: if (Cready) Targ->unHideTarget(Player,pc); break;
	case T_133: ItemTarget(Player,pt); break;//SetWipeTarget
	case T_134: Skills->Carpentry(Player,pi); break;
	case T_135: if (Cready) Targ->SetSpeechTarget(Player,pc); break;
	case T_136: Targ->XTeleport(Player,0); break;
		
	case T_150: if (Cready) pc->spattack=tempint[s]; break;
	case T_151: if (Cready) Targ->FullStatsTarget(s,pc); break;
	case T_152: Skills->BeggingTarget(Player,pc); break;
	case T_153: Skills->AnimalLoreTarget(Player,pc); break;
	case T_154: Skills->ForensicsTarget(Player,pi); break;
	case T_155: target(Player, T_156, "What item do you want to poison?");	break;
	case T_156: Skills->PoisoningTarget(Player,pi); break;
		
	case T_160: if (Iready) Skills->Inscribe(Player,pi); break;
		
	case T_162: Skills->LockPick(Player,pi); break;
		
	case T_164: Skills->Wheel(Player,pi, YARN); break;
	case T_165: Skills->Loom(Player,pi); break;
	case T_166: Skills->Wheel(Player,pi, THREAD); break;
	case T_167: Skills->Tailoring(Player,pi); break;
		
	case T_170: if (Iready) Targ->LoadCannon(Player,pi); break;
	case T_171: OldMagic->BuildCannon(Player,targpos); break;
	case T_172: Skills->Fletching(Player,pi); break;
	case T_173: Skills->MakeDough(Player,pi); break;
	case T_174: Skills->MakePizza(Player,pi); break;
	case T_175: if (Cready) pc->SetPoison(tempint[s]); break;
	case T_176: if (Cready) pc->PoisonStart(pc->getSlot(),tempint[s]); break;
	case T_177: if (Cready) pc->spadelay=tempint[s]; break;
	case T_178: if (Cready) pc->advobj=tempint[s]; break;
	case T_179: if (Cready) SetInvulFlag(Player,pc); break;
	case T_180: if (Iready) Skills->Tinkering(Player,pi); break;
	case T_181: Skills->PoisoningTarget(Player,pi); break;
		
	case T_183: Skills->TinkerAxel(Player,pi); break;
	case T_184: Skills->TinkerAwg(Player,pi); break;
	case T_185: Skills->TinkerClock(Player,pi); break;
	case T_186: vialtarget(Player,pt->Tserial); break;
		
	case T_198: Tiling(Player,pt); break;
	case T_199: Targ->Wiping(Player,pt); break;
		
	case T_200: Commands->SetItemTrigger(Player,pi); break;
	case T_201: Commands->SetNPCTrigger(Player,pc); break;
	case T_202: if (Iready) Commands->SetTriggerType(Player,pi); break;
	case T_203: Commands->SetTriggerWord(Player,pc); break;
	case T_204: if (Iready) triggerItem(Player,pi); break;
	case T_205: Skills->StealingTarget(Player,pt->Tserial); break;
	case T_206: if (Cready) Targ->CanTrainTarget(Player,pc); break;
	case T_207: ExpPotionTarget(Player,targpos); break;
	case T_209: if (Cready) pc->split=tempint[s]; break;
	case T_210: if (Cready) pc->splitchnc=tempint[s]; break;
	case T_212: Commands->Possess(Player); break;
	case T_213: Skills->PickPocketTarget(Player); break;
	case T_214: TeleStuff1(Player,pt); break;
	case T_215: TeleStuff2(Player,pt,targpos); break;
		
	case T_220: if (Cready) Guilds->Recruit(Player,pc); break;
	case T_221: if (Cready) Guilds->TargetWar(Player,pc); break;
	case T_223: if (Cready) Targ->SquelchTarg(Player,pc); break;//Squelch
	case T_224: if (Iready) Targ->PlVBuy(Player,pi); break;//PlayerVendors
	case T_225: if (Cready) Targ->Priv3XTarget(Player,pc); break; // SETPRIV3 +/- target
	case T_226: if (Cready) Targ->ShowPriv3Target(s,pc); break; // SHOWPRIV3
	case T_227: if (Cready) Targ->HouseOwnerTarget(Player,pc); break; // cj aug11/99
	case T_228: if (Cready) Targ->HouseEjectTarget(Player,pc); break; // cj aug11/99
	case T_229: if (Cready) Targ->HouseBanTarget(Player,pc); break; // cj aug12/99
	case T_230: if (Cready) Targ->HouseFriendTarget(Player,pc); break; // cj aug 12/99
	case T_231: if (Cready) Targ->HouseUnlistTarget(Player,pc); break; // cj aug 12/99
	case T_232: Targ->HouseLockdown(Player,pi); break; // Abaddon 17th December 1999
	case T_233: Targ->HouseRelease(Player,pi); break; // Abaddon 17th December 1999
	case T_234: Targ->HouseSecureDown(Player,pi); break; // Ripper
	case T_235: Targ->BanTarg(s, pc); break;
	case T_236: Skills->RepairTarget(Player,pi); break; //Ripper..Repairing item
	case T_237: Skills->SmeltItemTarget(Player,pi); break; //Ripper..Smelting item
		//taken from 6904t2(5/10/99) - AntiChrist
	case T_238: Targ->UnBanTarg(s, pc); break;
	case T_240: if (Cready) Targ->SetMurderCount(s,pc); break; // Abaddon 13 Sept 1999
		
	case T_245: if (!Cready && !Iready) buildHousePass2(Player,addx[s]);	break;
	case T_246: if (!Cready && !Iready) AddTarget(Player,pt); break;
	case T_247: if (Cready) Targ->ShowSkillTarget(Player,pc);break;
	//case T_248: Targ->MenuPrivTarg(s);break; // menupriv target
	case T_249: if (Iready) Targ->UnglowTarget(Player,pi);break; // unglow
	case T_250: if (Cready) Priv3Target(s,pc); break; // meta gm target
	case T_251: Targ->NewXTarget(s,pc,pi); break; // NEWX
	case T_252: Targ->NewYTarget(s,pc,pi); break; // NEWY
	case T_253: Targ->IncXTarget(s,pc,pi); break; // INCX
	case T_254: Targ->IncYTarget(s,pc,pi); break; // INCY
	case T_255: Targ->GlowTarget(Player,pi); break; // glow
		
	case T_258: if (Cready) SetcTarget(Player,pc);break;
	case T_259: if (Cready) Npcs->TempDupeNpcRevert(pc); break;
//	case T_260: { cInczTarget		T(s);	T.process(); } break;
	case T_260: if (Cready || Iready) IncZTarget(Player,pc,pi,addx[s]); break;
	case T_261: if (Iready) SetiTarget(Player,pi);break;
	case T_262: if (Cready) CloneNPCTarget(Player, pc); break;
	case T_263: if (Cready) PermActTarget(Player, pc); break;
	case T_264: if (Cready) npctalkall(pc,xtext[s]); break;
		
	case T_299: TilingScript(Player, pt, targpos); break;
	case T_300: KillNpcs(Player,targpos); break;
	case T_301: SwapNpcsTarget(Player, pt); break;
	case T_302: if (Cready) MakeNpcItem(Player, pc); break;

	case T_350: if (Cready) PartySystem->RequestAuthorization(Player,pc); break;
	case T_351: if (Cready) PartySystem->RemoveMemberTarg(Player,pc); break;
		
	default:
		LogErrorVar("Fallout of switch statement, multitarget(), value=(%i)",pt->Tnum);
	}

	Player->target1serial = pt->Tserial;	// remember what they targeted
}

void target(P_PLAYER Player, eTargets targettype, char *txt) // Send targetting cursor to client
{
	if (Player)
		SndTarget(Player,targettype,txt);
}
