/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

/////////////////////
// Name:	teleport.cpp
// Purpose: functions that handle the teleporting
// History:	cut from wolfpack.cpp by Duke, 26.10.00
// Remarks:	not necessarily ALL those functions
//

#include "teleport.h"

#include "mapstuff.h"
#include "regions.h"
#include "scriptc.h"
#include "skills.h"
#include "SndPkg.h"
#include "mapS.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "teleport.cpp"

struct tele_locations_st 
{
	cCoord destination, origem;
};

std::multimap <int, tele_locations_st> tele_locations; // can't use a map here, so using a multimap with x as the key :(


void teleporters(P_CHAR pc)
{
	multimap<int,tele_locations_st>::iterator iter;
	multimap<int,tele_locations_st>::iterator upper;
	
	iter=tele_locations.lower_bound(pc->pos.x);
	upper=tele_locations.upper_bound(pc->pos.x);
	// if not found, iter=upper=end()
	
	int loopexit=0;
	for ( ; iter != upper; ++iter)
	{
		if ( iter->second.origem.IsSameXY(pc->pos)
			&& (iter->second.origem.z == illegal_z || pc->pos.z == iter->second.origem.z))
		{
			if (!pc->npc)
			{
				P_PLAYER Player = MakePlayerRef(pc);
				if (!Player) return;
				Player->telePets(iter->second.destination);
			}
			pc->teleTo(iter->second.destination);
			return;
		}	
		if (loopexit++ > MAXLOOPS) break;
	}
}

void read_in_teleport(void)
{
	char text[256];
	char seps[]	= " ,\t\n";
	char *token;
	Script *Scp;

	tele_locations.clear();

	Scp=getScript(teleport_script);
	if (!Scp)
	{
		Console.send("ERROR: Teleport Data not found\n");
		error=1;
		keeprun=0;
		return;
	}

	Scp->Open();

	int loopexit=0;
	while(Scp->NextLineRaw() && ++loopexit<MAXLOOPS*17)
	{
		strncpy(text, script1, 255);

		if (text[0]!=';')
		{
			tele_locations_st dummy;

			token = strtok( text, seps );
			if (token==NULL) break;
			
			dummy.origem.x = atoi(token);

			token = strtok( NULL, seps );
			if (token==NULL) break;

			dummy.origem.y = atoi(token);

			token = strtok( NULL, seps );
			if (token==NULL) break;

			if (token[0] == 'A')
				dummy.origem.z = illegal_z;
			else
				dummy.origem.z = atoi(token);

			token = strtok( NULL, seps );
			if (token==NULL) break;

			dummy.destination.x = atoi(token);

			token = strtok(NULL, seps );
			if (token==NULL) break;

			dummy.destination.y = atoi(token);
			token = strtok(NULL, seps);
			if (token==NULL) break;

			dummy.destination.z = atoi(token);
			token = strtok(NULL,seps);

			if (token==NULL) break;
			if (!isalpha(token[0])) // ... z map name
			{
				dummy.destination.map=atoi(token);
				dummy.origem.map=atoi(token);
			}
			else // ... z name
			{
				dummy.destination.map=AllMaps->getdefaultmap();
				dummy.origem.map=AllMaps->getdefaultmap();
			}

			tele_locations.insert(make_pair(dummy.origem.x, dummy));

			//sprintf(temp,"%i %i %i %i -> %i %i %i %i\n", dummy.origem.x, dummy.origem.y, dummy.origem.z, dummy.origem.map, dummy.destination.x, dummy.destination.y, dummy.destination.z, dummy.destination.map);
			//LogError(temp);
			
		}
	}

	Scp->Close();
}

int validtelepos(P_PLAYER pc)
{
	int z=-1;

	if (pc->pos.map==0)
	{
	  if ((pc->pos.x>=1397)&&(pc->pos.x<=1400)&&
		(pc->pos.y>=1622)&&(pc->pos.y<=1630))
		z=28;
	  if ((pc->pos.x>=1510)&&(pc->pos.x<=1537)&&
		(pc->pos.y>=1455)&&(pc->pos.y<=1456))
		z=15;
	}
	return z;
}

void advancementobjects(P_CHAR pc, int adv, int allways)
{
	if (!pc) return;

	if (pc->advobj!=0 && !allways)
	{
		pc->Msg("You have already used an advancement object with this character.");
		return;
	}

	staticeffect(pc, 0x373A, 0, 15);
	soundeffect2(pc, 0x01E9);
	pc->advobj=adv;

	Script* Scp=findSection(advance_script,"ADVANCEMENT",adv);
	if (!Scp)
	{
		LogWarningVar("ADVANCEMENT OBJECT: Script section %i not found.",adv);
		pc->advobj=0;
		return;
	}

	int loopexit=0;
	while (Scp->NextLineSplitted() && (++loopexit < 150))	// no more than 150 lines in one section
	{
		int rc = 0;
		if (!strcmp("STR",script1) || !strcmp("STRENGTH",script1))
		{
			pc->setStr(getstatskillvalue(script2));
		}
		if (!strcmp("DEX",script1) || !strcmp("DEXTERITY",script1))
		{
			pc->setDex(getstatskillvalue(script2));
		}
		if (!strcmp("INT",script1) || !strcmp("INTELLIGENCE",script1))
		{
			pc->setInt(getstatskillvalue(script2));
			pc->setDecInt(0);
		}
		
		rc = Npcs->applySkillTags(Scp,pc);
		if (rc) {if (rc==1) continue; else return;}
		
		rc = pc->applyCommonTags();
		if (rc) {if (rc==1) continue; else return;}
		
		if (!strcmp("DYEHAIR",script1))
		{
			cItem* Hair=pc->GetItemOnLayer(0x0B);
			if (Hair)
			{
				Hair->setColor(hstr2num(script2));
				Hair->refresh();
			}
		}
		
		if (!strcmp("DYEBEARD",script1))
		{
			cItem* Beard=pc->GetItemOnLayer(0x10);
			if (Beard)
			{
				Beard->setColor(hstr2num(script2));
				Beard->refresh();
			}
		}
		
		if (!strcmp("KILLHAIR",script1))
		{
			cItem* Hair=pc->GetItemOnLayer(0x0B);
			if (Hair)
				Items->DeleItem(Hair);
		}
		
		if (!strcmp("KILLBEARD",script1))
		{
			cItem* Beard=pc->GetItemOnLayer(0x10);
			if (Beard)
				Items->DeleItem(Beard);
		}
		
		if (!strcmp("KILLPACK",script1))
		{
			cItem* Pack=pc->GetItemOnLayer(0x15);
			if (Pack)
				Items->DeleItem(Pack);
		}
		if (!strcmp("ITEM",script1))
		{
			int x=str2num(script2);
			scpMark m; Scp->Suspend(m);
			P_ITEM pRetitem = Items->CreateScriptItem(x);
			Scp->Resume(m);
			strcpy(script1, "DUMMY");
			
			if (pRetitem!=NULL)
			{
				if (pRetitem->layer==0x0B || pRetitem->layer==0x10)
				{
					pRetitem->SetContSerial(pc->serial);
				}
				else
				{
					P_ITEM Pack=pc->GetItemOnLayer(0x15);
					if (Pack)
					{
						pRetitem->SetContSerial(Pack->serial);
						pRetitem->SetRandPosInCont(Pack);
					}
				}
				pRetitem->refresh();
			}
		}
		
		if (!strcmp(script1,"SKIN"))
		{
			pc->skin = pc->xskin = static_cast<UI16>(hstr2num(script2));
		}
		
		if (!strcmp("POLY",script1))
		{
			short x=hstr2num(script2);
			pc->setId(x);
			pc->setXid(x);
		}
		
		if (!strcmp("ADVOBJ",script1))
		{
			pc->advobj=str2num(script2);
		}
		
		if (!strcmp("RACE", script1))
		{
			pc->setRace(str2num(script2));
		}
	}

	SndRemoveNpcToAll(pc);	// force client to forget about the char. Needed for color change
	updatechar(pc);
	Scp->Close();
}


// dead code ???!
void monstergate(P_CHAR pc_s, int npcNum)
{
	int tmp, z, lovalue, hivalue;

	if (!pc_s) return;
	if (pc_s->npc) return;

	int storeval=-1;

	Script* Scp = findSection(npc_script, "NPC", npcNum);
	if (!Scp) return;

	cSpawnContext CT;
	CT.Init();

	pc_s->title[0] = 0;
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
	{
		if (pc_s->Wears(pi) &&
			pi->layer!=0x15 && pi->layer!=0x1D &&
			pi->layer!=0x10 && pi->layer!=0x0B)
		{
			if (!CT.MyPack)
			{
				CT.MyPack=pc_s->getBackpack();
			}
			if (!CT.MyPack)
			{
				scpMark m; Scp->Suspend(m);
				CT.MyPack = pc_s->addBackpack();
				if (!CT.MyPack) return;
				Scp->Resume(m);

				CT.LastItem=CT.MyPack;
			}

			if (CT.MyPack) 
			{
				P_ITEM pi2 = pi->unconst();
				pi2->SetRandPosInCont(CT.MyPack);
				pi2->SetContSerial(CT.MyPack->serial);
				pi2->layer=0x00;
				
				SndRemoveItemToAll(pi2);
				pi2->refresh();
			}
		}
		else if (pc_s->Wears(pi) &&
			(pi->layer==0x0B || pi->layer==0x10))
		{
			Items->DeleItem(pi->unconst());
		}
	}

	int loopexit=0;
	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS) )
	{
		int rc = pc_s->applyCommonTags();			// tags common to scp, wsc and monstergate
		if (rc) {if (rc==1) continue; else return;}
		
		rc = pc_s->applyBirthLootTags(Scp, npcNum, &CT);	// loot related tags in scp and monstergate
		if (rc) {if (rc==1) continue; else return;}
		
		if (!strcmp("NAMELIST", script1))
		{
			scpMark m; Scp->Suspend(m);
			setrandomname(pc_s,script2);
			Scp->Resume(m);
			
			strcpy(script1, "DUMMY"); // To prevent accidental exit of loop.
		}
		if (!strcmp("ID",script1))
		{
			tmp=hstr2num(script2);
			pc_s->setId(tmp);
			pc_s->setXid(tmp);
		}
		if (!strcmp("SKIN",script1))
		{
			pc_s->skin = pc_s->xskin = hstr2num(script2);
		}
		
		if (!strcmp("ITEM",script1))
		{
			storeval=str2num(script2);
			scpMark m; Scp->Suspend(m);
			CT.LastItem = Items->CreateScriptItem(storeval);
			Scp->Resume(m);
			if (CT.LastItem)
			{
				CT.LastItem->SetContSerial(pc_s->serial);
				if (CT.LastItem->layer==0)
				{
					Console.send("Warning: Bad NPC Script %d with problem item %d executed!\n", npcNum, storeval);
				}
			}
			strcpy(script1, "DUMMY"); // Prevents unexpected matchups...
		}
		
		if (!strcmp("COLOR",script1))
		{
			if (CT.LastItem)
			{
				CT.LastItem->setColor(hstr2num(script2));
			}
		}
		if (!strcmp("COLORLIST",script1))
		{
			if (CT.LastItem)
			{
				scpMark m; Scp->Suspend(m);
				storeval=addrandomcolor(str2num(script2));
				CT.LastItem->setColor(storeval);
				Scp->Resume(m);
				strcpy(script1, "DUMMY"); // To prevent accidental exit of loop.
			}
		}
		
		//--------------------- NEW STAT & SKILL FORMAT ----------------
		//Handle Stats
		if (!strcmp("STR",script1) || !strcmp("STRENGTH",script1))
		{
			pc_s->setStr(getstatskillvalue(script2));
			pc_s->fullHp();
		}
		if (!strcmp("DEX",script1) || !strcmp("DEXTERITY",script1))
		{
			pc_s->setDex(getstatskillvalue(script2));
			pc_s->fullStm();
		}
		if (!strcmp("INT",script1) || ! strcmp("INTELLIGENCE",script1))
		{
			pc_s->setInt(getstatskillvalue(script2));
			pc_s->setDecInt(0);
			pc_s->fullMana();
		}
		//Done Handling Stats
		
		//Handle Skills
		rc = Npcs->applySkillTags(Scp,pc_s);
		if (rc) {if (rc==1) continue; else return;}
		
		//Handle Extras
		if (!strcmp("DAMAGE",script1) || !strcmp("ATT",script1))
		{
			gettokennum(script2, 0);
			lovalue=str2num(gettokenstr);
			gettokennum(script2, 1);
			hivalue=str2num(gettokenstr);
			pc_s->lodamage = lovalue;
			pc_s->hidamage = lovalue;
			if (hivalue) pc_s->hidamage = hivalue;
		}
		if (!strcmp("DEF",script1)) pc_s->def = getstatskillvalue(script2);
		//Done Handling Extras
		
		//Handle Obsolete Stuff
		if (!strcmp("SKILL", script1))
		{
			gettokennum(script2, 0);
			z=str2num(gettokenstr);
			gettokennum(script2, 1);
			pc_s->setRealSkill(z,str2num(gettokenstr));
		}
		//Done Handling Obsolete Stuff
	}
	Scp->Close();

	//Now find real 'skill' based on 'baseskill' (stat modifiers)
	int sk=SkillManager->getAllSkillsValue();
	for(z=0;z<sk;z++)
	{
		SkillManager->updateSkillLevel(pc_s,z);
	}
	updatechar(pc_s);
	staticeffect(pc_s, 0x373A, 0, 15);
	soundeffect2(pc_s, 0x01E9);
}

// new function 
////////////////////////////////// 
// This will be just a body type switching 
// and coloring item. 
// Uses the integer number of the monster type 
// from the npc file. 
// Gate has to be of type 84. 
// 
// 
void polycolorgate(P_CHAR pc_s, int x) 
{ 
	if (!pc_s) return; 
	if (pc_s->npc) return; 

	Script* Scp = findSection(npc_script, "NPC", x); 
	if (!Scp) return; 

	int loopexit = 0; 
	while (Scp->NextLineSplitted() &&(++loopexit < MAXLOOPS)); 
	{ 
		if (!strcmp("ID", script1))
		{ 
			int tmp = hstr2num(script2); 
			pc_s->setId(tmp);
			pc_s->setXid(tmp);
		} 
		else if (!strcmp("SKIN", script1))
		{ 
			pc_s->skin = pc_s->xskin = static_cast<UI16>(hstr2num(script2));
		} 
	} 
	Scp->Close(); 
  
	updatechar(pc_s); 
	staticeffect(pc_s, 0x373A, 0, 15); 
	soundeffect2(pc_s, 0x01E9); 
} 
// 
// 
// Aldur 
////////////////////////////////// 

void objTeleporters(UOXSOCKET s, P_CHAR pc_s)
{
	PC_ITEM pmi;
	int loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(pc_s->pos);
	while((pmi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (pmi->pos.IsSameXY(pc_s->pos) &&
			((abs(pmi->pos.z) + 10) >= abs(pc_s->pos.z)) &&((abs(pmi->pos.z) - 10) <= abs(pc_s->pos.z)))
		{
			if (pmi->isType(ITY_WalkoverTeleporter) && (pmi->morex + pmi->morey + pmi->morez >0))
			{
				cCoord newPos(pmi->morex,pmi->morey,pmi->morez,pmi->morem);
				pc_s->travelTo(newPos,s);
			}
			
			// advancement objects
			if (pmi->isType(ITY_80) && !pc_s->npc)
			{
				if (pmi->getMore1_4() == 0
					|| pc_s->serial == pmi->getMore1_4())
					advancementobjects(pc_s, pmi->morex, 0);
			}
			
			if (pmi->isType(ITY_81) && !pc_s->npc)
			{
				if (pmi->getMore1_4() == 0
					|| pc_s->serial == pmi->getMore1_4())	// restrict a gate's use by setting its MORE values to a char's serial #
					advancementobjects(pc_s, pmi->morex, 1);
			}
			
			// damage objects
			if (!pc_s->isInvul() && pmi->isType(ITY_DamageObj))
			{
				pc_s->reduceHp(pmi->morex + RandomNum(pmi->morey, pmi->morez));
			}
			// monster gates
			if (pmi->isType(ITY_MonsterGate))
				monstergate(pc_s, pmi->morex);

			if (pmi->isType(ITY_84)) 		// This will be just a body type switching item. (Aldur)
				polycolorgate(pc_s, pmi->morex); 
			
			// sound objects
			if (pmi->isType(ITY_SoundObj))
			{
				if ((unsigned)RandomNum(1, 100) <= pmi->morez)
					soundeffect3(pmi, (pmi->morex << 8) + pmi->morey);
			}
		}
	}
}

static int moongate1;
static int moongate2;
static cCoord MoongatePositions[8];

void addMoongate(PC_ITEM pi)
{
	if (pi->isType(ITY_Moongate1))
		moongate1 = pi->serial;
	if (pi->isType(ITY_Moongate2))
		moongate2 = pi->serial;
}

void loadMoongatePositions(int pairnum)
{
	Script* Scp = findSection(location_script,"MOONGATE",1);
	if (!Scp) return;

	int i = 0;
	while (Scp->NextLineSplitted())
	{
		if (Scp->getKeyVal("POS",MoongatePositions[i]))
		{
			i++;
			if (i>7) break;
		}
	}
	if (i < 8)
		LogWarning("Not enough positions in section moongate 1");
	Scp->Close();
}

void moveMoongates(int moonchanged)
{
	P_ITEM mg1 = FindItemBySerial(moongate1);
	P_ITEM mg2 = FindItemBySerial(moongate2);
	if (!mg1 || !mg2) return;

	if (moonchanged == 1)
	{
		mg1->MoveTo(MoongatePositions[timeLight.moon1]);
		mg1->refresh();
		mg2->setMorePos(MoongatePositions[timeLight.moon1]);
	}
	else
	{
		mg2->MoveTo(MoongatePositions[timeLight.moon2]);
		mg2->refresh();
		mg1->setMorePos(MoongatePositions[timeLight.moon2]);
	}
}

