/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "walking.h"

#include <cmath>
#include <time.h>
#include "boats.h"
#include "Client.h"
#include "guilds.h"
#include "house.h"
#include "magic.h"
#include "mapstuff.h"
#include "msgboard.h"
#include "multi.h"
#include "npcinfo.h"
#include "regions.h"
#include "scriptc.h"
#include "skills.h"
#include "SndPkg.h"
#include "sregions.h"
#include "teleport.h"
#include "TmpEff.h"
#include "trigger.h"
#include "weight.h"
#include "tiledata.h"
#include "mapS.h"
#include "network.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "walking.cpp"

int xycount;
#define XYMAX 5000 // Maximum items LoneWolf can handle on one X/Y square
CArray<XYMAX, unitile_st>xyblock("xyblock");

#define EXTRAMARGIN 6
// margin for real (proper) npcs send area
#define REALSENDMARGIN 18


///////////////
// Name:	getLeftDir, getRightDir
// history:	by Duke, 14.7.2001
// Purpose:	calculates the adjacent direction
//
int getLeftDir(int dir)	// counterclockwise
{
	dir &= 7;	// make sure it's valid
	return dir==0 ? 7 : dir-1;
}
int getRightDir(int dir)	// clockwise
{
	dir &= 7;
	return dir==7 ? 0 : dir+1;
}

///////////////
// Name:	getXYfromDir
// history:	by Duke, 14.7.2001
// Purpose:	calculates and changes the given coords one step into the given direction
//
void GetXYfromDir(int dir, int *x, int *y)
{
	switch(dir&0x07)
	{
	case 0: (*y)--;				break; // N
	case 1: (*x)++; (*y)--;		break; // NO
	case 2: (*x)++;				break; // O
	case 3: (*x)++; (*y)++;		break; // SO
	case 4: (*y)++;				break; // S
	case 5: (*x)--; (*y)++;		break; // SW
	case 6: (*x)--;				break; // W
	case 7: (*x)--; (*y)--;		break; // NW
	}
}
void GetXYfromDir(int dir, cCoord& pos)
{
	int x = pos.x;
	int y = pos.y;
	GetXYfromDir(dir,&x,&y);
	pos.x = x;
	pos.y = y;
}
void GetValidXYfromDir(int dir, cCoord& pos)
{
	int x = pos.x;
	int y = pos.y;
	GetXYfromDir(dir,&x,&y);
	////////// out of bounds/world checks (LB) ///////////

	int max_x = AllMaps->GetMapWidth(pos.map)  * 8;
	int max_y = AllMaps->GetMapHeight(pos.map) * 8;

	if (y<0) y=0; else if (y>=max_y) y = max_y;
	if (x<0) x=0; else if (x>=max_x) x = max_x;
	// note: though item/char->pos.x/y UNSIGNED, here it got signedness back via (signed) int. why we need to check <0

	pos.x = x;
	pos.y = y;
}

///////////////
// Name:	WalkHandleAllowance
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	checks if the Char is allowed to move at all i.e. not frozwn, overloaded...
//
bool WalkHandleAllowance(UOXSOCKET s, P_CHAR pc, int sequence)
{
	if (!pc) return false;
	
	if (pc->stablemaster_serial>0 && s<0) return false; // shouldnt be called for stabled pets, just to be on the safe side

	if (s!=-1)
	{
		if (walksequence[s]+1!=sequence && sequence!=256)
		{
			deny(s, pc, sequence);
			return false;
		}
	}

	if (!pc->dead && !pc->npc && !pc->isGMorCounselor())
	{
		if (!Weight->CheckWeight(pc, s) || pc->Stm() < 2) // this has to executes, even if s==-1, thus that many !=-1's
		{
			if (s!=-1)
			{
				pc->Msg("You are too fatigued to move.");
				walksequence[s]=-1;
				teleport(pc, pc->pos); // old pos = currpos
				deny(s,pc,sequence); // !!!
				return false;
			}
		}
	}

	// dont put a if s!=-1 here !!!
	if (pc->IsFrozen()) // lord binary !!! 
	{
		teleport(pc, pc->pos); // old pos = currpos
		if (s>-1)
		{
			if (pc->IsCasting())
				pc->Msg("You cannot move while casting.");
			else
				pc->Msg("You are frozen and cannot move.");
			deny(s,pc,sequence);
		}
		return false;
	} 
	return true;
}

///////////////
// Name:	WalkHandleRunning
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	handles running, stealth etc.
//
bool WalkHandleRunning(UOXSOCKET s, P_CHAR pc, int dir)
{
	if (!pc) return true;
	if (dir&0x80) 
	{ //AntiChrist -- if running
		//AntiChrist - we have to unhide always if running
		pc->unhide();
		//Running stamina - AntiChrist
		pc->regen2=uiCurrentTime+(SrvParms->staminarate*MY_CLOCKS_PER_SEC); //Don't regenerate stamina while running
		pc->running++;

		// Horse Stamina loss bug Fixed by blackwind. 
		// Without loss of stamina players will be able to runaway for ever..
		if (!pc->dead)
		{
			if (((!pc->onhorse && pc->running > SrvParms->runningstaminasteps)*2) ||
				(pc->onhorse && pc->running > SrvParms->runningstaminasteps*2*2) )
			{ //The first *2 it's because i noticed that a step(animation) correspond to 2 walking calls
				pc->running=0;
				pc->reduceStm(1);
			}
		}
	}
	else 
	{
		pc->running=0;
		if (pc->stealth>-1) 
		{ //AntiChrist - Stealth
			if (pc->stealth > 0 &&		// give them at least one step, regardless of skill (Duke)
				pc->stealth > (SrvParms->maxstealthsteps*pc->Skill(STEALTH))/1000)
			{
				pc->unhide();
			}
			pc->stealth++;
		}
		else
		{
			pc->unhide();
		}
	}
	return true;
}

///////////////
// Name:	WalkCollectBlockers
// history:	cut from walking() by Duke, 20.11.2000
// Purpose:	Collects Landscape plus static and dynamic items in an array
//
// Remark LB 14'th nov 2000: will rewrite this stuff when I have time again for wp-project...
// dont delete the rem'd out Console.send's in meanwhile, thx !
//
void WalkCollectBlockers(cCoord& tpos)
{
	xycount=0;

	int mapid = 0;
	signed char mapz = Map->AverageMapElevation(tpos, mapid);
	if (mapz != illegal_z)
	{
		const landData *land;
		land = TileData->SeekLand(mapid);
		
		xyblock[xycount].type=0;
		xyblock[xycount].basez = mapz;
		xyblock[xycount].id = mapid;
		xyblock[xycount].flags=land->flags;
		xyblock[xycount].height=0;
		xyblock[xycount].weight=255;

		//printf("0 %s %s %08x %04x\n", land->name, pc->name, land->flags, mapid);

		xycount++;
	}

	PC_ITEM pi;
	int loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(tpos);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (!pi->isMulti()) // no multi
		{
			if (pi->pos.IsSameXY(tpos))
			{
				const tileData *tile;
				tile = TileData->SeekTile(pi->id());
				xyblock[xycount].type=1;
				xyblock[xycount].basez=pi->pos.z;
				xyblock[xycount].id=pi->id();
				xyblock[xycount].flags=tile->flags;			
				xyblock[xycount].height=tile->height;
				xyblock[xycount].weight=tile->weight;

				//printf("1 %s %08x %04x\n", pc->name, tile->flags, pi->id());

				xycount++;
			}
		}
		else // if multi tile
		{
			if (inRange(pi->pos, tpos, BUILDRANGE))
			{
				int LoopExit=0;
				singleMultiPart mult;
				assert(pi->id()>=0x4000);
				
				Multi->multiIteratorStart(pi->id()-0x4000);
				while (Multi->multiIteratorNext(mult) && ++LoopExit<MAXLOOPS)				
				{					
					if (mult.visibility && (pi->pos.x+mult.x == tpos.x) && (pi->pos.y+mult.y == tpos.y))
					{
						const tileData *tile;
						tile = TileData->SeekTile(mult.tile);				
						xyblock[xycount].type=2;
						xyblock[xycount].basez=mult.z+pi->pos.z;
						xyblock[xycount].id=mult.tile;
						xyblock[xycount].flags=tile->flags;					
						xyblock[xycount].height=tile->height;
						xyblock[xycount].weight=255;
						
						//printf("2:%s %s %i %08x %04x\n",tile->name, pc->name, mult.visibility, tile->flags, mult.tile);
						
						xycount++;
					}
				}
			}
		}
	}

	AllMaps->mapStaticIteratorStart(tpos);
	loopexit = 0;
	cellData stat;
	while ( AllMaps->mapStaticIteratorNext(stat) && (++loopexit < MAXLOOPS))
	{	
		const tileData *tile;
		tile = TileData->SeekTile(stat.TID);
		xyblock[xycount].type=2;
		xyblock[xycount].basez=stat.z;
		xyblock[xycount].id=stat.TID;
		xyblock[xycount].flags=tile->flags;	
		xyblock[xycount].height=tile->height;
		xyblock[xycount].weight=255;

		//printf("3:%s %s %08x %04x\n",tile->name, pc->name, tile->flags, stat.TID);

		xycount++;
	}								
}

///////////////
// Name:	WalkEvaluateBlockers
// history:	cut from walking() by Duke, 20.11.2000
// Purpose:	Decides if something in the array blocks the walker
//
// if HELL freezes I'll fully understand this cruft :)
// (=absolutely needs to be rewritten sometime soon)

#define W_ONLINE 1
#define W_DEAD 2
#define W_GM 4
#define W_CANMOVEALL 8
#define W_CANONLYSWIM 0x10

void WalkEvaluateBlockers(const int props, const signed char oldz, signed char *pz, signed char *pdispz)
{
	signed char z, seekz,dispz=0;
	int num;
	int i;
	z=-128;
	int loopexit=0;
		
	char gmbody;
	if (props&W_GM) gmbody=1; else gmbody=0;
	if (props&W_DEAD) gmbody=1; else gmbody=0;//ghosts can walk trhou doors
	// NOTE: this is clearly a bug (for living GMs gmbody is 0), but the fix enables a different bug (Duke, 14.9.03)

	do
	{
		seekz=127;
		num=-1;
		for (i=0;i<xycount;i++)
		{
			if (xyblock[i].basez+xyblock[i].height < seekz)
			{
				num=i;
				seekz=xyblock[i].basez+xyblock[i].height;
			}
		}
		
		if (num==-1)
		{
#ifdef DEBUG
			Console.send("(walking) Error?\n");
#endif
			xycount=0;
			z=-128;
		}
		else
		{
			unitile_st& Blocker = xyblock[num];
			if (Blocker.type!=0)	// not a map tile
			{
				if (Blocker.height==0) Blocker.height++;
				if (Blocker.id!=1 && Blocker.basez<=oldz+MaxZstep)
				{
					if (TileData->isWet(Blocker.flags) && !(props&W_GM || props&W_DEAD))
					{
						//Console.send("A\n");
						z=-128;
					}
					else
					{
						if (gmbody)	//ghosts and GMs can walk through doors
						{
							if ( (Blocker.weight==255 && !(props&W_CANMOVEALL)) || Blocker.type==2 )
								if ( Blocker.weight==255 || Blocker.type==2 )
								{
									if (TileData->isClimbable(Blocker.flags))
									{													
										if (Blocker.basez<oldz+MaxZstep)
										{
											//Console.send("x3: seekz: %i\n",seekz);
											z=Blocker.basez+Blocker.height;
											dispz=Blocker.basez+(Blocker.height/2);
											
										}
										else
										{
											if (props&W_GM || props&W_DEAD || TileData->isDoor(Blocker.flags))
											{
												//Console.send("x4: seekz: %i\n",seekz);
												dispz=z=Blocker.basez;
											}
											else
											{
												//Console.send("B\n");
												z=-128;
											}
										}
									}
									else
									{
										if (Blocker.basez+Blocker.height<oldz+MaxZstep)
										{
											//Console.send("x1: seekz: %i\n",seekz);
											dispz=z=Blocker.basez+Blocker.height;
										}
										else
										{
											if (props&W_GM || props&W_DEAD || TileData->isDoor(Blocker.flags))
											{
												//Console.send("x2: seekz: %i\n",seekz);
												dispz=z=Blocker.basez;
											}
											else
											{
												z=-128;
											}
										}
									}
								}
						}
						else
						{
							//Console.send("Blocker.flags=%08x, pc->priv);
							if (TileData->isBlocking(Blocker.flags) /*&& (!(Blocker.flag1&1)))*/ && !(props&W_GM)) // lb, castle walk bugfix without sideffects with very.
								
							{
								//Console.send("D (ID=%x)\n", Blocker.id);
								if (Blocker.basez+Blocker.height>oldz)
									z=-128;
							}
							else
							{
								if (TileData->isClimbable(Blocker.flags))
								{
									//Console.send("ladder-z: %i height: %i\n", Blocker.basez,Blocker.height);
									if (Blocker.basez < oldz+MaxZstep)
									{
										//Console.send("ladder-z2: %i height: %i seekz: %i\n", Blocker.basez,Blocker.height,seekz);
										z=Blocker.basez+Blocker.height;
										dispz=Blocker.basez+(Blocker.height/2);
									}
									else
									{													
										if (props&W_GM)
											
										{
											//Console.send("x4: seekz: %i\n",seekz);
											
											dispz=z=Blocker.basez;
										}
										else
										{
											//Console.send("E\n");
											z=-128;
										}
									}
								}
								else
								{
									if (Blocker.basez+Blocker.height < oldz+MaxZstep)
									{
										//Console.send("x5: seekz: %i base: %i height: %i oldz: %i\n",seekz, Blocker.basez, Blocker.height, oldz);
										dispz=z=Blocker.basez+Blocker.height;
									}
									else
									{
										if (props&W_GM)
										{
											// Console.send("x6: seekz: %i\n",seekz);
											dispz=z=Blocker.basez;
										}
										else
										{
											if (TileData->isPlatform(Blocker.flags))
											{
												//Console.send("F: %08x\n", Blocker.flags);
												z=-128;
											}
										}
									}
								}
							}
						}
					}
				}
			}
			else
			{
				if ((TileData->isWet(Blocker.flags) || TileData->isBlocking(Blocker.flags)) && !(props&W_GM))
				{
					//Console.send("G\n");
					if (!(props&W_ONLINE)) z=-128; // changed from c to s, LB 16.9.99
				}
				else
				{
					if (z==-128 || Blocker.basez+Blocker.height < oldz+MaxZstep)
					{
						//Console.send("x7: seekz: %i\n",seekz);
						dispz=z=Blocker.basez;
					}
				}
			}
			
			if (xycount==0) break;
			memcpy(&xyblock[num],&xyblock[xycount-1],sizeof(unitile_st));
			
			xycount--;
		}
	}
	while (xycount>0  && (++loopexit < MAXLOOPS) );
	
	*pz=z; *pdispz=dispz;
}

void WalkCheckBlockers(cCoord& newPos, const signed char oldz, signed char *pz, signed char *pdispz, const int props = 0)
{
	if (props & W_CANONLYSWIM)
	{
		if (Boats->IsFreeWater2(newPos))
		{
			*pz = oldz;
			*pdispz = oldz;
		}
	}
	else
	{
		WalkCollectBlockers(newPos);
	
		WalkEvaluateBlockers(props, oldz, pz, pdispz);
	}
}

///////////////
// Name:	WalkHandleBlocking
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	Handles a 'real move' if the Char is not only changing direction
//
// Remark LB 14'th nov 2000: will rewrite this stuff when I have time again for wp-project...
// dont delete the rem'd out Console.send's in meanwhile, thx !

bool WalkHandleBlocking(UOXSOCKET s, P_CHAR pc, int dir, cCoord& newPos)
{
	if (!pc) return false;

	signed char z, dispz=0;
	int props = 0;
	if (s!=-1) props += W_ONLINE;
	if (pc->dead) props += W_DEAD;
	if (pc->isGM()) props += W_GM;
	if (pc->CanMoveAll()) props += W_CANMOVEALL;

	WalkCheckBlockers(newPos, pc->pos.z, &z, &dispz, props);
	
	// check if player is banned from a house - crackerjack 8/12/99

	if (!pc->npc)	// this is also called for npcs .. LB
	{
		PC_ITEM House = findHouseAt(newPos);
		if (House) 
		{
			int sx, sy, ex, ey;		
			int j=on_hlist(House, pc->serial);
			
			if (j==H_BAN) 
			{
				Map->MultiArea(House,&sx,&sy,&ex,&ey);
				pc->Msg("You are banned from that location.");
				cCoord banPos = newPos;
				banPos.x=ex;
				banPos.y=ey+1;
				pc->teleTo(banPos);
				return false;
			}
			else if ((j==H_FRIEND || pc->Owns(House) || IsCoOwner(House,pc)) && isTent(House->id()))
			{
				// do this only for tents because they have no doors
				// house refreshment code moved to dooruse()
				Map->MultiArea(House,&sx,&sy,&ex,&ey);
				if (newPos.x==ex || newPos.x==sx ||
					newPos.y==ey || newPos.y==sy )	// ONLY when they step on the border of the rectangle
					refreshhouse(s,pc,House->unconst());
			}
		}
	}

	if (pc->multis != -1)	// if they were on a boat
	{
		if (findBoatAt(newPos)==NULL)	// if they're no longer on a boat...
			pc->SetMultiSerial(-1);
	}
	
	if (z==-128)
	{
		return false;
	}

	newPos.z = z;
	return true;
}

void updatemobilesinvisrange(UOXSOCKET si, P_CHAR pc, P_CHAR Watcher, int dir)
{
	unsigned char extmove[17]={0,};

	LongToCharPtr(pc->serial,&extmove[1]);
	ShortToCharPtr(pc->id(),&extmove[5]);
	PosToPtrXY(pc->pos,&extmove[7]);
	extmove[11] = pc->dispz;
	extmove[12] = dir;
					
	// running stuff
	if (pc->npc && pc->war)				extmove[12] = dir | 0x80; // Skyfire	
//	if (pc->npc && pc->IsFollowing())	extmove[12] = dir | 0x80;
	
	//** Lb's flying creatures stuff ***/
					
	if (pc->npc)
	{
		if (NpcInfo_canItFly(pc->id())) // can it fly ?
		{
			if (pc->fly_steps>0)
			{
				pc->fly_steps--;
				extmove[12] |= 0x80; // run mode = fly for that ones that can fly
			}
			else
			{
				unsigned int fly_p=18; // flying probability = 1/fly_p each step (if it doesnt fly)
				unsigned char fly_steps_max=27;
				
				int d = 0;
				if (fly_p != 0)
					d = rand()%fly_p;
				if (d == 0) 
				{
					if (fly_steps_max != 0)
						pc->fly_steps = (rand()%fly_steps_max) + 2; 
					else pc->fly_steps += 2;
				}
			}
		}
	}
					
	ShortToCharPtr(pc->skin, &extmove[13]);
					
	if (pc->war) extmove[15] = 0x40;
	else extmove[15] = 0x00;
	if (pc->isHidden()) extmove[15] |= 0x80;
	if (pc->dead && !pc->war) extmove[15] |= 0x80; // Ripper
	if (pc->Poisoned()) extmove[15] |= 0x04; // AntiChrist -- thnx to SpaceDog
	int guild;
	guild = Guilds->Compare(pc, Watcher);
	if (guild == 1) 	 extmove[16] = 2; // Same guild (Green)
	else if (guild == 2) extmove[16] = 5; // Enemy guild.. set to orange
	else
	{
	   if (pc->isMurderer()) extmove[16] = 6;		// If a bad, show as red.
	   else if (pc->isInnocent()) extmove[16] = 1;		// If a good, show as blue.
	   else if (pc->flag == 0x08) extmove[16] = 2;		// green (guilds)
	   else if (pc->flag == 0x10) extmove[16] = 5;		// orange (guilds)
	   else  extmove[16] = 3;		// grey
	}
	if (!Watcher->isSameAs(pc)) 
	{
	   XStartPkt(0x77, 17);
	   XAddPkt(&extmove[0+1], 17-1);
	   XFinishPkt(si);
	}
}

///////////////
// Name:	WalkSendToPlayers
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	shows the char at the new position to other players in range
//
bool WalkSendToPlayers(P_CHAR pc, int dir, const cCoord& old_pos, const cCoord& new_pos)
{
	if (!pc) return true;
	bool isinsendarea,walkintosendarea; 

	P_CHAR Watcher;	
	AllPlayersOnScreenIterator api(pc->pos);	
	for (Watcher = api.First(); Watcher != NULL; Watcher = api.Next())
	{	
		{			
			if ( (pc->isHidden() && !Watcher->isGM()) || Watcher->isSameAs(pc) ) continue;
	
			UOXSOCKET si = api.getSocket();

			bool seen_on_oldpos = isonPlayersScreen(old_pos, Watcher->pos, EXTRAMARGIN); // EXTRAMARGIN -> very important, extended range for large creatures (eg dragons), even for smaller creatures, eg players, it looks too jumpy sendrange being exacly screensize
			bool seen_on_newpos = isonPlayersScreen(new_pos, Watcher->pos, EXTRAMARGIN);

			if (EXTRAMARGIN > 2)
			{
				// defined vis-range is bigger "real" sends-range
				isinsendarea = abs(Watcher->pos.x - new_pos.x)<=REALSENDMARGIN && abs(Watcher->pos.y - new_pos.y)<=REALSENDMARGIN;
				walkintosendarea = isinsendarea && (abs(Watcher->pos.x - old_pos.x)>REALSENDMARGIN || abs(Watcher->pos.y - old_pos.y)>REALSENDMARGIN);
				// ^^^ true if pc walks into send area from non-send area
			}

			if (seen_on_oldpos)
			{			
				if (seen_on_newpos) 
				{
					/**************************************/
					/**** mobile STAYS within vis-range ***/
					/**************************************/
					
					if (EXTRAMARGIN>2)
					{
						if (walkintosendarea) // mobile ENTERS "REAL" vis-range, send 0x78
						{
							impowncreate(si, pc, 1);
						}
						else
						{
							updatemobilesinvisrange(si, pc, Watcher, dir); // otherwise send 0x77
						}
					}
					else // no extramargin >2, unconditionaly update only (0x77)
					{
						updatemobilesinvisrange(si, pc, Watcher, dir);
					}
				}

				 /********************************/
				 /**** mobile LEAVES vis-range ***/
				 /********************************/

				else 
				{  
					SndRemoveNpc(si, pc, false);
				}
			}
			else // !seen_on_oldpos
			{
				if (seen_on_newpos)  // mobile ENTERS "REAL" vis-range, send 0x78
				{
					/********************************/
					/**** mobile ENTERS vis-range ***/
					/********************************/
					
					if (isinsendarea)
					{
						impowncreate(si, pc, 1); // mobile ENTERS vis-range, send 0x78
					}
				}
				// else
				// mobile INVISIBLE on old and new position, do nothing
			}
		}
	}	
	return true;
}

///////////////
// Name:	WalkHandleCharsAtNewPos
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	sends the newly visible Chars to the screen and checks for shoving
//
bool WalkHandleCharsAtNewPos(UOXSOCKET s, P_CHAR pc, const cCoord& old_pos, const cCoord& new_pos)
{
	if (!pc) return true;

	bool isinsendarea,walkintosendarea; 
	cChar* pc_i;
	int loopexit=0;
	RegGrid3x3Iterator4Chars rg3ic(pc->pos);
	while((pc_i = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{			
		if (pc_i->stablemaster_serial<=0) // dont send stabled pets to let them appear invisible
		{
			if ( !pc_i->isOfflinePlayer() || pc->isGM() )	//Let GMs see logged out players		
			{
				if (old_pos.IsSameXY(new_pos) || pc_i->isSameAs(pc) )	// just turning ?
					continue;											//  no multiple shoving
				
				bool seen_on_oldpos = isonPlayersScreen(old_pos, pc_i->pos, EXTRAMARGIN); //  EXTRAMARGIN >- important, see above
				bool seen_on_newpos = isonPlayersScreen(new_pos, pc_i->pos, EXTRAMARGIN);		
				
				if (EXTRAMARGIN > 2)
				{
					// defined vis-range could be > "real" send-range
					isinsendarea = abs(pc_i->pos.x - new_pos.x)<=REALSENDMARGIN && abs(pc_i->pos.y - new_pos.y)<=REALSENDMARGIN;
					walkintosendarea = isinsendarea && (abs(pc_i->pos.x - old_pos.x)>REALSENDMARGIN || abs(pc_i->pos.y - old_pos.y)>REALSENDMARGIN);
					// ^^^ true if pc walks into send area from non-send area
				}
				
				if (seen_on_oldpos)
				{
					if (!seen_on_newpos ) // (not moving) mobile leaves vis-range of moving player, send 0x1d	
					{
						SndRemoveNpc(s, pc_i, false);
					}
					else // seen_on_newpos ->  (not moving) mobile remaining in vis-range of moving player, basicly do nothing
					{
						if (EXTRAMARGIN > 2)
						{
							if (walkintosendarea) // except (not moving) mobile ENTERS "REAL" vis-range of moving player, send 0x78
							{
								impowncreate(s, pc_i, 1);
							}
						}
						/*
						else
						{
						//impowncreate(s, pc_i, 1);
						}
						*/
					}
				}
				else 
				{
					if (seen_on_newpos) // (not moving) mobile enteres vis range of moving player.
					{
						if (EXTRAMARGIN > 2)
						{
							if (isinsendarea)
							{
								impowncreate(s, pc_i, 1); // (not moving) mobile ENTERS "REAL" vis-range of moving player, send 0x78
							}
						}
						else
						{
							impowncreate(s, pc_i, 1);
						}
					}
				}
			}
			
		   if (!(
				pc->id()==0x03DB ||
				pc->id()==0x0192 ||
				pc->id()==0x0193 ||
				pc->isGMorCounselor()
				))
			{
				if ((pc_i->InGame() || (pc_i->npc && !pc_i->isDummy()))) //cant collide with dummys (thrawn)
				{
					if (pc_i->pos == pc->pos)
					{
						if (pc_i->isHidden() && !pc_i->dead && !pc_i->isInvul() && !pc_i->isGM())
						{
							pc->Msg("Being perfectly rested, you shoved something invisible out of the way.");
							pc->reduceStm(4);
						}
						else if (!pc_i->isHidden() && !pc_i->dead && !pc_i->isInvul() && !pc_i->isGM()) // ripper..GMs and ghosts dont get shoved.)
						{
							if (s!=-1)	
							{
								sprintf(temp,"Being perfectly rested, you shove %s out of the way.", pc_i->name);
								pc->Msg(temp);
							}
							pc->reduceStm(4);
						}
						else if (!pc_i->isGMorCounselor() && !pc_i->isInvul())//A normal player (No priv1(Not a gm))
						{
							pc->Msg("Being perfectly rested, you shove something invisible out of the way.");
							pc->reduceStm(4);
						}
					}
				}
			}
		}
	}
	return true;
}



///////////////
// Name:	WalkHandleItemsAtNewPos
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	sends the newly visible items to the screen and checks for item effects
//
bool WalkHandleItemsAtNewPos(UOXSOCKET s, P_CHAR pc, const cCoord& old_pos, const cCoord& new_pos)
{
	if (!pc) return true;
	
	PC_ITEM pi;
	int loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(pc->pos);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (pc->pos.IsSameXY(pi->pos) && pc->pos.z>=pi->pos.z && pc->pos.z<=pi->pos.z+5)
		{
			OldMagic->CheckFieldEffect(pc, pi);
			
			if (pi->trigger!=0)
			{
				if (pi->trigtype==1 && !pc->dead)
				{
					if (pi->disabled<=uiCurrentTime)//AntiChrist
					{
						triggerItem(pc,pi->unconst());  //When player steps on a trigger
					}
				}
			}
		}
		
		if ( pi->id()>=0x407C && pi->id()<=0x407E ) // bug alert, very strange, needs rewite ! -> sending castles and houses earlier
		{
			int di=dist(pi->pos, pc->pos); // buildrange and visrange is NOT euclidean metrics -> dist is -> wrong ! (*sqrt(2) would do it)
			if (di<=BUILDRANGE && di>=VISRANGE)								
			{			
				senditem(s, pi);
			}
		}	


		bool item_visible_old = isonPlayersScreen(old_pos, pi->pos, 0); // 0 is essential, otherwise (eg 4) some items aint send
		bool item_visible_new = isonPlayersScreen(new_pos, pi->pos, 0);
		if (item_visible_new && !item_visible_old) senditem(s, pi);
		
						 
	}
	return true;
}


void WalkingHandleRainSnow(UOXSOCKET s, P_PLAYER pc)
{
	if (!pc) return;
	
	/********* LB's no rain & snow in buildings stuff ***********/
	if (!pc->npc && pc->InGame()) // check for being in buildings (for weather) only for PC's
	{
		if (wtype!=0) // check only neccasairy if it rains or snows ...
		{
			int j=indungeon(pc); // dung-check
			int i=Map->StaticTop(pc->pos,pc->pos.z); // static check
			
			// dynamics-check
			int x=Map->DynamicElevation(pc->pos,pc->pos.z);
			if (x!=-127) if (Boats->GetBoat(pc)) x=-127; // check for dynamic buildings except boats
			if (x==1 || x==0) x=-127; // 1 seems to be the multi-borders
			// bugfix LB
			
			int kk=noweather[s];
			if (j || i || x!=-127 ) noweather[s]=1; else noweather[s]=0; // no rain & snow in static buildings+dungeons;
			if (kk-noweather[s]!=0) /*weather(y,0);*/ j=j; // iff outside-inside changes resend weather ...
			// needs to be de-rem'd if weather is available again
		}
	}
}


void WalkingHandleGlowingItems(P_CHAR pc)
{
	if (!pc) return;
	if (pc->InGame())
	{
		int loopexit=0;
		int ci=0;
		P_ITEM pi;
		while ( ((pi=SearchGlowers(pc->serial,&ci)) != NULL) && (++loopexit < MAXLOOPS) )
		{
			pc->glowHalo(pi);
		}
	}
}


unsigned char calcWalkingNotority(P_CHAR Player)
{
	if (Player->isHidden()) return 0; // not sure, copied from old wrong code, have to check into this

	/*
	notoriety:   
	
	0 = invalid/across server line
	1 = innocent (blue)
	2 = guilded/ally (green)
	3 = attackable but not criminal (gray)
	4 = criminal (gray)
	5 = enemy (orange)
	6 = murderer (red)
	7 = unknown use (translucent (like 0x4000 hue))
	*/

	return 1;
}

void walking(UOXSOCKET s, P_CHAR pc, int dir, int sequence, cCoord& newPos)
{	
	if (!WalkHandleAllowance(s,pc,sequence))		// check sequence, frozen, weight etc.
	{		
		return;
	}

	WalkHandleRunning(s,pc,dir);

	cCoord old_pos(pc->pos);

	if ((dir&0x0F)==pc->dir)		// not just turning
	{
		if ((dir&0x0F) > 7)
		{
			Console.send("ERROR: bad dir in walking()\n");
			Console.send("Caused by character %s. dir: %i dir&0x0f: %i dir-passed : %i dp&0x0f : %i\n",pc->name,pc->dir,pc->dir&0x0f,dir,dir&0x0f);
			// sysmessage(calcSocketFromChar(c), "Dont desparate, I'm working on that damned fu**** bug, Lord Binary");
			// pc->dir=0;
			// teleport(c);
			if (s!=-1) deny(s, pc, sequence);
			return;
		}
		if (s!=-1)	// for npcs, newPos is assumed to be *pre-checked*
		{
			if (!WalkHandleBlocking(s,pc,dir&0x0F,newPos))	// this also determines the correct Z at newPos
			{			
				if (s!=-1) deny(s, pc, sequence); // lb, crashfix
				return;
			}
		}

		pc->MoveTo(newPos);	//Char mapRegions
	}

	if (s!=-1)
	{
		unsigned char walkok[4]="\x22\x00\x01";
		walkok[1]=sequence;
		walkok[2]=calcWalkingNotority(pc);
		
		XStartPkt(0x22, 3);
		XAddPkt(&walkok[0+1], 3-1);
		XFinishPkt(s);
		
		walksequence[s]=sequence;
		if (walksequence[s]>255) walksequence[s]=1; // initially starts with 0 but after first 255 always 1's following 255's (not 0)
		
		static int fw2=0;
		
		if (SrvParms->antifastwalk)
		{
			unsigned char addkey[9]={0xbf,0x0,0x9,0x0,0x2,0x0,0x0,0x0,0x0};
			LongToCharPtr(0x88888800+(fw2), addkey+5);
			XStartPkt(0xBF, 9);
			XAddPkt(&addkey[0+3], 9-3);
			XFinishPkt(s);
			fw2++;
		}
	}
	
	if (s!=-1)
	{
		if (SrvParms->shoeswearout > 0)
		{
			if (rand()%SrvParms->shoeswearout == 0)
			{
				P_ITEM Shoe = pc->GetItemOnLayer(3);
				if (Shoe)
				{
					char tn[222];
					Shoe->getName(tn);	// get the name first, item might be deleted by WearOut
					
					if (Shoe->WearOut(1))	// has item been destroyed ? (100% chance to take off 1 hp)
					{
						sprintf(temp,"Your %s have been destroyed",tn);
						pc->Msg(temp);
					}
				}
			}
		}
	}

	WalkSendToPlayers(pc,dir, old_pos, pc->pos);
	
	if (dir>-1 && (dir&0x0F)<8) { pc->dir=(dir&0x0F);}
	else { LogErrorVar("dir-screwed : %i",dir); }
	
	if (s!=-1)
	{
		WalkHandleCharsAtNewPos(s, pc, old_pos, pc->pos);
		WalkHandleItemsAtNewPos(s, pc, old_pos, pc->pos);
	}
	
	if (!pc->pos.IsSameXY(old_pos))
	{
		if (pc->npc==0 || pc->questType || pc->tamed)
			objTeleporters(s,pc);
		teleporters(pc);
	}
	
	// WalkingHandleRainSnow(s,c); // while rain and snow are disabled its a waste of CPU cycles
	
	WalkingHandleGlowingItems(pc);

	OldMagic->GateCollision(pc);
	
	checkregion(pc); 
	
	if (Skills->GetCombatSkill(pc) == ARCHERY)  // -Frazurbluu- add in changes for archery skill, and dexterity
	{											//  possibly weapon speed?? maybe not, cause crossbows notta running shooting
		if (pc->HasTarget())
		{
			if (pc->timeout>= uiCurrentTime)
				pc->timeout= uiCurrentTime + (3*CLOCKS_PER_SEC);
		}
	} 
}


void walking2(P_CHAR pc) // Only for switching to combat mode
{
	if (!pc) return;

	if (!pc->war) // we have to execute this no matter if invisble or not LB
	{
		pc->setAttacker(NULL);
		pc->ResetTarget();
	}

	//unsigned int i;
	//int sendit;
	//printf("halodri\n");
	// seems like nonsense to me, we dont need to resend players when fights start
	// animations get send otherwhere	

	/*
	for (i = 0; i < now; i++)
	{
		if (perm[i])
		{
			P_CHAR Watcher = MAKE_CHAR_REF(currchar(i));
			if (!Watcher) continue;
			if (isonPlayersScreen(pc->pos, Watcher->pos, 0))
			{
				if ((pc->isHidden() || (pc->dead && !pc->war)) && !Watcher->isSameAs(pc) && !Watcher->isGM())
				{
					if (!Watcher->dead)
					{
						//SndRemoveNpc(i, pc);
						sendit = 0;
					}
					else
						sendit = 1;
				} else  sendit = 1; // LB 9-12-99 client 1.26.2 fix
				if (sendit)
				{
					updatemobilesinvisrange(i, pc, Watcher, pc->dir);
				}
												
			}
		}
	}
	*/
}

int npcSelectRandomDir(P_CHAR pc_i)
{
	int j=rand()%40;
	if (j>7 && j<33) // Let's move in the same direction lots of the time.  Looks nicer.
		return pc_i->dir;
	else
		return j%8;
}

int npcSelectDir(P_CHAR pc_i, int j)
{
	if (pc_i->blocked)
	{
		short x=0;
		if (j/2.0!=j/2)
			x=1;
		if (pc_i->blocked<=2)
			j =pc_i->dir2;	/* =(j-2-x)%8; //works better  ????*/
		else
		{
			if (rand()%2) j=pc_i->dir2 = static_cast<char>((j-2-x)%8);
			else j=pc_i->dir2 = static_cast<char>((j+2+x)%8);
		}
	}
	if (j<0)
		j=rand()%8;
	return j%8;
}

int npcSelectDirWarOld(P_CHAR pc_i, int j)
{
	if (pc_i->blocked)
	{
		short x=0;
		if (j/2.0!=j/2)	// odd dir, that is diagonal
			x=1;
		if (pc_i->blocked<=2)
			j =pc_i->dir2 =(j-2-x)%8; //works better  ????
		else
		{
			if (rand()%2) j=pc_i->dir2=(j-2-x)%8;
			else j=pc_i->dir2=(j+2+x)%8;
		}
	}
	if (j<0)
		j=rand()%8;
	return j%8;
}

class cWalkingAI
{
private:
	short results;
	cCoord startPos;
	int targetX, targetY;
	int zz;
	P_CHAR Npc;
public:
	void Init(P_CHAR pNpc, int desiredDir, int tx, int ty)
	{
		results = 0;
		startPos = pNpc->pos;
		targetX = tx;
		targetY = ty;
		zz = pNpc->pos.z;
		Npc = pNpc;
	}

	short dirBitChecked(int dir) { int i=(1<<dir);
	return i; }
	//return (1<<dir); }
	short dirBitBlocked(int dir) { return (1<<(dir+8)); }

	bool TileBlocks(int dir)		// check if tile blocks
	{
		if (results & dirBitChecked(dir))	// avoid double checking
			return (results & dirBitBlocked(dir)) ? true : false;
		cCoord chkPos = startPos;
		GetValidXYfromDir(dir,chkPos);		// get coords of the location we want to walk
		results |= dirBitChecked(dir);
		if (!validNPCMove(chkPos, zz, Npc))
			results |= dirBitBlocked(dir);
		return (results & dirBitBlocked(dir)) ? true : false;
	}

	bool DirBlocks(int dir)		// check if we can reach tile, return true if blocked
	{
		if (TileBlocks(dir))
			return true;		// the tile itself is blocking
		if (!(dir & 1))			// square ie. N,S,W or E
			return false;		// perfect! we can just go where we want to go
		else					// diagonal ie. NE,SE,SW or SE ??
		{						// We'll have to check the adjacent tiles too
			if (!TileBlocks(getLeftDir(dir)))
				return false;
			if (!TileBlocks(getRightDir(dir)))
				return false;
		}
		return true;
	}
	
	double dist(int xa, int ya, int xb, int yb) // Distance between position a and b
	{
		return hypot( abs(xa-xb), abs(ya-yb) ); // hypot is POSIX standard
	}
	
	double distToTarget(int dir, int map)
	{
		cCoord chkPos1 = startPos;
		GetValidXYfromDir(dir,chkPos1);		// get coords of the location we want to check
		return dist(startPos.x,startPos.y,chkPos1.x,chkPos1.y);
	}

	int ClosestDir(int dir1, int dir2, int map)
	{
		if ( distToTarget(dir1, map) < distToTarget(dir2,map) )
			return dir1;
		else
			return dir2;
	}

	int PreferredDir(int dir1, int dir2, int map)
	{
		int dir1stTry = ClosestDir(dir1,dir2, map);
		if (!DirBlocks(dir1stTry))
			return dir1stTry;
		int dir2ndTry = (dir1stTry==dir1 ? dir2 : dir1);
		if (!DirBlocks(dir2ndTry))
			return dir2ndTry;
		return -1;
	}

	int SelectDirWar(int desiredDir, int map)
	{
		int dir = -1;
		// step 1: let's see if we can just go where we want to go
		if (!DirBlocks(desiredDir))
			return desiredDir;				// perfect!

		
		// step 2: the direct way is blocked, find another way...
		int dirLeft1  = getLeftDir (desiredDir);
		int dirRight1 = getRightDir(desiredDir);

		dir = PreferredDir(dirLeft1,dirRight1, map);
		if (dir != -1)
			return dir;

		
		// step 3: let's try to go sideways...
		int dirLeft2  = getLeftDir (dirLeft1);
		int dirRight2 = getRightDir(dirRight1);

		dir = PreferredDir(dirLeft2,dirRight2,map);
		if (dir != -1)
			return dir;

		
		// step 4: let's try to go backwards...
		int dirLeft3  = getLeftDir (dirLeft2);
		int dirRight3 = getRightDir(dirRight2);

		dir = PreferredDir(dirLeft3,dirRight3,map);
		if (dir != -1)
			return dir;


		// step 5: only one place left to go...
		int opposite  = getLeftDir (dirLeft3);
		if (!DirBlocks(opposite))
			return opposite;

		return -1;				// we are totally blocked !
	}
	
};

void handle_scriptedwalkingnpc(P_CHAR pc, int num)
{   
	map<int, CScriptedWalking *>::iterator itty;
	
	itty = scriptedwalkingnpcs.find(pc->serial);
	if (itty != scriptedwalkingnpcs.end()) // found, script object for npc already created
	{
		CScriptedWalking *sw=itty->second;
		bool action=sw->nextstep(); // walking script handler gets npc into next step
		
		if (action) // if there  an action left, do it
		{
			char dir;
			char old_dir=pc->dir;
			cCoord pos;
			sw->querycurrent(dir, pos);
			npcwalk(pc, dir, 0);
			if (old_dir!=dir) npcwalk(pc,dir,0);
			
			//printf("d:%i x:%i y:%i\n", dir, pos.x, pos.y);
		}
		else
		{
			// e.g delete npc, not good for testing ... but basicly a good idea if it's used one day for non testing purposes
		}
	}
	else
	{
		CScriptedWalking *sw = new CScriptedWalking(num, pc); // not found, create new script object
		scriptedwalkingnpcs.insert(pair<int, CScriptedWalking *>(pc->serial, sw)); // store in map
		
		/// note: if used one day beyond testing, cleanup of CScriptedWalking ojects need to be done in delete npcs routines
		/// currently not done ...
		
		// now draw npc at startloc
		
		char dir;
		cCoord pos;
		sw->querycurrent(dir, pos);
		pc->dir=dir;
		pc->teleTo(pos);
		
	}
}

//NEW NPCMOVEMENT ZIPPY CODE STARTS HERE -- AntiChrist meging codes --
void npcMovement2(P_CHAR Npc)//Lag fix
{
	if (!Npc) return;
	int j, noTurn=0;
	int dir2walk = -1;
	
	if (Npc->isScriptedwalkingAi())
	{
		int num=Npc->getScriptedwalkingAi();
		handle_scriptedwalkingnpc(Npc, num);
		Npc->setNextMoveTime();
		return;
	}
	
	if (Npc->war && Npc->npcWander != 5) 
	{
		P_CHAR Attacker = Npc->getAttacker();
		if (Attacker)
		{
			j = Npc->calcDirTo(Attacker);
			if (Npc->dist(Attacker)>1 || j != Npc->dir)
			{
				if (Attacker->npc || Attacker->InGame())
				{
#if 0
					// this code is disabled because there is a bug (?) in validNPCMove() calling DoesStaticBlock()
					// eg. 'brambles' are considered 'non-blocking' but the client doesn't let you go there !
					// thus the new code called here is running into the same probs as the old code :(
					// I have no clue yet :(  (Duke 18.7.01)
					cWalkingAI wai;
					wai.Init(Npc, j, Attacker->pos.x, Attacker->pos.y);
					j=wai.SelectDirWar(j,Attacker->pos.map);
					if (j==-1) return;	// totally blocked. Don't walk.
#else
					j=npcSelectDirWarOld(Npc,j);	// a very simple pathfinding algorithm
#endif
					dir2walk = j;
				}
			}
		}
	} 
	else
	{
		switch(Npc->npcWander)
		{
		case 0: // No movement
			break;
		case 1: // Follow the follow target
			P_CHAR ToFollow;
			ToFollow = Npc->GetFollowTarget();
			if (!ToFollow)
			{
				Npc->npcWander=4;	// wander freely again
				return;
			}
			
			if (ToFollow->dead || ToFollow->disabledTimer.IsRunning()) break; //antichrist
			
			// Dupois - Added April 4, 1999
			if (Npc->questDestRegion==Npc->region)	// Has the Escortee reached the destination ??
			{
				MsgBoardQuestEscortArrive(Npc, MakePlayerRef(ToFollow));	// Pay the Escortee and free the NPC
			}
			// End - Dupois
			
			j = Npc->calcDirTo(ToFollow);
			if (Npc->dist(ToFollow) > 1 || j != Npc->dir)
			{
				dir2walk=npcSelectDir(Npc,j);
			}
			break;
		case 2:		// Wander freely, in a defined circle
		case 3:		// Wander freely, within a defined box
		case 4:		// wander freely, avoiding obstacles
			// fall-through for cases 2+3 intended
			dir2walk = npcSelectRandomDir(Npc);
			break;
		case 5: //FLEE!!!!!!
			{
				P_CHAR Target = Npc->GetTarget();
				if (!Target) break;
				if (Npc->dist(Target)<15)
				{
					j=(Npc->calcDirTo(Target)+4)%8;	// opposite direction
					dir2walk=npcSelectDir(Npc,j);
				}
				break;
			}
		default:
			LogErrorVar("Bad npcWander [%i] in npcMovement2()",Npc->npcWander);
		}
	}
	
	if (dir2walk != -1)
	{
		if (dir2walk == Npc->dir) noTurn=1;

		int boundary = 0;
		if (Npc->npcWander == 2) boundary = 2;	// circle
		if (Npc->npcWander == 3) boundary = 1;	// rectangle
		if (Npc->war) boundary = 0;				// war mode ? forget about boundaries !!

		npcwalk(Npc,dir2walk,boundary);
	}

	if (noTurn)	// turning doesn't take time, so check them right again next round
	{
		if (Npc->npcWander==1)			// if following
			Npc->setNextMoveTime(65);	// 65% of the normal delay
		else
			Npc->setNextMoveTime();
	}
}

int checkBoundingBox(const cCoord& chkPos, int fx1, int fy1, int fz1, int fx2, int fy2)
{
	if (chkPos.x >= ((fx1<fx2)?fx1:fx2) && chkPos.x <= ((fx1<fx2)?fx2:fx1))
		if (chkPos.y >= ((fy1<fy2)?fy1:fy2) && chkPos.y <= ((fy1<fy2)?fy2:fy1))
			if (fz1==-1 || abs(fz1-Map->Height(chkPos, fz1))<=5)
				return 1;
			return 0;
}

int checkBoundingCircle(const cCoord& chkPos, int fx1, int fy1, int fz1, int radius)
{
	if ( (chkPos.x-fx1)*(chkPos.x-fx1) + (chkPos.y-fy1)*(chkPos.y-fy1) <= radius * radius)
		if (fz1==-1 || abs(fz1-Map->Height(chkPos, fz1))<=5)
			return 1;
		return 0;
}

int checkBounds(P_CHAR pc, const cCoord& chkPos, int type)
{
	int move=0;
	switch (type)
	{
	case 0: move=1;break;
	case 1: move=checkBoundingBox(chkPos, pc->fx1, pc->fy1, pc->fz1, pc->fx2, pc->fy2);break;
	case 2: move=checkBoundingCircle(chkPos, pc->fx1, pc->fy1, pc->fz1, pc->fx2);break;
	default: move=0;	// invalid type given
	}
	return move;
}

// Restructured & bugfixed by LB 17 Septemer 1999
// Thx to JM for help.

void npcwalk(P_CHAR pc, int j, int type)   //type is npcwalk mode (0 for normal, 1 for box, 2 for circle)
{
	char v1,v2,v3,valid;

	if (!pc) return;

	signed char oldz = pc->pos.z;
	//Bug Fix -- Zippy
	if (pc->IsFrozen()) return;//Frozen - Don't send them al the way to walking to check this, just do it here.

	//Console.send("  npcw-2 c-dir: %i j-dor: %i\n",pc->dir,j);


	//Console.send("callwe-w: %i t: %i dir: %i\n",caller,type,j);

	/////////// LB's flying creatures stuff, flying animation if they stand still ///////
  
	if (NpcInfo_canItFly(pc->id())) // can it fly ?
	{
		if (pc->fly_steps>0)
		{
			if (rand()%3==0) pc->DoAction(0x13); // flying animation
		} 
	}
	
	/////////////////////////////////////////////////////////////////////////////////////

	//Console.send("name: %s j-dir:%i, c-dir:%i type:%i\n",pc->name,j,pc->dir,type);

	cCoord newPos = pc->pos;
	if (pc->dir == j)  // If we're moving, not changing direction
	{
		signed char zr=illegal_z, dispz=0;
		GetValidXYfromDir(pc->dir,newPos);	// get coords of the location we want to walk

		if (!checkBounds(pc, newPos, type)) return;	// check circle/rectangle *before* checking blocking (Duke, 14.06.05)

		switch(j) // Switch, based on the direction we're going to move.
		{
		case 0: // North
		case 2:
		case 4:
		case 6:
			valid=validNPCMove(newPos, oldz, pc, zr, dispz);
			if (valid)
			{
				walking(-1,pc,j,256,newPos);
			}
			break;
			
		case 1: // Northeast
		case 3:
		case 5:
		case 7:
			v1 = validNPCMove(newPos, oldz, pc, zr, dispz); // check northeast
			if (!v1) valid=0;	// if it blocks -> no move 
			else				// if not, check if both, N & E are blocking. 
								// if so it blocks too, otherwise its ok.
			{
				cCoord leftPos = pc->pos;
				GetValidXYfromDir(getLeftDir(pc->dir),leftPos);	// get coords of the left location we want to pass
				
				v2 =  validNPCMove(leftPos, oldz, pc);
				if (v2) valid=1; 
				else
				{ 
					cCoord rightPos = pc->pos;
					GetValidXYfromDir(getRightDir(pc->dir),rightPos);	// get coords of the right location we want to pass

					v3= validNPCMove(rightPos, oldz, pc);
					if (v3) valid=1; else valid=0;
				}
			}
			
			// LB remark: NW-block AND (N-block OR W-block) is the same more compact, BUT ...
			// could be slower, depending on the compilers optimization.
			// I dont trust msvc, thus the "hand-optimization

			//Console.send("val: %i v1%i v2%i v3%i\n",valid,v1,v2,v3);

			if (valid)
			{
				//Console.send("calling w\n");
				walking(-1,pc,j,256,newPos);
			}
			break;
		}
	}
	else 
	{
		//Console.send("changing dir?\n");
		walking(-1,pc,j,256,newPos);
	}
}

int validNPCMove(cCoord& newPos, signed char oldz, P_CHAR Npc, signed char& zr,  signed char& dispz)
{
	if (!Npc) return 0;

	Npc->blocked++;

	P_ITEM pi;
	int loopexit=0;
	RegCellIterator4Items rcii(newPos);
	while((pi = rcii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (!pi->pos.IsSameXY(newPos)) continue;
		if (pi->isMulti()) continue;

		const tileData *tile;
		tile = TileData->SeekTile(pi->id());
		if (pi->pos.z+tile->height>oldz+1 && pi->pos.z<oldz+MaxZstep)
		{
			// bugfix found by JustMichael, moved by crackerjack
			// 8/2/99 makes code run faster too - one less loop :)
			if (pi->id()==0x3946 || pi->id()==0x3956) return 0;	// energy field
			if (pi->id()<=0x03E2) return 0;						// all kinds of walls and such
			if (pi->id()>0x0854 && pi->id()<0x0866) return 0;	// *some* iron gates and wooden fences/gates ???
			
			if (pi->isType(ITY_Door))
			{
				if (Npc->npc && (strlen(Npc->title) > 0 || Npc->npcaitype != 0))
				{
					// Console.send("doors!!!\n");
					dooruse(NULL, pi);
				}
				Npc->blocked=0;
				return 0;
			}
		}
	}

	// experimental check for bad spawning/walking places, not optimized in any way (Duke, 3.9.01)
	int mapid = 0;
#if 0
	map_st map = Map->SeekMap0( x, y );
	mapid = map.id;
#else
	signed char mapz = Map->AverageMapElevation(newPos, mapid);	// just to get the map-ID
	if (mapz != illegal_z)
#endif

	{
		if ((mapid >= 0x25A && mapid <= 0x261) ||	// cave wall
			(mapid >= 0x266 && mapid <= 0x26D) ||	// cave wall
			(mapid >= 0x2BC && mapid <= 0x2CB) )	// cave wall
			return 0;
		if ( mapid >= 0x0A8 && mapid <= 0x0AB) 	// water (ocean ?)
			return 0;
//		land_st land;
//		Map->SeekLand(mapid, &land);
	}
	
	
	// check if other NPC's block
	if (speed.antinpcaggregation)
	{	  
		cChar* pc;
		int loopexit=0;
		RegGrid3x3Iterator4Chars rg3ic(newPos);
		while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
		{
			if ((pc->npc || pc->automode) && !Npc->isSameAs(pc)) // no self-avoidance ;-)
			{
				int p = newPos.distance(pc->pos);		
				if (p<=0) return 0;			// walking to fellow npc, that's blocking for sure 
				else 
				{
					if (!Npc->IsFollowing())
					{
						int pepper=rand()%100; // kind of avoidance, with small probability they wont move even if no DIRECT collision but within radius <=3
						if (pepper>=97 && p==3) return 0;
						if (pepper>=91 && p==2) return 0;
						if (pepper>=87 && p==1) return 0;
					}
				}
			}
		}
	}

	if (NpcInfo_canOnlySwim(Npc->id()) || Npc->isWaterCreatureBody())
		WalkCheckBlockers(newPos, oldz, &zr, &dispz, W_CANONLYSWIM);
	else
		WalkCheckBlockers(newPos, oldz, &zr, &dispz);
	if (zr != illegal_z)
	{
		newPos.z = zr;
		Npc->blocked = 0;
		return 1;
	}
	return 0;
}

int validNPCMove(cCoord& newPos, signed char oldz, P_CHAR Npc)
{
	signed char zr=illegal_z, dispz=0;
	return validNPCMove(newPos, oldz, Npc, zr, dispz);
}

bool canNpcStandHere(short x, short y);
bool canNpcStepHere (short x, short y, int capabilities, signed char oldZ);
bool canNpcSpawnHere(short x, short y, int capabilities);


/*

*/

bool CScriptedWalking::nextstep(void)
{

  if (maxlist<=0) return false; // didnt load correctly

  totalstep++; 

  currentstepinlist++;

  int maxsteps=data[currentlist].steps;
  currdir=data[currentlist].dir;

  //printf("m:%i\n", totalstep);

  if (currentstepinlist>=(maxsteps))
  {
	 currentlist++;

	 if ( (currentlist>=maxlist) && repeated) currentlist=0;
	 else if ( (currentlist>=maxlist) && !repeated) { currentlist=maxlist-1; return false; }

	 currentstepinlist=0;
  }

  GetXYfromDir(currdir, currpos);
  currpos.z=Map->Height(currpos, currpos.z);

  return true;
}

void CScriptedWalking::querycurrent(char& dir_curr, cCoord& curr_pos)
{
   dir_curr=this->currdir;
   curr_pos=this->currpos;
}

CScriptedWalking::CScriptedWalking(int i,P_CHAR pc)
{
  totalstep=0;
  currentlist=0;
  currentstepinlist=0;
  repeated=false;
  currdir=0;
  start.x=0;start.y=0;start.z=0;start.map=0;

  this->loadscript(i, pc);

  maxlist=data.size();

}

CScriptedWalking::~CScriptedWalking(void)
{
   data.clear();
}

void CScriptedWalking::loadscript(int i, P_CHAR Player)
{
	Script* Scp = new Script("walkingtest.scp");
	if (!Scp->Open())
	{
		LogWarning("walkingtest.scp not found");
		delete Scp;
		return;
	}

	char sect[512];
	sprintf(sect, "PATH %i", i);
	if (!Scp->find(sect))
	{
		LogWarningVar("Undefined path in walkingtest.scp %i", i);
		Scp->Close();
		delete Scp;
		return;
	}

	cCoord st;
	int dx=0,dy=0;
	char dir;

	int loopexit=0;
	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS))
	{	
		dir=99;
		if (!(strcmp(script1, "REPEAT")))
		{
			if (!(strcmp(script2,"NO")))
			{
				repeated=false;
			} else repeated=true;
		} 
		else if (!(strcmp(script1, "START")))
		{
			char tmp[512];
			strncpy(tmp, script2,   511);
			strncpy(script1, "POS", 511);
			strncpy(script2, tmp+4, 511); // horrible horrible hack, splittes the line again - without START

			if (!(strcmp(script2, "PLAYER")))
			{
			  if (Player!=NULL) st=Player->pos;				 												
			} 
			else Scp->getKeyVal("POS", st);			

			continue;
		}
		else if (!(strcmp(script1, "STARTDELTA")))
		{								
		   sscanf(script2,"%i %i",&dx,&dy);
		}
		else if(!(strcmp(script1,"S")))  dir = 4;		
		else if(!(strcmp(script1,"W")))  dir = 6;	
		else if(!(strcmp(script1,"N")))  dir = 0;
		else if(!(strcmp(script1,"E")))  dir = 2;
		else if(!(strcmp(script1,"NW"))) dir = 7;
		else if(!(strcmp(script1,"NE"))) dir = 1;  
		else if(!(strcmp(script1,"SE"))) dir = 3;
		else if(!(strcmp(script1,"SW"))) dir = 5;

		if (dir!=99)
		{
			scriptedWalkingData dummy; 
			dummy.dir=dir;
			dummy.steps=atoi(script2);
			data.push_back(dummy);
		}
	}

	start.x = st.x+dx;
	start.y = st.y+dy;
	start.map = st.map;
	start.z = Map->Height(st, 0);	

	currpos=start;

	if (data.size()>0) currdir = data[0].dir;

	Scp->Close();
	delete Scp;
	
	/*printf("repeated:%i\n", repeated);
	printf("sx:%i sy:%i sz:%i sm:%i\n",st.x, st.y,st.z,st.map);
	printf("dx:%i dy:%i\n",dx,dy);
	printf("x:%i y:%i z:%i m:%i\n", start.x, start.y, start.z, start.map);
	int a;
	for (a=0; a<data.size(); a++) printf("d:%i s:%i\n", data[a].dir, data[a].steps);
	*/
}

